#!/bin/bash -euET

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

if type hg > /dev/null 2>&1
then
	if [ `hg stat -q | wc -l` -eq 0 ]
	then
		if [ "${1:-}" = '-d' ]
		then
			hg log -r . --template '{date|date}\n' | awk '{printf ("%s %s %2d %s %s %s\n", $1, $2, $3, $4, $6, $5)}'
		elif [ "${1:-}" = '-s' ]
		then
			hg log -r . --template '{date|hgdate}\n' | awk '{printf ("%s\n", $1)}'

		else
			echo `hg log -r . --template '{node}\n'`
		fi
	else
		if [ \( "${1:-}" = '-d' \) -o \( "${1:-}" = '-s' \) ]
		then
			CLEANEXIT=1
			exit 1
		else
			echo '(Repository is in flux and there is no hg revision id for it)'
		fi
	fi
else
	if [ \( "${1:-}" = '-d' \) -o \( "${1:-}" = '-s' \) ]
	then
		CLEANEXIT=1
		exit 1
	else
		echo '(Mercurial not installed)'
		CLEANEXIT=1
		exit 1
	fi
fi

#-----

CLEANEXIT=1
exit 0

:<<=cut

=for comment
Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

=head1 NAME

hgrevid - Prints the Hg changeset id for this version.

=head1 SYNOPSIS

B<hgrevid> [B<-d>] [B<-s>]

=head1 DESCRIPTION

B<hgrevid> prints the hg changeset id for this version, if it is
defined.  Otherwise reports the source code is in flux.

=head1 OPTIONS

=over 7

=item B<-d>

Instead, print the date this version was committed, in a format
suitable for giving to touch -d.  This is useful for giving Hg
checkouts the same mtimes no matter when they are checked out.

=item B<-s>

Instead, print the date this version was committed, in seconds since
the epoch.

=back

=head1 SEE ALSO

L<http://www.workver.com>       the Working Version homepage

=head1 COPYRIGHT AND DISCLAIMERS

Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

This program is distributed in the hope that it will be useful, but
without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.

=head1 LICENSE

All rights reserved.

=head1 AUTHOR

Brian Bartholomew <bb@workver.com>

=cut

