#!/usr/bin/perl

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

use strict;

my $infile = $ARGV[0];

if ($infile eq '') {
	die "Usage: knit infile.knit\n";
}

&Knit::process ($infile);

exit (0);

#-----

package Knit;

$Knit::fmt = 'output format not assigned yet';

sub process {

	my $infile = shift;

	my $inbase;
	my $inknit;
	my $html_file;
	my $latex_file;
	my $sh_file;
	my $html_out;
	my $latex_out;
	my $sh_out;

	$inbase = $infile;
	$inbase =~ s/^(.*)\.knit$/\1/;

	$inknit = $inbase . '.knit';
	$html_file = $inbase . '.html';
	$latex_file = $inbase . '.tex';
	$sh_file = $inbase . '.sh';

	# defines &doc
	require ($inknit);

	$Knit::fmt = 'html';
	$html_out = &doc;

	$Knit::fmt = 'latex';
	$latex_out = &doc;

	$Knit::fmt = 'sh';
	$sh_out = &doc;

	open (HTML, '>', $html_file) || die "FATAL open '$html_file' failed with error '$!'\n";
	open (LATEX, '>', $latex_file) || die "FATAL open '$latex_file' failed with error '$!'\n";
	open (SH, '>', $sh_file) || die "FATAL open '$sh_file' failed with error '$!'\n";
	print HTML $html_out;
	print LATEX $latex_out;
	print SH $sh_out;

	close (HTML);
	close (LATEX);
	close (SH);
}

sub comment {

	my $in = shift;

	return (&h_lit ('<!-- ' . $in . " -->\n") .
		&l_lit ('% ' . $in . "\n") .
		&s_lit ('# ' . $in . "\n"));
}

sub h_lit {

	my $in = shift;

	if ($Knit::fmt eq 'html' ) {
		return ($in);
	}
}

sub l_lit {

	my $in = shift;

	if ($Knit::fmt eq 'latex' ) {
		return ($in);
	}
}

sub s_lit {

	my $in = shift;

	if ($Knit::fmt eq 'sh' ) {
		return ($in);
	}
}

sub percent {

	return (&l_lit ('\%') . &h_lit ('%') . &s_lit ('%'));
}

sub newline {

	my $in = shift;

	return ($in . "\n");
}

sub par {

	my $in = shift;

	return (&h_lit ('<p>' . $in . "</p>\n") .
		&l_lit ($in . "\n") .
		&s_lit ($in . "\n"));
}

sub strong {

	my $in = shift;

	return (&h_lit ('<strong>' . $in . "</strong>") .
		&l_lit ("\\emph{$in}") .
		&s_lit ("*$in*"));
}

sub figure {

	my $inhash = shift;

	return (&h_lit (
'<p><center><table class="figure">
') .
		(($inhash->{'caption'} ne '') ? (
			&h_lit (
'<caption alt="' . $inhash->{'alt'} . '" align="bottom">' . $inhash->{'caption'} . '</caption>
')
		) : (
			''
		)) . 
		&h_lit (
'<tr><td><object data="' . $inhash->{'file'} . '.svg" type="image/svg+xml"></object></td></tr>
</table></center></p>

') .
		&l_lit (
'\begin{figure}[h]
\centering
\includegraphics[width=0.8\textwidth]{' . $inhash->{'file'} . '}
') . 
		(($inhash->{'caption'} ne '') ? (
			&l_lit (
'\caption{' . $inhash->{'caption'} . '}
')
		) : (
			''
		)) . 
		&l_lit (
'\label{' . $inhash->{'label'} . '}
\end{figure}

') .
		&s_lit (''));
}
