#!/bin/bash -euET

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

function print_version {

	echo 'mk-usage-version version @@@ HFVER @@@
Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew
All rights reserved.
Written by Brian Bartholomew.' 1>&2
}

function print_usage {

	cat << 'EOF' 1>&2
Usage: mk-usage-version FILE sh|pl

       FILE    Refers to FILE.pod or FILE.sh
EOF
}

if [ "${1:-}" = '--help' ]
then
	print_version
	echo 1>&2
	print_usage
	CLEANEXIT=1
	exit 0
fi

if [ "${1:-}" = '--version' ]
then
	print_version
	CLEANEXIT=1
	exit 0
fi

#-----

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

POD=$1
GENLANG=$2

BINNAME=`basename $POD`

if [ "$GENLANG" = "sh" ]
then
	cat << END

#-----

function print_version {

	echo '$BINNAME version @@@ HFVER @@@
Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew
All rights reserved.
Written by Brian Bartholomew.' 1>&2
}

function print_usage {

	cat << 'EOF' 1>&2
END
elif [ "$GENLANG" = "pl" ]
then
	cat << END

#-----

sub print_version {

	warn ("$BINNAME version @@@ HFVER @@@
Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew
All rights reserved.
Written by Brian Bartholomew.
");
}

sub print_usage {

	warn << 'EOF'
END
else
	echo "FATAL unknown format $GENLANG" 1>&2
	exit 1
fi

# Prefer "perldoc | col -b | expand" to "pod2tex" because pod2text marks italics with asterisks
perldoc $POD.pod | \
col -b | \
expand | \
awk '
/^SYNOPSIS/, ((/^[A-Z]/) && ($1 != "SYNOPSIS")) {
	if ((!/^[A-Z]/) && (!/^$/)) {
		if (usage != 1) {
			gsub (/^ +/, "")
			print "Usage: " $0
			usage = 1
		} else {
			print $0
		}
	}
}

/^OPTIONS/, ((/^[A-Z]/) && ($1 != "OPTIONS")) {
	if ((!/^[A-Z]/) && (!/^$/)) {
		if (options != 1) {
			print ""
			print $0
			options = 1
		} else {
			print $0
		}
	}
}
'

if [ "$GENLANG" = "sh" ]
then
	if [ -e $POD.help ]
	then
		echo
		cat $POD.help
	fi

	cat << 'END'
EOF
}

if [ "${1:-}" = '--help' ]
then
	print_version
	echo 1>&2
	print_usage
	CLEANEXIT=1
	exit 0
fi

if [ "${1:-}" = '--version' ]
then
	print_version
	CLEANEXIT=1
	exit 0
fi

#-----

END
elif [ "$GENLANG" = "pl" ]
then
	if [ -e $POD.help ]
	then
		echo
		cat $POD.help
	fi

	cat << 'END'
EOF
}

if ($ARGV[0] eq '--help') {
	&print_version;
	warn ("\n");
	&print_usage;
	exit (0);
}

if ($ARGV[0] eq '--version') {
	&print_version;
	exit (0);
}

#-----

END
else
	echo "FATAL unknown format $GENLANG" 1>&2
	exit 1
fi

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#-----

CLEANEXIT=1
exit 0

:<<=cut

=for comment
Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

=head1 NAME

mk-usage-version - Generate bash and perl print_version and print_usage functions from pod.

=head1 SYNOPSIS

B<mk-usage-version> I<foo> B<sh>|B<pl>

=head1 DESCRIPTION

B<mk-usage-version> generates functions in bash and perl to print the
version and the usage, where the usage is taken from the SYNOPSIS and
OPTIONS sections of the pod.

=head1 OPTIONS

=over 7

=item I<foo>

Refers to foo.pod

=back

=head1 SEE ALSO

http://www.workver.com       the Working Version homepage

=head1 COPYRIGHT AND DISCLAIMERS

Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

This program is distributed in the hope that it will be useful, but
without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.

=head1 LICENSE

All rights reserved.

=head1 AUTHOR

Brian Bartholomew <bb@workver.com>

=cut

