#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

RUN_COPROCESS=true
RUN_DIFFDB=true
RUN_DROPDB=true
RUN_FORKDB=true
RUN_HF_BOOTSTRAP_PULL_HFDIST=true
RUN_HF_BOOTSTRAP_PULL_TAR=true
RUN_HF_BOOTSTRAP_PUSH_HFDIST=true
RUN_HF_BOOTSTRAP_PUSH_HFDIST_TEST=true
RUN_HF_BOOTSTRAP_PUSH_RDIST=true
RUN_HF_BOOTSTRAP_PUSH_RSYNC=true
RUN_HF_BOOTSTRAP_PUSH_SCP=true
RUN_HF_BOOTSTRAP_PUSH_TAR=true
RUN_HFCHECK=true
RUN_HF_ENCODE_DECODE=true
RUN_HF=true
RUN_HFGEN=true
RUN_HFLAYERS=true
RUN_HF_RELOADDB=true
RUN_HF_ROOT=true
RUN_HF_SL=true
RUN_HF_VERSION=true
RUN_HF_WINNOW_VERSIONS=true
RUN_LSDB=true
RUN_OE=true
RUN_OEL=true
RUN_OEWATCH=true
RUN_OH_OMAR=true

#  when this script runs tex/hgrevid.tex becomes stale, but we use the
#  old value as a description of the executables running
function whenwasrun() {
	echo "Run on $(date)"
	echo "  with hgrevid $(< tex/hgrevid.tex)"
}

#-----

if [ $(pwd) != $(hg root) ]
then
	echo "FATAL must be run from top level directory, aka hg root" 1>&2
	CLEANEXIT=1
	exit 1
fi

CLEANEXIT=1
echo 'This script must only be run after this sequence of operations:

	hg commit
	make clean
	make
	make install

If all this is true press RET, otherwise to cancel do ^C
'
read -e -p '> ' -i 'yes' YESNO
CLEANEXIT=0
echo

#----- XXX

# wkhtmltopdf --zoom 0.7 byroom.html byroom.pdf
#
# after shrunk down to 14 rows and 'generated by / date time' header is removed
#
# then inkscape to chop away unwanted size, whiteout overflow gray background
# save as pdf with document size set to drawing size

#----- hfdist, sl

if $RUN_HF_SL
then
	echo "#----- hfdist" 1>&2

	# Used for hfdist and sl
	rm -rf /tmp/mkrunoutput /tmp/mkrunoutput2
	mkdir /tmp/mkrunoutput /tmp/mkrunoutput2
	echo foo > /tmp/mkrunoutput/a-file
	mkdir /tmp/mkrunoutput/b-dir
	chmod 755 /tmp/mkrunoutput/b-dir /tmp/mkrunoutput
	chmod 644 /tmp/mkrunoutput/a-file
	touch -t 200109082146.40 /tmp/mkrunoutput/b-dir /tmp/mkrunoutput/a-file /tmp/mkrunoutput

#-- hfdist

	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-flag--version.tex
	echo '{\footnotesize
\begin{verbatim}
$ hfdist --version' >> tex/runoutput/hf-flag--version.tex
	/@@@ HFTOP @@@/bin/hfdist --version 2>&1 | expand | fold >> tex/runoutput/hf-flag--version.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-flag--version.tex

	whenwasrun | sed 's/^/% /' > tex/runoutput/hfdist.tex
	echo '{\footnotesize
\begin{verbatim}
$ hfdist -v 21 -wd mkrunoutput mkrunoutput2' >> tex/runoutput/hfdist.tex
	(cd /tmp ; /@@@ HFTOP @@@/bin/hfdist -v 21 -wd mkrunoutput mkrunoutput2 2>&1) | expand | fold >> tex/runoutput/hfdist.tex
	echo '\end{verbatim}}' >> tex/runoutput/hfdist.tex

#-- sl

	echo "#----- sl" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/sl.tex
	echo '{\footnotesize
\begin{verbatim}
$ sl -k mkrunoutput' >> tex/runoutput/sl.tex
	(cd /tmp ; /@@@ HFTOP @@@/bin/sl -k mkrunoutput 2>&1) | expand | fold >> tex/runoutput/sl.tex
	echo '\end{verbatim}}' >> tex/runoutput/sl.tex
	rm -rf /tmp/mkrunoutput /tmp/mkrunoutput2

#-- sl-sum

	echo "#----- sl-sum" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/sl-sum.tex
	echo '{\footnotesize
\begin{verbatim}
$ sl-sum -k FOO' >> tex/runoutput/sl-sum.tex
	(cd /tmp ; /@@@ HFTOP @@@/bin/sl-sum -k FOO 2>&1) | expand | fold >> tex/runoutput/sl-sum.tex
	echo '\end{verbatim}}' >> tex/runoutput/sl-sum.tex
	rm -rf /tmp/mkrunoutput /tmp/mkrunoutput2

#-- hflog

	echo "#----- hflog" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hflog.tex
	echo '{\footnotesize
\begin{verbatim}
$ hflog log-file XXX' >> tex/runoutput/hflog.tex
	(cd /tmp ; echo XXX /@@@ HFTOP @@@/bin/hflog log-file XXX 2>&1) | expand | fold >> tex/runoutput/hflog.tex
	echo '\end{verbatim}}' >> tex/runoutput/sl.tex
	echo '{\footnotesize
\begin{verbatim}
$ hflog -l log-file XXX' >> tex/runoutput/hflog.tex
	(cd /tmp ; echo XXX /@@@ HFTOP @@@/bin/hflog -l log-file XXX 2>&1) | expand | fold >> tex/runoutput/hflog.tex
	echo '\end{verbatim}}' >> tex/runoutput/hflog.tex
	echo '{\footnotesize
\begin{verbatim}
$ hflog -c log-file XXX' >> tex/runoutput/hflog.tex
	(cd /tmp ; echo XXX /@@@ HFTOP @@@/bin/hflog -c log-file XXX 2>&1) | expand | fold >> tex/runoutput/hflog.tex
	echo '\end{verbatim}}' >> tex/runoutput/hflog.tex
	rm -rf /tmp/mkrunoutput /tmp/mkrunoutput2
fi

#-- hfcheck

if $RUN_HFCHECK
then
	echo "#----- hfcheck" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hfcheck.tex
	echo '{\footnotesize
\begin{verbatim}
$ hfcheck XXX' >> tex/runoutput/hfcheck.tex
	(cd /tmp ; echo XXX /@@@ HFTOP @@@/bin/hfcheck XXX 2>&1) | expand | fold >> tex/runoutput/hfcheck.tex
	echo '\end{verbatim}}' >> tex/runoutput/hfcheck.tex
	rm -rf /tmp/mkrunoutput /tmp/mkrunoutput2
fi

#-- hflayers

if $RUN_HFLAYERS
then
	echo "#----- hflayers" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hflayers.tex
	echo '{\footnotesize
\begin{verbatim}
$ hflayers /src/here/foo /target/there/foo XXX | hfdist -l -' >> tex/runoutput/hflayers.tex
	(cd /tmp ; echo "$ hflayers /src/here/foo /target/there/foo
XXX"2>&1) | expand | fold >> tex/runoutput/hflayers.tex
	echo '\end{verbatim}}' >> tex/runoutput/hflayers.tex
fi

#-- hf

if $RUN_HF
then
	echo "#----- hf" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf /src/here/foo /target/there/foo XXX | hfdist -l -' >> tex/runoutput/hf.tex
	(cd /tmp ; echo "$ hf
XXX"2>&1) | expand | fold >> tex/runoutput/hf.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf.tex
fi

#----- oh, omar

if $RUN_OH_OMAR
then
	# Used for oh and omar
	rm -rf /tmp/mkrunoutput3
	mkdir /tmp/mkrunoutput3
	chmod 755 /tmp/mkrunoutput3
	touch -t 200109082146.40 /tmp/mkrunoutput3

#-- oh

	echo "#----- oh" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/oh.tex
	whenwasrun | sed 's/^/#	/' > tex/runoutput/oh-doc.sh
	echo '#' >> tex/runoutput/oh-doc.sh

	MSG='init -- creates an empty Hg repository and omar
        support to allow version control of a Linux root partition filetree'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#    /' >> tex/runoutput/oh-doc.sh
	echo '{\footnotesize
\begin{verbatim}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh
	MSG='$ oh init'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	(cd /tmp/mkrunoutput3 ; /@@@ HFTOP @@@/bin/oh init 2>&1) | tee /dev/tty | expand | fold | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	echo '\end{verbatim}}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh

	MSG='wrap -- massage the filetree so that it is ready for an Hg commit'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#    /' >> tex/runoutput/oh-doc.sh
	echo '#' >> tex/runoutput/oh-doc.sh
	echo '{\footnotesize
\begin{verbatim}' >> tex/runoutput/oh.tex
	MSG='$ touch oh/foo oh/bar
$ mkdir oh/baz
$ oh wrap'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	(cd /tmp/mkrunoutput3 ; touch oh/foo oh/bar ; mkdir oh/baz ; /@@@ HFTOP @@@/bin/oh wrap 2>&1) | tee /dev/tty | expand | fold | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	echo '\end{verbatim}}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh

	MSG='status -- what state is the repository in?'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#    /' >> tex/runoutput/oh-doc.sh
	echo '#' >> tex/runoutput/oh-doc.sh
	echo '{\footnotesize
\begin{verbatim}' >> tex/runoutput/oh.tex
	MSG='$ oh status'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	(cd /tmp/mkrunoutput3 ; /@@@ HFTOP @@@/bin/oh status 2>&1) | tee /dev/tty | expand | fold | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	echo '\end{verbatim}}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh

	MSG='commit -- do the Hg commit'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#    /' >> tex/runoutput/oh-doc.sh
	echo '{\footnotesize
\begin{verbatim}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh
	MSG='$ oh commit '"'"'Initial checkin'"'"
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	(cd /tmp/mkrunoutput3 ; /@@@ HFTOP @@@/bin/oh commit 'Initial checkin' 2>&1) | tee /dev/tty | expand | fold | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	echo '\end{verbatim}}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh


	MSG='unwrap -- massage the filetree so that it is ready to have files written'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#    /' >> tex/runoutput/oh-doc.sh
	echo '{\footnotesize
\begin{verbatim}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh
	MSG='$ oh unwrap'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	(cd /tmp/mkrunoutput3 ; /@@@ HFTOP @@@/bin/oh unwrap 2>&1) | tee /dev/tty | expand | fold | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	echo '\end{verbatim}}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh

	MSG='fix -- make working set files consistent with present hg revision'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#    /' >> tex/runoutput/oh-doc.sh
	echo '{\footnotesize
\begin{verbatim}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh
	MSG='$ rm -r oh/baz   # oops!
$ oh fix -y'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	(cd /tmp/mkrunoutput3 ; rm -r oh/baz ; /@@@ HFTOP @@@/bin/oh fix -y 2>&1) | tee /dev/tty | expand | fold | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	echo '\end{verbatim}}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh

	MSG='snapshot -- wrap, commit, unwrap'
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#    /' >> tex/runoutput/oh-doc.sh
	echo '{\footnotesize
\begin{verbatim}' >> tex/runoutput/oh.tex
	echo '#' >> tex/runoutput/oh-doc.sh
	MSG='$ oh unwrap
[ ...output deleted... ]
$ touch oh/a1 oh/b1 oh/c1
$ oh snapshot '"'"'More mods'"'"
	echo "$MSG" | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	(cd /tmp/mkrunoutput3 ; /@@@ HFTOP @@@/bin/oh unwrap > /dev/null 2>&1 ; touch oh/a1 oh/b1 oh/c1 ; /@@@ HFTOP @@@/bin/oh snapshot 'More mods' 2>&1) | tee /dev/tty | expand | fold | tee -a tex/runoutput/oh.tex | sed 's/^/#	/' >> tex/runoutput/oh-doc.sh
	echo '\end{verbatim}}' >> tex/runoutput/oh.tex

#-- omar

	echo "#----- omar" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/omar.tex
	echo '{\footnotesize
\begin{verbatim}
$ omar -cvf ../filelist.omar .' >> tex/runoutput/omar.tex
	(cd /tmp/mkrunoutput3/oh ; /@@@ HFTOP @@@/bin/omar -cvf ../filelist.omar . 2>&1) | expand | fold >> tex/runoutput/omar.tex
	echo '\end{verbatim}}' >> tex/runoutput/omar.tex
	echo '{\footnotesize
\begin{verbatim}
$ cat ../filelist.omar' >> tex/runoutput/omar.tex
	(cd /tmp/mkrunoutput3/oh ; cat ../filelist.omar 2>&1) | expand | fold >> tex/runoutput/omar.tex
	echo '\end{verbatim}}' >> tex/runoutput/omar.tex

	rm -rf /tmp/mkrunoutput3
fi

#----- coprocess

if $RUN_COPROCESS
then
	echo "#----- coprocess" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/coprocess.tex
	echo '{\footnotesize
\begin{verbatim}
$ coprocess XXX YYY' >> tex/runoutput/coprocess.tex
	echo /@@@ HFTOP @@@/bin/coprocess XXX YYY 2>&1 | tee /dev/tty | expand | fold >> tex/runoutput/coprocess.tex
	echo '\end{verbatim}}' >> tex/runoutput/coprocess.tex
fi

#----- hfgen

if $RUN_HFGEN
then
	echo "#----- hfgen" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hfgen.tex
	echo '{\footnotesize
\begin{verbatim}
$ hfgen' >> tex/runoutput/hfgen.tex
	(cd gen ; /@@@ HFTOP @@@/bin/hfgen 2>&1) | tee /dev/tty | expand | fold >> tex/runoutput/hfgen.tex
	echo '\end{verbatim}}' >> tex/runoutput/hfgen.tex
fi

#----- hfcheck

if $RUN_HFCHECK
then
	echo "#----- hfcheck" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hfcheck.tex
	echo '{\footnotesize
\begin{verbatim}
$ hfcheck XXX' >> tex/runoutput/hfcheck.tex
	(cd gen ; echo /@@@ HFTOP @@@/bin/hfcheck XXX 2>&1) | tee /dev/tty | expand | fold >> tex/runoutput/hfcheck.tex
	echo '\end{verbatim}}' >> tex/runoutput/hfcheck.tex
fi

#----- oe

if $RUN_OE
then
	echo "#----- oe" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/oe.tex
	echo '{\footnotesize
\begin{verbatim}
$ oe -j 6 _fork10 bash << '"'"EOF"'"'
> num=`expr $RANDOM / 8000` ; echo sleep $num ; sleep $num ; exit $num
> EOF' >> tex/runoutput/oe.tex
	/@@@ HFTOP @@@/bin/oe -j 6 _fork10 bash << 'EOF' 2>&1 | tee /dev/tty | expand | fold >> tex/runoutput/oe.tex
num=`expr $RANDOM / 8000` ; echo sleep $num ; sleep $num ; exit $num
EOF
	echo '\end{verbatim}}' >> tex/runoutput/oe.tex
fi

#----- oel

if $RUN_OEL
then
	echo "#----- oel" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/oel.tex
	echo '{\footnotesize
\begin{verbatim}
$ oel' >> tex/runoutput/oel.tex
	/@@@ HFTOP @@@/bin/oel 2>&1 | expand | fold >> tex/runoutput/oel.tex
	echo '\end{verbatim}}' >> tex/runoutput/oel.tex

	whenwasrun | sed 's/^/% /' > tex/runoutput/oelserver.tex
	echo '{\footnotesize
\begin{verbatim}
$ oel server' >> tex/runoutput/oelserver.tex
	/@@@ HFTOP @@@/bin/oel server 2>&1 | expand | fold >> tex/runoutput/oelserver.tex
	echo '\end{verbatim}}' >> tex/runoutput/oelserver.tex

	whenwasrun | sed 's/^/% /' > tex/runoutput/oelfork.tex
	echo '{\footnotesize
\begin{verbatim}
$ oel _fork localhost3 blah' >> tex/runoutput/oelfork.tex
	/@@@ HFTOP @@@/bin/oel _fork localhost3 blah 2>&1 | expand | fold >> tex/runoutput/oelfork.tex
	echo '\end{verbatim}}' >> tex/runoutput/oelfork.tex

	whenwasrun | sed 's/^/% /' > tex/runoutput/oelbig.tex
	echo "{\footnotesize
\begin{verbatim}
$ oel 'a b' 'c -d' ix -ix e 'localhost foofile' server -gear -client" >> tex/runoutput/oelbig.tex
	/@@@ HFTOP @@@/bin/oel 'a b' 'c -d' ix -ix e 'localhost foofile' server -gear -client 2>&1 | expand | fold >> tex/runoutput/oelbig.tex
	echo '\end{verbatim}}' >> tex/runoutput/oelbig.tex
fi

#----- oewatch

if $RUN_OEWATCH
then
	echo "#----- oewatch" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/oewatch.tex
	echo '{\footnotesize
\begin{verbatim}
XXX' >> tex/runoutput/oe.tex
	echo /@@@ HFTOP @@@/bin/oewatch XXX 2>&1 | tee /dev/tty | expand | fold >> tex/runoutput/oewatch.tex
	echo '\end{verbatim}}' >> tex/runoutput/oewatch.tex
fi

#----- hf-reloaddb

if $RUN_HF_RELOADDB
then
	echo "#----- hf-reloaddb" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-reloaddb.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-reloaddb -y' >> tex/runoutput/hf-reloaddb.tex
	/@@@ HFTOP @@@/bin/hf-reloaddb -y 2>&1 | tee /dev/tty | head -20 | expand | fold >> tex/runoutput/hf-reloaddb.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-reloaddb.tex
	echo '\emph{[... mysqldump output deleted ...]}' >> tex/runoutput/hf-reloaddb.tex
fi

#----- hf-encode

if $RUN_HF_ENCODE_DECODE
then
	echo "#----- hf-decode" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-decode.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-decode < FOO XXX' >> tex/runoutput/hf-decode.tex
	/@@@ HFTOP @@@/bin/hf-decode < FOO XXX 2>&1 | tee /dev/tty | head -20 | expand | fold >> tex/runoutput/hf-decode.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-decode.tex

#----- hf-decode

	echo "#----- hf-encode" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-encode.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-encode < FOO XXX' >> tex/runoutput/hf-encode.tex
	/@@@ HFTOP @@@/bin/hf-encode < FOO XXX 2>&1 | tee /dev/tty | head -20 | expand | fold >> tex/runoutput/hf-encode.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-encode.tex
fi

#----- lsdb

if $RUN_LSDB
then
	echo "#----- lsdb" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/lsdb.tex
	echo '{\footnotesize
\begin{verbatim}
$ lsdb' >> tex/runoutput/lsdb.tex
	/@@@ HFTOP @@@/bin/lsdb 2>&1 | expand | fold >> tex/runoutput/lsdb.tex
	echo '\end{verbatim}}' >> tex/runoutput/lsdb.tex
fi

#----- forkdb

if $RUN_FORKDB
then
	echo "#----- forkdb" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/forkdb.tex
	echo '{\footnotesize
\begin{verbatim}
$ forkdb hfexample1 hfexample2' >> tex/runoutput/forkdb.tex
	/@@@ HFTOP @@@/bin/forkdb hfexample1 hfexample2 2>&1 | tee /dev/tty | head -20 | expand | fold >> tex/runoutput/forkdb.tex
	echo '\end{verbatim}}' >> tex/runoutput/forkdb.tex
	echo '\emph{[... mysqldump output deleted ...]}' >> tex/runoutput/forkdb.tex
fi

#----- diffdb

if $RUN_DIFFDB
then
	echo "#----- diffdb" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/diffdb.tex
	echo '{\footnotesize
\begin{verbatim}
$ diffdb hfexample1 hfexample2' >> tex/runoutput/diffdb.tex
	/@@@ HFTOP @@@/bin/diffdb hfexample1 hfexample2 2>&1 | expand | fold >> tex/runoutput/diffdb.tex
	echo '\end{verbatim}}' >> tex/runoutput/diffdb.tex
fi

#----- dropdb

if $RUN_DROPDB
then
	echo "#----- dropdb" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/dropdb.tex
	echo '{\footnotesize
\begin{verbatim}
$ dropdb hfexample2' >> tex/runoutput/dropdb.tex
	/@@@ HFTOP @@@/bin/dropdb hfexample2 2>&1 | expand | fold >> tex/runoutput/dropdb.tex
	echo '\end{verbatim}}' >> tex/runoutput/dropdb.tex
fi

#----- hf-version

if $RUN_HF_VERSION
then
	echo "#----- hf-version" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-version.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-version' >> tex/runoutput/hf-version.tex
	/@@@ HFTOP @@@/bin/hf-version 2>&1 | expand | fold >> tex/runoutput/hf-version.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-version.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-version -a' >> tex/runoutput/hf-version.tex
	/@@@ HFTOP @@@/bin/hf-version -a 2>&1 | expand | fold >> tex/runoutput/hf-version.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-version.tex
fi


#----- hf-root

if $RUN_HF_ROOT
then
	echo "#----- hf-root" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-root.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-root' >> tex/runoutput/hf-root.tex
	/@@@ HFTOP @@@/bin/hf-root 2>&1 | expand | fold >> tex/runoutput/hf-root.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-root.tex
fi

#----- hf-winnow-versions

if $RUN_HF_WINNOW_VERSIONS
then
	echo "#----- hf-winnow-versions" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-winnow-versions.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-winnow-versions' >> tex/runoutput/hf-winnow-versions.tex
	/@@@ HFTOP @@@/bin/hf-winnow-versions 2>&1 | expand | fold >> tex/runoutput/hf-winnow-versions.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-winnow-versions.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-winnow-versions -d' >> tex/runoutput/hf-winnow-versions.tex
	/@@@ HFTOP @@@/bin/hf-winnow-versions -d 2>&1 | expand | fold >> tex/runoutput/hf-winnow-versions.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-winnow-versions.tex
fi

#----- hf-bootstrap-pull-hfdist

if $RUN_HF_BOOTSTRAP_PULL_HFDIST
then
	echo "#----- hf-bootstrap-pull-hfdist" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-bootstrap-pull-hfdist.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-bootstrap-pull-hfdist' >> tex/runoutput/hf-bootstrap-pull-hfdist.tex
	echo '[ XXX omitted ]' 2>&1 | expand | fold >> tex/runoutput/hf-bootstrap-pull-hfdist.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-bootstrap-pull-hfdist.tex
	echo '\emph{[... hf output deleted ...]}' >> tex/runoutput/hf-bootstrap-pull-hfdist.tex
fi

#----- hf-bootstrap-pull-tar

if $RUN_HF_BOOTSTRAP_PULL_TAR
then
	echo "#----- hf-bootstrap-pull-tar" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-bootstrap-pull-tar.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-bootstrap-pull-tar' >> tex/runoutput/hf-bootstrap-pull-tar.tex
	echo '[ XXX omitted ]' 2>&1 | expand | fold >> tex/runoutput/hf-bootstrap-pull-tar.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-bootstrap-pull-tar.tex
	echo '\emph{[... tar output deleted ...]}' >> tex/runoutput/hf-bootstrap-pull-tar.tex
fi

#----- hf-bootstrap-push-hfdist

if $RUN_HF_BOOTSTRAP_PUSH_HFDIST
then
	echo "#----- hf-bootstrap-push-hfdist" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-bootstrap-push-hfdist.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-bootstrap-push-hfdist' >> tex/runoutput/hf-bootstrap-push-hfdist.tex
	echo '[ XXX omitted ]' 2>&1 | expand | fold >> tex/runoutput/hf-bootstrap-push-hfdist.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-bootstrap-push-hfdist.tex
	echo '\emph{[... hf output deleted ...]}' >> tex/runoutput/hf-bootstrap-push-hfdist.tex
fi

#----- hf-bootstrap-push-hfdistdist-test

if $RUN_HF_BOOTSTRAP_PUSH_HFDIST_TEST
then
	echo "#----- hf-bootstrap-push-hfdist-test" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-bootstrap-push-hfdist-test.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-bootstrap-push-hfdist-test' >> tex/runoutput/hf-bootstrap-push-hfdist-test.tex
	echo '[ XXX omitted ]' 2>&1 | expand | fold >> tex/runoutput/hf-bootstrap-push-hfdist-test.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-bootstrap-push-hfdist-test.tex
fi

#----- hf-bootstrap-push-rsync

if $RUN_HF_BOOTSTRAP_PUSH_RSYNC
then
	echo "#----- hf-bootstrap-push-rsync" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-bootstrap-push-rsync.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-bootstrap-push-rsync' >> tex/runoutput/hf-bootstrap-push-rsync.tex
	echo '[ XXX omitted ]' 2>&1 | expand | fold >> tex/runoutput/hf-bootstrap-push-rsync.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-bootstrap-push-rsync.tex
	echo '\emph{[... rsync output deleted ...]}' >> tex/runoutput/hf-bootstrap-push-rsync.tex
fi

#----- hf-bootstrap-push-tar

if $RUN_HF_BOOTSTRAP_PUSH_TAR
then
	echo "#----- hf-bootstrap-push-tar" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-bootstrap-push-tar.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-bootstrap-push-tar' >> tex/runoutput/hf-bootstrap-push-tar.tex
	echo '[ XXX omitted ]' 2>&1 | expand | fold >> tex/runoutput/hf-bootstrap-push-tar.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-bootstrap-push-tar.tex
	echo '\emph{[... tar output deleted ...]}' >> tex/runoutput/hf-bootstrap-push-tar.tex
fi

#----- hf-bootstrap-push-rdist

if $RUN_HF_BOOTSTRAP_PUSH_RDIST
then
	echo "#----- hf-bootstrap-push-rdist" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-bootstrap-push-rdist.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-bootstrap-push-rdist' >> tex/runoutput/hf-bootstrap-push-rdist.tex
	echo '[ XXX omitted ]' 2>&1 | expand | fold >> tex/runoutput/hf-bootstrap-push-rdist.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-bootstrap-push-rdist.tex
	echo '\emph{[... rdist output deleted ...]}' >> tex/runoutput/hf-bootstrap-push-rdist.tex
fi

#----- hf-bootstrap-push-scp

if $RUN_HF_BOOTSTRAP_PUSH_SCP
then
	echo "#----- hf-bootstrap-push-scp" 1>&2
	whenwasrun | sed 's/^/% /' > tex/runoutput/hf-bootstrap-push-scp.tex
	echo '{\footnotesize
\begin{verbatim}
$ hf-bootstrap-push-scp' >> tex/runoutput/hf-bootstrap-push-scp.tex
	echo '[ XXX omitted ]' 2>&1 | expand | fold >> tex/runoutput/hf-bootstrap-push-scp.tex
	echo '\end{verbatim}}' >> tex/runoutput/hf-bootstrap-push-scp.tex
	echo '\emph{[... scp output deleted ...]}' >> tex/runoutput/hf-bootstrap-push-scp.tex
fi
