#!/usr/bin/perl

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

use Getopt::Std;

if ($ARGV[0] eq '--version') {

	&version;
	exit (0);
}

if ($ARGV[0] eq '--help') {

	&version;
	warn ("\n");
	&usage;
	exit (0);
}

if (!(getopts ('d:'))) {
	&usage;
	exit (1);
}

if (($opt_d ne '') && ($opt_d ne 'oracle8') && ($opt_d ne 'mysql3') &&
	($opt_d ne 'mysql4') && ($opt_d ne 'mysql5')) {
	&usage;
	exit (1);
}

open (TEXSQL, ">&3") || die "FATAL must redirect fd 3 somewhere, quitting\n";
open (DOT, ">&4") || die "FATAL must redirect fd 4 somewhere, quitting\n";
open (TEXTBL, ">&5") || die "FATAL must redirect fd 5 somewhere, quitting\n";
open (PERLTBL, ">&6") || die "FATAL must redirect fd 6 somewhere, quitting\n";

#----- Read the rdb tables into perl tables -----------------------------------

foreach $file (@ARGV) {

	$tablename = $file;
	$tablename =~ s/\.tbl$//;
	$tablename =~ s|^.*/||;

	push (@tablelist, $tablename);

	$rowcount = 0;
	open (FILE, "column bytes index name type totbl tocol description < $file | headchg -del |") || die "FATAL could not run column with error '$!', quitting";
	while (<FILE>) {
		$line = $_;
		chop ($line);
		@fields = split (/\t/, $line, 7);
		$evalline = "\$tables{\"$tablename,$rowcount\"} = [ \@fields ];";
		eval ($evalline);
		$rowcount++;
	}
	close (FILE);
}

#----- Before walking any tables ----------------------------------------------

&textblprint('%
%  Mechanically generated, edits will be lost
%

');

&dotprint('#
#  Mechanically generated, edits will be lost
#

digraph g {
        graph [size="7.5,8.0",margin="0.5",rankdir=LR];
	node [shape=record];

');

&perltblprint('#
#  Mechanically generated, edits will be lost
#

package HF::Schema;
use strict;

#
#  @foo = @{$HF::Schema::schema{"hosts"}{"columns"}};
#  print "hosts columns are @foo\n";
#
#  print "bytes of name column are " .
#	$HF::Schema::schema{"hosts"}{"table"}{"name"}{"bytes"} . "\n";
#
#  @tablelist = sort (keys (%HF::Schema::schema));
#  print "tables are @foo\n";
#

');

@schema = ();

#----- For all tables: pass one, delete foreign keys --------------------------

if ($opt_d eq 'oracle8') {
	&texsqlprint ("
\\subsection{deleting foreign keys}
");
}

foreach $tablename (@tablelist) {

	@deleteforeign = ();
	$rowcount = 0;
	while (exists ($tables{"$tablename,$rowcount"})) {

		@fields = @{$tables{"$tablename,$rowcount"}};

#----- For each row in pass one -----------------------------------------------

		if ($fields[1] =~ /f/) {	# foreign key
			if (($fields[4] ne '') && ($fields[5] ne '')) {
				push (@deleteforeign, sprintf ("alter table %s
        drop constraint %s_%s_constraint;\n",
					$tablename, $tablename, $fields[2],
					$fields[2], $fields[4], $fields[5]));

			} else {
				die "FATAL empty table or column field for foreign key, quitting";
			}
		}

		$rowcount++;
	}

#----- After each table in pass one -------------------------------------------

	if ($#deleteforeign >= 0) {
		if ($opt_d eq 'oracle8') {
			&sqlprint (join ("\n", @deleteforeign) . "\n");

			&texsqlprint ("
\\subsubsection{$tablename table}

");
			&texsqlcode (join ("\n", @deleteforeign));
		}
	}
}

#----- For all tables: pass two, remove primary keys --------------------------

if ($opt_d eq 'oracle8') {
	&texsqlprint ("
\\subsection{deleting primary keys}
");
}

foreach $tablename (@tablelist) {

	@deleteprimary = ();
	$rowcount = 0;
	while (exists ($tables{"$tablename,$rowcount"})) {

		@fields = @{$tables{"$tablename,$rowcount"}};

#----- For each row in pass two -----------------------------------------------

		if ($fields[1] =~ /k/) {	# primary key
			push (@deleteprimary, sprintf ("alter table %s
        drop primary key;\n\ndrop sequence %s_seq;\n",
				$tablename, $tablename));

		}

		$rowcount++;
	}

#----- After each table in pass two -------------------------------------------

	if ($#deleteprimary >= 0) {
		if ($opt_d eq 'oracle8') {
			&sqlprint (join ("\n", @deleteprimary) . "\n");

			&texsqlprint ("
\\subsubsection{$tablename table}

");
			&texsqlcode (join ("\n", @deleteprimary));
		}
	}
}

#----- For all tables: pass three, tables -------------------------------------

&texsqlprint ("
\\subsection{creating tables}
");

foreach $tablename (@tablelist) {

	if (($opt_d eq 'oracle8') || ($opt_d eq 'mysql3')) {
		$createtable = "drop table $tablename;\n";
	} elsif (($opt_d eq 'mysql4') || ($opt_d eq 'mysql5')) {
		$createtable = "drop table if exists $tablename;\n";
	} else {
		die "can't happen";
	}
$createtable .= "
create table $tablename (
";

	&textblprint("
\\subsection{$tablename table}

\\footnotesize{
\\begin{tabular}{rp{0.2in}p{0.75in}p{0.6in}p{0.52in}p{0.52in}p{1.8in}}
\\toprule
\Bytes & Idx & Name & Type & Points to Table & Points to Column & Description \\\\
\\midrule
");

	&dotprint (sprintf ("\ttable_%s [label=\"%s table", $tablename, $tablename));

#----- For each row in pass three ---------------------------------------------

	$bytes = 0;
	$rowcount = 0;
	@createcolumns = ();
	$keycolumn = '';
	@schema_columns = ();
	@schema_table = ();
	while (exists ($tables{"$tablename,$rowcount"})) {

		@fields = @{$tables{"$tablename,$rowcount"}};

# XXX add the documentation to the column?
		if ($opt_d eq 'oracle8') {
			push (@createcolumns, sprintf ("        %-20s %-20s default null",
				$fields[2], $fields[3]));
		} elsif ($opt_d eq 'mysql3') {
			if ($fields[1] =~ /k/) {	# primary key
				push (@createcolumns, sprintf ("        %-20s %-20s not null auto_increment primary key",
					$fields[2], $fields[3]));
			} else {
				push (@createcolumns, sprintf ("        %-20s %-20s default null",
					$fields[2], $fields[3]));
			}
		} elsif ($opt_d eq 'mysql4') {
			if ($fields[1] =~ /k/) {	# primary key
				push (@createcolumns, sprintf ("        %-20s %-20s not null primary key",
					$fields[2], $fields[3]));
			} else {
				push (@createcolumns, sprintf ("        %-20s %-20s ",
					$fields[2], $fields[3]));
			}
		} elsif ($opt_d eq 'mysql5') {
			if ($fields[1] =~ /k/) {	# primary key
				push (@createcolumns, sprintf ("        %-20s %-20s not null",
					$fields[2], $fields[3]));
				$keycolumn = sprintf ("        primary key (%s)", $fields[2]);
			} else {
				push (@createcolumns, sprintf ("        %-20s %-20s ",
					$fields[2], $fields[3]));
			}
		} else {
			die "FATAL can't happen, quitting";
		}

		&dotprint (sprintf ("|<%s> %s", $fields[2], $fields[2]));

		$bytes += $fields[0];
		@texfields = map { s/_/\\_/g ; $_ } @fields;
		&textblprint (join (" & ", @texfields) . " \\\\\n");

		push (@schema_columns, "\t\t\t'$fields[2]'");
		push (@schema_table, "\t\t\t'$fields[2]' => {
				'bytes' => '$fields[0]',
				'index' => '$fields[1]',
				'type' => '$fields[3]',
				'totbl' => '$fields[4]',
				'tocol' => '$fields[5]',
				'description' => q{$fields[6]}
			}");

		$rowcount++;
	}

#----- After each table in pass three -----------------------------------------

	if (($opt_d eq 'mysql5') && ($keycolumn ne '')) {
		push (@createcolumns, $keycolumn);
	}

	$createtable .= join (",\n", @createcolumns) . "\n)";

	if (($opt_d eq 'oracle8') || ($opt_d eq 'mysql3')) {
		$createtable .= ";\n";
	} elsif (($opt_d eq 'mysql4') || ($opt_d eq 'mysql5')) {
###		$createtable .= " engine = InnoDB;\n";
#  Switch to MyISAM to get information_schema.tables.update_time working.
		$createtable .= " engine = MyISAM;\n";
	} else {
		die "can't happen";
	}

	&sqlprint ("$createtable\n");

	&texsqlprint ("
\\subsubsection{$tablename table}

");
	&texsqlcode ($createtable);

	&dotprint ("\"];\n");

	&textblprint (sprintf ('\midrule
%d & & & & & & \\\\
\bottomrule
\end{tabular}
}

', $bytes));

	push (@schema, "\t'$tablename' => {
		'columns' => [
" . join (",\n", @schema_columns) . "
		],
		'table' => {
" . join (",\n", @schema_table) . '
		}
	}');
}

#----- For all tables: pass four, indices -------------------------------------

&texsqlprint ("
\\subsection{creating indices}
");

foreach $tablename (@tablelist) {

	@createindices = ();
	$rowcount = 0;
	while (exists ($tables{"$tablename,$rowcount"})) {

		@fields = @{$tables{"$tablename,$rowcount"}};

#----- For each row in pass four ----------------------------------------------

		if ($fields[1] =~ /i/) {	# index

			#  Indices get dropped when table is replaced earlier
			push (@createindices, sprintf ("create index %s_%s_index on %s (%s);\n",
				$tablename, $fields[2],
				$tablename, $fields[2],
				$tablename, $fields[2]));
		}

		$rowcount++;
	}

#----- After each table in pass four ------------------------------------------

	if ($#createindices >= 0) {
		if ($opt_d eq 'oracle8') {
			&sqlprint (join ("\n", @createindices) . "\n");

			&texsqlprint ("
\\subsubsection{$tablename table}

");
			&texsqlcode (join ("\n", @createindices));
		}
	}
}

#----- For all tables: pass five, primary keys --------------------------------

if ($opt_d eq 'oracle8') {
	&texsqlprint ("
\\subsection{creating primary keys}
");
}

foreach $tablename (@tablelist) {

	@createprimary = ();
	$rowcount = 0;
	while (exists ($tables{"$tablename,$rowcount"})) {

		@fields = @{$tables{"$tablename,$rowcount"}};

#----- For each row in pass five ----------------------------------------------

		if ($fields[1] =~ /k/) {	# primary key
			push (@createprimary, sprintf ("alter table %s
        add primary key (%s);\n\ncreate sequence %s_seq nocycle;\n",
				$tablename, $fields[2], $tablename));

		}

		$rowcount++;
	}

#----- After each table in pass five ------------------------------------------

	if ($#createprimary >= 0) {
		if ($opt_d eq 'oracle8') {
			&sqlprint (join ("\n", @createprimary) . "\n");

			&texsqlprint ("
\\subsubsection{$tablename table}

");
			&texsqlcode (join ("\n", @createprimary));
		}
	}
}

#----- For all tables: pass six, foreign keys ---------------------------------

if ($opt_d eq 'oracle8') {
	&texsqlprint ("
\\subsection{creating foreign keys}
");
}

foreach $tablename (@tablelist) {

	@createforeign = ();
	$rowcount = 0;
	while (exists ($tables{"$tablename,$rowcount"})) {

		@fields = @{$tables{"$tablename,$rowcount"}};

#----- For each row in pass six -----------------------------------------------

		if ($fields[1] =~ /f/) {	# foreign key
			if (($fields[4] ne '') && ($fields[5] ne '')) {
				push (@createforeign, sprintf ("alter table %s
        add constraint %s_%s_constraint
        foreign key (%s) references %s(%s);\n",
					$tablename, $tablename, $fields[2],
					$fields[2], $fields[4], $fields[5]));

				&dotprint (sprintf ("\ttable_%s:%s -> table_%s:%s [dir=\"back\"];\n",
					$fields[4], $fields[5],
					$tablename, $fields[2]));
			} else {
				die "FATAL empty table or column field for foreign key, quitting";
			}
		}

		$rowcount++;
	}

#----- After each table in pass six -------------------------------------------

	if ($#createforeign >= 0) {
		if ($opt_d eq 'oracle8') {
			&sqlprint (join ("\n", @createforeign) . "\n");

			&texsqlprint ("
\\subsubsection{$tablename table}

");
			&texsqlcode (join ("\n", @createforeign));
		}
	}
}

#----- After all table walks --------------------------------------------------

&dotprint('}
');

&perltblprint ('%HF::Schema::schema = (
' . join (",\n", @schema) . '
);

1;
');

#----- Subroutines ------------------------------------------------------------

sub sqlprint {

	my ($string) = @_;

	print STDOUT $string;
}

#------------------------------------------------------------------------------

sub texsqlprint {

	my ($string) = @_;

	print TEXSQL $string;
}

#------------------------------------------------------------------------------

sub texsqlcode {

	my ($string) = @_;

	print TEXSQL "\\begin{quote}
\\begin{verbatim}
${string}\\end{verbatim}
\\end{quote}
";
}

#------------------------------------------------------------------------------

sub dotprint {

	my ($string) = @_;

	print DOT $string;
}

#------------------------------------------------------------------------------

sub textblprint {

	my ($string) = @_;

	print TEXTBL $string;
}

#------------------------------------------------------------------------------

sub perltblprint {

	my ($string) = @_;

	print PERLTBL $string;
}

#------------------------------------------------------------------------------

sub usage {

	warn 'Usage: tblburst [-d oracle8|mysql3|mysql4|mysql5] *.tbl > ddl1.sql \
	3> ddl1.tex 4> tbl.dot 5> tbl.tex 6> tbl.pl

	--help
	--version

SQL on fd 1
SQL in LaTeX on fd 3
graphviz dot on fd 4
LaTeX tables on fd 5
perl tables on fd 6
';
}

#------------------------------------------------------------------------------

sub version {

	warn ("tblburst version 3.1
Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew
All rights reserved.
Written by Brian Bartholomew.
");
}
