#!/bin/bash -euET

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#
#  Usage:
#
#	client-standalone-precise-smaller.sh
#

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

# Add new software respositories

###add-apt-repository -y 'deb http://packages.medibuntu.org/ precise free non-free'
add-apt-repository -y 'deb http://archive.canonical.com/ubuntu precise partner'

# add-apt-repository creates failure during apt-get update, sources aren't available here, deb-src line is wrong
###### add-apt-repository -y 'deb http://download.virtualbox.org/virtualbox/debian precise contrib non-free'
if ! grep '^deb http://download.virtualbox.org/virtualbox/debian precise contrib non-free$' /etc/apt/sources.list > /dev/null
then
	echo 'deb http://download.virtualbox.org/virtualbox/debian precise contrib non-free' >> /etc/apt/sources.list
fi

# Add new software respositories for gnome classic

add-apt-repository -y ppa:gnome3-team/gnome3
add-apt-repository -y ppa:ferramroberto/gnome3

# Add PGP keys

###wget -q http://packages.medibuntu.org/medibuntu-key.gpg -O - | apt-key add -

wget -q http://download.virtualbox.org/virtualbox/debian/oracle_vbox.asc -O- | apt-key add -

# Get updates since the cd

apt-get -y update
apt-get -y dist-upgrade

#-----

# ssh adjustments
sed \
	-e 's/#   ForwardX11 no/    ForwardX11 yes/' \
	-e 's/#   ForwardX11Trusted yes/    ForwardX11Trusted yes/' \
	-e 's/HashKnownHosts yes/HashKnownHosts no/' \
	< /etc/ssh/ssh_config > /tmp/ssh_config
cat /tmp/ssh_config > /etc/ssh/ssh_config

# Prompt for license approvals
apt-get -y install ubuntu-restricted-extras

# gnome classic
apt-get -y install gnome-shell gnome-shell-extensions-common gnome-tweak-tool

# Switch from Unity user interface to gnome
sed 's/^user-session=ubuntu$/user-session=gnome-classic/' < /etc/lightdm/lightdm.conf > /tmp/pp
cat /tmp/pp > /etc/lightdm/lightdm.conf
rm /tmp/pp

apt-get -y install emacs23 emacs23-el

apt-get -y install auctex

apt-get -y install texlive texlive-doc-en dvipdfmx lmodern tipa prosper latex-beamer latex-xcolor texpower texpower-manual texpower-examples texlive-generic-extra cm-super texlive-latex-extra texlive-full

# Paper size
texconfig-sys dvips paper letter
texconfig-sys pdftex paper letter
# took 'overwrite with maintainer's version' choice
texconfig-sys dvipdfm paper letter
texconfig-sys xdvi paper us
texconfig-sys mode ljfour
### echo letter > /etc/papersize	# is already set to letter

apt-get -y install xpdf

apt-get -y install gv

apt-get -y install ntp ntp-doc

apt-get -y install enscript

apt-get -y install thunderbird thunderbird-gnome-support
### latex-xft-fonts

apt-get -y install openoffice.org openoffice.org-starter-guide libooolib-perl openoffice.org-report-builder fig2sxd writer2latex writer2latex-manual libwriter2latex-java-doc openoffice.org-presentation-minimizer odt2txt bsh-doc libcommons-logging-java-doc libgcj-doc libhsqldb-java-doc libjline-java-doc libsac-java-doc libxalan2-java-doc libxerces2-java-doc openclipart-openoffice.org openoffice.org-dtd-officedocument1.0

# NOT openoffice.org-dmaths, creates a separate ugly toolbar menu

apt-get -y install mercurial

# Fix problem that keeps inserted USB drives from being recognized
if ! ls -l /usr/lib/dbus-1.0/dbus-daemon-launch-helper | grep 'root messagebus'
then
	ls -l /usr/lib/dbus-1.0/dbus-daemon-launch-helper
	chgrp messagebus /usr/lib/dbus-1.0/dbus-daemon-launch-helper
	chmod 4754 /usr/lib/dbus-1.0/dbus-daemon-launch-helper
	ls -l /usr/lib/dbus-1.0/dbus-daemon-launch-helper
fi

#-----

apt-get -y install virtualbox virtualbox-guest-additions
apt-get -y install dkms

apt-get -y install gnuplot gnuplot-doc libgd-tools

apt-get -y install mplayer libggi-target-x nas libdvdcss2 libggimisc2 pulseaudio mplayer-doc pulseaudio-module-hal pulseaudio-module-x11 libdvdcss2

apt-get -y install xloadimage xli

# unit of measurement conversion
apt-get -y install units

apt-get -y install ispell spell

# -- Graphics editing

apt-get -y install imagemagick imagemagick-doc

apt-get -y install netpbm html2ps

# Adobe Photoshop-type bitmap editor
apt-get -y install gimp gimp-data-extras gimp-dcraw gimp-help-common gimp-help-en gimp-helpbrowser gimp-resynthesizer gimp-texturize

# Adobe Illustrator-type vector editors
apt-get -y install inkscape python-lxml-dbg python-numpy-doc python-numpy-dbg python-uniconvertor-dbg libsvg-perl python-reportlab-doc

#-----

CLEANEXIT=1
exit 0
