#!/bin/bash -euET

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#
#  Usage:
#
#	client-standalone-precise.sh
#

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

# Add new software respositories

add-apt-repository -y 'deb http://cran.R-project.org/bin/linux/ubuntu precise/'
###add-apt-repository -y 'deb http://packages.medibuntu.org/ precise free non-free'
add-apt-repository -y 'deb http://archive.canonical.com/ubuntu precise partner'

# add-apt-repository creates failure during apt-get update, sources aren't available here, deb-src line is wrong
###### add-apt-repository -y 'deb http://download.virtualbox.org/virtualbox/debian precise contrib non-free'
if ! grep '^deb http://download.virtualbox.org/virtualbox/debian precise contrib non-free$' /etc/apt/sources.list > /dev/null
then
	echo 'deb http://download.virtualbox.org/virtualbox/debian precise contrib non-free' >> /etc/apt/sources.list
fi

# Add new software respositories for gnome classic

add-apt-repository -y ppa:gnome3-team/gnome3
add-apt-repository -y ppa:ferramroberto/gnome3

# Add PGP keys

# Current key for R
gpg --keyserver keyserver.ubuntu.com --recv-key E084DAB9
gpg -a --export E084DAB9 | apt-key add -

###wget -q http://packages.medibuntu.org/medibuntu-key.gpg -O - | apt-key add -

wget -q http://download.virtualbox.org/virtualbox/debian/oracle_vbox.asc -O- | apt-key add -

# Get updates since the cd

apt-get -y update
apt-get -y dist-upgrade

#-----

# ssh adjustments
sed \
	-e 's/#   ForwardX11 no/    ForwardX11 yes/' \
	-e 's/#   ForwardX11Trusted yes/    ForwardX11Trusted yes/' \
	-e 's/HashKnownHosts yes/HashKnownHosts no/' \
	< /etc/ssh/ssh_config > /tmp/ssh_config
cat /tmp/ssh_config > /etc/ssh/ssh_config

# Prompt for license approvals
apt-get -y install ubuntu-restricted-extras
# On amd64, pulls in 137 Meg of i386 libraries
### apt-get -y install acroread

# gnome classic
apt-get -y install gnome-shell gnome-shell-extensions-common gnome-tweak-tool

# Switch from Unity user interface to gnome
sed 's/^user-session=ubuntu$/user-session=gnome-classic/' < /etc/lightdm/lightdm.conf > /tmp/pp
cat /tmp/pp > /etc/lightdm/lightdm.conf
rm /tmp/pp

apt-get -y install build-essential gcc-4.4-doc libstdc++6-4.4-doc

apt-get -y install emacs23 emacs23-el

apt-get -y install auctex

### apt-get -y install gnuserv

#apt-get -y install gnus w3m w3m-img w3m-el
# gnus asks for xemacs21 which breaks ess install
apt-get -y install w3m w3m-img w3m-el

apt-get -y install mailcrypt

### apt-get -y install bbdb

### apt-get -y install vm

### apt-get -y install metamail # no such package

apt-get -y install texlive texlive-doc-en dvipdfmx lmodern tipa prosper latex-beamer latex-xcolor texpower texpower-manual texpower-examples texlive-generic-extra cm-super texlive-latex-extra texlive-full

# Paper size
texconfig-sys dvips paper letter
texconfig-sys pdftex paper letter
# took 'overwrite with maintainer's version' choice
texconfig-sys dvipdfm paper letter
texconfig-sys xdvi paper us
texconfig-sys mode ljfour
### echo letter > /etc/papersize	# is already set to letter

apt-get -y install gfortran gfortran-multilib gfortran-doc gfortran-4.4-multilib gfortran-4.4-doc libgfortran3-dbg
### if [ `uname -m` = 'x86_64' ]
### then
### 	apt-get -y install lib32mudflap0
### else
### 	apt-get -y install lib64mudflap0
### fi

# XXX can't find package refblas3
#apt-get -y install refblas3 refblas3-doc

apt-get -y install r-base curl r-doc-info r-doc-pdf r-doc-html r-base-latex r-base-dev r-mathlib cdbs debhelper tclreadline autoconf autoconf2.13 autoconf-archive autoconf-doc libtool automake gettext-doc libtool-doc

apt-get -y install ess pspp
# Suggested packages:
#   xlispstat, no installation candidate

apt-get -y install xpdf

apt-get -y install gv

apt-get -y install ntp ntp-doc

apt-get -y install enscript

apt-get -y install psutils

apt-get -y install pdftk

apt-get -y install page-crunch

apt-get -y install screen byobu

apt-get -y install thunderbird thunderbird-gnome-support
### latex-xft-fonts

### for i in '// save space of copying mail to home dir and indexing it' \
###	'pref("mail.server.default.autosync_offline_stores", false);' \
###	'pref("mail.server." + serverFromAccount + ".autosync_offline_stores", false);' \
###	'pref("mail.server." + serverFromAccount + ".offline_download", false);'
### do
###	if ! grep -q "^$i\$" /etc/thunderbird/syspref.js
###	then
###		echo "$i" >> /etc/thunderbird/syspref.js
###	fi
### done

#### SKIP DUE TO previous install of nonfree restricted stuff, flash
#### already works on youtube
##### apt-get -y install flashplugin-nonfree msttcorefonts ttf-xfree86-nonfree xfs
# XXX do I really want xfs and the fonts?

### Already in
# apt-get -y install java-common
### default-jre default-jdk

apt-get -y install mh-e

### cli-uno-bridge
### openoffice.org-evolution
### openoffice.org-dev-doc
### openoffice.org-dev openoffice.org-gcj
### wpd2sxw
### wps2sxw
apt-get -y install openoffice.org openoffice.org-starter-guide libooolib-perl openoffice.org-report-builder fig2sxd writer2latex writer2latex-manual libwriter2latex-java-doc openoffice.org-presentation-minimizer odt2txt bsh-doc libcommons-logging-java-doc libgcj-doc libhsqldb-java-doc libjline-java-doc libsac-java-doc libxalan2-java-doc libxerces2-java-doc openclipart-openoffice.org openoffice.org-dtd-officedocument1.0

# NOT openoffice.org-dmaths, creates a separate ugly toolbar menu

apt-get -y install filezilla

apt-get -y install tracker

apt-get -y install finger

apt-get -y install fetchmail

apt-get -y install expect

apt-get -y install mercurial
apt-get -y install rcs
apt-get -y install git-core git-doc gitk git-gui

### apt-get -y install mkpasswd

apt-get -y install rdist

apt-get -y install hwinfo
apt-get -y install perl-doc

###perl -MCPAN -e shell
###install Mknod
###^D

### apt-get -y install sysstat

# Fix problem that keeps inserted USB drives from being recognized
if ! ls -l /usr/lib/dbus-1.0/dbus-daemon-launch-helper | grep 'root messagebus'
then
	ls -l /usr/lib/dbus-1.0/dbus-daemon-launch-helper
	chgrp messagebus /usr/lib/dbus-1.0/dbus-daemon-launch-helper
	chmod 4754 /usr/lib/dbus-1.0/dbus-daemon-launch-helper
	ls -l /usr/lib/dbus-1.0/dbus-daemon-launch-helper
fi

#-----

R --no-save << EOF

install.packages(c("CircStats"), repos="http://cran.us.r-project.org")
install.packages(c("Design"), repos="http://cran.us.r-project.org")
install.packages(c("GeneTS"), repos="http://cran.us.r-project.org")
install.packages(c("Hmisc"), repos="http://cran.us.r-project.org")
install.packages(c("ISwR"), repos="http://cran.us.r-project.org")
install.packages(c("MCMCpack"), repos="http://cran.us.r-project.org")
install.packages(c("Rcmdr"), repos="http://cran.us.r-project.org")
install.packages(c("SASmixed"), repos="http://cran.us.r-project.org")
install.packages(c("SuppDists"), repos="http://cran.us.r-project.org")
install.packages(c("akima"), repos="http://cran.us.r-project.org")
install.packages(c("assist"), repos="http://cran.us.r-project.org")
install.packages(c("brlr"), repos="http://cran.us.r-project.org")
install.packages(c("car"), repos="http://cran.us.r-project.org")
install.packages(c("clines"), repos="http://cran.us.r-project.org")
install.packages(c("cluster"), repos="http://cran.us.r-project.org")
install.packages(c("coda"), repos="http://cran.us.r-project.org")
install.packages(c("combinat"), repos="http://cran.us.r-project.org")
install.packages(c("covRobust"), repos="http://cran.us.r-project.org")
install.packages(c("diptest"), repos="http://cran.us.r-project.org")
install.packages(c("e1071"), repos="http://cran.us.r-project.org")
install.packages(c("effects"), repos="http://cran.us.r-project.org")
install.packages(c("effects"), repos="http://cran.us.r-project.org")
install.packages(c("ellipse"), repos="http://cran.us.r-project.org")
install.packages(c("emplik"), repos="http://cran.us.r-project.org")
install.packages(c("energy"), repos="http://cran.us.r-project.org")
install.packages(c("evd"), repos="http://cran.us.r-project.org")
install.packages(c("exactLoglinTest"), repos="http://cran.us.r-project.org")
install.packages(c("exactRankTests"), repos="http://cran.us.r-project.org")
install.packages(c("fastICA"), repos="http://cran.us.r-project.org")
install.packages(c("fda"), repos="http://cran.us.r-project.org")
install.packages(c("fork"), repos="http://cran.us.r-project.org")
install.packages(c("g.data"), repos="http://cran.us.r-project.org")
install.packages(c("gee"), repos="http://cran.us.r-project.org")
install.packages(c("geepack"), repos="http://cran.us.r-project.org")
install.packages(c("genetics"), repos="http://cran.us.r-project.org")
install.packages(c("glmmML"), repos="http://cran.us.r-project.org")
install.packages(c("gregmisc"), repos="http://cran.us.r-project.org")
install.packages(c("gss"), repos="http://cran.us.r-project.org")
install.packages(c("ipred"), repos="http://cran.us.r-project.org")
install.packages(c("kernlab"), repos="http://cran.us.r-project.org")
install.packages(c("lars"), repos="http://cran.us.r-project.org")
install.packages(c("lasso2"), repos="http://cran.us.r-project.org")
install.packages(c("leaps"), repos="http://cran.us.r-project.org")
install.packages(c("lme4"), repos="http://cran.us.r-project.org")
install.packages(c("locfit"), repos="http://cran.us.r-project.org")
install.packages(c("logistf"), repos="http://cran.us.r-project.org")
install.packages(c("logspline"), repos="http://cran.us.r-project.org")
install.packages(c("mclust"), repos="http://cran.us.r-project.org")
install.packages(c("mda"), repos="http://cran.us.r-project.org")
install.packages(c("mix"), repos="http://cran.us.r-project.org")
install.packages(c("msm"), repos="http://cran.us.r-project.org")
install.packages(c("multcomp"), repos="http://cran.us.r-project.org")
install.packages(c("mvnormtest"), repos="http://cran.us.r-project.org")
install.packages(c("mvtnorm"), repos="http://cran.us.r-project.org")
install.packages(c("nortest"), repos="http://cran.us.r-project.org")
install.packages(c("pamr"), repos="http://cran.us.r-project.org")
install.packages(c("panel"), repos="http://cran.us.r-project.org")
install.packages(c("pastecs"), repos="http://cran.us.r-project.org")
install.packages(c("polspline"), repos="http://cran.us.r-project.org")
install.packages(c("pps"), repos="http://cran.us.r-project.org")
install.packages(c("pspline"), repos="http://cran.us.r-project.org")
install.packages(c("quantreg"), repos="http://cran.us.r-project.org")
install.packages(c("randomForest"), repos="http://cran.us.r-project.org")
install.packages(c("session"), repos="http://cran.us.r-project.org")
install.packages(c("sm"), repos="http://cran.us.r-project.org")
install.packages(c("sma"), repos="http://cran.us.r-project.org")
install.packages(c("tensor"), repos="http://cran.us.r-project.org")
install.packages(c("tree"), repos="http://cran.us.r-project.org")
install.packages(c("tseries"), repos="http://cran.us.r-project.org")
install.packages(c("wavethresh"), repos="http://cran.us.r-project.org")
install.packages(c("wle"), repos="http://cran.us.r-project.org")
install.packages(c("xgobi"), repos="http://cran.us.r-project.org")
install.packages(c("xtable"), repos="http://cran.us.r-project.org")
install.packages(c("zicounts"), repos="http://cran.us.r-project.org")

install.packages("VGAM", repos="http://www.stat.auckland.ac.nz/~yee")

# Mapping packages

install.packages(c("geoR"), repos="http://cran.us.r-project.org")
install.packages(c("geoRglm"), repos="http://cran.us.r-project.org")
install.packages(c("gstat"), repos="http://cran.us.r-project.org")
install.packages(c("maps"), repos="http://cran.us.r-project.org")
install.packages(c("maptools"), repos="http://cran.us.r-project.org")
install.packages(c("spatstat"), repos="http://cran.us.r-project.org")
install.packages(c("splancs"), repos="http://cran.us.r-project.org")
install.packages(c("sp"), repos="http://cran.us.r-project.org")
install.packages(c("spBayes"), repos="http://cran.us.r-project.org")

### source("http://bioconductor.org/getBioC.R")
### getBioC()

quit()
EOF

#-----

### apt-get -y install virtualbox virtualbox-guest-additions
apt-get -y install dkms
apt-get -y install virtualbox-4.3 ### virtualbox-guest-additions

# texmacs
apt-get -y install texlive-extra-utils dvi2tty lacheck texmacs-extra-fonts libjpeg-progs libtiff-tools librsvg2-bin libtiff-opengl dvidvi xfig-doc

### # gv with gnome gui flavor
### apt-get -y install gnome-gv

apt-get -y install pdfedit

apt-get -y install gnuplot gnuplot-doc libgd-tools

# Multi-language xterm
### apt-get -y install mlterm mlterm-tools unifont xfonts-efont-unicode ttf-vlgothic ttf-baekmuk t1-cyrillic mlterm-im-uim mlterm-im-m17nlib mlterm-im-scim m17n-docs xfonts-efont-unicode-ib ttf-arphic-bsmi00lp ttf-arphic-gbsn00lp unifont-bin

apt-get -y install mplayer libggi-target-x nas libdvdcss2 libggimisc2 pulseaudio mplayer-doc pulseaudio-module-hal pulseaudio-module-x11 libdvdcss2

apt-get -y install xloadimage xli

# unit of measurement conversion
apt-get -y install units

# Floppy drive access
apt-get -y install mtools

# DOS file conversion
apt-get -y install tofrodos

# An un-packager
apt-get -y install unrar

# file synchronization
apt-get -y install unison-gtk

# tk extension
apt-get -y install blt

# XXX what is citadel server and why it is listening on an IP?
### # Mail Handler
### apt-get -y install nmh
apt-get -y install nmh mh-book ### exmh
### metamail

apt-get -y install ispell spell

# -- Math-type things

# ancestor of Maple
apt-get -y install maxima xmaxima wxmaxima

# alternative to maple
apt-get -y install yacas yacas-doc

# clone of Matlab
apt-get -y install octave3.2 octave3.2-info octave3.2-doc octave3.2-htmldoc octave3.2-headers octave3.2-emacsen libhdf5-doc

# -- Graphics editing

apt-get -y install imagemagick imagemagick-doc

apt-get -y install netpbm html2ps

# Adobe Photoshop-type bitmap editor
apt-get -y install gimp gimp-data-extras gimp-dcraw gimp-help-common gimp-help-en gimp-helpbrowser gimp-resynthesizer gimp-texturize

# Adobe Illustrator-type vector editors
apt-get -y install inkscape python-lxml-dbg python-numpy-doc python-numpy-dbg python-uniconvertor-dbg libsvg-perl python-reportlab-doc
### apt-get -y install dia dia-gnome
### apt-get -y install xfig xfig-doc transfig xfig-libs

# -- Sysadmin

# top for network traffic flows
apt-get -y install etherape

# Ethernet sniffer with GUI
apt-get -y install wireshark adns-tools

# where are the packets getting lost
apt-get -y install traceroute

# GUI for diff
# tkdiff
apt-get -y install tkcvs
# python-gtkmozembed
apt-get -y install meld python-gtkspell

# graph visualization
apt-get -y install graphviz graphviz-doc

# What IP ports are open?
apt-get -y install nmap

# Foreign software package handling
# lsb-rpm
apt-get -y install alien lintian

# Break apart MS .mdi paper scanner files
apt-get -y install foremost

# -- Other

# Solitare card game
### apt-get -y install spider

# Opera browser
### wget -O - http://deb.opera.com/archive.key | apt-key add -
### apt-get -y install opera

### # 3D modelers
### apt-get -y install blender yafray
### apt-get -y install k3d aqsis

### # 2D CAD
### apt-get -y install qcad qcad-doc partlibrary

### # Page layout
### apt-get -y install scribus scribus-template scribus-doc

### # Sound editing
### apt-get -y install audacity lame lame-extras
### apt-get -y install ardour-gtk ardour-doc fil-plugins tap-plugins

### # Structured xml editor
### apt-get -y install conglomerate

### # Planitarium simulator
### apt-get -y install celestia-gnome

### apt-get -y install python2.4

### # Windows simulator
### apt-get -y install wine

#-----

CLEANEXIT=1
exit 0
