#!/bin/bash -euET

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#
#  Usage:
#
#	INSTALL-mysql.sh
#

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

### trap ': ${0##*/} line number $LINENO' DEBUG

#-----

set -x
### set -o functrace

if [ "`id -u`" -eq 0 ]
then
	echo "FATAL should be run as database developer user, not root, quitting" 1>&2
	exit 1
fi

if ! type lsb_release > /dev/null 2>&1
then
	echo "FATAL unexpected Linux release or distribution, no lsb_release program, quitting" 1>&2
	exit 1
fi

#  If you do apt-get purge mysql-blah, also do apt-get autoremove and rm -r /var/lib/mysql to remove data

case `lsb_release -a | grep Description: | sed 's/ /_/g'` in

	*Ubuntu_14.04.5_LTS)
		sudo apt-get -y install mysql-server mysql-client mysql-workbench emma whois
		sudo mysql_secure_installation
	;;

	*)
		echo "FATAL unexpected Linux release or distribution, quitting" 1>&2
		exit 1
	;;

esac

touch ~/.hfgen-password
chmod 600 ~/.hfgen-password
HFGENDBPW=$(mkpasswd $(ps auxww | md5sum | awk '{print $1}'))
echo $HFGENDBPW > ~/.hfgen-password

touch ~/.my.cnf
chmod 600 ~/.my.cnf
MYMYSQLPW=$(mkpasswd $(ps auxww | md5sum | awk '{print $1}'))
cat > ~/.my.cnf << EOF
[mysql]
user=$USER
password=$MYMYSQLPW
[mysqladmin]
user=$USER
password=$MYMYSQLPW
[mysqldump]
user=$USER
password=$MYMYSQLPW
EOF

echo "Enter mysql password for user root"
mysql -uroot -p mysql << EOF
create user '$USER' identified by '$MYMYSQLPW';
grant all privileges on *.* to '$USER' identified by '$MYMYSQLPW' with grant option;
grant all privileges on *.* to '$USER'@'localhost' identified by '$MYMYSQLPW' with grant option;
create user 'hfgen' identified by '$HFGENDBPW';
-- insert for hfgen is to allow populating temporary tables
grant select, insert, create temporary tables on *.* to 'hfgen' identified by '$HFGENDBPW';
grant select, insert, create temporary tables on *.* to 'hfgen'@'localhost' identified by '$HFGENDBPW';
flush privileges;
select user, host, password from user order by user, host;
EOF

cat << EOF
Should resemble:

+------------------+-----------+-------------------------------------------+
| user             | host      | password                                  |
+------------------+-----------+-------------------------------------------+
| debian-sys-maint | localhost | *8A5EEA3FD894D4BE3E449282B0D31AD3FB4DD40D |
| hfgen            | %         | *4ACF7B10EEC231C80957A6000AEACA4251B67A4E |
| hfgen            | localhost | *4ACF7B10EEC231C80957A6000AEACA4251B67A4E |
| root             | 127.0.0.1 | *B53B98C73AA2B9044DE83A11A1EC1A192A9DF0AB |
| root             | localhost | *B53B98C73AA2B9044DE83A11A1EC1A192A9DF0AB |
| root             | ubuntu    | *B53B98C73AA2B9044DE83A11A1EC1A192A9DF0AB |
| YOUR-ACCOUNT     | %         | *318626A64BF2A9BDD8F8C2BCDF5C92B3F13D3C88 |
| YOUR-ACCOUNT     | localhost | *318626A64BF2A9BDD8F8C2BCDF5C92B3F13D3C88 |
+------------------+-----------+-------------------------------------------+
EOF

# Log into mysql as user running script, not hfgen
mysql mysql << EOF
show tables;
EOF

mysql -uhfgen -p$(< ~/.hfgen-password) mysql << EOF
show tables;
EOF

# Remove record of plaintext passwords
rm -f ~/.mysql_history

#-----

CLEANEXIT=1
exit 0
