#!/bin/bash -euET

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#
#  Show which capture steps have been completed and how long they took
#
#  Usage:
#
#	./pull-times.sh
#

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

if ! make -q pull
then
	echo "ERROR 'make pull' is not built, major remaining steps include:
" 2>&1
	make -n pull |& grep 's.*\.sh'
	echo
fi

function duration_of_logs ()
{
	NETSUM=0
	for i in `/bin/ls -1tr repo/m/s*.log`
	do
		START=`head -1 $i | awk '$7 ~ /[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]/ {print $7}'`
		END=`tail -1 $i | awk '$7 ~ /[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]/ {print $7}'`
		if echo $START | grep -q '[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]' && echo $END | grep -q '[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]'
		then
			# expr exits with 1 when math result is 0
			NET=`expr $END - $START + 1`
			NETSUM=`expr $NETSUM + $NET`
			printf '%-35s start %10d end %10d net %10d seconds\n' $i $START $END $NET
#		else
#			printf 'total %10d seconds\n' $NETSUM
#			return 0
		fi
	done
}

duration_of_logs
printf 'total %10d seconds\n' $NETSUM

#-----

CLEANEXIT=1
exit 0
