#!/bin/bash -euET

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#
#  A step in capturing servers and clients
#

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

: ${ALLFARHOSTS:?FATAL env var ALLFARHOSTS not defined, should be list of far hosts, quitting}

for i in $ALLFARHOSTS
do
	mkdir -p repo/h/$i/s3
	hfdist -wd repo/h/$i/s2/d repo/h/$i/s3/d
	hfdist -wd repo/h/$i/s2/g repo/h/$i/s3/g
	hfdist -wd repo/h/$i/s2/h/$i repo/h/$i/s3/h/$i
	hfdist -wd repo/h/$i/s2/o repo/h/$i/s3/o
	hfdist -wd repo/h/$i/s2/p/$i repo/h/$i/s3/p/$i
	hfdist -wd repo/h/$i/s2/w repo/h/$i/s3/w
done

set -- $ALLFARHOSTS
FIRSTHOST=$1 ; shift
for i
do
	# these dirtree differences should be removed
	hfdist -wd repo/h/$FIRSTHOST/s3/g/usr/share/emacs23/site-lisp/ess	repo/h/$i/s3/g/usr/share/emacs23/site-lisp/ess
	hfdist -wd repo/h/$FIRSTHOST/s3/g/var/cache/apt				repo/h/$i/s3/g/var/cache/apt
	hfdist -wd repo/h/$FIRSTHOST/s3/g/var/cache/apt-xapian-index		repo/h/$i/s3/g/var/cache/apt-xapian-index
	hfdist -wd repo/h/$FIRSTHOST/s3/g/var/cache/fontconfig			repo/h/$i/s3/g/var/cache/fontconfig
	hfdist -wd repo/h/$FIRSTHOST/s3/g/var/cache/man				repo/h/$i/s3/g/var/cache/man
	hfdist -wd repo/h/$FIRSTHOST/s3/g/var/lib/apt				repo/h/$i/s3/g/var/lib/apt
	hfdist -wd repo/h/$FIRSTHOST/s3/g/var/lib/dpkg				repo/h/$i/s3/g/var/lib/dpkg
	hfdist -wd repo/h/$FIRSTHOST/s3/g/var/log				repo/h/$i/s3/g/var/log

	# these individual file differences should be removed
	hflayers copyone repo/h/$FIRSTHOST/s3/g/usr/lib/graphviz/config6			repo/h/$i/s3/g/usr/lib/graphviz/config6 | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/usr/lib/i386-linux-gnu/gdk-pixbuf-2.0/2.10.0/loaders.cache		repo/h/$i/s3/g/usr/lib/i386-linux-gnu/gdk-pixbuf-2.0/2.10.0/loaders.cache | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/usr/lib/jvm/java-6-openjdk-amd64/jre/lib/amd64/server/classes.jsa	repo/h/$i/s3/g/usr/lib/jvm/java-6-openjdk-amd64/jre/lib/amd64/server/classes.jsa | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/usr/lib/x86_64-linux-gnu/gdk-pixbuf-2.0/2.10.0/loaders.cache		repo/h/$i/s3/g/usr/lib/x86_64-linux-gnu/gdk-pixbuf-2.0/2.10.0/loaders.cache | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/usr/share/applications/mimeinfo.cache		repo/h/$i/s3/g/usr/share/applications/mimeinfo.cache | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/usr/share/glib-2.0/schemas/gschemas.compiled	repo/h/$i/s3/g/usr/share/glib-2.0/schemas/gschemas.compiled | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/usr/share/icons/gnome/icon-theme.cache		repo/h/$i/s3/g/usr/share/icons/gnome/icon-theme.cache | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/usr/share/icons/hicolor/icon-theme.cache	repo/h/$i/s3/g/usr/share/icons/hicolor/icon-theme.cache | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/usr/share/info/dir.old				repo/h/$i/s3/g/usr/share/info/dir.old | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/usr/share/info/dir				repo/h/$i/s3/g/usr/share/info/dir | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/var/cache/ldconfig/aux-cache			repo/h/$i/s3/g/var/cache/ldconfig/aux-cache | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/var/lib/doc-base/info/files-old.yml		repo/h/$i/s3/g/var/lib/doc-base/info/files-old.yml | hfdist -wd -l -
	hflayers copyone repo/h/$FIRSTHOST/s3/g/var/lib/doc-base/info/files.yml			repo/h/$i/s3/g/var/lib/doc-base/info/files.yml | hfdist -wd -l -

	FILES=`cd repo/h/$FIRSTHOST/s3/g && echo var/backups/dpkg.status.*`
	for j in $FILES
	do
		hflayers copyone repo/h/$FIRSTHOST/s3/g/$j repo/h/$i/s3/g/$j | hfdist -wd -l -
	done

	FILES=`cd repo/h/$FIRSTHOST/s3/g && echo var/backups/apt.*`
	for j in $FILES
	do
		hflayers copyone repo/h/$FIRSTHOST/s3/g/$j repo/h/$i/s3/g/$j | hfdist -wd -l -
	done

	# XXX these differences look like errors, and should be understood and/or fixed
	# modes of "mlocate"
	hfdist -wd repo/h/$FIRSTHOST/s3/g/etc/cron.daily repo/h/$i/s3/g/etc/cron.daily
	# modes of "sys"
	hfdist -wd repo/h/$FIRSTHOST/s3/g/sys repo/h/$i/s3/g/sys
	# mtime of "cron" dir, existance of "paths.pyc"
	hfdist -wd repo/h/$FIRSTHOST/s3/g/usr/lib/mailman/cron repo/h/$i/s3/g/usr/lib/mailman/cron

	# remove superfluous mtime differences produced by apt-get upgrades
	for j in d g o w
	do
		hflayers both109 repo/h/$FIRSTHOST/s3/$j repo/h/$i/s3/$j | hfdist -wd -k -L -l -
	done
done

touch repo/m/s3.timestamp

#-----

CLEANEXIT=1
exit 0
