#!/bin/bash -euET

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#
#  A step in capturing servers and clients
#

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

: ${FARHOST:?FATAL env var FARHOST not defined, should be name of far host, quitting}

NOW=`cat repo/m/s4.now`

rm -rf repo/h/$FARHOST/s4/s repo/h/$FARHOST/s4/k
mkdir -p repo/h/$FARHOST/s4/s repo/h/$FARHOST/s4/k
hfdist -wd repo/h/$FARHOST/s3/d			repo/h/$FARHOST/s4/d
hfdist -wd repo/h/$FARHOST/s3/g			repo/h/$FARHOST/s4/g
hfdist -wd repo/h/$FARHOST/s3/h/$FARHOST	repo/h/$FARHOST/s4/h/$FARHOST
hfdist -wd repo/h/$FARHOST/s3/o			repo/h/$FARHOST/s4/o
hfdist -wd repo/h/$FARHOST/s3/p/$FARHOST	repo/h/$FARHOST/s4/p/$FARHOST
hfdist -wd repo/h/$FARHOST/s3/w			repo/h/$FARHOST/s4/w

cd repo/h/$FARHOST/s4

function fixleadinpath ()
{
	SRC="$1"
	DST="$2"
	PTH="$3"

	UPDIR=$(dirname $PTH)
	while [ "$UPDIR" != '.' ]
	do
		(cd "$SRC" ; tar -cf - --no-recursion "$UPDIR") | (cd "$DST" ; tar -xpf -)
		UPDIR=$(dirname $UPDIR)
	done
}

function splitfor103 ()
{
	if [ -e "$1/$3" -o -h "$1/$3" ]
	then
		echo "splitfor103 $1 $2 $3"

		rm -rf "$2/$FARHOST/$3"
		(cd "$1" ; tar -cf - "$3") | (cd "$2/$FARHOST" ; tar -xpf -)

		fixleadinpath "$1" "$2/$FARHOST" "$3"
		truncate -s 0 "$1/$3"
		touch -d @$NOW "$1/$3"
	else
		echo "skipping splitfor103 $1 $2 $3"
	fi
}

function emptyout ()
{
	if [ -e "$1/$2" -o -h "$1/$2" ]
	then
		echo "emptyout $1 $2"
		truncate -s 0 "$1/$2"
		touch -d @$NOW "$1/$2"
	else
		echo "skipping emptyout $1 $2"
	fi
}

function to_s ()
{
	if [ -e "$1/$2" -o -h "$1/$2" ]
	then
		echo "to_s $1 $2"
		mkdir -p s/$FARHOST

		rm -rf "s/$FARHOST/$2"
		(cd "$1" ; tar -cf - "$2") | (cd "s/$FARHOST" ; tar -xpf -)

		fixleadinpath "$1" "s/$FARHOST" "$2"
		touch -r $(dirname "$1/$2") /tmp/s4.time
		rm -rf "$1/$2"
		touch -r /tmp/s4.time $(dirname "$1/$2")
		rm /tmp/s4.time
	else
		echo "skipping to_s $1 $2"
	fi
}

function to_p ()
{
	if [ -e "$1/$2" -o -h "$1/$2" ]
	then
		echo "to_p $1 $2"
		mkdir -p p/$FARHOST

		rm -rf "p/$FARHOST/$2"
		(cd "$1" ; tar -cf - "$2") | (cd "p/$FARHOST" ; tar -xpf -)

		fixleadinpath "$1" "p/$FARHOST" "$2"
		touch -r $(dirname "$1/$2") /tmp/s4.time
		rm -rf "$1/$2"
		touch -r /tmp/s4.time $(dirname "$1/$2")
		rm /tmp/s4.time
	else
		echo "skipping to_p $1 $2"
	fi
}

function to_k ()
{
	if [ -e "$1/$2" -o -h "$1/$2" ]
	then
		echo "to_k $1 $2"
		mkdir -p k/$FARHOST

		rm -rf "k/$FARHOST/$2"
		(cd "$1" ; tar -cf - "$2") | (cd "k/$FARHOST" ; tar -xpf -)

		fixleadinpath "$1" "k/$FARHOST" "$2"
		touch -r $(dirname "$1/$2") /tmp/s4.time
		rm -rf "$1/$2"
		touch -r /tmp/s4.time $(dirname "$1/$2")
		rm /tmp/s4.time
	else
		echo "skipping to_k $1 $2"
	fi
}

function to_p_lead ()
{
	if [ -e "$1/$2" -o -h "$1/$2" ]
	then
		echo "to_p_lead $1 $2 $3"

		mkdir -p p/$FARHOST/$3
		rm -rf "p/$FARHOST/$3/$2"
		(cd "$1" ; tar -cf - "$2") | (cd "p/$FARHOST/$3" ; tar -xpf -)

### XXX ?	fixleadinpath "$1" "p/$FARHOST/$3" "$2"

		touch -r $(dirname "$1/$2") /tmp/s4.time
		rm -rf "$1/$2"
		touch -r /tmp/s4.time $(dirname "$1/$2")
		rm /tmp/s4.time
	else
		echo "skipping to_p_lead $1 $2 $3"
	fi
}

splitfor103 g h etc/passwd
#splitfor103 g h etc/passwd~
splitfor103 g h etc/passwd-
splitfor103 g h var/backups/passwd.bak

splitfor103 g h etc/shadow
splitfor103 g h etc/shadow-
splitfor103 g h var/backups/shadow.bak

splitfor103 g h etc/gshadow
splitfor103 g h etc/gshadow-
splitfor103 g h var/backups/gshadow.bak

splitfor103 g h etc/group
#splitfor103 g h etc/group~
splitfor103 g h etc/group-
splitfor103 g h var/backups/group.bak

splitfor103 g h etc/hosts
splitfor103 g h etc/hostname
splitfor103 g h etc/mailname

splitfor103 g h etc/aliases

splitfor103 g h etc/cups/subscriptions.conf
splitfor103 g h etc/cups/subscriptions.conf.O

to_p g etc/fstab

to_p g etc/udev/rules.d/70-persistent-net.rules

#splitfor103 g h etc/resolv.conf
to_p g etc/resolv.conf

to_s g etc/ssh

#to_s g etc/ssl/private
to_s g etc/ssl
to_s g var/lib/dovecot/ssl-parameters.dat

to_s g var/lib/mailman/data/adm.pw
to_p g var/lib/mailman

to_s g root/.ssh
to_s g home/ubntinst/.ssh

to_k g initrd.img
to_k g initrd.img.old
to_k g vmlinuz
to_k g vmlinuz.old
to_k g lib/modules
to_k g lib/firmware
to_k g usr/src
to_k g boot
to_k g etc/apt/apt.conf.d/01autoremove-kernels
to_k g var/lib/dkms/virtualbox
to_k g var/lib/initramfs-tools

to_p g etc/sysstat

to_p g etc/apache2/apache2.conf
to_p g etc/apache2/sites-available/default
to_p g etc/apache2/sites-available/default-ssl

to_p g etc/postfix/main.cf
to_p g etc/postfix/master.cf

to_p g etc/X11/xorg.conf.failsafe

to_p g etc/event.d
to_p g etc/event.disabled
to_p g etc/init/hvc0.conf

to_p g etc/dovecot/dovecot.conf
to_p g etc/dovecot/dovecot.conf_DISABLED
to_p g etc/rc0.d
to_p g etc/rc1.d
to_p g etc/rc2.d
to_p g etc/rc3.d
to_p g etc/rc4.d
to_p g etc/rc5.d
to_p g etc/rc6.d
to_p g var/lib/update-rc.d

to_p g etc/init/tty1.conf
to_p g etc/init/tty2.conf
to_p g etc/init/tty3.conf
to_p g etc/init/tty4.conf
to_p g etc/init/tty5.conf
to_p g etc/init/tty6.conf

to_p g cdrom

to_p g var/spool/cron/crontabs

to_p g var/cache/bind
to_p g var/cache/cups
to_p g var/cache/lightdm
to_p g var/lib/AccountsService
to_p g var/lib/alsa
to_p g var/lib/amavis
to_p g var/lib/apt/periodic
to_p g var/lib/apt-xapian-index
to_p g var/lib/clamav
to_p g var/lib/dhcp
to_p g var/lib/lightdm
to_p g var/lib/logrotate
to_p g var/lib/mlocate/mlocate.db
to_p g var/lib/mysql
to_p g var/lib/NetworkManager
to_p g var/lib/ntp/ntp.drift
to_p g var/lib/plymouth
to_p g var/lib/postfix
to_p g var/lib/postgrey
to_p g var/lib/rfkill/saved-state
to_p g var/lib/spamassassin
to_p g var/lib/squirrelmail
to_p g var/lib/sudo
to_p g var/lib/udisks	
to_p g var/lib/update-manager
to_p g var/lib/update-notifier
to_p g var/lib/upower
to_p g var/lib/urandom
to_p g var/lib/ureadahead
to_p g var/lib/xkb
to_p g var/spool/anacron
to_p g var/spool/postfix

to_p g etc/lvm/cache/.cache

to_p_lead o VBoxGuestAdditions-4.1.12 opt

emptyout g etc/mtab

echo "PROGRESS $0 end"

cd ../../../..
touch repo/h/$FARHOST/s4.timestamp

#-----

CLEANEXIT=1
exit 0
