#!/bin/bash -euET

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#
#  A step in capturing servers and clients
#

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

: ${ALLFARHOSTS:?FATAL env var ALLFARHOSTS not defined, should be list of far hosts, quitting}

echo -e 'repo\tmd5\tlines\tfarhost
15\t32\t6N\t15' > repo/m/s8.rdb

for i in $ALLFARHOSTS
do
	sed "s/\$/	$i/" < repo/h/$i/s6/s6.md5sums | sort >> repo/m/s8.rdb
done

set -- $ALLFARHOSTS
FIRSTHOST=$1 ; shift
BADCMP=0
for i
do
	cmp -s repo/h/$FIRSTHOST/s6/s6.md5sums repo/h/$i/s6/s6.md5sums || BADCMP=1
done

if [ $BADCMP -eq 1 ]
then
	echo "
ERROR perhost differences remain; debug with:
ERROR tkdiff repo/h/{THISHOST,THATHOST}/s5/REPO.sl
"
	LC_ALL=C sorttbl repo lines farhost < repo/m/s8.rdb | ptbl
	echo
	exit 3
fi

for i in d g o w
do
	(set -x ; hfdist -wd repo/h/$FIRSTHOST/s4/$i repo/m/$i)
done

for i in h p s k
do
	for j in $ALLFARHOSTS
	do
		(set -x ; hfdist -wd repo/h/$j/s4/$i/$j repo/m/$i/$j)
	done
done

touch repo/m/s8.timestamp

#-----

CLEANEXIT=1
exit 0
