#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#  Two expected uses of signals: 1) "^C" of hfcheck on command line, 2) "kill PID" for PID in CHECKPOINT list
trap '
trap - INT
CLEANEXIT=1
kill -s INT $$' INT

export HFGENDB=${HFGENDB:-@@@ EXAMPLEDB @@@}
HFTOP='/@@@ HFTOP @@@'
GENTOPINST="$HFTOP/gen"
GENVERDIR="hfgen.out/$HFGENDB"

# Doesn't use getops because hostspec negations collide with dashed
# option parsing

WRITE=''
DELETE=''
CHECKSUM=''
CHECK=''
VERBOSE=''
OPTX=''
JOBS='4'
THOROUGHNESS=''
TARGETROOT=''
HOSTCONNECTION=''
LIMITTO=''
LIMITSKIP=''
LIMITIS=''
EXAMINE=''

while :
do
	case "${1:-}" in
		-w)	WRITE='-w'
			shift
			;;
		-d)	DELETE='-d'
			shift
			;;
		-wd)	WRITE='-w'
			DELETE='-d'
			shift
			;;
		-k)	CHECKSUM='-k'
			shift
			;;
		-c)	CHECK='-c'
			shift
			;;
		-v)	VERBOSE="${2:-}"
			if [ -z "$VERBOSE" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				VERBOSE="-v $VERBOSE"
				shift 2
			fi
			;;
		-x)	OPTX="${2:-}"
			if [ -z "$OPTX" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				OPTX="-x $OPTX"
				shift 2
			fi
			;;
		-j)	JOBS="${2:-}"
			if [ -z "$JOBS" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				shift 2
			fi
			;;
		-t)	THOROUGHNESS="${2:-}"
			if [ -z "$THOROUGHNESS" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				THOROUGHNESS="-t $THOROUGHNESS"
				shift 2
			fi
			;;
		-r)	TARGETROOT="${2:-}"
			if [ -z "$TARGETROOT" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				TARGETROOT="-r $TARGETROOT"
				shift 2
			fi
			;;
		-h)	HOSTCONNECTION="${2:-}"
			if [ -z "$HOSTCONNECTION" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				HOSTCONNECTION="-h $HOSTCONNECTION"
				shift 2
			fi
			;;
		-l)	LIMITTO="${2:-}"
			if [ -z "$LIMITTO" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				LIMITIS="-l $LIMITTO"
				shift 2
			fi
			;;
		-L)	LIMITSKIP="${2:-}"
			if [ -z "$LIMITSKIP" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				for I in $LIMITSKIP
				do
					LIMITIS="$LIMITIS -L $I"
				done
				shift 2
			fi
			;;
		-e)	EXAMINE='-e'
			shift
			;;
		*)	break
			;;
	esac
done

if [ $# -eq 0 ]
then
	print_version
	echo 1>&2
	print_usage
	CLEANEXIT=1
	exit 0
fi

if [ \( -n "$LIMITTO" \) -a \( -n "$LIMITSKIP" \) ]
then
	echo "ERROR can't use -l and -L together" 1>&2
	print_usage
	CLEANEXIT=1
	exit 1
fi

#-----

echo "NOTICE HFGENDB=$HFGENDB" 1>&2

# Want hfcheck to run both in hg repo and against installed location.
# So it looks for dir hfgen.out or symlink to it in the current dir,
# and if it doesn't find it, uses the one in the install location

if [ ! -d hfgen.out ]
then
	echo "NOTICE hfgen.out is not in cd, using data in $GENTOPINST/$GENVERDIR" 1>&2
	GENROOT="$GENTOPINST/$GENVERDIR"
else
	GENROOT="$(pwd)/$GENVERDIR"
fi

#-----

HFCHECKTMP="/tmp/hfcheck.$$"
HOSTSPEC="$@"

echo -g $GENVERDIR $WRITE $DELETE $CHECKSUM $CHECK $VERBOSE $OPTX $THOROUGHNESS $TARGETROOT $HOSTCONNECTION $LIMITIS $EXAMINE | \
	oe -q -l -s -j $JOBS "$HOSTSPEC" "$GENROOT/hfcheck-int/hfcheck-oe-int" 9> $HFCHECKTMP

HFCHECKTMPINSIDE=$(< $HFCHECKTMP)

cd $HFCHECKTMPINSIDE/hosts

echo "hflog results are:"
for i in $(/bin/ls -1 | sort -t. -k 1,1 -n -k 2,2)
do
	if [ -f $i/hfcheck.hflog ]
	then
		hflog -n $i/hfcheck.hflog
	fi
done

echo "stdout and exits are:"
for i in $(/bin/ls -1 | sort -t. -k 1,1 -n -k 2,2)
do
	(echo $i ; cat $i/oe_stdout) | fmt | awk '{ printf ("%-15s  %-25s  %-25s  exit ", $1, $2, $3) }'
	cat $i/oe_exit
done

echo "bash commands for detailed logs:"
for i in $(/bin/ls -1 | sort -t. -k 1,1 -n -k 2,2)
do
	echo "hflog $HFCHECKTMPINSIDE/hosts/$i/hfcheck.hflog | less"
done
echo "hflog $HFCHECKTMPINSIDE/hosts/*/hfcheck.hflog | less
less $HFCHECKTMPINSIDE/hosts/*/oe_stderr"
if [ -n "$EXAMINE" ]
then
	echo "sl -tr $HFCHECKTMPINSIDE/hosts/*/examine | less"
fi
echo "To cleanup rm -r $HFCHECKTMPINSIDE"

rm -f $HFCHECKTMP
