#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

gen_EXT_SH := \
	gen/hfcheck.sh \
	gen/hfgen.sh \
	gen/hfmenu.sh \
	gen/hf-reloaddb.sh

gen_EXT_PL := \
	gen/hf-mtimedb.pl

gen_EXT_POD := \
	$(patsubst %.sh,%.pod,$(gen_EXT_SH)) \
	$(patsubst %.pl,%.pod,$(gen_EXT_PL))

gen_HELP := \
	gen/hfcheck.help

gen_OTHERS := \
	gen/module.mk \
	gen/GNUmakefile

gen_SOURCES := \
	$(gen_EXT_SH) \
	$(gen_EXT_PL) \
	$(gen_EXT_POD) \
	$(gen_HELP) \
	$(gen_OTHERS)

gen_MAN := \
	$(patsubst %.pod,%.1,$(gen_EXT_POD))

gen_BASH_TESTME := \
	gen/autofs/Generate \
	gen/backups/Generate \
	gen/bind/Generate \
	gen/dhcp/Generate \
	gen/emailaliases/Generate \
	gen/empldir/Generate \
	gen/hfcheck-int/Generate \
	gen/hfmenu-int/Generate \
	gen/hostlists/Generate \
	gen/mailandhostname/Generate \
	gen/network/Generate \
	gen/passwdgroup/check-sanity \
	gen/printers/Generate \
	gen/website/Generate

gen_PERL_TESTME := \
	gen/hfdist/Generate \
	gen/maps/Generate \
	gen/passwdgroup/Generate \
	gen/website/gsub.pl

gen_BASH_RESULTS := \
	gen/hfcheck \
	gen/hfgen \
	gen/hfmenu \
	gen/hf-reloaddb

gen_PERL_RESULTS := \
	gen/hf-mtimedb

gen_RESULTS := \
	$(gen_BASH_RESULTS) \
	$(gen_PERL_RESULTS) \
	$(gen_MAN)

gen_PIECES := \
	autofs \
	backups \
	bind \
	dhcp \
	emailaliases \
	empldir \
	hfcheck-int \
	hfdist \
	hfmenu-int \
	hostlists \
	mailandhostname \
	maps \
	network \
	passwdgroup \
	printers \
	website

# Dependencies deliberately hidden.  make install will install
# repeatedly using old md5 values.  make clean will update md5 values.
gen/md5:
	cat $(gen_SOURCES) | md5sum | awk '{print $$1}' > gen/md5

gen/hfcheck: gen/hfcheck.help

.PHONY: build_gen
build_gen: $(gen_RESULTS) gen/md5 ## Build gen

.PHONY: clean_gen
clean_gen: ## Clean gen
	rm -f $(strip $(gen_RESULTS) gen/md5)
	rm -rf gen/hfgen.out/*

.PHONY: install_gen
install_gen: installstep1 $(gen_RESULTS) gen/md5 ## Install gen
	cp -a gen/hfcheck install/$(HFTOP)/bin/hfcheck
	cp -a gen/hfgen install/$(HFTOP)/bin/hfgen
	cp -a gen/hfmenu install/$(HFTOP)/bin/hfmenu
	cp -a gen/hf-mtimedb install/$(HFTOP)/bin/hf-mtimedb
	cp -a gen/hf-reloaddb install/$(HFTOP)/bin/hf-reloaddb
	cp -a $(gen_MAN) install/$(HFTOP)/man/man1
	(cd gen ; cp -a $(gen_PIECES) ../install/$(HFTOP)/gen)
	(cd gen ; cp -a $(gen_PIECES) ../install/$(HFGENTOP)/hfexample)
	ln -s /$(HFSITETOP)/hfgen.out install/$(HFTOP)/gen/hfgen.out
	ln -s /$(HFSITETOP)/hfgen.out install/$(HFGENTOP)/hfexample/hfgen.out

.PHONY: run_hfgen
run_hfgen: ## Run hfgen in source gen area
	cd gen ; ./hfgen
