#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

package HF::Harmonize;
use strict;

use Socket;
use Digest::MD5;
use File::Path 'make_path';

sub setup {

	$HF::Harmonize::md5handle = Digest::MD5->new;
}

sub new {

	my $proto = shift;
	my $issource = shift;

	my $class = ref ($proto) || $proto;

	my $self = {

		'issource'	=> $issource,
###		'md5handle'	=> Digest::MD5->new
	};

	bless ($self, $class);

	return ($self);
}

sub source_main {

	my $self = shift;

	my $from;
	my $layersptr;
	my $returnval;

	if ($self->{'issource'} != 1) {
		HF::Report->report (4, 1, "FATAL Harmonize object not set up to be source");
	}

	$HF::Globals::source_report->[$HF::Globals::hostrow] = { };
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'fail_audit'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'fail_connection'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'fail_layers'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'fail_total'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_b'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_c'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_d'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_f'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_f_bytes'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_l'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_p'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_s'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'sent_f'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'sent_f_bytes'} = 0;
	$HF::Globals::source_report->[$HF::Globals::hostrow]{'sent_f_tries'} = 0;

	$HF::Globals::wasskipped = [ ];

	$layersptr = HF::Layers->new;
	if (!($layersptr->source_sanitycheck)) {
		HF::Report->report (2, 1, "FATAL source layers not sane, aborting", 'fail_layers');
		&to_other_end ('abort');
		$returnval = 2;
	} elsif (!($layersptr->source_start)) {
		HF::Report->report (2, 1, "FATAL source layers didn't start, aborting", 'fail_layers');
		&to_other_end ('abort');
		$returnval = 2;
	} else {
		if ($HF::Globals::islocal == 0) {
			&HF::Remote::remote_status_write;
		}
		if (($HF::Globals::opt_x == 2) || ($HF::Globals::opt_x == 20)) {
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " starting");
			$HF::Globals::opt_x_output = [ ];
			$layersptr->walk ($HF::Globals::source_layers, 'source');
			$HF::Globals::source_skips->[$HF::Globals::hostrow] = $HF::Globals::wasskipped;
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " finished");
			&to_other_end ('opt_x');
			$returnval = 0;
		} elsif ($HF::Globals::opt_x == 21) {
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " starting");
			$HF::Globals::opt_x_output = [ ];
			$layersptr->log_walk ($HF::Globals::source_layers, 'source');
			$HF::Globals::source_report->[$HF::Globals::hostrow]{'opt_x_output'} = $HF::Globals::opt_x_output;
			$HF::Globals::source_skips->[$HF::Globals::hostrow] = $HF::Globals::wasskipped;
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " finished");
			&to_other_end ('opt_x');
			$returnval = 0;
		} elsif ($HF::Globals::opt_x == 22) {
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " starting");
			$HF::Globals::opt_x_output = [ ];
			$layersptr->diskusage_walk ($HF::Globals::source_layers, 'source');
			$layersptr->diskusage_report;
			$HF::Globals::source_report->[$HF::Globals::hostrow]{'opt_x_output'} = $HF::Globals::opt_x_output;
			$HF::Globals::source_skips->[$HF::Globals::hostrow] = $HF::Globals::wasskipped;
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " finished");
			&to_other_end ('opt_x');
			$returnval = 0;
		} elsif (($HF::Globals::opt_x == 3) || ($HF::Globals::opt_x == 30) || ($HF::Globals::opt_x == 31) || ($HF::Globals::opt_x == 32)) {
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " waiting");
			$from = &from_other_end;
			if ($from eq 'abort') {
				HF::Report->report (2, 1, "PROGRESS aborting", 'fail_layers');
				$returnval = 2;
			} elsif ($from eq 'opt_x') {
				$HF::Globals::source_skips->[$HF::Globals::hostrow] = $HF::Globals::wasskipped;
				HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " completed");
				$returnval = 0;
			} else {
				HF::Report->report (4, 1, "FATAL should have received opt_x, instead received '" . $from . "'");
			}
		} else {
			$from = &from_other_end;
			if ($from eq 'abort') {
				HF::Report->report (2, 1, "PROGRESS aborting", 'fail_layers');
				$returnval = 2;
			} elsif ($from eq 'walk') {
				&to_other_end ('walk');
				&source_send_ftw ($layersptr);
				&source_wantedfiles;
				&to_other_end ('nomoresourcetreewalk');
				$HF::Globals::source_skips->[$HF::Globals::hostrow] = $HF::Globals::wasskipped;
				$returnval = 0;
			} else {
				HF::Report->report (4, 1, "FATAL should have received walk, instead received '" . $from . "'");
			}
		}
	}

	return ($returnval);
}

sub source_send_ftw {

	my $layersptr = shift;

	my $checkpointtime;
	my $walkhash;
	my $cyclecount;
	my $now;

	$checkpointtime = time + 1;

	while ($walkhash = $layersptr->source_next, defined ($walkhash->{'inode'})) {

# HF::Report->report (0, 13, &HF::Layers::print_hash ($walkhash));

		if (($HF::Globals::opt_x == 1) && ((($walkhash->{'mode'} & 0170000) == 0100000) && ($walkhash->{'size'} > 1))) {
			$walkhash->{'size'} = 1;
		}

		&to_other_end (join ("\t",
			'walk',
			$walkhash->{'encname'},
			$walkhash->{'dev'},
			$walkhash->{'inode'},
			$walkhash->{'mode'},
			$walkhash->{'links'},
			$walkhash->{'owner'},
			$walkhash->{'group'},
			$walkhash->{'rdev'},
			$walkhash->{'size'},
			$walkhash->{'mtime'},
			$walkhash->{'enctarget'},
			$walkhash->{'encpath'},
			$walkhash->{'checksum'},
			$walkhash->{'treatment'}));

		$cyclecount++;
		if (($HF::Globals::opt_X == 1) && (($cyclecount % 5000) == 0)) {
			HF::Report->report (0, 4, "PROGRESS sleeping to reduce workload");
			sleep (1);
		}
		if (($cyclecount % 50000) == 0) {
			&source_wantedfiles;
		}

		$now = time;
		if ($now > $checkpointtime) {
			HF::Report->report (0, 4, "PROGRESS '" . $walkhash->{'encname'} . "'");
			$checkpointtime = $now + 1;
		}
	}
}

sub source_wantedfiles {

	my $from;
	my @fromarr;
	my $wantedfilenames;
	my $key;
	my $localname;
	my $localpath;
	my $treatment;
	my $bufsize;
	my $readres;
	my $buf;
	my $checkpointtime;
	my $now;
	my $sentcount;
	my $filesize;
	my $percent;
	my $totsentmegs;

# -----

	$checkpointtime = time + 1;

	$HF::Globals::source_report->[$HF::Globals::hostrow]{'sent_f_tries'}++;
	&to_other_end ('suspendcompare');

# -----

	while (($from = &from_other_end) ne "\t\t") {
		if ($from eq 'abort') {
			HF::Report->report (3, 3, "PROGRESS aborting");
		}
		@fromarr = split ("\t", $from);
		$wantedfilenames->{$fromarr[0]} = join ("\t", $fromarr[1], $fromarr[2]);
	}

# -----

	# Since no cmpname treatment given to sort, order resembles
	# but is not identical to walk; still, approximate sort is far
	# better than no sort
	foreach $key (sort (keys (%{$wantedfilenames}))) {

		$localname = HF::Encode->decode ($key);
		($localpath, $treatment) = split ("\t", $wantedfilenames->{$key});
		$localpath = HF::Encode->decode ($localpath);

		if ($HF::Globals::dowrite == 1) {
			if (!(open (FILEGUTS, "<", $localpath))) {
				HF::Report->report (2, 2, "FAILURE can't open '" . HF::Encode->encode ($localpath) . "' for sending contents, error '$!'", 'fail_layers');
			} else {
				$HF::Globals::source_report->[$HF::Globals::hostrow]{'sent_f'}++;
				&to_other_end (join ("\t", $key, 'writethisfile'));

				$sentcount = 0;
				if ($HF::Globals::opt_x == 1) {
					$bufsize = 1;
				} else {
					if (($treatment == 104) || ($treatment == 114)) {
						$bufsize = 0;
					} else {
						$bufsize = 10000;
					}
				}
				while ($readres = read (FILEGUTS, $buf, $bufsize), (($readres != 0) && (defined ($readres)))) {
					$sentcount += $readres;
					# This perl number works when counting higher than 4 Gig
					$HF::Globals::source_report->[$HF::Globals::hostrow]{'sent_f_bytes'} += $readres;
					&to_other_end (HF::Marshal::marshal ($buf), 1);

					$now = time;
					if ($now > $checkpointtime) {
						$filesize = (lstat ($localpath))[7];
						if (($HF::Globals::opt_x == 1) && ($filesize >= 1)) {
							$filesize = 1;
						}

						if ($filesize > 0) {
							$percent = ($sentcount/$filesize) * 100;
						} else {
							$percent = 100;
						}
						$totsentmegs = $HF::Globals::source_report->[$HF::Globals::hostrow]{'sent_f_bytes'}/1000000;
						HF::Report->report (0, 4, sprintf ("PROGRESS total sent %s MB; %s of %s bytes, %d%% of '%s'",
							&HF::Misc::commify_number (sprintf ("%0.1f", $totsentmegs)), &HF::Misc::commify_number ($sentcount),
							&HF::Misc::commify_number ($filesize), $percent, $key));
						$checkpointtime = $now + 1;
					}
					if ($HF::Globals::opt_x == 1) {
						last;
					}
				}

				if (!(defined ($readres))) {
					HF::Report->report (2, 2, "FAILURE can't read '" . HF::Encode->encode ($localpath) . "', error '$!'", 'fail_connection');
				}

				&to_other_end (HF::Marshal::marshal (''), 1);
			}
			close (FILEGUTS);
		}
	}

# -----

	&to_other_end (join ("\t", '', ''));

	return (1);
}

sub target_main {

	my $self = shift;

	my $from;
	my $layersptr;
	my $returnval;

	if ($self->{'issource'} != 0) {
		HF::Report->report (4, 1, "FATAL Harmonize object not set up to be target");
	}

	$HF::Globals::target_differences->[$HF::Globals::hostrow] = [ ];

	$HF::Globals::target_report->[$HF::Globals::hostrow] = { };
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_delete'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_reload'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_total'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'fail_audit'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'fail_connection'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'fail_layers'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'fail_total'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'fix_dir_mtimes'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'fix_hardlinks'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_b'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_c'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_d'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_f'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_f_bytes'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_l'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_p'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_s'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'target_diff_rows'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'target_diff_zeroed'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'want_sent_f'} = 0;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'want_sent_f_bytes'} = 0;

	$HF::Globals::wasskipped = [ ];

	$HF::Globals::layersptr = HF::Layers->new;
	if (!($HF::Globals::layersptr->target_sanitycheck)) {
		HF::Report->report (2, 1, "FATAL target layers not sane, aborting", 'fail_layers');
		&to_other_end ('abort');
		$returnval = 2;
	} elsif (!($HF::Globals::layersptr->target_start)) {
		HF::Report->report (2, 1, "FATAL target layers didn't start, aborting", 'fail_layers');
		&to_other_end ('abort');
		$returnval = 2;
	} else {
		if ($HF::Globals::islocal == 0) {
			&HF::Remote::remote_status_write;
		}
		if (($HF::Globals::opt_x == 3) || ($HF::Globals::opt_x == 30)) {
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " starting");
			$HF::Globals::opt_x_output = [ ];
			$HF::Globals::layersptr->walk ($HF::Globals::target_layers, 'target');
			$HF::Globals::target_skips->[$HF::Globals::hostrow] = $HF::Globals::wasskipped;
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " finished");
			&to_other_end ('opt_x');
			$returnval = 0;
		} elsif ($HF::Globals::opt_x == 31) {
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " starting");
			$HF::Globals::opt_x_output = [ ];
			$HF::Globals::layersptr->log_walk ($HF::Globals::target_layers, 'target');
			$HF::Globals::target_report->[$HF::Globals::hostrow]{'opt_x_output'} = $HF::Globals::opt_x_output;
			$HF::Globals::target_skips->[$HF::Globals::hostrow] = $HF::Globals::wasskipped;
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " finished");
			&to_other_end ('opt_x');
			$returnval = 0;
		} elsif ($HF::Globals::opt_x == 32) {
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " starting");
			$HF::Globals::opt_x_output = [ ];
			$HF::Globals::layersptr->diskusage_walk ($HF::Globals::target_layers, 'target');
			$HF::Globals::layersptr->diskusage_report;
			$HF::Globals::target_report->[$HF::Globals::hostrow]{'opt_x_output'} = $HF::Globals::opt_x_output;
			$HF::Globals::target_skips->[$HF::Globals::hostrow] = $HF::Globals::wasskipped;
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " finished");
			&to_other_end ('opt_x');
			$returnval = 0;
		} elsif (($HF::Globals::opt_x == 2) || ($HF::Globals::opt_x == 20) || ($HF::Globals::opt_x == 21) || ($HF::Globals::opt_x == 22)) {
			HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " waiting");
			$from = &from_other_end;
			if ($from eq 'abort') {
				HF::Report->report (2, 1, "PROGRESS aborting", 'fail_layers');
				$returnval = 2;
			} elsif ($from eq 'opt_x') {
				$HF::Globals::target_skips->[$HF::Globals::hostrow] = $HF::Globals::wasskipped;
				HF::Report->report (0, 2, "PROGRESS opt_x " . $HF::Globals::opt_x . " completed");
				$returnval = 0;
			} else {
				HF::Report->report (4, 1, "FATAL should have received opt_x, instead received '" . $from . "'");
			}
		} else {
			&to_other_end ('walk');
			$from = &from_other_end;
			if ($from eq 'abort') {
				HF::Report->report (2, 1, "PROGRESS aborting", 'fail_layers');
				$returnval = 2;
			} elsif ($from eq 'walk') {
				&target_compare;
				if ($HF::Globals::examine eq '') {
					if ($HF::Globals::donotfixhardlinks == 0) {
						&target_harmonize_hardlinks;
					}
					&target_harmonize_dir_mtimes;
				}
				$HF::Globals::target_skips->[$HF::Globals::hostrow] = $HF::Globals::wasskipped;
				$returnval = 0;
			} else {
				HF::Report->report (4, 1, "FATAL should have received walk, instead received '" . $from . "'");
			}
		}
	}

	$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_total'} =
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'} +
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'} +
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_reload'} +
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_delete'};

	return ($returnval);
}

sub target_harmonize_dir_mtimes {

	my $key;
	my $dirmtime;
	my $realname;
	my $dirpath;
	my @dirpathlist;
	my @realnamelist;
	my @whichlayer;

HF::Report->report (0, 19, "TRACE sub target_harmonize_dir_mtimes");

	HF::Report->report (0, 4, "PROGRESS fixing dir mtimes");

	# does not need cmpname treatment because foo-/ and foo/ can
	# be touched in either order, as they are at the same level
	foreach $key (sort {$b cmp $a} (keys (%{$HF::Globals::dirstotouch}))) {

		$dirmtime = $HF::Globals::dirmtimes->{$key};
		# source symlink replacing target dir which was wrong type can register source-nonexistent
		# dirs to touch when target dir's subdirs are deleted
		if (defined ($dirmtime)) {
			@dirpathlist = sort (keys (%{$HF::Globals::dirpaths->{$key}}));
			@realnamelist = ( );
			foreach $dirpath (@dirpathlist) {
				push (@realnamelist, HF::Encode->decode ($dirpath));
			}

			# If the dir doesn't exist on the target, it didn't match to have encpath recorded
			# during the walk.  Search to tell what layer's walk path it should be on.
			if ($realnamelist[0] eq '') {
				$realname = HF::Encode->decode ($key);
				@whichlayer = &which_layer_for_new_entry ($realname);
				$realname = $whichlayer[0];
###&HF::Misc::joinpaths ($HF::Globals::target_layers->[0]{'mount'}[1], $realname);
				@realnamelist = ($realname);
			}
			foreach $realname (@realnamelist) {
				$HF::Globals::target_report->[$HF::Globals::hostrow]{'fix_dir_mtimes'}++;
				if ($HF::Globals::dowrite == 1) {
					if (!(utime ($dirmtime, $dirmtime, $realname))) {
						HF::Report->report (2, 2, "FAILURE 1 can't utime '" . HF::Encode->encode ($realname) . "', error '$!'", 'fail_layers');
					}
				}
			}
		}
	}
}

sub target_harmonize_hardlinks {

	my $source_row;
	my $target_row;
	my $link_slots;
	my $ofsource;
	my $slot;
	my $devino;
	my @thisrow;
	my $claimedcount;
	my @tmpmsg;

HF::Report->report (0, 19, "TRACE sub target_harmonize_hardlinks");

	HF::Report->report (0, 4, "PROGRESS fixing hardlinks");

	#  Rearrange into new data structure
	$link_slots = [ ];
	foreach $ofsource (1, 0) {
		$link_slots->[$ofsource] = [ ];
		$slot = 0;
		foreach $devino (keys (%{$HF::Globals::linkarray->[$ofsource]})) {
			@thisrow = @{$HF::Globals::linkarray->[$ofsource]{$devino}};
			$claimedcount = shift (@thisrow);
			# Since no cmpname treatment given to sort, order is similar
			# but not identical to walk
			@{$link_slots->[$ofsource][$slot]} = sort (@thisrow);
			unshift (@{$link_slots->[$ofsource][$slot]}, $devino, $claimedcount);
			$slot++;
		}
		# Since no cmpname treatment given to sort, order is similar
		# but not identical to walk
		@{$link_slots->[$ofsource]} = sort {$a->[2] cmp $b->[2]} @{$link_slots->[$ofsource]};
	}

	#  Test and repair
	$source_row = 0;
	$target_row = 0;

	while (($#{$link_slots->[1][$source_row]} != -1) || ($#{$link_slots->[0][$target_row]} != -1)) {

		if (($#{$link_slots->[1][$source_row]} != -1) && ($link_slots->[1][$source_row][1] != ($#{$link_slots->[1][$source_row]} - 1))) {
#####			$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_unfound_hardlinks'} += ($#{$link_slots->[1][$source_row]} - 1);
			@tmpmsg = @{$link_slots->[1][$source_row]};
			HF::Report->report (2, 2, "WARNING transferred only " . ($#tmpmsg - 1) . " of " . $tmpmsg[1] . " hardlinks for source '" .
				join ("', '", @tmpmsg[2 .. $#tmpmsg]) . "'", 'fail_audit');
		}
		if (($#{$link_slots->[0][$target_row]} != -1) && ($link_slots->[0][$target_row][1] != ($#{$link_slots->[0][$target_row]} - 1))) {
#####			$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_unfound_hardlinks'} += ($#{$link_slots->[0][$target_row]} - 1);
			@tmpmsg = @{$link_slots->[0][$target_row]};
			if ($HF::Globals::dowrite == 1) {
				HF::Report->report (2, 2, "WARNING found only " . ($#tmpmsg - 1) . " of " . $tmpmsg[1] . " hardlinks for target '" .
					join ("', '", @tmpmsg[2 .. $#tmpmsg]) . "'", 'fail_audit');
			} else {
				HF::Report->report (2, 2, "WARNING found only " . ($#tmpmsg - 1) . " of " . $tmpmsg[1] . " hardlinks for target '" .
					join ("', '", @tmpmsg[2 .. $#tmpmsg]) . "'");
			}
		}

		if ($#{$link_slots->[0][$target_row]} == -1) {
			&target_harmonize_hardlinks_walk_col ($link_slots->[1], undef, $source_row, undef);
			$source_row++;
			next;
		} elsif ($#{$link_slots->[1][$source_row]} == -1) {
			&target_harmonize_hardlinks_walk_col (undef, $link_slots->[0], undef, $target_row);
			$target_row++;
			next;
		} elsif ($link_slots->[1][$source_row][2] lt $link_slots->[0][$target_row][2]) {
			&target_harmonize_hardlinks_walk_col ($link_slots->[1], undef, $source_row, undef);
			$source_row++;
			next;
		} elsif ($link_slots->[1][$source_row][2] gt $link_slots->[0][$target_row][2]) {
			&target_harmonize_hardlinks_walk_col (undef, $link_slots->[0], undef, $target_row);
			$target_row++;
			next;
		} elsif ($link_slots->[1][$source_row][2] eq $link_slots->[0][$target_row][2]) {
			&target_harmonize_hardlinks_walk_col ($link_slots->[1], $link_slots->[0], $source_row, $target_row);
			$source_row++;
			$target_row++;
			next;
		} else {
			HF::Report->report (4, 1, "FATAL should never get here");
		}
	}
}

sub target_harmonize_hardlinks_walk_col {

	my $source_slots = shift;
	my $target_slots = shift;
	my $source_row = shift;
	my $target_row = shift;

	my $source_col;
	my $target_col;

HF::Report->report (0, 19, "TRACE sub target_harmonize_hardlinks_walk_col");

	$source_col = 3;
	$target_col = 3;

	while (($source_slots->[$source_row][$source_col] ne '') || ($target_slots->[$target_row][$target_col] ne '')) {

		if ($target_slots->[$target_row][$target_col] eq '') {
			&target_harmonize_hardlinks_inspect_or_create_link ($source_slots, $target_slots, $source_row, $target_row, $source_col, $target_col, 1);
			$source_col++;
			next;
		} elsif ($source_slots->[$source_row][$source_col] eq '') {
			&target_harmonize_hardlinks_separate_link ($source_slots, $target_slots, $source_row, $target_row, $source_col, $target_col);
			$target_col++;
			next;
		} elsif ($source_slots->[$source_row][$source_col] lt $target_slots->[$target_row][$target_col]) {
			&target_harmonize_hardlinks_inspect_or_create_link ($source_slots, $target_slots, $source_row, $target_row, $source_col, $target_col, 1);
			$source_col++;
			next;
		} elsif ($source_slots->[$source_row][$source_col] gt $target_slots->[$target_row][$target_col]) {
			&target_harmonize_hardlinks_separate_link ($source_slots, $target_slots, $source_row, $target_row, $source_col, $target_col);
			$target_col++;
			next;
		} elsif ($source_slots->[$source_row][$source_col] eq $target_slots->[$target_row][$target_col]) {
			&target_harmonize_hardlinks_inspect_or_create_link ($source_slots, $target_slots, $source_row, $target_row, $source_col, $target_col, 0);
			$source_col++;
			$target_col++;
			next;
		} else {
			HF::Report->report (4, 1, "FATAL should never get here");
		}
	}
}

sub target_harmonize_hardlinks_inspect_or_create_link {

	my $source_slots = shift;
	my $target_slots = shift;
	my $source_row = shift;
	my $target_row = shift;
	my $source_col = shift;
	my $target_col = shift;
	my $create = shift;

	my $linkedfile;
	my $linkedfiledec;
	my $seenpath;
	my $seenpathdec;
	my $realname;
	my @whichlayer;
	my $linkedmasterhash;
	my $linkedhash;

HF::Report->report (0, 19, "TRACE sub target_harmonize_hardlinks_inspect_or_create_link");

	# XXX repeats lstat of master link for each col
	$linkedfile = $source_slots->[$source_row][2];
	$seenpath = $HF::Globals::linkpaths->{$linkedfile};
	$linkedfiledec = HF::Encode->decode ($linkedfile);
	if ($seenpath ne '') {
		$realname = HF::Encode->decode ($seenpath);
	} else {
		@whichlayer = &which_layer_for_new_entry ($linkedfiledec);
		$realname = $whichlayer[0];
	}
	$linkedmasterhash = HF::Harmonize->stattohash ($realname);
	$linkedmasterhash->{'encname'} = $linkedfile;
	$linkedmasterhash->{'name'} = $linkedfiledec;

	$linkedfile = $source_slots->[$source_row][$source_col];
	$seenpath = $HF::Globals::linkpaths->{$linkedfile};
	$linkedfiledec = HF::Encode->decode ($linkedfile);
	if ($seenpath ne '') {
		$realname = HF::Encode->decode ($seenpath);
	} else {
		@whichlayer = &which_layer_for_new_entry ($linkedfiledec);
		$realname = $whichlayer[0];
#&HF::Misc::joinpaths ($HF::Globals::target_layers->[0]{'mount'}[1], $linkedfiledec);
	}
	$linkedhash = HF::Harmonize->stattohash ($realname);
	$linkedhash->{'encname'} = $linkedfile;
	$linkedhash->{'name'} = $linkedfiledec;

	# Record so don't process target file for separate after it's been relinked here
	$HF::Globals::linkfixed->{$linkedfile} = 1;

	if (($linkedmasterhash->{'dev'} != $linkedhash->{'dev'}) && ($create == 0)) {
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'}++;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'fix_hardlinks'}++;
		HF::Report->report (1, 3, "DIFFERENCE 1 nonmatching inodes, files '" . $linkedmasterhash->{'encname'} . "' and '" . $linkedhash->{'encname'} .
			"' are supposed to be hardlinked");
		HF::Report->report (2, 2, "FAILURE files '" . $linkedmasterhash->{'encname'} . "' and '" . $linkedhash->{'encname'} .
			"' are supposed to be hardlinked, but they are on different devices on the remote end so there is no fixing it", 'fail_audit');
		if (defined ($HF::Globals::outputplbig)) {
			push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
				[ $linkedhash->{'name'}, 'relink', undef, { %$linkedmasterhash }, { %$linkedhash } ]);
		}
	} elsif (($linkedmasterhash->{'inode'} != $linkedhash->{'inode'}) || (!(defined ($linkedmasterhash->{'inode'}))) || (!(defined ($linkedhash->{'inode'})))) {
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'}++;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'fix_hardlinks'}++;
		HF::Report->report (1, 3, "DIFFERENCE 2 nonmatching inodes, files '" . $linkedmasterhash->{'encname'} . "' and '" . $linkedhash->{'encname'} .
			"' are supposed to be hardlinked");
		if (defined ($HF::Globals::outputplbig)) {
			push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
				[ $linkedhash->{'name'}, 'relink', undef, { %$linkedmasterhash }, { %$linkedhash } ]);
		}
		if (($HF::Globals::dowrite == 1) && ($HF::Globals::dodelete == 1)) {
			&target_register_dir_to_touch (HF::Encode->encode (&HF::Misc::dirpart ($linkedfile)));
			if (!(unlink ($linkedhash->{'path'}))) {
				HF::Report->report (2, 2, "FAILURE 1 can't unlink '" . $linkedhash->{'encpath'} . "', error '$!'", 'fail_layers');
			}
			if (!(link ($linkedmasterhash->{'path'}, $linkedhash->{'path'}))) {
				HF::Report->report (2, 2, "FAILURE can't link '" . $linkedhash->{'encpath'} . "' to '" . $linkedmasterhash->{'encpath'} .
					"', error '$!'", 'fail_layers');
			}
		}
	}
}

sub target_harmonize_hardlinks_separate_link {

	my $source_slots = shift;
	my $target_slots = shift;
	my $source_row = shift;
	my $target_row = shift;
	my $source_col = shift;
	my $target_col = shift;

	my $linkedfile;
	my $linkedfiledec;
	my $seenpath;
	my $seenpathdec;
	my $realname;
	my @whichlayer;
	my $temp;
	my $linkedhash;

HF::Report->report (0, 19, "TRACE sub target_harmonize_hardlinks_separate_link");

	$linkedfile = $target_slots->[$target_row][$target_col];
	$seenpath = $HF::Globals::linkpaths->{$linkedfile};
	$linkedfiledec = HF::Encode->decode ($linkedfile);
	if ($seenpath ne '') {
		$realname = HF::Encode->decode ($seenpath);
	} else {
		@whichlayer = &which_layer_for_new_entry ($linkedfiledec);
		$realname = $whichlayer[0];
#&HF::Misc::joinpaths ($HF::Globals::target_layers->[0]{'mount'}[1], $linkedfiledec);
	}

	if (-e $realname) {	# may already have been deleted during walk

		# Don't process target file for separate after it's been processed earlier to relink
		if (exists ($HF::Globals::linkfixed->{$linkedfile})) {
			return;
		}

		$linkedhash = HF::Harmonize->stattohash ($realname);
		$linkedhash->{'encname'} = $linkedfile;
		$linkedhash->{'name'} = $linkedfiledec;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'}++;
		HF::Report->report (1, 3, "DIFFERENCE hardlinked file '" . $linkedfile . "' is not supposed to be hardlinked where it is");
		if (defined ($HF::Globals::outputplbig)) {
			push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
				[ $linkedhash->{'name'}, 'unlink', undef, undef, { %$linkedhash } ]);
		}

		$temp = HF::Misc::makeuniqfilename ($realname);

		if (($HF::Globals::dowrite == 1) && ($HF::Globals::dodelete == 1)) {
			&target_register_dir_to_touch (HF::Encode->encode (&HF::Misc::dirpart ($linkedfile)));
			if (!(&HF::Misc::cleansystem ('cp', '-a', $realname, $temp))) {
				HF::Report->report (2, 2, "FAILURE can't cp -a '" . $realname . "' '" . $temp . "', error '$!'", 'fail_layers');
			}
			if (!(unlink ($realname))) {
				HF::Report->report (2, 2, "FAILURE 2 can't unlink '" . $realname . "', error '$!'", 'fail_layers');
			}
			if (!(&HF::Misc::cleansystem ('mv', $temp, $realname))) {
				HF::Report->report (2, 2, "FAILURE can't mv '" . $temp . "' '" . $realname . "', error '$!'", 'fail_layers');
			}
		}
	}
}

sub hash_to_ls {

	my $ftwhash = shift;

	my $line;

	if (!(defined ($ftwhash->{'inode'}))) {
		return ('hash_to_ls NO FILE');
	}

	if ($ftwhash->{'type'} == 0100000) {	# file
		$line = '-';
	} elsif ($ftwhash->{'type'} == 0040000) {	# dir
		$line = 'd';
	} elsif ($ftwhash->{'type'} == 0120000) {	# symlink
		$line = 'l';
	} elsif ($ftwhash->{'type'} == 0020000) {	# char dev
		$line = 'c';
	} elsif ($ftwhash->{'type'} == 0060000) {	# block dev
		$line = 'b';
	} elsif ($ftwhash->{'type'} == 0010000) {	# fifo
		$line = 'p';
	} elsif ($ftwhash->{'type'} == 0140000) {	# socket
		$line = 's';
	} else {
		HF::Report->report (4, 1, sprintf ("FATAL hash_to_ls unknown type %04o for encpath '%s'", $ftwhash->{'type'}, $ftwhash->{'encpath'}));
	}

	$line .= sprintf (' 0%04o %2d %5d %5d', $ftwhash->{'perms'}, $ftwhash->{'links'}, $ftwhash->{'owner'}, $ftwhash->{'group'});

	if ($ftwhash->{'type'} == 0100000) {
		$line .= sprintf (' %10d', $ftwhash->{'size'});
	} elsif (($ftwhash->{'type'} == 0020000) || ($ftwhash->{'type'} == 0060000)) {
		$line .= sprintf (' %10d', $ftwhash->{'rdev'});
	} else {
		$line .= '           ';
	}

#$ftwhash->{'encpath'}
	$line .= sprintf (" %10d %s", $ftwhash->{'mtime'}, $ftwhash->{'encname'});

	if (($ftwhash->{'type'} == 0100000) && ($ftwhash->{'checksum'} ne '')) {
		$line .= ' ' . $ftwhash->{'checksum'};
	} elsif ($ftwhash->{'type'} == 0120000) {
		$line .= ' -> ' . $ftwhash->{'enctarget'};
	}

	return ($line);
}

sub target_get_source {

	my $from;
	my $result;
	my @fromarr;
	my $cmd;

	$from = &from_other_end;

	@fromarr = split ("\t", $from);
	$cmd = shift (@fromarr);
	if ($cmd eq 'abort') {
		HF::Report->report (3, 1, 'PROGRESS aborting');
	} elsif ($cmd eq 'nomoresourcetreewalk') {
		return (undef);
	} elsif ($cmd eq 'suspendcompare') {
		&target_wantedfiles;
		# Go back to get data that caller is expecting
		return (&target_get_source);
	} elsif ($cmd eq 'walk') {
		$result->{'encname'} = shift (@fromarr);
		$result->{'dev'} = shift (@fromarr);
		$result->{'inode'} = shift (@fromarr);
		$result->{'mode'} = shift (@fromarr);
		$result->{'links'} = shift (@fromarr);
		$result->{'owner'} = shift (@fromarr);
		$result->{'group'} = shift (@fromarr);
		$result->{'rdev'} = shift (@fromarr);
		$result->{'size'} = shift (@fromarr);
		$result->{'mtime'} = shift (@fromarr);
		$result->{'enctarget'} = shift (@fromarr);
		$result->{'encpath'} = shift (@fromarr);
		$result->{'checksum'} = shift (@fromarr);
		$result->{'treatment'} = shift (@fromarr);

		$result->{'name'} = HF::Encode->decode ($result->{'encname'});
		$result->{'target'} = HF::Encode->decode ($result->{'enctarget'});
		$result->{'path'} = HF::Encode->decode ($result->{'encpath'});

		$result->{'type'} = $result->{'mode'} & 0170000;
		$result->{'perms'} = $result->{'mode'} & 07777;

		# To make target_compare work properly, the character
		# "/" needs to compare lower than any other character,
		# such as "-".  If this is not done, then "foo-"
		# compares as earlier than "foo/", and the compare
		# walk misbehaves.

		$result->{'cmpname'} = $result->{'name'};
		$result->{'cmpname'} =~ s|/|\000|g;

		&target_register_links ($result, 1);
		&target_register_dir_mtimes ($result);

		if (($HF::Globals::opt_x == 5) && (defined ($result->{'inode'}))) {
			HF::Report->report (1, 1, "SOURCE " . &hash_to_ls ($result));
		}

		return ($result);
	} else {
		HF::Report->report (4, 1, "FATAL unknown command '$cmd'");
	}
}

sub target_wantedfiles {

# XXX should come from system headers
	my $MAXFILENAME = 100;

	my $key;
	my $from;
	my @fromarr;
	my $localname;
	my $localpath;
	my $realname;
	my $tempname;
	my $domv;
	my $canwrite;
	my $block;
	my $askedwant;
	my $askedhave;
	my $foundhave;
	my $temp1;
	my $temp2;
	my $temp3;
	my @whichlayer;
	my $examinepath;
	my $make_path_error;
	my $totfilecount;
	my $totfilebytes;

# -----

HF::Report->report (0, 19, "TRACE sub target_wantedfiles");

	if ($HF::Globals::dowrite == 1) {
		HF::Report->report (0, 4, "PROGRESS target sending file contents wanted");
		# Since no cmpname treatment given to sort, order is
		# similar but not identical to walk
		$totfilecount = 0;
		$totfilebytes = 0;
		foreach $key (sort (keys (%{$HF::Globals::wantfilecontents}))) {

			if ($HF::Globals::opt_x == 4) {
				HF::Report->report (0, 2, "WANT name '" . $HF::Globals::wantfilecontents->{$key}{'want'}{'encname'} .
					"' path '" . $HF::Globals::wantfilecontents->{$key}{'want'}{'encpath'} .
					"' treatment '" . $HF::Globals::wantfilecontents->{$key}{'want'}{'treatment'} . "'");
			}
			$totfilecount++;
			$totfilebytes += $HF::Globals::wantfilecontents->{$key}{'want'}{'size'};
			&to_other_end (join ("\t", $HF::Globals::wantfilecontents->{$key}{'want'}{'encname'},
				$HF::Globals::wantfilecontents->{$key}{'want'}{'encpath'},
				$HF::Globals::wantfilecontents->{$key}{'want'}{'treatment'}));
		}
		if ($totfilecount > 0) {
			HF::Report->report (0, 4, "PROGRESS target wants " . &HF::Misc::commify_number ($totfilecount) . " files totaling " .
				&HF::Misc::commify_number ($totfilebytes) . " bytes");
		}
	}

# -----

	&to_other_end (join ("\t", '', '', ''));

# -----

	while ($from = &from_other_end, @fromarr = split ("\t", $from), (!(($fromarr[0] eq '') && ($fromarr[1] eq '')))) {

		$localname = HF::Encode->decode ($fromarr[0]);
		$localpath = HF::Encode->decode ($fromarr[1]);

		if (($HF::Globals::opt_x == 4) || ($HF::Globals::opt_x == 40)) {
			HF::Report->report (0, 2, "GOT  name '" . $fromarr[0] . "'");
			if ($HF::Globals::opt_x == 40) {
				HF::Report->report (0, 2, "SAVED IN WANTFILECONTENTS want
                 " . &HF::Layers::print_hash ($HF::Globals::wantfilecontents->{$localname}{'want'}));
				HF::Report->report (0, 2, "SAVED IN WANTFILECONTENTS have
                 " . &HF::Layers::print_hash ($HF::Globals::wantfilecontents->{$localname}{'have'}));
			}
		}

		# collect real name
		$temp1 = { %{$HF::Globals::wantfilecontents->{$localname}{'want'}} };

		if ($HF::Globals::examine ne '') {
			$temp1->{'startdir'} = $HF::Globals::examine;
			$temp1->{'path'} = HF::Misc::joinpaths ($HF::Globals::examine, $temp1->{'name'});
			$temp1->{'encpath'} = HF::Encode->encode ($temp1->{'path'});

			$examinepath = &HF::Misc::dirpart ($temp1->{'path'});
			if (!(-e $examinepath)) {
				make_path ($examinepath, { mode => 0755, error => \$make_path_error });	# ogmt not relevant
				if (@$make_path_error) {
					# Only show first error that make_path returns
					HF::Report->report (2, 1, "FAILURE mkdir(s) of target examine interior dir '" .
						HF::Encode->encode ($examinepath) . "' failed with error '" .
						join (' ', %{${$make_path_error}[0]}) . "'", 'fail_layers');
				}
			} elsif ((!(-d $HF::Globals::examine)) || (-l $HF::Globals::examine)) {
				HF::Report->report (2, 1, "FAILURE target examine interior dir '" . HF::Encode->encode ($examinepath) .
					"' is not dir", 'fail_layers');
			}
		} else {
			if ($HF::Globals::wantfilecontents->{$localname}{'have'}{'startdir'} ne '') {
				$temp1->{'startdir'} = $HF::Globals::wantfilecontents->{$localname}{'have'}{'startdir'};
				$temp1->{'path'} = $HF::Globals::wantfilecontents->{$localname}{'have'}{'path'};
				$temp1->{'encpath'} = $HF::Globals::wantfilecontents->{$localname}{'have'}{'encpath'};
			} else {
###				@whichlayer = &which_layer_for_new_entry ($fromarr[0]);
				@whichlayer = &which_layer_for_new_entry ($temp1->{'name'});
				$temp1->{'startdir'} = $HF::Globals::target_layers->[$whichlayer[1]]{'mount'}[1];
###				$temp1->{'path'} = HF::Misc::joinpaths ($temp1->{'startdir'}, $temp1->{'name'});
				$temp1->{'path'} = $whichlayer[0];
				$temp1->{'encpath'} = HF::Encode->encode ($temp1->{'path'});
			}
		}

		# make temp name
		$temp2 = { %{$temp1} };	# copy top level of object
		$temp2->{'path'} = HF::Misc::makeuniqfilename ($temp2->{'path'});
		$temp2->{'encpath'} = HF::Encode->encode ($temp2->{'path'});
		$temp2->{'name'} = undef;	# not constructed
		$temp2->{'encname'} = undef;	# not constructed

		$temp3 = { };	# mostly empty, so as to adjust every stat
		$temp3->{'startdir'} = $temp2->{'startdir'};
		$temp3->{'path'} = $temp2->{'path'};
		$temp3->{'encpath'} = $temp2->{'encpath'};

# -----

		if ($HF::Globals::dowrite == 0) {
			HF::Report->report (4, 1, 'FATAL should not be writing file contents');
		}

		if (!(open (FILEGUTS, ">", $temp2->{'path'}))) {
			HF::Report->report (2, 2, "FAILURE can't open '" . HF::Encode->encode ($temp2->{'path'}) . "', error '$!'", 'fail_layers');
		}

		while (length ($block = &HF::Marshal::unmarshal ($HF::Globals::hosts->[$HF::Globals::hostrow]{'read'})) gt 0) {
			print FILEGUTS $block;
		}

		close (FILEGUTS);

# -----

HF::Report->report (0, 19, "TRACE calling target_inspect_and_update_outside from target_wantedfiles");
		&target_inspect_and_update_outside ($temp2, $temp3, 1);

		# rename temp name to real name
		if (!(rename ($temp2->{'path'}, $temp1->{'path'}))) {
			HF::Report->report (2, 2, "FAILURE can't rename '" . HF::Encode->encode ($temp2->{'path'}) . "' to '" . HF::Encode->encode ($temp1->{'path'}) . "', error '$!'", 'fail_layers');
		}

		&target_register_dir_to_touch (HF::Encode->encode (&HF::Misc::dirpart ($localname)));
	}

	undef ($HF::Globals::wantfilecontents);
	undef ($HF::Globals::makeuniqfilename_used);

	return (1);
}

sub to_other_end {

	my $arg = shift;
	my $suppressnewline = shift;

	if (!($suppressnewline)) {
		$arg .= "\n";
	}

	if (!(print {$HF::Globals::hosts->[$HF::Globals::hostrow]{'write'}} $arg)) {
		HF::Report->report (2, 2, 'FAILURE Harmonize write of comm to other end', 'fail_connection');
		return (undef);
	} else {
		return (1);
	}
}

sub from_other_end {

	my $inline;

	$inline = $HF::Globals::hosts->[$HF::Globals::hostrow]{'read'}->getline;
	if (!(defined ($inline))) {
		HF::Report->report (3, 1, 'FATAL Harmonize read of comm from other end, result is undef');
	}
	chop ($inline);

	return ($inline);
}

sub target_register_links {

	my $input = shift;
	my $ofsource = shift;

	my $dev;
	my $ino;

	if (($input->{'links'} > 1) &&
		(($input->{'type'} == 0100000) ||
		 ($input->{'type'} == 0020000) ||
		 ($input->{'type'} == 0060000) ||
		 ($input->{'type'} == 0010000) ||
		 ($input->{'type'} == 0140000))) {

		$dev = $input->{'dev'};
		$ino = $input->{'inode'};

		if (exists ($HF::Globals::linkarray->[$ofsource]{"$dev,$ino"})) {
			if (($ofsource == 1) && ($input->{'links'} != $HF::Globals::linkarray->[$ofsource]{"$dev,$ino"}[0])) {
				HF::Report->report (2, 2, "FAILURE 1 inconsistent link count for '" . $input->{'encname'} .
					"', was " . $HF::Globals::linkarray->[$ofsource]{"$dev,$ino"}[0] .
					", now " . $input->{'links'}, 'fail_audit');
			}
			push (@{$HF::Globals::linkarray->[$ofsource]{"$dev,$ino"}}, $input->{'encname'});
		} else {
			push (@{$HF::Globals::linkarray->[$ofsource]{"$dev,$ino"}}, $input->{'links'}, $input->{'encname'});
		}
	}

	return (1);
}

sub target_register_link_paths {

	my $input = shift;

	if (($input->{'links'} > 1) &&
		(($input->{'type'} == 0100000) ||
		 ($input->{'type'} == 0020000) ||
		 ($input->{'type'} == 0060000) ||
		 ($input->{'type'} == 0010000) ||
		 ($input->{'type'} == 0140000))) {

		$HF::Globals::linkpaths->{$input->{'encname'}} = $input->{'encpath'};
	}

	return (1);
}

sub target_register_dir_to_touch {

	my $dir = shift;

	if ($dir eq '') {
		$dir = '.';
	}

	$HF::Globals::dirstotouch->{$dir} = 1;

	return (1);
}

sub target_register_dir_mtimes {

	my $input = shift;

	my $dir;

	if ($input->{'type'} == 0040000) {

		$dir = $input->{'encname'};
		if ($dir eq '') {
			$dir = '.';
		}

		$HF::Globals::dirmtimes->{$dir} = $input->{'mtime'};
	}

	return (1);
}

sub target_register_dir_paths {

	my $input = shift;

	my $dir;

	if ($input->{'type'} == 0040000) {
		$dir = $input->{'encname'};
		if ($dir eq '') {
			$dir = '.';
		}
		$HF::Globals::dirpaths->{$dir}{$input->{'encpath'}} = 1;

		# swap+reverse means multiple layers on target have same dir name;
		# touch them all
		if (exists ($input->{'encpath_content'})) {
			$HF::Globals::dirpaths->{$dir}{$input->{'encpath_content'}} = 1;
		}
	}

	return (1);
}

sub target_inspect_and_update_outside {

	my $want = shift;
	my $have = shift;
	my $quiet = shift;

	my $didchowngroup;

# 0100000  Regular file
# 0040000  Directory
# 0120000  Symbolic link
# 0020000  Character device
# 0060000  Block device
# 0010000  FIFO
# 0140000  Socket

HF::Report->report (0, 19, "TRACE sub target_inspect_and_update_outside, want encpath " . $want->{'encpath'} . " have encpath " . $have->{'encpath'});

	$didchowngroup = 0;

	# check owner
	if (($want->{'type'} != 0120000) &&	# not symlink
		(($quiet == 1) ||
			(($want->{'treatment'} != 101) && ($have->{'treatment'} != 101) &&
			 ($want->{'treatment'} != 102) && ($have->{'treatment'} != 102) &&
			 ($want->{'treatment'} != 109) && ($have->{'treatment'} != 109)))) {
HF::Report->report (0, 19, "TRACE target_inspect_and_update_outside, checking owner");

		if ((!(defined ($have->{'owner'}))) || ($want->{'owner'} != $have->{'owner'})) {
			if ($quiet == 0) {
				$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'}++;
				HF::Report->report (1, 3, "DIFFERENCE wrong owner, want " . $want->{'owner'} . " have " . $have->{'owner'} . " for '" . $want->{'encname'} . "'");
				if (defined ($HF::Globals::outputplbig)) {
					push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
						[ $have->{'name'}, 'fix', 'owner', { %$want }, { %$have } ]);
				}
			}
			if (($HF::Globals::dowrite == 1) && (($HF::Globals::examine eq '') || ($quiet == 1))) {
				if ($> == 0) {
					$didchowngroup = 1;
					if (!(chown ($want->{'owner'}, $have->{'group'}, $have->{'path'}))) {
						HF::Report->report (2, 2, "FAILURE 1 can't chown '" . $have->{'encpath'} . "', error '$!'", 'fail_layers');
					}
###				} else {
###					HF::Report->report (2, 2, "FAILURE can't chown '" . $have->{'encpath'} . "', not root", 'fail_layers');
				}
			}
			$have->{'owner'} = $want->{'owner'};	# make group check logic work
		}
	}

	# check group
	if (($want->{'type'} != 0120000) &&	# not symlink
		(($quiet == 1) ||
			(($want->{'treatment'} != 101) && ($have->{'treatment'} != 101) &&
			 ($want->{'treatment'} != 102) && ($have->{'treatment'} != 102) &&
			 ($want->{'treatment'} != 109) && ($have->{'treatment'} != 109)))) {
HF::Report->report (0, 19, "TRACE target_inspect_and_update_outside, checking group");

		if ((!(defined ($have->{'group'}))) || ($want->{'group'} != $have->{'group'})) {
			if ($quiet == 0) {
				$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'}++;
				HF::Report->report (1, 3, "DIFFERENCE wrong group, want " . $want->{'group'} . " have " . $have->{'group'} . " for '" . $want->{'encname'} . "'");
				if (defined ($HF::Globals::outputplbig)) {
					push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
						[ $have->{'name'}, 'fix', 'group', { %$want }, { %$have } ]);
				}
			}
			if (($HF::Globals::dowrite == 1) && (($HF::Globals::examine eq '') || ($quiet == 1))) {
				if ($> == 0) {
					$didchowngroup = 1;
					if (!(chown ($have->{'owner'}, $want->{'group'}, $have->{'path'}))) {
						HF::Report->report (2, 2, "FAILURE 2 can't chown '" . $have->{'encpath'} . "', error '$!'", 'fail_layers');
					}
###				} else {
###					HF::Report->report (2, 2, "FAILURE can't chgrp '" . $have->{'encpath'} . "', not root", 'fail_layers');
				}
			}
		}
	}

	# check mode
	if (($want->{'type'} != 0120000) &&	# not symlink
		(($quiet == 1) ||
			(($want->{'treatment'} != 102) && ($have->{'treatment'} != 102) &&
			 ($want->{'treatment'} != 109) && ($have->{'treatment'} != 109)))) {
HF::Report->report (0, 19, "TRACE target_inspect_and_update_outside, checking perms");
		if ((!(defined ($have->{'perms'}))) || ($want->{'perms'} != $have->{'perms'})) {
			if ($quiet == 0) {
				$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'}++;
				HF::Report->report (1, 3, sprintf ("DIFFERENCE wrong perms, want 0%04o have 0%04o for '%s'", $want->{'perms'}, $have->{'perms'}, $want->{'encname'}));
				if (defined ($HF::Globals::outputplbig)) {
					push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
						[ $have->{'name'}, 'fix', 'perms', { %$want }, { %$have } ]);
				}
			}
			if (($HF::Globals::dowrite == 1) && (($HF::Globals::examine eq '') || ($quiet == 1))) {
				if (!(chmod ($want->{'perms'}, $have->{'path'}))) {
					HF::Report->report (2, 2, "FAILURE 1 can't chmod '" . $have->{'encpath'} . "', error '$!'", 'fail_layers');
				}
			}
		}
		if ((($want->{'perms'} & 07000) != 0) && ($didchowngroup == 1)) {
			if (($HF::Globals::dowrite == 1) && (($HF::Globals::examine eq '') || ($quiet == 1))) {
				# to put back the suid/sgid/sticky that chown/chgrp stripped
				if (!(chmod ($want->{'perms'}, $have->{'path'}))) {
					HF::Report->report (2, 2, "FAILURE 2 can't chmod '" . $have->{'encpath'} . "', error '$!'", 'fail_layers');
				}
			}
		}
	}

	# check mtime
HF::Report->report (0, 19, "TRACE target_inspect_and_update_outside may check mtime, encpath " . $want->{'encpath'} . " have " . $have->{'mtime'} . " want " . $want->{'mtime'});

	if (($want->{'type'} != 0120000) &&	# not symlink
		(($quiet == 1) || (
			($want->{'treatment'} != 102) && ($have->{'treatment'} != 102) &&
			($want->{'treatment'} != 104) && ($have->{'treatment'} != 104) &&
			($want->{'treatment'} != 105) && ($have->{'treatment'} != 105) &&
			($want->{'treatment'} != 107) && ($have->{'treatment'} != 107) &&
			($want->{'treatment'} != 113) && ($have->{'treatment'} != 113) &&
			(!((($want->{'treatment'} == 109) || ($have->{'treatment'} == 109)) && ($want->{'type'} == 0100000)))))) {

HF::Report->report (0, 19, "TRACE target_inspect_and_update_outside checking mtime, encname " . $have->{'encname'} . " encpath " . $want->{'encpath'} . " have " . $have->{'mtime'} . " want " . $want->{'mtime'});

		if ((!(defined ($have->{'mtime'}))) || ($want->{'mtime'} != $have->{'mtime'})) {
			if ($quiet == 0) {
				$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'}++;
				HF::Report->report (1, 3, "DIFFERENCE wrong mtime, want " . $want->{'mtime'} . " have " . $have->{'mtime'} . " for '" . $want->{'encname'} . "'");
				if (defined ($HF::Globals::outputplbig)) {
					push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
						[ $have->{'name'}, 'fix', 'mtime', { %$want }, { %$have } ]);
				}
			}
			if (($HF::Globals::dowrite == 1) && (($HF::Globals::examine eq '') || ($quiet == 1))) {
HF::Report->report (0, 19, "TRACE target_inspect_and_update_outside updating mtime, encname " . $have->{'encname'} . " encpath " . $want->{'encpath'} . " have " . $have->{'mtime'} . " want " . $want->{'mtime'});
				if (!(utime ($want->{'mtime'}, $want->{'mtime'}, $have->{'path'}))) {
					HF::Report->report (2, 2, "FAILURE 2 can't utime '" . $have->{'encpath'} . "', error '$!'", 'fail_layers');
				}
			}
		}
	}
}

sub target_delete {

	my $have = shift;

HF::Report->report (0, 19, "TRACE sub target_delete, have encpath " . $have->{'encpath'});

	if (($HF::Globals::dowrite == 1) && ($HF::Globals::examine eq '')) {
		if ($have->{'type'} == 0040000) {	# dir
			if ($HF::Globals::dodelete == 1) {
				if (!(&HF::Misc::cleansystem ('rm', '-rf', $have->{'path'}))) {
					HF::Report->report (2, 2, "FAILURE can't rm -rf '" . $have->{'encpath'} . "', error '$!'", 'fail_layers');
				}
			}
		} else {
			&target_un_register_links ($have);
			if ($HF::Globals::dodelete == 1) {
				if (!(unlink ($have->{'path'}))) {
					HF::Report->report (2, 2, "FAILURE 3 can't unlink '" . $have->{'encpath'} . "', error '$!'", 'fail_layers');
				}
			}
		}
	}
}

sub target_un_register_links {

	my $have = shift;

	my $dev;
	my $ino;
	my $count;
	my @linknames;

HF::Report->report (0, 19, "TRACE sub target_un_register_links, have encpath " . $have->{'encpath'});

	$dev = $have->{'dev'};
	$ino = $have->{'inode'};
	if (exists ($HF::Globals::linkarray->[0]{"$dev,$ino"})) {
		$count = $HF::Globals::linkarray->[0]{"$dev,$ino"}[0];
		if ($#{$HF::Globals::linkarray->[0]{"$dev,$ino"}} > 1) {
			@linknames = @{$HF::Globals::linkarray->[0]{"$dev,$ino"}};
			shift (@linknames);
			@linknames = grep ($_ ne $have->{'encname'}, @linknames);
			$HF::Globals::linkarray->[0]{"$dev,$ino"} = [ --$count, @linknames ];
		} else {
			delete ($HF::Globals::linkarray->[0]{"$dev,$ino"});
		}
	}
}

sub create_non_file {

	my $want = shift;

	my $status;
	my $shortsocketname;
	my $shortsocketdir;

HF::Report->report (0, 19, "TRACE sub create_non_file, want encpath " . $want->{'encpath'});

	if ($want->{'type'} == 0040000) {	# dir
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'}++;
		if ($HF::Globals::dowrite == 1) {
			if (!($status = mkdir ($want->{'path'}, $want->{'perms'}))) {
				HF::Report->report (2, 2, "FAILURE can't mkdir '" . $want->{'encpath'} . "', error '$!'", 'fail_layers');
			}
		}
	} elsif ($want->{'type'} == 0120000) {	# symlink
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'}++;
		if ($HF::Globals::dowrite == 1) {
			if (!($status = symlink ($want->{'target'}, $want->{'path'}))) {
				HF::Report->report (2, 2, "FAILURE can't symlink '" . $want->{'encpath'} . "' pointing to '" . $want->{'enctarget'} . "', error '$!'", 'fail_layers');
			}
		}
	} elsif ($want->{'type'} == 0020000) {	# char dev
		if ($> == 0) {
			$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'}++;
			if ($HF::Globals::dowrite == 1) {
				if (!($status = &HF::Misc::mknod ($want->{'path'}, $want->{'mode'}, $want->{'rdev'}))) {
					HF::Report->report (2, 2, "FAILURE 1 can't mknod '" . $want->{'encpath'} . "' as char, error '$!'", 'fail_layers');
				}
			}
###		} else {
###			HF::Report->report (2, 2, "FAILURE can't mknod '" . $want->{'encpath'} . "' as char, not root", 'fail_layers');
		}
	} elsif ($want->{'type'} == 0060000) {	# block dev
		if ($> == 0) {
			$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'}++;
			if ($HF::Globals::dowrite == 1) {
				if (!($status = &HF::Misc::mknod ($want->{'path'}, $want->{'mode'}, $want->{'rdev'}))) {
					HF::Report->report (2, 2, "FAILURE 2 can't mknod '" . $want->{'encpath'} . "' as block, error '$!'", 'fail_layers');
				}
			}
###		} else {
###			HF::Report->report (2, 2, "FAILURE can't mknod '" . $want->{'encpath'} . "' as block, not root", 'fail_layers');
		}
	} elsif ($want->{'type'} == 0010000) {	# fifo
		# non-root mknod supposed to work for fifo under Linux
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'}++;
		if ($HF::Globals::dowrite == 1) {
			if (!($status = &HF::Misc::mknod ($want->{'path'}, $want->{'mode'}, 0))) {
				HF::Report->report (2, 2, "FAILURE 3 can't mknod '" . $want->{'encpath'} . "', error '$!'", 'fail_layers');
			}
		}
	} elsif ($want->{'type'} == 0140000) {	# socket
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'}++;
		if ($HF::Globals::dowrite == 1) {
			if (-e $want->{'path'}) {
				if (!($status = unlink ($want->{'path'}))) {
					HF::Report->report (2, 2, "FAILURE 4 can't unlink '" . $want->{'encpath'} . "', error '$!'", 'fail_layers');
				}
			}
			if (!($status = socket (SOCKETFH, &PF_UNIX, &SOCK_STREAM, 0))) {
				HF::Report->report (2, 2, "FAILURE can't socket '" . $want->{'encpath'} . "', error '$!'", 'fail_layers');
			}

			# silent truncation of pathnames longer than around 108 in bind(); chdir to dir first to shorten pathname
			$shortsocketname = &HF::Misc::basepart ($want->{'path'});
			$shortsocketdir = &HF::Misc::dirpart ($want->{'path'});
			if (($want->{'path'} eq '/') || ($shortsocketname eq '/')) {
				HF::Report->report (4, 1, "FATAL socket name should not be slash for path '" . $want->{'encpath'} . "'");
			}
			if (length ($shortsocketname) >= 50) {
				HF::Report->report (2, 1, "WARNING socket basename may be too long for bind '" . $want->{'encpath'} . "'");
			}
			if (!($status = opendir (MAKESOCKETDIRH, '.'))) {
				HF::Report->report (3, 1, "FATAL can't opendir '.'");
			}
			if (!($status = chdir ($shortsocketdir))) {
				HF::Report->report (3, 1, "FATAL can't chdir into '" . HF::Encode->encode ($shortsocketdir) . "' to make socket '" . $want->{'encpath'} . "'");
			}
			if (!($status = bind (SOCKETFH, &sockaddr_un ($shortsocketname)))) {
				HF::Report->report (2, 2, "FAILURE can't bind '" . $want->{'encpath'} . "' using name '" . HF::Encode->encode ($shortsocketname) . "', error '$!'", 'fail_layers');
			}
			if (!($status = chdir (MAKESOCKETDIRH))) {
				HF::Report->report (3, 1, "FATAL can't chdir back to normal cwd after making socket '" . $want->{'encpath'} . "'");
			}
			if (!($status = closedir (MAKESOCKETDIRH))) {
				HF::Report->report (3, 1, "FATAL can't closedir '.' after making socket '" . $want->{'encpath'} . "'");
			}

			close (SOCKETFH);
		}
	} else {
		HF::Report->report (4, 1, sprintf ("FATAL unknown type %04o for encpath '%s'", $want->{'type'}, $want->{'encpath'}));
	}
}

sub target_create {

	my $want = shift;
	my $have = shift;

	my $temp1;
	my $temp2;
	my @whichlayer;

HF::Report->report (0, 19, "TRACE sub target_create, want encpath " . $want->{'encpath'} . " have encpath " . $have->{'encpath'});

	if ($want->{'type'} == 0100000) {	# file
		&target_ask_source_for_file_contents ($want, $have);
	} else {
		# construct target's version of layers result for filename
		$temp1 = { %{$want} };	# copy top level of object

		if ($have->{'startdir'} ne '') {
			$temp1->{'startdir'} = $have->{'startdir'};
			$temp1->{'path'} = $have->{'path'};
			$temp1->{'encpath'} = $have->{'encpath'};
		} else {
			@whichlayer = &which_layer_for_new_entry ($temp1->{'name'});
			$temp1->{'startdir'} = $HF::Globals::target_layers->[$whichlayer[1]]{'mount'}[1];
			$temp1->{'path'} = $whichlayer[0];
###&HF::Misc::joinpaths ($HF::Globals::target_layers->[$whichlayer[1]]{'mount'}[1], $temp1->{'name'});
			$temp1->{'encpath'} = HF::Encode->encode ($temp1->{'path'});
		}

		# mostly empty, so as to adjust every stat
		$temp2 = { };
		$temp2->{'name'} = $temp1->{'name'};
		$temp2->{'encname'} = $temp1->{'encname'};
		$temp2->{'path'} = $temp1->{'path'};
		$temp2->{'encpath'} = $temp1->{'encpath'};
		$temp2->{'startdir'} = $temp1->{'startdir'};
		# create it
		if (($HF::Globals::dowrite == 1) && ($HF::Globals::examine eq '')) {
			&create_non_file ($temp1);
HF::Report->report (0, 19, "TRACE calling target_inspect_and_update_outside from target_create");
			&target_inspect_and_update_outside ($temp1, $temp2, 1);
		}
		&target_register_dir_to_touch (HF::Encode->encode (&HF::Misc::dirpart ($want->{'name'})));
	}
}

sub target_check_and_fix {

	my $want = shift;
	my $have = shift;

	my $deleteit;
	my $wantcopy;
	my $havecopy;

HF::Report->report (0, 19, "TRACE sub target_check_and_fix, want encpath " . $want->{'encpath'} . " have encpath " . $have->{'encpath'} . " want owner " . $want->{'owner'} . " have owner " . $have->{'owner'});

	if ((($want->{'treatment'} != 0) && ($have->{'treatment'} != 0)) &&
		(!(	(($want->{'treatment'} == 104) && ($have->{'treatment'} == 104)) ||
			(($want->{'treatment'} == 109) && ($have->{'treatment'} == 109))))) {
# XXX handle this better
		HF::Report->report (3, 1, 'FATAL treatments collided, source treatment ' . $want->{'treatment'} .
			' target treatment ' . $have->{'treatment'} . " for '" . $want->{'encname'} . "'");
	}

	# reasons to delete and recreate

	$deleteit = 0;

	if ($want->{'type'} != $have->{'type'}) {
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_reload'}++;
		if (($have->{'treatment'} == 109) || ($have->{'treatment'} == 109)) {
			HF::Report->report (1, 3, sprintf ("DIFFERENCE wrong type, want 0%06o have 0%06o for '%s'",
				$want->{'type'}, $have->{'type'}, $have->{'encname'}));
			if (defined ($HF::Globals::outputplbig)) {
				push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
					[ $have->{'name'}, 'ignore', 'type', { %$want }, { %$have } ]);
			}
		} else {
			HF::Report->report (1, 3, sprintf ("DIFFERENCE wrong type, want 0%06o have 0%06o, reloading '%s'",
				$want->{'type'}, $have->{'type'}, $have->{'encname'}));
			$deleteit = 1;
			if (defined ($HF::Globals::outputplbig)) {
				$wantcopy = { %{$want} };	# copy top level of object
				$havecopy = { %{$have} };	# copy top level of object
				push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
					[ $have->{'name'}, 'replace', 'type', { %$wantcopy }, { %$havecopy } ]);
			}
		}
	} else {
		$have->{'pre_fix_mtime'} = $have->{'mtime'};
HF::Report->report (0, 19, "TRACE calling target_inspect_and_update_outside from 1 target_check_and_fix");
		&target_inspect_and_update_outside ($want, $have, 0);
		$have->{'mtime'} = $have->{'pre_fix_mtime'};

		if (($have->{'treatment'} == 103) || ($have->{'treatment'} == 113)) {
			$have->{'startdir'} = $have->{'startdir_content'};
			$have->{'path'} = $have->{'path_content'};
			$have->{'encpath'} = $have->{'encpath_content'};
			$have->{'rdev'} = $have->{'rdev_content'};
			$have->{'target'} = $have->{'target_content'};
			$have->{'enctarget'} = $have->{'enctarget_content'};
			$have->{'size'} = $have->{'size_content'};
			$have->{'checksum'} = $have->{'checksum_content'};
HF::Report->report (0, 19, "TRACE calling target_inspect_and_update_outside from 2 target_check_and_fix");
			&target_inspect_and_update_outside ($want, $have, 0);
		}

		if ($want->{'type'} == 0100000) {
			if (	($want->{'treatment'} != 102) && ($have->{'treatment'} != 102) &&
				($want->{'treatment'} != 104) && ($have->{'treatment'} != 104) &&
				($want->{'treatment'} != 105) && ($have->{'treatment'} != 105) &&
				($want->{'treatment'} != 109) && ($have->{'treatment'} != 109)) {

HF::Report->report (0, 19, "TRACE target_check_and_fix, checking size");
				if ($want->{'size'} != $have->{'size'}) {
					$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_reload'}++;
					HF::Report->report (1, 3, "DIFFERENCE file wrong size, want " . $want->{'size'} . " have " . $have->{'size'} . ", reloading '" . $have->{'encname'} . "'");
					if (defined ($HF::Globals::outputplbig)) {
						$wantcopy = { %{$want} };	# copy top level of object
						$havecopy = { %{$have} };	# copy top level of object
						push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
							[ $have->{'name'}, 'replace', 'size', { %$wantcopy }, { %$havecopy } ]);
					}
					$deleteit = 2;
				} elsif ($want->{'checksum'} ne '') {
					if ($have->{'checksum'} eq '') {
						$have->{'checksum'} = &getchecksum ($have->{'path'});
					}
					if ($want->{'checksum'} ne $have->{'checksum'}) {
						$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_reload'}++;
						HF::Report->report (1, 3, "DIFFERENCE file wrong checksum, want " . $want->{'checksum'} . " have " . $have->{'checksum'} . ", reloading '" . $have->{'encname'} . "'");
						if (defined ($HF::Globals::outputplbig)) {
							$wantcopy = { %{$want} };	# copy top level of object
							$havecopy = { %{$have} };	# copy top level of object
							push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
								[ $have->{'name'}, 'replace', 'checksum', { %$wantcopy }, { %$havecopy } ]);
						}
						$deleteit = 2;
					}
				} elsif (($want->{'mtime'} != $have->{'mtime'}) && (($want->{'treatment'} != 107) && ($have->{'treatment'} != 107))) {
HF::Report->report (0, 19, "TRACE target_check_and_fix, checking mtime");
					$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_reload'}++;
					HF::Report->report (1, 3, "DIFFERENCE file wrong mtime, want " . $want->{'mtime'} . " have " . $have->{'mtime'} . ", reloading '" . $have->{'encname'} . "'");
					if (defined ($HF::Globals::outputplbig)) {
						$wantcopy = { %{$want} };	# copy top level of object
						$havecopy = { %{$have} };	# copy top level of object
						push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
							[ $have->{'name'}, 'replace', 'mtime', { %$wantcopy }, { %$havecopy } ]);
					}
					$deleteit = 2;
				}
			} elsif (($want->{'treatment'} == 109) || ($have->{'treatment'} == 109)) {
				if ($want->{'checksum'} ne '') {
HF::Report->report (0, 19, "TRACE target_check_and_fix 2, checking mtime");
					if ($have->{'checksum'} eq '') {
						$have->{'checksum'} = &getchecksum ($have->{'path'});
					}
					if ($want->{'checksum'} eq $have->{'checksum'}) {
						if ($want->{'mtime'} != $have->{'mtime'}) {
							$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'}++;
							HF::Report->report (1, 3, "DIFFERENCE file wrong mtime, want " . $want->{'mtime'} . " have " . $have->{'mtime'} . ", fixing '" . $have->{'encname'} . "'");
							if (defined ($HF::Globals::outputplbig)) {
								push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
									[ $have->{'name'}, 'fix', 'mtime', { %$want }, { %$have } ]);
							}

							if ($HF::Globals::dowrite == 1) {
HF::Report->report (0, 19, "TRACE target_check_and_fix 2 updating mtime, encname " . $have->{'encname'} . " encpath " . $want->{'encpath'} . " have " . $have->{'mtime'} . " want " . $want->{'mtime'});
								if (!(utime ($want->{'mtime'}, $want->{'mtime'}, $have->{'path'}))) {
									HF::Report->report (2, 2, "FAILURE 3 can't utime '" . $have->{'encpath'} . "', error '$!'", 'fail_layers');
								}
							}
						}
					}
				}
			}
		} elsif (($want->{'type'} == 0120000) && ($want->{'target'} ne $have->{'target'}) &&
			(($want->{'treatment'} != 108) && ($have->{'treatment'} != 108))) {

			$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_reload'}++;
			HF::Report->report (1, 3, "DIFFERENCE symlink wrong target, want '" . $want->{'target'} . "' have '" . $have->{'target'} . "', recreating '" . $have->{'encname'} . "'");
			if (defined ($HF::Globals::outputplbig)) {
				$wantcopy = { %{$want} };	# copy top level of object
				$havecopy = { %{$have} };	# copy top level of object
				push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
					[ $have->{'name'}, 'replace', 'target', { %$wantcopy }, { %$havecopy } ]);
			}
			$deleteit = 1;
		} elsif ((($want->{'type'} == 0020000) || ($want->{'type'} == 0060000)) && ($want->{'rdev'} != $have->{'rdev'})) {
			$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_reload'}++;
			HF::Report->report (1, 3, "DIFFERENCE device wrong rdev, want " . $want->{'rdev'} . " have " . $have->{'rdev'} . ", recreating '" . $have->{'encname'} . "'");
			if (defined ($HF::Globals::outputplbig)) {
				$wantcopy = { %{$want} };	# copy top level of object
				$havecopy = { %{$have} };	# copy top level of object
				push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
					[ $have->{'name'}, 'replace', 'rdev', { %$wantcopy }, { %$havecopy } ]);
			}
			$deleteit = 1;
		}
	}

	if (($deleteit == 1) || ($deleteit == 2)) {
		&target_report_delete_if_dir ($have);
	}
	if ($deleteit == 1) {
		&target_delete ($have);
	}
	if (($deleteit == 1) || ($deleteit == 2)) {
		&target_create ($want, $have);
	}
}

sub target_ask_source_for_file_contents {

	my $want = shift;
	my $have = shift;

HF::Report->report (0, 19, "TRACE sub target_ask_source_for_file_contents, want encpath " . $want->{'encpath'} . " have encpath " . $have->{'encpath'});

	$HF::Globals::wantfilecontents->{$want->{'name'}}{'want'} = { %{$want} };	# copy top level of object
	if (defined ($have)) {
		$HF::Globals::wantfilecontents->{$want->{'name'}}{'have'} = { %{$have} };	# copy top level of object
	} else {
		$HF::Globals::wantfilecontents->{$want->{'name'}}{'have'} = { };
	}
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'want_sent_f'}++;
	$HF::Globals::target_report->[$HF::Globals::hostrow]{'want_sent_f_bytes'} += $want->{'size'};
}

sub target_get_target {

	my $result;

	$result = $HF::Globals::layersptr->target_next;

	# To make target_compare work properly, the character
	# "/" needs to compare lower than any other character,
	# such as "-".  If this is not done, then "foo-"
	# compares as earlier than "foo/", and the compare
	# walk misbehaves.

	$result->{'cmpname'} = $result->{'name'};
	$result->{'cmpname'} =~ s|/|\000|g;

	&target_register_links ($result, 0);
	&target_register_link_paths ($result);
	&target_register_dir_paths ($result);

	if (($HF::Globals::opt_x == 5) && (defined ($result->{'inode'}))) {
		HF::Report->report (1, 1, "TARGET " . &hash_to_ls ($result));
	}

	return ($result);
}

sub target_report_delete_if_dir {

	my $have = shift;

	my $deletedirname;
	my $deleteftw;
	my $deleteftwhash;
	my $deletebelowname;
	my $deletecounthere;

	if ($have->{'type'} == 0040000) {	# dir

		$deletedirname = $have->{'name'};

		$deleteftw = HF::FTWstream->new ($have->{'path'}, undef, 0);

		# discard the top dir, it has already been reported
		$deleteftwhash = $deleteftw->next;

		while ($deleteftwhash = $deleteftw->next, defined ($deleteftwhash->{'inode'})) {

			$deletebelowname = &HF::Misc::joinpaths ($deletedirname, $deleteftwhash->{'name'});

			$deletecounthere++;
			$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_delete'}++;

			if (($HF::Globals::opt_x == 6) && (($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'} +
				$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'} +
				$HF::Globals::target_report->[$HF::Globals::hostrow]{'compare_delete'} +
				$deletecounthere) > 100)) {

				if ($HF::Globals::islocal == 1) {
					$HF::Globals::local_report->[$HF::Globals::hostrow]{'abandoned_remote'} = 1;
					HF::Report->report (1, 3, "PROGRESS aborting after 100 filetree entries seen");
					&to_other_end ('abort');
					goto ABANDONED_REMOTE;
				} else {
					&to_other_end ('abort');
					HF::Report->report (3, 3, "PROGRESS aborting after 100 filetree entries seen");
				}
			}

			HF::Report->report (1, 3, "COMPARE 9 DELETE '" . HF::Encode->encode ($deletebelowname) . "'");
			if (defined ($HF::Globals::outputplbig)) {
				push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
					[ $deletebelowname, 'delete', undef, undef, { %$deleteftwhash } ]);
			}

			&target_un_register_links ($deleteftwhash);
		}
	}
}

sub target_compare {

	my $source;
	my $target;

	my $deleteftw;
	my $deleteftwhash;
	my $deletedirname;
	my $deletebelowname;

	my $gensource;

	HF::Report->report (0, 10, "COMPARE STATUS starting");

	$HF::Globals::linkarray = [ ];
	$HF::Globals::linkarray->[0] = { };
	$HF::Globals::linkarray->[1] = { };

	$source = &target_get_source;
	$target = &target_get_target;

	for (;;) {

		# To make target_compare work properly, the character
		# "/" needs to compare lower than any other character,
		# such as "-".  If this is not done, then "foo-"
		# compares as earlier than "foo/", and the compare
		# walk misbehaves.

		if (($source->{'inode'} eq '') && ($target->{'inode'} eq '')) {
			HF::Report->report (0, 10, "COMPARE STATUS no more files on either side");
			last;
		}

		if ($source->{'inode'} eq '') {
			HF::Report->report (0, 10, "COMPARE STATUS no more source files");
			do {
				if ($target->{'treatment'} == 207) {
					$HF::Globals::target_report->[$HF::Globals::hostrow]{'compare_check'}++;
					HF::Report->report (0, 11, "COMPARE 3 CHECK '" . $target->{'encname'} . "'");
HF::Report->report (0, 19, "TRACE sub target_compare, 1 applying genowner");
					$gensource = { %{$target} };
					$gensource->{'treatment'} = 0;
					$gensource->{'owner'} = $target->{'genowner'};
HF::Report->report (0, 19, "TRACE sub target_compare, 1 gensource owner " . $gensource->{'owner'});
					$gensource->{'group'} = $target->{'gengroup'};
					$gensource->{'perms'} = $target->{'genperms'};
					$gensource->{'mode'} = $target->{'genmode'};
					&target_check_and_fix ($gensource, $target);
				} elsif (($target->{'treatment'} != 105) && ($target->{'treatment'} != 109)) {
					$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_delete'}++;
					HF::Report->report (1, 3, "COMPARE 2 DELETE '" . $target->{'encname'} . "'");
					if (defined ($HF::Globals::outputplbig)) {
						push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
							[ $target->{'name'}, 'delete', undef, undef, { %$target } ]);
					}
					&target_report_delete_if_dir ($target);
					&target_delete ($target);
					&target_register_dir_to_touch (HF::Encode->encode (&HF::Misc::dirpart ($target->{'name'})));
				}

			} while ($target = &target_get_target, $target->{'inode'} ne '');
			last;
		}

		if ($target->{'inode'} eq '') {
			HF::Report->report (0, 10, "COMPARE STATUS no more target files");
			do {
				if (($source->{'treatment'} != 207) && ($source->{'treatment'} != 105) && ($source->{'treatment'} != 109)) {
					$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'}++;
					HF::Report->report (1, 3, "COMPARE 2 CREATE '" . $source->{'encname'} . "'");
					if (defined ($HF::Globals::outputplbig)) {
						push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
							[ $source->{'name'}, 'create', undef, { %$source }, undef ])
					}
					&target_create ($source, undef);
				}
			} while ($source = &target_get_source, $source->{'inode'} ne '');
			last;
		}

		if ($source->{'cmpname'} eq $target->{'cmpname'}) {
			HF::Report->report (0, 11, "COMPARE STATUS matching filenames '" . $source->{'encname'} . "'");
			$HF::Globals::target_report->[$HF::Globals::hostrow]{'compare_check'}++;
			HF::Report->report (0, 11, "COMPARE 1 CHECK '" . $target->{'encname'} . "'");
			&target_check_and_fix ($source, $target);
			$source = &target_get_source;
			$target = &target_get_target;
			next;
		}

		if ($source->{'cmpname'} lt $target->{'cmpname'}) {
			HF::Report->report (0, 10, "COMPARE STATUS source '" . $source->{'encname'} . "' lt target '" . $target->{'encname'} . "'");
			if (($source->{'treatment'} != 207) && ($source->{'treatment'} != 105) && ($source->{'treatment'} != 109)) {
				$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'}++;
				HF::Report->report (1, 3, "COMPARE 3 CREATE '" . $source->{'encname'} . "'");
				if (defined ($HF::Globals::outputplbig)) {
					push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
						[ $source->{'name'}, 'create', undef, { %$source }, undef ])
				}
				&target_create ($source, undef);
			}
			$source = &target_get_source;
			next;
		}

		if ($source->{'cmpname'} gt $target->{'cmpname'}) {
			HF::Report->report (0, 10, "COMPARE STATUS source '" . $source->{'encname'} . "' gt target '" . $target->{'encname'} . "'");
			if ($target->{'treatment'} == 207) {
				$HF::Globals::target_report->[$HF::Globals::hostrow]{'compare_check'}++;
				HF::Report->report (0, 11, "COMPARE 2 CHECK '" . $target->{'encname'} . "'");
HF::Report->report (0, 19, "TRACE sub target_compare, 2 applying genowner");
				$gensource = { %{$target} };
				$gensource->{'treatment'} = 0;
				$gensource->{'owner'} = $target->{'genowner'};
				$gensource->{'group'} = $target->{'gengroup'};
				$gensource->{'perms'} = $target->{'genperms'};
				$gensource->{'mode'} = $target->{'genmode'};
				&target_check_and_fix ($gensource, $target);
			} elsif (($target->{'treatment'} != 105) && ($target->{'treatment'} != 109)) {
				$HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_delete'}++;
				HF::Report->report (1, 3, "COMPARE 3 DELETE '" . $target->{'encname'} . "'");
				if (defined ($HF::Globals::outputplbig)) {
					push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
						[ $target->{'name'}, 'delete', undef, undef, { %$target } ]);
				}
				&target_report_delete_if_dir ($target);
				&target_delete ($target);
				&target_register_dir_to_touch (HF::Encode->encode (&HF::Misc::dirpart ($target->{'name'})));
			}
			$target = &target_get_target;
			next;
		}

		HF::Report->report (4, 1, 'FATAL 2 should never reach here');
	}

	HF::Report->report (0, 10, "COMPARE STATUS ending");
	return (1);
}

sub getchecksum {

	my $filename = shift;

	my $MD5;

	$HF::Harmonize::md5handle->reset;
	if (open ($MD5, "<", $filename)) {
		$HF::Harmonize::md5handle->addfile ($MD5);
		close ($MD5) || HF::Report->report (4, 1, "FATAL close of md5 fh for filename '$filename' failed, error '$!'");
		return ($HF::Harmonize::md5handle->hexdigest);
	} else {
		HF::Report->report (2, 2, "FAILURE 1 can't open '" . HF::Encode->encode ($filename) . "' for checksum, error '$!'", 'fail_layers');
		return (undef);
	}
}

sub stattohash {

	my $self = shift;
	my $path = shift;

	my $reshash;
	my $str;

	$reshash->{'name'} = $path;
	$reshash->{'encname'} = HF::Encode->encode ($path);

	$reshash->{'path'} = $reshash->{'name'};
#	$reshash->{'path_content'} = $reshash->{'path'};
	$reshash->{'encpath'} = $reshash->{'encname'};
#	$reshash->{'encpath_content'} = $reshash->{'encpath'};

	($reshash->{'dev'}, $reshash->{'inode'}, $reshash->{'mode'},
		$reshash->{'links'}, $reshash->{'owner'}, $reshash->{'group'},
		$reshash->{'rdev'}, $reshash->{'size'}, $reshash->{'atime'},
		$reshash->{'mtime'}, $reshash->{'ctime'},
		$reshash->{'blksize'}, $reshash->{'blocks'}) = lstat ($path);

	if (!(defined ($reshash->{'dev'}))) {
###		HF::Report->report (2, 2, "FAILURE 1 can't lstat '" . $reshash->{'encpath'} . "', error '$!'", 'fail_layers');
	}

	$reshash->{'type'} = $reshash->{'mode'} & 0170000;
	$reshash->{'perms'} = $reshash->{'mode'} & 07777;

	$reshash->{'target'} = undef;
	$reshash->{'enctarget'} = undef;

	$reshash->{'checksum'} = undef;

	return ($reshash);
}

sub which_layer_for_new_entry {

	my $try = shift;

	my $i;
	my $origtry;
	my $remainder;

#warn "@@@ which_layer_for_new_entry: try $try\n";
#warn "@@@ which_layer_for_new_entry: array bounds count " . $#{$HF::Globals::target_layers} . "\n";

	$origtry = $try;
	$try =~ s@^/@@;

	if ($#{$HF::Globals::target_layers} > 0) {
		# If the mount list's first entry is '', as conventional, or a later entry is '', it won't be found;
		# this is ok given the response to not-found is to put in layer 0 regardless of 0's mount point.

		while ($try ne '') {

			for ($i = $#{$HF::Globals::target_layers};  $i > -1;  $i--) {

#warn "@@@ which_layer_for_new_entry: try i $i $try eq " . " mount point '" . $HF::Globals::target_layers->[$i]{'mount'}[0] . "' mount source '" . $HF::Globals::target_layers->[$i]{'mount'}[1] . "'\n";

				if ($try eq $HF::Globals::target_layers->[$i]{'mount'}[0]) {
#warn "@@@ which_layer_for_new_entry: found " . $HF::Globals::target_layers->[$i]{'mount'}[0] . "\n";
					if (length ($try) > 0) {
						$remainder = substr ($origtry, length ($try) + 1);	# +1 for removed slash
					} else {
						$remainder = $origtry;
					}
					return (&HF::Misc::joinpaths ($HF::Globals::target_layers->[$i]{'mount'}[1], $remainder), $i);
				}
			}
			$try =~ s@(^|/)[^/]+$@@;
		}
#warn "@@@ which_layer_for_new_entry: not found\n";
	}

	return (&HF::Misc::joinpaths ($HF::Globals::target_layers->[0]{'mount'}[1], $origtry), 0);
}


BEGIN {

	&setup;
}

1;
