#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

package HF::Hosts;
use strict;

use Data::Dumper;

sub eval_file_to_tmp {

# XXX       do EXPR
# XXX           Uses the value of EXPR as a filename and executes the contents of the file as a Perl script.

	my $filename = shift;

	my $filecontents;
	my $evalcode;

	$HF::Globals::hosts_parsed = 0;
	$HF::Globals::hosts_file = $filename;

	if ($filename eq '-') {
		$filename = '/dev/stdin';
	}

	if (!(open (HOSTSFILE, "<", $filename))) {
		HF::Report->report (3, 1, "FATAL can't open hosts file '$filename', error '$!'");
	}
	{
		local ($/);
		$filecontents = <HOSTSFILE>;
	}
	close (HOSTSFILE);
	$evalcode = '$HF::Tmp::hosts = 
' . $filecontents . ';

1;';

	eval $evalcode;

	if ($@ eq '') {
		return (1);
	} else {
		HF::Report->report (3, 1, "FATAL eval error message for host file '$filename' is '$@'")
	}
}

sub parse_tmp_to_globals {

	my $i;
	my $iptr;
	my $optr;

	if ($#{$HF::Tmp::hosts->{'hosts'}} == -1) {
		HF::Report->report (2, 2, 'FAILURE no hosts, terminating parse', 'fail_connection');
		return (0);
	}

	for ($i = 0;  $i <= $#{$HF::Tmp::hosts->{'hosts'}};  $i++) {

		$iptr = $HF::Tmp::hosts->{'hosts'}[$i];

		$HF::Globals::hosts->[$i] = { };
		$optr = $HF::Globals::hosts->[$i];

		$optr->{'channel'} = $iptr->{'channel'};

		if (($iptr->{'channel'} eq 'fork') &&
			($iptr->{'ip'} eq '') &&
			($iptr->{'hostname'} eq '') &&
			(!(exists ($iptr->{'sshopts'})))) {

			# do nothing more

		} elsif (($iptr->{'channel'} eq 'null') &&
			($iptr->{'ip'} eq '') &&
			($iptr->{'hostname'} eq '') &&
			(!(exists ($iptr->{'sshopts'})))) {

			# do nothing more

		} elsif (($iptr->{'channel'} eq 'ssh') &&
			($iptr->{'ip'} ne '') &&
			($iptr->{'ip'} !~ /^-/) &&
			($iptr->{'hostname'} eq '')) {

			$optr->{'ip'} = $iptr->{'ip'};

			if (exists ($iptr->{'sshopts'})) {
				$optr->{'sshopts'} = $iptr->{'sshopts'};
			}

		} elsif (($iptr->{'channel'} eq 'ssh') &&
			($iptr->{'ip'} eq '') &&
			($iptr->{'hostname'} !~ /^-/) &&
			($iptr->{'hostname'} ne '')) {

			$optr->{'hostname'} = $iptr->{'hostname'};

			if (exists ($iptr->{'sshopts'})) {
				$optr->{'sshopts'} = $iptr->{'sshopts'};
			}

		} else {
			HF::Report->report (2, 2, "FAILURE bogus hosts entry $i, skipping", 'fail_connection');
		}

		if (($iptr->{'direction'} eq '') || ($iptr->{'direction'} eq 'push')) {
			if ($HF::Globals::swappushpull == 1) {
				$optr->{'direction'} = 'pull';
			} else {
				$optr->{'direction'} = 'push';
			}
		} elsif ($iptr->{'direction'} eq 'pull') {
			if ($HF::Globals::swappushpull == 1) {
				$optr->{'direction'} = 'push';
			} else {
				$optr->{'direction'} = 'pull';
			}
		} else {
			HF::Report->report (2, 2, "FAILURE bogus direction entry $i, skipping", 'fail_connection');
		}
	}

	$HF::Globals::hosts_parsed = 1;

	return (1);
}

sub display {

	my $dumper;
	my $all;

	$all = { 'hosts_file' => $HF::Globals::hosts_file,
'hosts_parsed' => $HF::Globals::hosts_parsed,
'hosts' => $HF::Globals::hosts };
	$dumper = Data::Dumper->new ([ $all ]);
	$dumper->Terse (1);

	return ($dumper->Dump);
}

1;
