#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

package HF::Layers;
use strict;

use Data::Dumper;
use Digest::MD5;
use File::Path 'make_path';

#-----

sub new {

	my $proto = shift;
	my $config = shift;

	my $class = ref ($proto) || $proto;

	my $self = {

		'md5handle'	=> Digest::MD5->new,
		'firstnextrun'	=> 1
	};

	bless ($self, $class);

	return ($self);
}

sub pointer_to_tmp_layers {

	my $pointer = shift;

	$HF::Globals::layers_parsed = 0;
	$HF::Globals::layers_file = undef;

	$HF::Tmp::layers = $pointer;
}

sub eval_file_to_tmp {

# XXX       do EXPR
# XXX          Uses the value of EXPR as a filename and executes the contents of the file as a Perl script.

	my $filename = shift;

	my $filecontents;
	my $evalcode;

	$HF::Globals::layers_parsed = 0;
	$HF::Globals::layers_file = $filename;

	if ($filename eq '-') {
		$filename = '/dev/stdin';
	}

	if (!(open (LAYERSFILE, "<", $filename))) {
		HF::Report->report (3, 1, "FATAL can't open layers file '$filename', error '$!'");
	}
	{
		local ($/);
		$filecontents = <LAYERSFILE>;
	}
	close (LAYERSFILE);
	$evalcode = '$HF::Tmp::layers =
' . $filecontents . ';

1;';

	eval $evalcode;

	if ($@ eq '') {
		return (1);
	} else {
		HF::Report->report (3, 1, "FATAL eval error of layers file '$filename', message is '$@'")
	}
}

sub parse_tmp_to_globals {

	my $i;
	my $cleansource;
	my $cleantarget;

	$cleansource = [ ];
	$cleantarget = [ ];

	if ($#{$HF::Tmp::layers->{'source_layers'}} == -1) {
		HF::Report->report (2, 1, 'FATAL no source_layers, terminating parse', 'fail_layers');
		return (0);
	}

	if ($#{$HF::Tmp::layers->{'target_layers'}} == -1) {
		HF::Report->report (2, 1, 'FATAL no target_layers, terminating parse', 'fail_layers');
		return (0);
	}

	for ($i = 0;  $i <= $#{$HF::Tmp::layers->{'source_layers'}};  $i++) {
		if ($HF::Tmp::layers->{'source_layers'}[$i]{'mount'}[2] ne 'DELETE_THIS_LAYER') {
			push (@{$cleansource}, $HF::Tmp::layers->{'source_layers'}[$i]);
		}
	}

	for ($i = 0;  $i <= $#{$HF::Tmp::layers->{'target_layers'}};  $i++) {
		if ($HF::Tmp::layers->{'target_layers'}[$i]{'mount'}[2] ne 'DELETE_THIS_LAYER') {
			push (@{$cleantarget}, $HF::Tmp::layers->{'target_layers'}[$i]);
		}
	}

	if ($HF::Globals::swaplayers == 1) {
		$HF::Globals::target_layers = $cleansource;
		$HF::Globals::source_layers = $cleantarget;
	} else {
		$HF::Globals::source_layers = $cleansource;
		$HF::Globals::target_layers = $cleantarget;
	}

	$HF::Globals::layers_parsed = 1;
}

sub display {

	#  doesn't use $self, can be called as OO or not

	my $self = shift;

	my $dumper;
	my $all;

	$all = { 'layers_file' => $HF::Globals::layers_file,
'layers_parsed' => $HF::Globals::layers_parsed,
'layers_source_sane' => $HF::Globals::layers_source_sane,
'layers_target_sane' => $HF::Globals::layers_target_sane,
'source_layers' => $HF::Globals::source_layers,
'target_layers' => $HF::Globals::target_layers };

	$dumper = Data::Dumper->new ([ $all ]);
	$dumper->Terse (1);

	return ($dumper->Dump);
}

sub source_sanitycheck {

	my $self = shift;
	my $i;
	my $j;
	my $mountdir;
	my $mounttype;
	my $treatment;
	my $filter;
	my $idmap;

	if ($HF::Globals::layers_parsed != 1) {
		HF::Report->report (4, 1, 'FATAL layers not parsed yet');
	}

	if ($#{$HF::Globals::source_layers} == -1) {
		HF::Report->report (2, 1, "FATAL source walk has zero layers", 'fail_layers');
		return (0);
	}

	for ($i = 0;  $i <= $#{$HF::Globals::source_layers};  $i++) {

		$mountdir = $HF::Globals::source_layers->[$i]{'mount'}[1];
		$mounttype = $HF::Globals::source_layers->[$i]{'mount'}[2];

		if (!(($mounttype eq 'all') || ($mounttype eq 'nonfiles') || ($mounttype eq 'nondirs') || ($mounttype eq 'onlydirs') ||
			($mounttype eq 'topdir') || ($mounttype eq 'sl') || ($mounttype eq 'glob') || ($mounttype eq 'null'))) {

			HF::Report->report (2, 1, "FATAL source walk layer $i unknown type '" . $mounttype . "'", 'fail_layers');
			return (0);
		}

		if (!(($mounttype eq 'glob') || ($mounttype eq 'sl') || ($mounttype eq 'null'))) {
			if (!(-e $mountdir)) {
				HF::Report->report (2, 1, "FATAL source walk layer $i dir '" . $mountdir . "' does not exist", 'fail_layers');
				return (0);
			} elsif ((!(-d $mountdir)) || (-l $mountdir)) {
				HF::Report->report (2, 1, "FATAL source walk layer $i dir '" . $mountdir . "' is not dir", 'fail_layers');
				return (0);
			}
		}

		if (($mounttype eq 'glob') && ($mountdir eq '')) {
			HF::Report->report (2, 1, "FATAL source walk layer $i glob is zero length", 'fail_layers');
			return (0);
		}

		if (($mounttype eq 'sl') && (!(-f $mountdir))) {
			HF::Report->report (2, 1, "FATAL source walk layer $i sl '" . $mountdir . "' is not file", 'fail_layers');
			return (0);
		}

		if (exists ($HF::Globals::source_layers->[$i]{'skips'})) {
			if ($#{$HF::Globals::source_layers->[$i]{'skips'}} == -1) {
				HF::Report->report (2, 1, "FATAL source walk layer $i has no skips list entries", 'fail_layers');
				return (0);
			}
			for ($j = 0;  $j <= $#{$HF::Globals::source_layers->[$i]{'skips'}};  $j++) {
				if ($HF::Globals::source_layers->[$i]{'skips'}[$j] eq '') {
					HF::Report->report (2, 1, "FATAL source walk layer $i skips $j is zero length", 'fail_layers');
					return (0);
				}
				eval { qr/$HF::Globals::source_layers->[$i]{'skips'}[$j]/ };
				if ($@) {
					HF::Report->report (2, 1, "FATAL source walk layer $i skips $j has bad syntax", 'fail_layers');
					return (0);
				}
			}
		}

		if (exists ($HF::Globals::source_layers->[$i]{'filters'})) {
			if ($#{$HF::Globals::source_layers->[$i]{'filters'}} == -1) {
				HF::Report->report (2, 1, "FATAL source walk layer $i has no filters list entries", 'fail_layers');
				return (0);
			}
			for ($j = 0;  $j <= $#{$HF::Globals::source_layers->[$i]{'filters'}};  $j++) {
				$filter = $HF::Globals::source_layers->[$i]{'filters'}[$j][0];
				if (!(($filter eq 'owner') || ($filter eq 'group') || ($filter eq 'perms') || ($filter eq 'mtime'))) {
					HF::Report->report (2, 1, "FATAL source walk layer $i filter $j has invalid filter '$filter'", 'fail_layers');
					return (0);
				}
				$filter = $HF::Globals::source_layers->[$i]{'filters'}[$j][1];
				if (!(($filter =~ /^[0-9]+$/) && (($filter + 0) == $filter))) {	# possibly octal string, then numeric
					HF::Report->report (2, 1, "FATAL source walk layer $i filter $j has invalid number '$HF::Globals::source_layers->[$i]{'filters'}[$j][1]'", 'fail_layers');
					return (0);
				}
			}
		}

		if (exists ($HF::Globals::source_layers->[$i]{'idmap'})) {
			if (!(exists ($HF::Globals::source_layers->[$i]{'idmap'}{'owner'}))) {
				HF::Report->report (2, 1, "FATAL source walk layer $i idmap has no owner entry", 'fail_layers');
				return (0);
			}
			if (!(exists ($HF::Globals::source_layers->[$i]{'idmap'}{'group'}))) {
				HF::Report->report (2, 1, "FATAL source walk layer $i idmap has no group entry", 'fail_layers');
				return (0);
			}
			foreach $j (sort (keys (%{$HF::Globals::source_layers->[$i]{'idmap'}{'owner'}}))) {
				if (!(($j eq 'default') || ($j =~ /^[0-9]+$/))) {
					HF::Report->report (2, 1, "FATAL source walk layer $i idmap owner key '$j' is not a number or 'default'", 'fail_layers');
					return (0);
				}
				$idmap = $HF::Globals::source_layers->[$i]{'idmap'}{'owner'}{$j};
				if (!($idmap =~ /^[0-9]+$/)) {
					HF::Report->report (2, 1, "FATAL source walk layer $i idmap owner key '$j' value '$idmap' is not a number", 'fail_layers');
					return (0);
				}
			}
			foreach $j (sort (keys (%{$HF::Globals::source_layers->[$i]{'idmap'}{'group'}}))) {
				if (!(($j eq 'default') || ($j =~ /^[0-9]+$/))) {
					HF::Report->report (2, 1, "FATAL source walk layer $i idmap group key '$j' is not a number or 'default'", 'fail_layers');
					return (0);
				}
				$idmap = $HF::Globals::source_layers->[$i]{'idmap'}{'group'}{$j};
				if (!($idmap =~ /^[0-9]+$/)) {
					HF::Report->report (2, 1, "FATAL source walk layer $i idmap group key '$j' value '$idmap' is not a number", 'fail_layers');
					return (0);
				}
			}
		}

		if ($#{$HF::Globals::source_layers->[$i]{'treatments'}} == -1) {
			HF::Report->report (2, 1, "FATAL source walk layer $i has no treatments list entries", 'fail_layers');
			return (0);
		}

		for ($j = 0;  $j <= $#{$HF::Globals::source_layers->[$i]{'treatments'}};  $j++) {
			$treatment = $HF::Globals::source_layers->[$i]{'treatments'}[$j][0];
			if (!(($treatment == 0) || ($treatment == 101) || ($treatment == 102) || ($treatment == 103) || ($treatment == 113) ||
				($treatment == 104) || ($treatment == 114) || ($treatment == 105) || ($treatment == 106) || ($treatment == 107) ||
				($treatment == 207) || ($treatment == 108) || ($treatment == 109))) {

				HF::Report->report (2, 1, "FATAL source walk layer $i treatments $j has invalid treatment '$treatment'", 'fail_layers');
				return (0);
			}

			if ($treatment != 0) {
###				if ($HF::Globals::source_layers->[$i]{'treatments'}[$j][1][0] eq '') {
###					HF::Report->report (2, 1, "FATAL source walk layer $i treatments $j has zero-length first entry of regexp array", 'fail_layers');
###					return (0);
###				}
				if ($treatment == 207) {
					if (!(	(($HF::Globals::source_layers->[$i]{'treatments'}[$j][2][0] =~ /^[0-9]+$/) &&
						(($HF::Globals::source_layers->[$i]{'treatments'}[$j][2][0] + 0) == $HF::Globals::source_layers->[$i]{'treatments'}[$j][2][0])) ||
						(($HF::Globals::source_layers->[$i]{'treatments'}[$j][2][1] =~ /^[0-9]+$/) &&
						(($HF::Globals::source_layers->[$i]{'treatments'}[$j][2][1] + 0) == $HF::Globals::source_layers->[$i]{'treatments'}[$j][2][2])) ||
						(($HF::Globals::source_layers->[$i]{'treatments'}[$j][2][2] =~ /^[0-9]+$/) &&
						(($HF::Globals::source_layers->[$i]{'treatments'}[$j][2][2] + 0) == $HF::Globals::source_layers->[$i]{'treatments'}[$j][2][2])))) {
						HF::Report->report (2, 1, "FATAL source walk layer $i treatments $j treatment 207 has invalid number in user/group/perms array", 'fail_layers');
						return (0);
					}
				}
			}
		}
	}

	$HF::Globals::layers_source_sane = 1;
	return (1);
}

sub source_start {

	my $self = shift;

	my $from;

	if ($HF::Globals::layers_source_sane != 1) {
		HF::Report->report (4, 1, 'FATAL source layers not known sane');
	}

	$self->start_layer ($HF::Globals::source_layers, 1);

	return (1);
}

sub source_next {

	my $self = shift;

	my $outhash;
	my $type;

	$outhash = $self->next ($HF::Globals::source_layers, 'source');
	$type = $outhash->{'type'};

	if ($type == 0100000) {		# file
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_f'}++;
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'}++;
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_f_bytes'} += $outhash->{'size'};
	} elsif ($type == 0040000) {	# dir
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_d'}++;
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'}++;
	} elsif ($type == 0120000) {	# symlink
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_l'}++;
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'}++;
	} elsif ($type == 0020000) {	# char dev
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_c'}++;
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'}++;
	} elsif ($type == 0060000) {	# block dev
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_b'}++;
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'}++;
	} elsif ($type == 0010000) {	# fifo
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_p'}++;
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'}++;
	} elsif ($type == 0140000) {	# socket
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_s'}++;
		$HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'}++;
	}
	# no else die, returning undefs indicates walk is done

	if (($HF::Globals::opt_x == 6) && (($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'} +
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'} + $HF::Globals::target_report->[$HF::Globals::hostrow]{'compare_delete'}) > 100)) {

		&HF::Harmonize::to_other_end ('abort');
		if ($HF::Globals::islocal == 1) {
			$HF::Globals::local_report->[$HF::Globals::hostrow]{'abandoned_remote'} = 1;
			HF::Report->report (1, 3, "PROGRESS aborting after 100 filetree entries seen");
			goto ABANDONED_REMOTE;
		} else {
			HF::Report->report (3, 3, "PROGRESS aborting after 100 filetree entries seen");
		}
	}

	return ($outhash);
}

sub target_sanitycheck {

	my $self = shift;
	my $i;
	my $j;
	my $mountdir;
	my $mounttype;
	my $treatment;
	my $filter;
	my $idmap;

	if ($HF::Globals::layers_parsed != 1) {
		HF::Report->report (4, 1, 'FATAL layers not parsed yet');
	}
	if ($#{$HF::Globals::target_layers} == -1) {
		HF::Report->report (2, 1, "FATAL target walk has zero layers", 'fail_layers');
		return (0);
	}

	for ($i = 0;  $i <= $#{$HF::Globals::target_layers};  $i++) {

		$mountdir = $HF::Globals::target_layers->[$i]{'mount'}[1];
		$mounttype = $HF::Globals::target_layers->[$i]{'mount'}[2];

		if (!(($mounttype eq 'all') || ($mounttype eq 'nonfiles') || ($mounttype eq 'nondirs') || ($mounttype eq 'onlydirs') ||
			($mounttype eq 'topdir') || ($mounttype eq 'sl') || ($mounttype eq 'glob') || ($mounttype eq 'null'))) {

			HF::Report->report (2, 1, "FATAL target walk layer $i unknown type '" . $mounttype . "'", 'fail_layers');
			return (0);
		}

		if (($mounttype eq 'null') && ($HF::Globals::dowrite == 1)) {
			HF::Report->report (2, 1, "FATAL target walk layer $i cannot be type null when writing", 'fail_layers');
			return (0);
		}

		if (!(($mounttype eq 'glob') || ($mounttype eq 'sl') || ($mounttype eq 'null'))) {
			# any target layer dir allowed to not exist; but if name does exist, it must be dir and not symlink
			if ((-e $mountdir) && ((!(-d $mountdir)) || (-l $mountdir))) {
				HF::Report->report (2, 1, "FATAL target walk layer $i dir '" . $mountdir . "' is not dir", 'fail_layers');
				return (0);
			}
		}

		if (($mounttype eq 'glob') && ($mountdir eq '')) {
			HF::Report->report (2, 1, "FATAL target walk layer $i glob is zero length", 'fail_layers');
			return (0);
		}

		if (($mounttype eq 'sl') && (!(-f $mountdir))) {
			HF::Report->report (2, 1, "FATAL target walk layer $i sl '" . $mountdir . "' is not file", 'fail_layers');
			return (0);
		}

		if (exists ($HF::Globals::target_layers->[$i]{'skips'})) {
			if ($#{$HF::Globals::target_layers->[$i]{'skips'}} == -1) {
				HF::Report->report (2, 1, "FATAL target walk layer $i has no skips list entries", 'fail_layers');
				return (0);
			}
			for ($j = 0;  $j <= $#{$HF::Globals::target_layers->[$i]{'skips'}};  $j++) {
				if ($HF::Globals::target_layers->[$i]{'skips'}[$j] eq '') {
					HF::Report->report (2, 1, "FATAL target walk layer $i skips $j is zero length", 'fail_layers');
					return (0);
				}
				eval { qr/$HF::Globals::target_layers->[$i]{'skips'}[$j]/ };
				if ($@) {
					HF::Report->report (2, 1, "FATAL target walk layer $i skips $j has bad syntax", 'fail_layers');
					return (0);
				}
			}
		}

		if (exists ($HF::Globals::target_layers->[$i]{'filters'})) {
			if ($#{$HF::Globals::target_layers->[$i]{'filters'}} == -1) {
				HF::Report->report (2, 1, "FATAL target walk layer $i has no filters list entries", 'fail_layers');
				return (0);
			}
			for ($j = 0;  $j <= $#{$HF::Globals::target_layers->[$i]{'filters'}};  $j++) {
				$filter = $HF::Globals::target_layers->[$i]{'filters'}[$j][0];
				if (!(($filter eq 'owner') || ($filter eq 'group') || ($filter eq 'perms') || ($filter eq 'mtime'))) {
					HF::Report->report (2, 1, "FATAL target walk layer $i filter $j has invalid filter '$filter'", 'fail_layers');
					return (0);
				}
				$filter = $HF::Globals::target_layers->[$i]{'filters'}[$j][1];
				if (!(($filter =~ /^[0-9]+$/) && (($filter + 0) == $filter))) {	# possibly octal string, then numeric
					HF::Report->report (2, 1, "FATAL target walk layer $i filter $j has invalid number '$HF::Globals::target_layers->[$i]{'filters'}[$j][1]'", 'fail_layers');
					return (0);
				}
			}
		}

		if (exists ($HF::Globals::target_layers->[$i]{'idmap'})) {
			if (!(exists ($HF::Globals::target_layers->[$i]{'idmap'}{'owner'}))) {
				HF::Report->report (2, 1, "FATAL source walk layer $i idmap has no owner entry", 'fail_layers');
				return (0);
			}
			if (!(exists ($HF::Globals::target_layers->[$i]{'idmap'}{'group'}))) {
				HF::Report->report (2, 1, "FATAL source walk layer $i idmap has no group entry", 'fail_layers');
				return (0);
			}
			foreach $j (sort (keys (%{$HF::Globals::target_layers->[$i]{'idmap'}{'owner'}}))) {
				if (!(($j eq 'default') || ($j =~ /^[0-9]+$/))) {
					HF::Report->report (2, 1, "FATAL source walk layer $i idmap owner key '$j' is not a number or 'default'", 'fail_layers');
					return (0);
				}
				$idmap = $HF::Globals::target_layers->[$i]{'idmap'}{'owner'}{$j};
				if (!($idmap =~ /^[0-9]+$/)) {
					HF::Report->report (2, 1, "FATAL source walk layer $i idmap owner key '$j' value '$idmap' is not a number", 'fail_layers');
					return (0);
				}
			}
			foreach $j (sort (keys (%{$HF::Globals::target_layers->[$i]{'idmap'}{'group'}}))) {
				if (!(($j eq 'default') || ($j =~ /^[0-9]+$/))) {
					HF::Report->report (2, 1, "FATAL source walk layer $i idmap group key '$j' is not a number or 'default'", 'fail_layers');
					return (0);
				}
				$idmap = $HF::Globals::target_layers->[$i]{'idmap'}{'group'}{$j};
				if (!($idmap =~ /^[0-9]+$/)) {
					HF::Report->report (2, 1, "FATAL source walk layer $i idmap group key '$j' value '$idmap' is not a number", 'fail_layers');
					return (0);
				}
			}
		}

		if ($#{$HF::Globals::target_layers->[$i]{'treatments'}} == -1) {
			HF::Report->report (2, 1, "FATAL target walk layer $i has no treatments list entries", 'fail_layers');
			return (0);
		}

		for ($j = 0;  $j <= $#{$HF::Globals::target_layers->[$i]{'treatments'}};  $j++) {
			$treatment = $HF::Globals::target_layers->[$i]{'treatments'}[$j][0];
			if (!(($treatment == 0) || ($treatment == 101) || ($treatment == 102) || ($treatment == 103) || ($treatment == 113) ||
				($treatment == 104) || ($treatment == 114) || ($treatment == 105) || ($treatment == 106) || ($treatment == 107) ||
				($treatment == 207) || ($treatment == 108) || ($treatment == 109))) {

				HF::Report->report (2, 1, "FATAL target walk layer $i treatments $j has invalid treatment '$treatment'", 'fail_layers');
				return (0);
			}

			if ($treatment != 0) {
###				if ($HF::Globals::target_layers->[$i]{'treatments'}[$j][1][0] eq '') {
###					HF::Report->report (2, 1, "FATAL target walk layer $i treatments $j has zero-length first entry of regexp array", 'fail_layers');
###					return (0);
###				}
				if ($treatment == 207) {

					if (!(	(($HF::Globals::target_layers->[$i]{'treatments'}[$j][2][0] =~ /^[0-9]+$/) &&
						(($HF::Globals::target_layers->[$i]{'treatments'}[$j][2][0] + 0) == $HF::Globals::target_layers->[$i]{'treatments'}[$j][2][0])) ||
						(($HF::Globals::target_layers->[$i]{'treatments'}[$j][2][1] =~ /^[0-9]+$/) &&
						(($HF::Globals::target_layers->[$i]{'treatments'}[$j][2][1] + 0) == $HF::Globals::target_layers->[$i]{'treatments'}[$j][2][2])) ||
						(($HF::Globals::target_layers->[$i]{'treatments'}[$j][2][2] =~ /^[0-9]+$/) &&
						(($HF::Globals::target_layers->[$i]{'treatments'}[$j][2][2] + 0) == $HF::Globals::target_layers->[$i]{'treatments'}[$j][2][2])))) {
						HF::Report->report (2, 1, "FATAL target walk layer $i treatments $j treatment 207 has invalid number in user/group/perms array", 'fail_layers');
						return (0);
					}
				}
			}
		}
	}

	$HF::Globals::layers_target_sane = 1;

	return (1);
}

sub target_start {

	my $self = shift;
	my $i;
	my $backquotecmd;
	my $backquoteres;
	my $make_path_error;
	my $mountdir;
	my $mounttype;
	my $dirbad;

	if ($HF::Globals::layers_target_sane != 1) {
		HF::Report->report (4, 1, 'FATAL target layers not known sane');
	}

	if ($HF::Globals::examine ne '') {
		if ((-e $HF::Globals::examine) && ((!(-d $HF::Globals::examine)) || (-l $HF::Globals::examine))) {
			HF::Report->report (2, 1, "FATAL target examine walk top dir '" . HF::Encode->encode ($HF::Globals::examine) .
				"' is not dir", 'fail_layers');
			return (0);
		}
		if (!(-e $HF::Globals::examine)) {
			if ($HF::Globals::dowrite == 1) {
				HF::Report->report (1, 3, "DIFFERENCE target examine walk top dir does not exist '" .
					HF::Encode->encode ($HF::Globals::examine) . "', making it");
				make_path ($HF::Globals::examine, { mode => 0755, error => \$make_path_error });	# ogmt not relevant
				if (@$make_path_error) {
					# Only show first error that make_path returns
					HF::Report->report (2, 1, "FATAL mkdir(s) of '" . HF::Encode->encode ($HF::Globals::examine) .
						"' failed with error '" . join (' ', %{${$make_path_error}[0]}) . "'", 'fail_layers');
					return (0);
				}
			} else {
				HF::Report->report (1, 3, "DIFFERENCE target examine walk top dir '" . HF::Encode->encode ($HF::Globals::examine) .
					"' does not exist, but -w will make it");
			}
		}
	}

	$dirbad = 0;
	for ($i = 0;  $i <= $#{$HF::Globals::target_layers};  $i++) {

		$mountdir = $HF::Globals::target_layers->[$i]{'mount'}[1];

		if ($HF::Globals::free_bytes > 0) {
			if (-e $mountdir) {
				# XXX improve dir quoting, more careful parsing of results, make sure is number
				$backquotecmd = q{df --block-size=1 '} . $mountdir . q{' | awk 'NR == 2 {print $4}'};
				$backquoteres = `$backquotecmd`;
				if (("$?" >> 8) != 0) {
					HF::Report->report (2, 2, "FAILURE can't df '" . HF::Encode->encode ($mountdir) . "', error '$!'", 'fail_layers');
				}
				chop ($backquoteres);
				if ($backquoteres <= $HF::Globals::free_bytes) {
					HF::Report->report (2, 1, "FATAL target walk layer $i has insufficient free bytes, want $HF::Globals::free_bytes have " . &HF::Misc::commify_number ($backquoteres), 'fail_layers');
					$dirbad = 1;
				}
			} else {
				HF::Report->report (2, 2, "FAILURE dir doesn't exist, can't check free bytes of '" . HF::Encode->encode ($mountdir) . "'", 'fail_layers');
			}
		}
		if ($HF::Globals::free_inodes > 0) {
			if (-e $mountdir) {
				# XXX improve dir quoting, more careful parsing of results, make sure is number
				$backquotecmd = q{df -i '} . $mountdir . q{' | awk 'NR == 2 {print $4}'};
				$backquoteres = `$backquotecmd`;
				if (("$?" >> 8) != 0) {
					HF::Report->report (2, 2, "FAILURE can't df -i '" . HF::Encode->encode ($mountdir) . "', error '$!'", 'fail_layers');
				}
				chop ($backquoteres);
				if ($backquoteres <= $HF::Globals::free_inodes) {
					HF::Report->report (2, 1, "FATAL target walk layer $i has insufficient free inodes, want $HF::Globals::free_inodes have " . &HF::Misc::commify_number ($backquoteres), 'fail_layers');
					$dirbad = 1;
				}
			} else {
				HF::Report->report (2, 2, "FAILURE dir doesn't exist, can't check free inodes of '" . HF::Encode->encode ($mountdir) . "'", 'fail_layers');
			}
		}
	}
	if ($dirbad == 1) {
		return (0);
	}

	$dirbad = 0;
	for ($i = 0;  $i <= $#{$HF::Globals::target_layers};  $i++) {

		$mountdir = $HF::Globals::target_layers->[$i]{'mount'}[1];
		$mounttype = $HF::Globals::target_layers->[$i]{'mount'}[2];

		if (!(($mounttype eq 'sl') || ($mounttype eq 'glob') || ($mounttype eq 'null'))) {
			if (!(-e $mountdir)) {
				if ($HF::Globals::dowrite == 1) {
				        $HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'}++;
					HF::Report->report (1, 3, "DIFFERENCE target walk dir $i does not exist '" . HF::Encode->encode ($mountdir) . "', making it");
					if (defined ($HF::Globals::outputplbig)) {
						push (@{$HF::Globals::target_differences->[$HF::Globals::hostrow]},
							[ "", "create", undef,
								{
								'cmpname' => '',
								'dev' => undef,
								'encname' => '.',
								'encpath' => HF::Encode->encode ($mountdir),
								'enctarget' => '',
								'group' => $),
								'inode' => 999999,
								'links' => 2,
								'mode' => 0040000 | 0755,
								'mtime' => time,
								'name' => '.',
								'owner' => $>,
								'path' => $mountdir,
								'perms' => 0755,
								'rdev' => 0,
								'size' => 0,
								'target' => '',
								'treatment' => 0,
								'type' => 0040000
								}, undef ]);
					}

					make_path ($mountdir, { mode => 0755, error => \$make_path_error });	# ogmt will be corrected in normal walk
					if (@$make_path_error) {
						# Only show first error that make_path returns
						HF::Report->report (2, 1, "FATAL mkdir(s) of " . HF::Encode->encode ($mountdir) .
							" failed with error '" . join (' ', %{${$make_path_error}[0]}) . "'", 'fail_layers');
						return (0);
					}
				} else {
					HF::Report->report (2, 1, "FATAL target walk top dir '" . HF::Encode->encode ($mountdir) . "' does not exist, but -w will make it", 'fail_layers');
					$dirbad = 1;
				}
			} elsif ((!(-d $mountdir)) || (-l $mountdir)) {
				HF::Report->report (2, 1, "FATAL target walk top dir '" . HF::Encode->encode ($mountdir) . "' is not dir", 'fail_layers');
				$dirbad = 1;
			}
		}
	}
	if ($dirbad == 1) {
		return (0);
	}

	$self->start_layer ($HF::Globals::target_layers, 0);

	return (1);
}

sub target_next {

	my $self = shift;
	my $outhash;
	my $type;

	$outhash = $self->next ($HF::Globals::target_layers, 'target');
	$type = $outhash->{'type'};

	if ($type == 0100000) {		# file
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_f'}++;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'}++;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_f_bytes'} += $outhash->{'size'};
	} elsif ($type == 0040000) {	# dir
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_d'}++;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'}++;
	} elsif ($type == 0120000) {	# symlink
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_l'}++;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'}++;
	} elsif ($type == 0020000) {	# char dev
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_c'}++;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'}++;
	} elsif ($type == 0060000) {	# block dev
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_b'}++;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'}++;
	} elsif ($type == 0010000) {	# fifo
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_p'}++;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'}++;
	} elsif ($type == 0140000) {	# socket
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_s'}++;
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'}++;
	}
	# no else die, returning undefs indicates walk is done

	if (($HF::Globals::opt_x == 6) && (($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'} +
		$HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'} + $HF::Globals::target_report->[$HF::Globals::hostrow]{'compare_delete'}) > 100)) {

		&HF::Harmonize::to_other_end ('abort');
		if ($HF::Globals::islocal == 1) {
			$HF::Globals::local_report->[$HF::Globals::hostrow]{'abandoned_remote'} = 1;
			HF::Report->report (1, 3, "PROGRESS aborting after 100 filetree entries seen");
			goto ABANDONED_REMOTE;
		} else {
			HF::Report->report (3, 3, "PROGRESS aborting after 100 filetree entries seen");
		}
	}

	return ($outhash);
}

sub print_layers {

	my $self = shift;
	my $layers = shift;

	my $i;
	my $j;
	my $k;

	for ($i = 0;  $i <= $#{$layers};  $i++) {

		print "mount point '" . $layers->[$i]{'mount'}[0] . "'\n" .
			"real source path '" . $layers->[$i]{'mount'}[1] . "'\n" .
			"type '" . $layers->[$i]{'mount'}[2] . "'\n";

		# doesn't have to be a skips section
		if (defined ($layers->[$i]{'skips'})) {
			for ($j = 0;  $j <= $#{$layers->[$i]{'skips'}};  $j++) {
				print "filetree skip regex '" . $layers->[$i]{'skips'}[$j] . "'\n";
			}
		}

		# doesn't have to be a filters section
		if (defined ($layers->[$i]{'filters'})) {
			for ($j = 0;  $j <= $#{$layers->[$i]{'filters'}};  $j++) {
				print "filter sets '" . $layers->[$i]{'filters'}[$j][0] . "' to " . $layers->[$i]{'filters'}[$j][1] . "\n";
			}
		}

		# doesn't have to be an idmap section
		if (defined ($layers->[$i]{'idmap'})) {
			foreach $j (sort (keys (%{$layers->[$i]{'idmap'}}))) {
				foreach $k (sort (keys (%{$layers->[$i]{'idmap'}{$j}}))) {
					print "idmap key '$j' subkey '$k' value '" . $layers->[$i]{'idmap'}{$j}{$k} . "'\n";
				}
			}
		}

		for ($j = 0;  $j <= $#{$layers->[$i]{'treatments'}};  $j++) {

			print "treatment '" . $layers->[$i]{'treatments'}[$j][0] . "'\n";

			# doesn't have to be treatments regexps
			if (defined ($layers->[$i]{'treatments'}[$j][1])) {
				for ($k = 0;  $k <= $#{$layers->[$i]{'treatments'}[$j][1]};  $k++) {
					print "stats replace regex '" . $layers->[$i]{'treatments'}[$j][1][$k] . "'\n";
				}
			}
			if ($layers->[$i]{'treatments'}[$j][0] == 207) {
				print "owner '" . $layers->[$i]{'treatments'}[$j][2][0] . "'\n";
				print "group '" . $layers->[$i]{'treatments'}[$j][2][1] . "'\n";
				print "perms '" . $layers->[$i]{'treatments'}[$j][2][2] . "'\n";
			}
		}
	}
}

sub start_layer {

	my $self = shift;
	my $layers = shift;
	my $issource = shift;

	my $i;
	my $j;
	my $skips;
	my $cmpname;

	for ($i = 0;  $i <= $#{$layers};  $i++) {

		if (($layers->[$i]{'mount'}[2] eq 'all') || ($layers->[$i]{'mount'}[2] eq 'nonfiles') || ($layers->[$i]{'mount'}[2] eq 'nondirs') ||
			($layers->[$i]{'mount'}[2] eq 'onlydirs') || ($layers->[$i]{'mount'}[2] eq 'topdir') || ($layers->[$i]{'mount'}[2] eq 'sl') ||
			($layers->[$i]{'mount'}[2] eq 'glob') || ($layers->[$i]{'mount'}[2] eq 'null')) {

			if ((defined ($layers->[$i]{'skips'})) && ($#{$layers->[$i]{'skips'}} > -1)) {
				$skips = '(' . join (')|(', @{$layers->[$i]{'skips'}}) . ')';
			} else {
				$skips = undef;
			}
			if (!($self->path_is_clean ($layers->[$i]{'mount'}[1]))) {
				HF::Report->report (4, 1, "FATAL path is not clean '" . $layers->[$i]{'mount'}[1] . "'");
			}
			if ($layers->[$i]{'mount'}[2] eq 'sl') {
				$layers->[$i]{'mount_gen'}[0] = HF::STLstream->new ($layers->[$i]{'mount'}[1], $skips, $HF::Globals::dochecksums);
				$layers->[$i]{'mount'}[1] = $layers->[$i]{'mount_gen'}[0]->getroot;
			} elsif ($layers->[$i]{'mount'}[2] eq 'glob') {
				$layers->[$i]{'mount_gen'}[0] = HF::GLOBstream->new ($layers->[$i]{'mount'}[1], $skips, $HF::Globals::dochecksums);
				$layers->[$i]{'mount'}[1] = $layers->[$i]{'mount_gen'}[0]->getroot;
			} elsif ($layers->[$i]{'mount'}[2] eq 'null') {
				$layers->[$i]{'mount_gen'}[0] = HF::NULLstream->new;
			} else {
				$layers->[$i]{'mount_gen'}[0] = HF::FTWstream->new ($layers->[$i]{'mount'}[1], $skips, $HF::Globals::dochecksums);
			}

			# doesn't have to be a skips section
			if (defined ($layers->[$i]{'skips'})) {
				$layers->[$i]{'skips_gen'} = '(' . join (')|(', @{$layers->[$i]{'skips'}}) . ')';
			}
			if ($layers->[$i]{'mount'}[0] ne '') {
				if (!($self->path_is_clean ($layers->[$i]{'mount'}[0]))) {
					HF::Report->report (4, 1, "FATAL path is not clean '" . $layers->[$i]{'mount'}[0] . "'");
				}
				$layers->[$i]{'mount_gen'}[2] = [ split ('/', $layers->[$i]{'mount'}[0]) ];
				$layers->[$i]{'mount_gen'}[3] = 0;
			} else {
				$layers->[$i]{'mount_gen'}[2] = [ ];
				$layers->[$i]{'mount_gen'}[3] = 0;
			}
		} else {
			HF::Report->report (4, 1, "FATAL walk type '" . $layers->[$i]{'mount'}[2] . "' unknown");
		}

		for ($j = 0;  $j <= $#{$layers->[$i]{'treatments'}};  $j++) {

			# doesn't have to be treatments regexps
			if (defined ($layers->[$i]{'treatments'}[$j][1])) {
				$layers->[$i]{'treatments_gen'}[$j] = '(' . join (')|(', @{$layers->[$i]{'treatments'}[$j][1]}) . ')';
			}
		}

# XXX OLD fake dirs on target side causes problems for file etc/foo when something mounted at etc/ssh
# Try putting fake dirs back on target side
#		if (($layers->[$i]{'mount_gen'}[3] < $#{$layers->[$i]{'mount_gen'}[2]}) && ($issource == 1)) {
		if ($layers->[$i]{'mount_gen'}[3] < $#{$layers->[$i]{'mount_gen'}[2]}) {
			# fake entries for mount point
			$layers->[$i]{'mount_gen'}[1]{'name'} = join ('/', @{$layers->[$i]{'mount_gen'}[2]}[0..$layers->[$i]{'mount_gen'}[3]]);
			$layers->[$i]{'mount_gen'}[3]++;
		       	if ($HF::Globals::verbosity >= 7) {
				HF::Report->report (1, 5, "ATTENTION 3 generating fake directory, issource " . $issource .
					" to glue in mount point '" . join ('/', @{$layers->[$i]{'mount_gen'}[2]}[0..$layers->[$i]{'mount_gen'}[3]]) . "'");
				HF::Report->report (1, 5, "ATTENTION 4 mount_gen-3 " . $layers->[$i]{'mount_gen'}[3] .
					" joined mount_gen-2 '" . join (' / ', @{$layers->[$i]{'mount_gen'}[2]}) . "'");
			} else {
				HF::Report->report (1, 5, "ATTENTION 5 generating fake directory for mount point '" .
					$layers->[$i]{'mount_gen'}[1]{'name'} . "'");
			}

			$layers->[$i]{'mount_gen'}[1]{'fake'} = 1;
			$layers->[$i]{'mount_gen'}[1]{'treatment'} = 0;
			$layers->[$i]{'mount_gen'}[1]{'type'} = 0040000;
			$layers->[$i]{'mount_gen'}[1]{'perms'} = 0755;
			$layers->[$i]{'mount_gen'}[1]{'mode'} = $layers->[$i]{'mount_gen'}[1]{'type'} | $layers->[$i]{'mount_gen'}[1]{'perms'};
			$layers->[$i]{'mount_gen'}[1]{'owner'} = 0;
			$layers->[$i]{'mount_gen'}[1]{'group'} = 0;
			$layers->[$i]{'mount_gen'}[1]{'dev'} = 999999;
			$layers->[$i]{'mount_gen'}[1]{'inode'} = 999999;
			$layers->[$i]{'mount_gen'}[1]{'links'} = 2;
			$layers->[$i]{'mount_gen'}[1]{'size'} = 4096;
			$layers->[$i]{'mount_gen'}[1]{'mtime'} = 0;
			$layers->[$i]{'mount_gen'}[1]{'blksize'} = 1024;
			$layers->[$i]{'mount_gen'}[1]{'blocks'} = 4;
			$layers->[$i]{'mount_gen'}[1]{'startdir'} = $layers->[$i]{'mount'}[0];
			$layers->[$i]{'mount_gen'}[1]{'path'} = '';
			$layers->[$i]{'mount_gen'}[1]{'encpath'} = '';
		} else {
			$layers->[$i]{'mount_gen'}[1] = $layers->[$i]{'mount_gen'}[0]->next;
			$layers->[$i]{'mount_gen'}[1]{'name'} = $layers->[$i]{'mount'}[0];
		}

		if ($layers->[$i]{'mount_gen'}[1]{'name'} eq '') {
			$layers->[$i]{'mount_gen'}[1]{'name'} = '.';
			$cmpname = '';
		} else {
			$cmpname = $layers->[$i]{'mount_gen'}[1]{'name'};
			$cmpname =~ s|/|\000|g;
		}
		$layers->[$i]{'mount_gen'}[1]{'cmpname'} = $cmpname;
		$layers->[$i]{'mount_gen'}[1]{'encname'} = HF::Encode->encode ($layers->[$i]{'mount_gen'}[1]{'name'});

		if (exists ($layers->[$i]{'filters'})) {
			for ($j = 0;  $j <= $#{$layers->[$i]{'filters'}};  $j++) {
				$layers->[$i]{'mount_gen'}[1]{$layers->[$i]{'filters'}[$j][0]} = $layers->[$i]{'filters'}[$j][1];
				if ($layers->[$i]{'filters'}[$j][0] eq 'perms') {
					$layers->[$i]{'mount_gen'}[1]{'mode'} = $layers->[$i]{'mount_gen'}[1]{'type'} | $layers->[$i]{'mount_gen'}[1]{'perms'};
				}
			}
		}

		if (exists ($layers->[$i]{'idmap'})) {
			if (exists ($layers->[$i]{'idmap'}{'owner'}{$layers->[$i]{'mount_gen'}[1]{'owner'}})) {
				$layers->[$i]{'mount_gen'}[1]{'owner'} = $layers->[$i]{'idmap'}{'owner'}{$layers->[$i]{'mount_gen'}[1]{'owner'}};
			} elsif (exists ($layers->[$i]{'idmap'}{'owner'}{'default'})) {
					$layers->[$i]{'mount_gen'}[1]{'owner'} = $layers->[$i]{'idmap'}{'owner'}{'default'};
			}
			if (exists ($layers->[$i]{'idmap'}{'group'}{$layers->[$i]{'mount_gen'}[1]{'group'}})) {
				$layers->[$i]{'mount_gen'}[1]{'group'} = $layers->[$i]{'idmap'}{'group'}{$layers->[$i]{'mount_gen'}[1]{'group'}};
			} elsif (exists ($layers->[$i]{'idmap'}{'group'}{'default'})) {
					$layers->[$i]{'mount_gen'}[1]{'group'} = $layers->[$i]{'idmap'}{'group'}{'default'};
			}
		}
	}
}

sub next {

	my $self = shift;
	my $layers = shift;
	my $end = shift;

	my $outhash = { };
	my $lowestname;
	my $i;
	my $j;
	my $k;
	my $thisname;
	my $type;
	my $treatment;
	my $oldtreatment;
	my $cmpname;
	my $fakeused;
	my $bypasstree;

	if ($self->{'firstnextrun'} == 0) {

		# Push this work-in-progress update up here from bottom of next() to
		# delay picking up the next subdir entry, which might be modified by
		# target_compare().  This avoids a stale cache problem.

		for ($i = 0;  $i <= $#{$layers};  $i++) {
			if ($layers->[$i]{'mount_gen'}[1]{'inplay'} == 1) {

# XXX OLD fake dirs on target side causes problems for file etc/foo when something mounted at etc/ssh
# Try putting fake dirs back on target side
#				if (($layers->[$i]{'mount_gen'}[3] < $#{$layers->[$i]{'mount_gen'}[2]}) && ($end eq "source")) {
				if ($layers->[$i]{'mount_gen'}[3] < $#{$layers->[$i]{'mount_gen'}[2]}) {
					# create fake dir entries down to this layer's mount point
					# so this layer is reachable properly in walk, with all intermediate dirs walked,
					# even if this layer's mount point is not shadowing dirs in a higher layer
					$layers->[$i]{'mount_gen'}[1]{'name'} = join ('/', @{$layers->[$i]{'mount_gen'}[2]}[0..$layers->[$i]{'mount_gen'}[3]]);
				       	if ($HF::Globals::verbosity >= 7) {
						HF::Report->report (1, 5, "ATTENTION 7 generating fake directory, end eq source " .
							($end eq "source") . " to glue in mount point '" . $layers->[$i]{'mount_gen'}[1]{'name'} . "'");
						HF::Report->report (1, 5, "ATTENTION 8   mount_gen-3 " . $layers->[$i]{'mount_gen'}[3] .
							" joined mount_gen-2 '" . join (' / ', @{$layers->[$i]{'mount_gen'}[2]}) . "'");
					} else {
						HF::Report->report (1, 5, "ATTENTION 9 generating fake directory to glue in mount point '" .
							$layers->[$i]{'mount_gen'}[1]{'name'} . "'");
					}
					$layers->[$i]{'mount_gen'}[3]++;

					$layers->[$i]{'mount_gen'}[1]{'fake'} = 1;
					$layers->[$i]{'mount_gen'}[1]{'treatment'} = 0;
					$layers->[$i]{'mount_gen'}[1]{'type'} = 0040000;
					$layers->[$i]{'mount_gen'}[1]{'perms'} = 0755;
					$layers->[$i]{'mount_gen'}[1]{'mode'} = $layers->[$i]{'mount_gen'}[1]{'type'} | $layers->[$i]{'mount_gen'}[1]{'perms'};
					$layers->[$i]{'mount_gen'}[1]{'owner'} = 0;
					$layers->[$i]{'mount_gen'}[1]{'group'} = 0;
					$layers->[$i]{'mount_gen'}[1]{'dev'} = 999999;
					$layers->[$i]{'mount_gen'}[1]{'inode'} = 999999;
					$layers->[$i]{'mount_gen'}[1]{'links'} = 2;
					$layers->[$i]{'mount_gen'}[1]{'size'} = 4096;
					$layers->[$i]{'mount_gen'}[1]{'mtime'} = 0;
					$layers->[$i]{'mount_gen'}[1]{'blksize'} = 1024;
					$layers->[$i]{'mount_gen'}[1]{'blocks'} = 4;
					$layers->[$i]{'mount_gen'}[1]{'startdir'} = '';
					$layers->[$i]{'mount_gen'}[1]{'path'} = '';
					$layers->[$i]{'mount_gen'}[1]{'encpath'} = '';
				} else {
					if (($layers->[$i]{'mount'}[2] eq 'all') || ($layers->[$i]{'mount'}[2] eq 'sl') ||
						($layers->[$i]{'mount'}[2] eq 'glob') || ($layers->[$i]{'mount'}[2] eq 'null')) {

						$layers->[$i]{'mount_gen'}[1] = $layers->[$i]{'mount_gen'}[0]->next;
					} elsif ($layers->[$i]{'mount'}[2] eq 'nonfiles') {
						do {
							$layers->[$i]{'mount_gen'}[1] = $layers->[$i]{'mount_gen'}[0]->next;
						} until ((!(defined ($layers->[$i]{'mount_gen'}[1]{'name'}))) || ($layers->[$i]{'mount_gen'}[1]{'type'} != 0100000));
					} elsif ($layers->[$i]{'mount'}[2] eq 'nondirs') {
						$layers->[$i]{'mount_gen'}[1] = $layers->[$i]{'mount_gen'}[0]->next;
						while ((defined ($layers->[$i]{'mount_gen'}[1]{'name'})) && ($layers->[$i]{'mount_gen'}[1]{'type'} == 0040000)) {
							# Discard all of walk under this dir; hope it's small
							$bypasstree = $layers->[$i]{'mount_gen'}[1]{'name'} . "/";
							do {
								$layers->[$i]{'mount_gen'}[1] = $layers->[$i]{'mount_gen'}[0]->next;
							} until ((!(defined ($layers->[$i]{'mount_gen'}[1]{'name'}))) ||
								(length ($layers->[$i]{'mount_gen'}[1]{'name'}) < length ($bypasstree)) ||
								(substr ($layers->[$i]{'mount_gen'}[1]{'name'}, 0, length ($bypasstree)) ne $bypasstree));
						}
					} elsif ($layers->[$i]{'mount'}[2] eq 'onlydirs') {
						do {
							$layers->[$i]{'mount_gen'}[1] = $layers->[$i]{'mount_gen'}[0]->next;
						} until ((!(defined ($layers->[$i]{'mount_gen'}[1]{'name'}))) || ($layers->[$i]{'mount_gen'}[1]{'type'} == 0040000));
					} elsif ($layers->[$i]{'mount'}[2] eq 'topdir') {
						$layers->[$i]{'mount_gen'}[1] = { };
					} else {
						HF::Report->report (4, 1, "FATAL walk type '" . $layers->[$i]{'mount'}[2] . "' unknown");
					}
					if (defined ($layers->[$i]{'mount_gen'}[1]{'name'})) {
						if ($layers->[$i]{'mount_gen'}[1]{'name'} eq '.') {
							$layers->[$i]{'mount_gen'}[1]{'name'} = $layers->[$i]{'mount'}[0];
						} elsif ($layers->[$i]{'mount'}[0] ne '') {
							$layers->[$i]{'mount_gen'}[1]{'name'} = $layers->[$i]{'mount'}[0] . '/' . $layers->[$i]{'mount_gen'}[1]{'name'};
		 				}
					}
				}

				if (defined ($layers->[$i]{'mount_gen'}[1]{'name'})) {
					if ($layers->[$i]{'mount_gen'}[1]{'name'} eq '.') {
						HF::Report->report (4, 1, "FATAL name should not be dot here, fix so that cmpname = empty");
					}
					if ($layers->[$i]{'mount_gen'}[1]{'name'} eq '') {
						HF::Report->report (4, 1, "FATAL name should not be empty here, fix so that cmpname = empty and name = dot");
					}
					$layers->[$i]{'mount_gen'}[1]{'encname'} = HF::Encode->encode ($layers->[$i]{'mount_gen'}[1]{'name'});
					$cmpname = $layers->[$i]{'mount_gen'}[1]{'name'};
					$cmpname =~ s|/|\000|g;
					$layers->[$i]{'mount_gen'}[1]{'cmpname'} = $cmpname;

					if (exists ($layers->[$i]{'filters'})) {
						for ($j = 0;  $j <= $#{$layers->[$i]{'filters'}};  $j++) {
							$layers->[$i]{'mount_gen'}[1]{$layers->[$i]{'filters'}[$j][0]} = $layers->[$i]{'filters'}[$j][1];
							if ($layers->[$i]{'filters'}[$j][0] eq 'perms') {
								$layers->[$i]{'mount_gen'}[1]{'mode'} = $layers->[$i]{'mount_gen'}[1]{'type'} | $layers->[$i]{'mount_gen'}[1]{'perms'};
							}
						}
					}

					if (exists ($layers->[$i]{'idmap'})) {
						if (exists ($layers->[$i]{'idmap'}{'owner'}{$layers->[$i]{'mount_gen'}[1]{'owner'}})) {
							$layers->[$i]{'mount_gen'}[1]{'owner'} = $layers->[$i]{'idmap'}{'owner'}{$layers->[$i]{'mount_gen'}[1]{'owner'}};
						} elsif (exists ($layers->[$i]{'idmap'}{'owner'}{'default'})) {
								$layers->[$i]{'mount_gen'}[1]{'owner'} = $layers->[$i]{'idmap'}{'owner'}{'default'};
						}
						if (exists ($layers->[$i]{'idmap'}{'group'}{$layers->[$i]{'mount_gen'}[1]{'group'}})) {
							$layers->[$i]{'mount_gen'}[1]{'group'} = $layers->[$i]{'idmap'}{'group'}{$layers->[$i]{'mount_gen'}[1]{'group'}};
						} elsif (exists ($layers->[$i]{'idmap'}{'group'}{'default'})) {
								$layers->[$i]{'mount_gen'}[1]{'group'} = $layers->[$i]{'idmap'}{'group'}{'default'};
						}
					}
				}
			}
		}
	}

	$lowestname = undef;
	for ($i = 0;  $i <= $#{$layers};  $i++) {

		$thisname = $layers->[$i]{'mount_gen'}[1]{'cmpname'};

		if ((defined ($thisname)) && ((!(defined ($lowestname))) || ($thisname lt $lowestname))) {
			$lowestname = $thisname;
		}
	}

	if (!(defined ($lowestname))) {
		return (undef);
	}

	$type = undef;
	for ($i = 0;  $i <= $#{$layers};  $i++) {
		if ($layers->[$i]{'mount_gen'}[1]{'cmpname'} eq $lowestname) {
			$layers->[$i]{'mount_gen'}[1]{'inplay'} = 1;
			if (!(defined ($type))) {
				$type = $layers->[$i]{'mount_gen'}[1]{'type'};
			} else {
				if ($layers->[$i]{'mount_gen'}[1]{'type'} != $type) {
					HF::Report->report (3, 1, "FATAL type incompatibility at source layer $i name '$lowestname' this layer type " .
						$self->type_to_string ($layers->[$i]{'mount_gen'}[1]{'type'}) . " previous layer type " . $self->type_to_string ($type));
				}
			}
		} else {
			$layers->[$i]{'mount_gen'}[1]{'inplay'} = 0;
		}
	}

	$outhash->{'cmpname'} = $lowestname;

	for ($i = 0;  $i <= $#{$layers};  $i++) {

		if ($layers->[$i]{'mount_gen'}[1]{'inplay'} == 1) {

			$outhash->{'whichlayers'} .= "$i";

			$treatment = undef;
			if ($layers->[$i]{'mount_gen'}[1]{'fake'} == 1) {
				$fakeused = 0;
				foreach $k ('name', 'encname', 'dev', 'inode', 'mode', 'type', 'perms', 'links', 'owner', 'group', 'size', 'mtime', 'blksize', 'blocks',              'startdir', 'path', 'encpath'                                                      ) {
					if (!(defined ($outhash->{$k}))) {
						$outhash->{$k} = $layers->[$i]{'mount_gen'}[1]{$k};
						$fakeused = 1;
					}
				}
				if ($fakeused == 1) {
					HF::Report->report (1, 5, "ATTENTION 11 fake mount point directory stats used for '" . $outhash->{'encname'} . "'", 'fail_layers');
				}
				$treatment = 0;
			} else {
				for ($j = 0;  $j <= $#{$layers->[$i]{'treatments'}};  $j++) {

					$oldtreatment = $treatment;
					if (defined ($layers->[$i]{'treatments_gen'}[$j])) {
						if ($layers->[$i]{'mount_gen'}[1]{'name'} =~ /$layers->[$i]{'treatments_gen'}[$j]/) {
							$treatment = $layers->[$i]{'treatments'}[$j][0];
						}
					} else {
						$treatment = $layers->[$i]{'treatments'}[$j][0];
					}

					if (((defined ($oldtreatment)) && (!(defined ($treatment)))) ||
						((!(defined ($oldtreatment))) && (defined ($treatment))) ||
						($treatment != $oldtreatment)) {

						if (($treatment == 108) && ($type != 0120000)) {	# symlink
							HF::Report->report (3, 1, "FATAL treatment 108 given to non-symlink '" . $layers->[$i]{'mount_gen'}[1]{'encname'} . "'");
						}

						if (($treatment == 0) || ($treatment == 101) || ($treatment == 102) || ($treatment == 105) || ($treatment == 107) || ($treatment == 207) ||
							($treatment == 108) || ($treatment == 109)) {

							foreach $k ('name', 'encname', 'dev', 'inode', 'mode', 'type', 'perms', 'links', 'owner', 'group', 'size', 'mtime', 'blksize', 'blocks',              'startdir', 'path', 'encpath', 'checksum', 'rdev', 'target', 'enctarget'           ) {
								$outhash->{$k} = $layers->[$i]{'mount_gen'}[1]{$k};
							}
						} elsif (($treatment == 103) || ($treatment == 113)) {
							if ($end eq "source") {
								foreach $k (                   'dev', 'inode',                          'links',                   'size',          'blksize', 'blocks',              'startdir', 'path', 'encpath', 'checksum', 'rdev', 'target', 'enctarget'           ) {
									$outhash->{$k} = $layers->[$i]{'mount_gen'}[1]{$k};
								}
								if (($type == 0100000) && ($outhash->{'checksum'} eq '')) {
									$outhash->{'checksum'} = $self->getchecksum ($outhash->{'path'});
								}
							} elsif ($end eq "target") {
								$outhash->{'startdir_content'} = $layers->[$i]{'mount_gen'}[1]{'startdir'};
								$outhash->{'path_content'} = $layers->[$i]{'mount_gen'}[1]{'path'};
								$outhash->{'encpath_content'} = $layers->[$i]{'mount_gen'}[1]{'encpath'};
								$outhash->{'rdev_content'} = $layers->[$i]{'mount_gen'}[1]{'rdev'};
								$outhash->{'target_content'} = $layers->[$i]{'mount_gen'}[1]{'target'};
								$outhash->{'enctarget_content'} = $layers->[$i]{'mount_gen'}[1]{'enctarget'};
								$outhash->{'size_content'} = $layers->[$i]{'mount_gen'}[1]{'size'};
								$outhash->{'checksum_content'} = $layers->[$i]{'mount_gen'}[1]{'checksum'};
							} else {
								HF::Report->report (4, 1, "FATAL unknown end $end");
							}
							if (!(defined ($outhash->{'perms'}))) {
								HF::Report->report (1, 5, "ATTENTION 12 no up layer is defining ogm for '" .
									$layers->[$i]{'mount_gen'}[1]{'encname'} . "', using content file values");
							}
							foreach $k ('name', 'encname', 'dev', 'inode', 'mode', 'type', 'perms', 'links', 'owner', 'group', 'size', 'mtime', 'blksize', 'blocks', 'treatment', 'startdir', 'path', 'encpath', 'checksum', 'rdev', 'target', 'enctarget'           ) {
								if (!(defined ($outhash->{$k}))) {
									$outhash->{$k} = $layers->[$i]{'mount_gen'}[1]{$k};
								}
							}
						} elsif ($treatment == 104) {
							foreach $k ('name', 'encname', 'dev', 'inode', 'mode', 'type', 'perms', 'links', 'owner', 'group', 'size', 'mtime', 'blksize', 'blocks',              'startdir', 'path', 'encpath', 'checksum', 'rdev', 'target', 'enctarget'           ) {
								if (!(defined ($outhash->{$k}))) {
									$outhash->{$k} = $layers->[$i]{'mount_gen'}[1]{$k};
								}
							}
							# mainly for benefit of -x 22|32
							if ($type == 0100000) {		# file
								$outhash->{'blocks'} = 0;
								$outhash->{'size'} = 0;
							}
						} elsif ($treatment == 114) {
							foreach $k ('name', 'encname', 'dev', 'inode', 'mode', 'type', 'perms', 'links', 'owner', 'group', 'size', 'mtime', 'blksize', 'blocks',              'startdir', 'path', 'encpath', 'checksum', 'rdev', 'target', 'enctarget'           ) {
								$outhash->{$k} = $layers->[$i]{'mount_gen'}[1]{$k};
							}
							if ($type == 0100000) {		# file
								$outhash->{'blocks'} = 0;
								$outhash->{'size'} = 0;
							}
						} elsif ($treatment == 106) {
							foreach $k ('name', 'encname', 'dev', 'inode', 'mode', 'type', 'perms', 'links', 'owner', 'group', 'size', 'mtime', 'blksize', 'blocks',              'startdir', 'path', 'encpath', 'checksum', 'rdev', 'target', 'enctarget'           ) {
								if ($type == 0040000) {	# dir
									if (!(defined ($outhash->{$k}))) {
										$outhash->{$k} = $layers->[$i]{'mount_gen'}[1]{$k};
									}
								} else {
									$outhash->{$k} = $layers->[$i]{'mount_gen'}[1]{$k};
								}
							}
						} else {
							HF::Report->report (4, 1, "FATAL unknown treatment " . $treatment);
						}
						if ($treatment == 207) {
HF::Report->report (0, 19, "TRACE sub next, genowner " . $outhash->{'genowner'});
							$outhash->{'genowner'} = $layers->[$i]{'treatments'}[$j][2][0];
							$outhash->{'gengroup'} = $layers->[$i]{'treatments'}[$j][2][1];
							$outhash->{'genperms'} = $layers->[$i]{'treatments'}[$j][2][2];
							$outhash->{'genmode'} = $outhash->{'type'} | $outhash->{'perms'};
						}
					}
				}
			}
			if (!(defined ($treatment))) {
				HF::Report->report (4, 1, "FATAL no treatment");
			}
			$outhash->{'treatment'} = $treatment;
		}
	}

	foreach $i ('name', 'encname', 'dev', 'inode', 'mode', 'type', 'perms', 'links', 'owner', 'group', 'size', 'mtime', 'blksize', 'blocks', 'treatment', 'startdir', 'path', 'encpath'                                                      ) {
		if (!(defined ($outhash->{$i}))) {
			HF::Report->report (4, 1, "FATAL insufficient config for source field '$i' name '" . $layers->[$i]{'mount_gen'}[1]{'encname'} . "'");
		}
	}

	if ((($treatment == 103) || ($treatment == 113)) && ($end eq "target")) {
		foreach $i ('startdir_content', 'path_content', 'encpath_content', 'rdev_content', 'size_content') {
			if (!(defined ($outhash->{$i}))) {
				HF::Report->report (4, 1, "FATAL insufficient config for source field '$i' name '" . $layers->[$i]{'mount_gen'}[1]{'encname'} . "'");
			}
		}
		if (($type == 0120000) && ((!(defined ($outhash->{'target_content'}))) || (!(defined ($outhash->{'enctarget_content'}))))) {
			HF::Report->report (4, 1, "FATAL insufficient config for source field 'target_content' and/or 'enctarget_content', name '" . $layers->[$i]{'mount_gen'}[1]{'encname'} . "'");
		}
	}

	if ((($type == 0020000) || ($type == 0060000)) && (!(defined ($outhash->{'rdev'})))) {
		HF::Report->report (4, 1, "FATAL insufficient config for source field 'rdev', name " . $layers->[$i]{'mount_gen'}[1]{'encname'} . "'");
	}
	if (($type == 0120000) && ((!(defined ($outhash->{'target'}))) || (!(defined ($outhash->{'enctarget'}))))) {
		HF::Report->report (4, 1, "FATAL insufficient config for source field 'target' and/or 'enctarget', name '" . $layers->[$i]{'mount_gen'}[1]{'encname'} . "'");
	}

	if (($type != 0040000) && ((!(defined ($outhash->{'path'}))) || (!(defined ($outhash->{'encpath'}))))) {
		HF::Report->report (4, 1, "FATAL insufficient config for source field 'path' or 'encpath', name " . $layers->[$i]{'mount_gen'}[1]{'encname'} . "'");
	}

	$self->{'firstnextrun'} = 0;

	return ($outhash);
}

sub walk {

	my $self = shift;
	my $layers = shift;
	my $end = shift;

	my $whichlayers;
	my $walkhash;
	my $treatment;
	my $whichnext;

	if ($end eq 'source') {
		$whichnext = sub { $self->source_next };
	} elsif ($end eq 'target') {
		$whichnext = sub { $self->target_next };
	} else {
		HF::Report->report (4, 1, 'FATAL end not source or target');
	}

###	while ($walkhash = $self->next ($layers, $end), defined ($walkhash)) {
	while ($walkhash = &$whichnext, defined ($walkhash->{'inode'})) {

		if (length ($walkhash->{'whichlayers'}) gt 1) {
			$whichlayers = 'LAYERS';
		} else {
			$whichlayers = 'layers';
		}
		if ($walkhash->{'treatment'} gt 0) {
			$treatment = 'TMT';
		} else {
			$treatment = 'tmt';
		}
		HF::Report->report (1, 2, sprintf ("WALK %s %s %-10s %s %3d name %-30s     path %s", $end, $whichlayers, $walkhash->{'whichlayers'},
			$treatment, $walkhash->{'treatment'}, $walkhash->{'encname'}, $walkhash->{'encpath'}));

		if (($HF::Globals::opt_x == 20) || ($HF::Globals::opt_x == 30)) {
			HF::Report->report (1, 2, sprintf ("WALK " . &HF::Layers::print_hash ($walkhash)));
		}
	}
}

sub diskusage_walk {

	my $self = shift;
	my $layers = shift;
	my $end = shift;

	my $walkhash;
	my $whichnext;
	my $now;
	my $checkpointtime;

	if ($end eq 'source') {
		$whichnext = sub { $self->source_next };
	} elsif ($end eq 'target') {
		$whichnext = sub { $self->target_next };
	} else {
		HF::Report->report (4, 1, 'FATAL end not source or target');
	}

	$HF::Globals::diskusage = { };
	$HF::Globals::diskusage{'.'} = 0;

	$checkpointtime = time + 1;
###	while ($walkhash = $self->next ($layers, $end), defined ($walkhash)) {
	while ($walkhash = &$whichnext, defined ($walkhash->{'inode'})) {

		$self->diskusage_record ($walkhash);

		$now = time;
		if ($now > $checkpointtime) {
			HF::Report->report (0, 4, "PROGRESS '" . $walkhash->{'encname'} . "'");
			$checkpointtime = $now + 1;
		}
	}
}

sub diskusage_record {

	my $self = shift;
	my $walkhash = shift;

	my @splitpath;
	my $i;
	my $subpath;

	if ($walkhash->{'encname'} ne '.') {
		$walkhash->{'duencname'} = './' . $walkhash->{'encname'};
	}

	@splitpath = split (/\//, $walkhash->{'duencname'});

	for ($i = $#splitpath;  $i >= 0;  $i--) {

		if (($i == $#splitpath) && ($walkhash->{'type'} != 0040000)) {	# Directory
			next;
		}

		# The character "/" needs to compare lower than any other
		# character, such as "-".  If this is not done, then "foo-"
		# compares as earlier than "foo/", and the sort of directories
		# is broken.
		$subpath = join ("\000", @splitpath[0 .. $i]);

		$HF::Globals::diskusage{$subpath} += $walkhash->{'size'};
	}
}

sub diskusage_report {

	my $self = shift;

	my $key;
	my $path;
	my $rounded;

	foreach $key (sort (keys (%HF::Globals::diskusage))) {

		$path = $key;
		$path =~ s|\000|/|g;

		$rounded = int (($HF::Globals::diskusage{$key} / 1024) + 0.5);

		push (@{$HF::Globals::opt_x_output}, "$rounded\t$path");
	}
}

sub log_walk {

	my $self = shift;
	my $layers = shift;
	my $end = shift;

	my $walkhash;
	my $whichnext;
	my $now;
	my $checkpointtime;

	if ($end eq 'source') {
		$whichnext = sub { $self->source_next };
	} elsif ($end eq 'target') {
		$whichnext = sub { $self->target_next };
	} else {
		HF::Report->report (4, 1, 'FATAL end not source or target');
	}

	$checkpointtime = time + 1;
###	while ($walkhash = $self->next ($layers, $end), defined ($walkhash)) {
	while ($walkhash = &$whichnext, defined ($walkhash->{'inode'})) {

		push (@{$HF::Globals::opt_x_output}, [ $walkhash->{'name'}, undef, undef, { %$walkhash }, undef ]);

		$now = time;
		if ($now > $checkpointtime) {
			HF::Report->report (0, 4, "PROGRESS '" . $walkhash->{'encname'} . "'");
			$checkpointtime = $now + 1;
		}
	}
}

sub type_to_string {

	my $self = shift;
	my $type = shift;

	my $res;

	if ($type == 0100000) {		# file
		$res = 'f';
	} elsif ($type == 0040000) {	# dir
		$res = 'd';
	} elsif ($type == 0120000) {	# symlink
		$res = 'l';
	} elsif ($type == 0020000) {	# char dev
		$res = 'c';
	} elsif ($type == 0060000) {	# block dev
		$res = 'b';
	} elsif ($type == 0010000) {	# fifo
		$res = 'p';
	} elsif ($type == 0140000) {	# socket
		$res = 's';
	} else {
		$res = sprintf ("?=%o", $type);
	}

	return ($res);
}

sub getchecksum {

	my $self = shift;
	my $filename = shift;

	my $MD5;

	$self->{'md5handle'}->reset;
	if (open ($MD5, "<", $filename)) {
		$self->{'md5handle'}->addfile ($MD5);
		close ($MD5) || HF::Report->report (4, 1, "FATAL close of md5 fh for filename '$filename' failed, error '$!'");
		return ($self->{'md5handle'}->hexdigest);
	} else {
		HF::Report->report (2, 2, "FAILURE 2 can't open '" . HF::Encode->encode ($filename) . "' for checksum, error '$!'", 'fail_layers');
		return (undef);
	}
}

sub print_hash {

	my $hash = shift;

	return (sprintf ("name(E)    '%s'
                 encname    '%s'
                 target(E)  '%s'
                 enctarget  '%s'
                 path(E)    '%s'
         path_content(E)    '%s'
                 encpath    '%s'
         encpath_content    '%s'
              startdir(E)   '%s'
      startdir_content(E)   '%s'
                 cmpname(E) '%s'
                 dev        %10d  inode     %10d  mode         0%06o
                 type          0%06o  perms          0%04o  links     %10d
                 owner      %10d  group     %10d  rdev      %10d
                 size       %10d                        mtime     %10d
                                        blksize   %10d  blocks    %10d
                 checksum   %s
                 treatment  %3d",
				HF::Encode->encode ($hash->{'name'}),
				$hash->{'encname'},
				HF::Encode->encode ($hash->{'target'}),
				$hash->{'enctarget'},
				HF::Encode->encode ($hash->{'path'}),
				HF::Encode->encode ($hash->{'path_content'}),
				$hash->{'encpath'},
				$hash->{'encpath_content'},
				HF::Encode->encode ($hash->{'startdir'}),
				HF::Encode->encode ($hash->{'startdir_content'}),
				HF::Encode->encode ($hash->{'cmpname'}),
				$hash->{'dev'},
				$hash->{'inode'},
				$hash->{'mode'},
				$hash->{'type'},
				$hash->{'perms'},
				$hash->{'links'},
				$hash->{'owner'},
				$hash->{'group'},
				$hash->{'rdev'},
				$hash->{'size'},
				$hash->{'mtime'},
				$hash->{'blksize'},
				$hash->{'blocks'},
				$hash->{'checksum'},
				$hash->{'treatment'}));
}

sub path_is_clean {

	my $in = shift;

	my @pieces;
	my $j;

	if ($in eq '') {
		return (0);
	} elsif ($in =~ /\/\//) {
		return (0);
	} elsif (($in ne '/') && ($in =~ /\/$/)) {
		return (0);
	}

	@pieces = split ('/', $in, -1);

	foreach $j (@pieces) {
		if ($j eq '..') {
			return (0);
		} elsif ($j eq '.') {
			return (0);
		}
	}

	return (1);
}

1;
