#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

package HF::Local;
use strict;

use Data::Dumper;

sub main {

	my $dumper;
	my $all;
	my $highreturnval;
	my $pwdresult;
	my $dump_filter;
	my $grand_diff_total;
	my $grand_fail_total;
	my $i;
	my $returnval;

# -----

	$HF::Globals::islocal = 1;
	if (!(defined ($HF::Globals::dowrite))) {
		$HF::Globals::dowrite = 0;
	}
	if (!(defined ($HF::Globals::dodelete))) {
		$HF::Globals::dodelete = 0;
	}
	if (!(defined ($HF::Globals::dochecksums))) {
		$HF::Globals::dochecksums = 0;
	}
	if (!(defined ($HF::Globals::checkreadiness))) {
		$HF::Globals::checkreadiness = 0;
	}
	if (($HF::Globals::dodelete) && (!($HF::Globals::dowrite))) {
		HF::Report->report (3, 1, "FATAL option -d without option -w is not sensible");
	}
	if (!(defined ($HF::Globals::opt_x))) {
		$HF::Globals::opt_x = 0;
	}
	if (!(defined ($HF::Globals::opt_X))) {
		$HF::Globals::opt_X = 0;
	}
	if (!(defined ($HF::Globals::swappushpull))) {
		$HF::Globals::swappushpull = 0;
	}
	if (!(defined ($HF::Globals::swaplayers))) {
		$HF::Globals::swaplayers = 0;
	}
	if (!(defined ($HF::Globals::examine))) {
		$HF::Globals::examine = '';
	}
	if (!(defined ($HF::Globals::free_bytes))) {
		$HF::Globals::free_bytes = 0;
	}
	if (!(defined ($HF::Globals::free_inodes))) {
		$HF::Globals::free_inodes = 0;
	}
	if (defined ($HF::Globals::outputplbig)) {
		# outputpl used for filename, outputplbig overrides
		$HF::Globals::outputpl = $HF::Globals::outputplbig;
	}
	if (!(defined ($HF::Globals::donotfixhardlinks))) {
		$HF::Globals::donotfixhardlinks = 0;
	}

# -----

	HF::Report->report (0, 4, "PROGRESS local starting up, pid " . $$);
	HF::Report->report (0, 6, "PROGRESS local hftop " .
		$HF::Globals::hftop . ", hg version " .
		$HF::Globals::hgversion);

	$pwdresult = `/bin/pwd 2>&1`;
	chop ($pwdresult);
	# "$?" to fix emacs syntax highlighting
	if (("$?" >> 8) != 0) {
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_connection'}++;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_no_remote_answer'}++;	##### KEEP
		HF::Report->report (3, 1, "FATAL /bin/pwd failed with '$pwdresult', error '$!'");
	}
	HF::Report->report (0, 6, "PROGRESS local " .
		"dowrite " . $HF::Globals::dowrite . ", " .
		"dodelete " . $HF::Globals::dodelete . ", " .
		"dochecksums " . $HF::Globals::dochecksums . ", " .
		"donotfixhardlinks " . $HF::Globals::donotfixhardlinks . ", " .
		"opt_x " . $HF::Globals::opt_x . ", " .
		"opt_X " . $HF::Globals::opt_X . ", " .
		"swappushpull " . $HF::Globals::swappushpull . ", " .
		"swaplayers " . $HF::Globals::swaplayers . ", " .
		"examine '" . $HF::Globals::examine . "', " .
		"free_bytes " . $HF::Globals::free_bytes . ", " .
		"free_inodes " . $HF::Globals::free_inodes . ", " .
		"pwd " . $pwdresult);

# -----

	if (defined ($HF::Globals::outputpl)) {
		if (!(open (OUTPUTFILE, ">", $HF::Globals::outputpl))) {
			HF::Report->report (3, 1, "FATAL can't open '$HF::Globals::outputpl' for log file, error $!");
		}
	}

	$highreturnval = 0;
	for ($HF::Globals::hostrow = 0;  $HF::Globals::hostrow <= $#{$HF::Globals::hosts};  $HF::Globals::hostrow++) {

		$HF::Globals::local_report->[$HF::Globals::hostrow] = { };
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'time_begin'} = time;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'pid'} = $$;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'hftop'} = $HF::Globals::hftop;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'hgversion'} = $HF::Globals::hgversion;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'dowrite'} = $HF::Globals::dowrite;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'dodelete'} = $HF::Globals::dodelete;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'dochecksums'} = $HF::Globals::dochecksums;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'opt_x'} = $HF::Globals::opt_x;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'opt_X'} = $HF::Globals::opt_X;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'donotfixhardlinks'} = $HF::Globals::donotfixhardlinks;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'checkreadiness'} = $HF::Globals::checkreadiness;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'swappushpull'} = $HF::Globals::swappushpull;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'swaplayers'} = $HF::Globals::swaplayers;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'examine'} = $HF::Globals::examine;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'free_bytes'} = $HF::Globals::free_bytes;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'free_inodes'} = $HF::Globals::free_inodes;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'outputpl'} = $HF::Globals::outputpl;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'outputplbig'} = $HF::Globals::outputplbig;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'pwd'} = $pwdresult;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'euid'} = $>;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_audit'} = 0;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_connection'} = 0;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_layers'} = 0;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_no_remote_answer'} = 0;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_total'} = 0;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_wrong_remote_version'} = 0;
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'abandoned_remote'} = 0;

		$HF::Globals::local_errors->[$HF::Globals::hostrow] = [ ];

		$returnval = &for_a_hostrow;
		if ($returnval != 0) {
			if ($returnval > $highreturnval) {
				$highreturnval = $returnval;
			}
		}

		$HF::Globals::local_report->[$HF::Globals::hostrow]{'time_end'} = time;

		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_total'} =
			$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_connection'} +
			$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_audit'} +
			$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_layers'};

		HF::Report->report (0, 5,          "PROGRESS              file       dir   symlink      pipe   chardev    blkdev    socket       all    file byte seen");
		HF::Report->report (0, 5, sprintf ("PROGRESS source %10s %9s %9s %9s %9s %9s %9s %9s %17s",
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_f'}),
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_d'}),
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_l'}),
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_p'}),
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_c'}),
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_b'}),
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_s'}),
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_all'}),
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'seen_f_bytes'})));
		HF::Report->report (0, 5, sprintf ("PROGRESS target %10s %9s %9s %9s %9s %9s %9s %9s %17s",
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_f'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_d'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_l'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_p'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_c'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_b'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_s'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_all'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'seen_f_bytes'})));
		HF::Report->report (0, 5,          "PROGRESS");
		HF::Report->report (0, 5,          "PROGRESS    want byte sent want file      file byte sent file sent   fixlink dir mtime  diff tot  fail tot");
		HF::Report->report (0, 5, sprintf ("PROGRESS %17s %9s   %17s %9s %9s %9s %9s %9s",
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'want_sent_f_bytes'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'want_sent_f'}),
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'sent_f_bytes'}),
			&HF::Misc::commify_number ($HF::Globals::source_report->[$HF::Globals::hostrow]{'sent_f'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'fix_hardlinks'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'fix_dir_mtimes'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_total'}),
			&HF::Misc::commify_number (
				$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_total'} +
				$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_total'})));
		HF::Report->report (0, 5,          "PROGRESS");
		HF::Report->report (0, 5,          "PROGRESS diffs create     modify     reload     delete | fails  connection      audit     layers");
		HF::Report->report (0, 5, sprintf ("PROGRESS   %10s %10s %10s %10s | local  %10s %10s %10s",
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_create'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_modify'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_reload'}),
			&HF::Misc::commify_number ($HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_delete'}),
			&HF::Misc::commify_number ($HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_connection'}),
			&HF::Misc::commify_number ($HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_audit'}),
			&HF::Misc::commify_number ($HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_layers'})));
		HF::Report->report (0, 5, sprintf ("PROGRESS                                               | remote %10s %10s %10s",
			&HF::Misc::commify_number ($HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_connection'}),
			&HF::Misc::commify_number ($HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_audit'}),
			&HF::Misc::commify_number ($HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_layers'})));
	}

# -----

	if (defined ($HF::Globals::outputpl)) {
		$all = {
			'hosts_file' => $HF::Globals::hosts_file,
			'hosts' => $HF::Globals::hosts,
			'layers_file' => $HF::Globals::layers_file,
			'source_layers' => $HF::Globals::source_layers,
			'target_layers' => $HF::Globals::target_layers,
			'local_report' => $HF::Globals::local_report,
			'remote_report' => $HF::Globals::remote_report,
			'source_report' => $HF::Globals::source_report,
			'target_report' => $HF::Globals::target_report,
			'local_errors' => $HF::Globals::local_errors,
			'remote_errors' => $HF::Globals::remote_errors
		};
		if (defined ($HF::Globals::outputplbig)) {
			$all->{'target_differences'} = $HF::Globals::target_differences;
			$all->{'source_skips'} = $HF::Globals::source_skips;
			$all->{'target_skips'} = $HF::Globals::target_skips;
		}

		# Remove unwanted internals such as the read and write filehandles,
		# mount_gen, and treatments_gen from the perl report
		$dump_filter = q|
sub dump_filter {

	my ($hash) = @_;

	# return an array ref containing the hash keys to dump
	# in the order that you want them to be dumped

	return [

|;

		for ($HF::Globals::hostrow = 0;  $HF::Globals::hostrow <= $#{$HF::Globals::hosts};  $HF::Globals::hostrow++) {
			$dump_filter .= q|		$hash eq $all->{'hosts'}[| . $HF::Globals::hostrow .
				q|] ? (sort (grep {(($_ ne 'read') && ($_ ne 'write'))} keys %$hash)) :
|;
		}

		for ($i = 0;  $i <= $#{$HF::Globals::source_layers};  $i++) {
			$dump_filter .= q|		$hash eq $all->{'source_layers'}[| . $i .
				q|] ? (sort (grep {(($_ ne 'mount_gen') && ($_ ne 'skips_gen') && ($_ ne 'treatments_gen'))} keys %$hash)) :
|;
		}

		for ($i = 0;  $i <= $#{$HF::Globals::target_layers};  $i++) {
			$dump_filter .= q|		$hash eq $all->{'target_layers'}[| . $i .
				q|] ? (sort (grep {(($_ ne 'mount_gen') && ($_ ne 'skips_gen') && ($_ ne 'treatments_gen'))} keys %$hash)) :
|;
		}

		$dump_filter .= q|
		# Sort keys in default order for all other hashes
		(sort keys %$hash)
	];
}

1;
|;

		eval ($dump_filter) || die "$!";

		$dumper = Data::Dumper->new ([$all]);
		$dumper->Sortkeys (\&dump_filter);
		$dumper->Terse (1);
		print OUTPUTFILE $dumper->Dump;
		close (OUTPUTFILE);
	}

# -----

	for ($HF::Globals::hostrow = 0;  $HF::Globals::hostrow <= $#{$HF::Globals::hosts};  $HF::Globals::hostrow++) {
		$grand_diff_total += $HF::Globals::target_report->[$HF::Globals::hostrow]{'diff_total'};
		$grand_fail_total += (
			$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_total'} +
			$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_total'});
	}

       	if ($HF::Globals::verbosity >= 1) {
		print $HF::Report::forparsefh "HFDIST_DIFF_TOTAL=" . $grand_diff_total . "\n" . "HFDIST_FAIL_TOTAL=" . $grand_fail_total . "\n";
	}

# -----

	HF::Report->report (0, 4, "PROGRESS local shutting down");

	return ($highreturnval);
}

sub for_a_hostrow {

	my $from;
	my $dumper;
	my $globals;
	my $harmonizeptr;
	my $reportstuff;
	my $returnval;

# -----

	&start_remote;

# -----

	$from = &fromremote;
	if (!(defined ($from))) {
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_wrong_remote_version'}++;	##### KEEP
		HF::Report->report (2, 2, "FAILURE no remote answer", 'fail_connection');
		return (1);
	}

# -----

	$HF::Globals::local_report->[$HF::Globals::hostrow]{'checkreadiness'} = 0;
	if (!(($from eq 'hfdist wire protocol 1.24.1') ||
		($from eq 'hfdist wire protocol 1.24.2') ||
		($from eq 'hfdist wire protocol 1.24.3'))) {

		if (!(&toremote ('shutdown'))) {
			return (1);
		}
		HF::Report->report (2, 2, "FAILURE wrong remote version '$from'", 'fail_connection');
		return (1);
	}

	$HF::Globals::local_report->[$HF::Globals::hostrow]{'checkreadiness'} = 1;
	if ($HF::Globals::checkreadiness == 1) {
		if (!(&toremote ('shutdown'))) {
			return (1);
		}
		HF::Report->report (0, 4, "PROGRESS readiness check completed");
		return (0);
	}

# -----

	$dumper = Data::Dumper->new ([
$HF::Globals::hosts_file,
$HF::Globals::hosts,
$HF::Globals::hostrow,
$HF::Globals::layers_file,
$HF::Globals::source_layers,
$HF::Globals::target_layers,
$HF::Globals::layers_parsed,
$HF::Globals::outputpl,
$HF::Globals::outputplbig,
$HF::Globals::stderrcols
],[
'HF::Globals::hosts_file',
'HF::Globals::hosts',
'HF::Globals::hostrow',
'HF::Globals::layers_file',
'HF::Globals::source_layers',
'HF::Globals::target_layers',
'HF::Globals::layers_parsed',
'HF::Globals::outputpl',
'HF::Globals::outputplbig',
'HF::Globals::stderrcols'
]);

	$globals = $dumper->Dump;
	chop ($globals);
	if (!(&toremote (HF::Encode->encode ($globals)))) {
		return (1);
	}

# -----

	$HF::Globals::local_comm_error_goto_ABANDONED_REMOTE = 1;

	if ($HF::Globals::hosts->[$HF::Globals::hostrow]{'direction'} eq 'pull') {
		$harmonizeptr = HF::Harmonize->new (0);
		$returnval = $harmonizeptr->target_main;
	} else {
		$harmonizeptr = HF::Harmonize->new (1);
		$returnval = $harmonizeptr->source_main;
	}

	$HF::Globals::local_comm_error_goto_ABANDONED_REMOTE = 0;

# -----

	$from = &fromremote;
	if (!(defined ($from))) {
		return (1);
	}
	$reportstuff = HF::Encode->decode ($from);
	if ($reportstuff eq 'abort') {
		HF::Report->report (1, 3, 'PROGRESS commanded to abort');
		return (1);
	} elsif (($reportstuff eq 'walk') || ($reportstuff eq 'opt_x')) {
		$from = &fromremote;
		if (!(defined ($from))) {
			return (1);
		}
		$reportstuff = HF::Encode->decode ($from);
	}

	eval $reportstuff;

	if ($@ ne '') {
		HF::Report->report (4, 1, "FATAL eval failed of reportstuff, error '$@'
$reportstuff
");
	}

# -----

	$from = &fromremote;
	if (!(defined ($from))) {
		return (1);
	}
	if ($from ne 'goodbye') {
		HF::Report->report (4, 1, "FATAL protocol error, should have received goodbye, did receive '$from'");
	}

# -----

	if ($returnval != 0) {
		return ($returnval);
	} elsif (($HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_connection'} +
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_audit'} +
		$HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_layers'} +
		$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_total'}) == 0) {

		return (0);
	}

ABANDONED_REMOTE: return (1);
}

sub start_remote {

	my @cmd;
	my $popenresult = { };

	if ($HF::Globals::hosts_parsed != 1) {
		HF::Report->report (4, 1, 'FATAL hosts unparsed');
	}

	if ($HF::Globals::remotecmd ne '') {
		@cmd = $HF::Globals::remotecmd;
	} else {
		@cmd = $HF::Globals::hftop . '/bin/hfdist';
	}
	push (@cmd, '-S');
	if ($HF::Globals::dowrite == 1) {
		push (@cmd, '-w');
	}
	if ($HF::Globals::dodelete == 1) {
		push (@cmd, '-d');
	}
	if ($HF::Globals::dochecksums == 1) {
		push (@cmd, '-k');
	}
	if ($HF::Globals::donotfixhardlinks == 1) {
		push (@cmd, '-L');
	}
	push (@cmd, '-v', $HF::Globals::verbosity);
	if ($HF::Globals::opt_x != 0) {
		push (@cmd, '-x', $HF::Globals::opt_x);
	}
	if ($HF::Globals::opt_X != 0) {
		push (@cmd, '-X', $HF::Globals::opt_X);
	}
	if ($HF::Globals::swappushpull == 1) {
		push (@cmd, '-r');
	}
	if ($HF::Globals::swaplayers == 1) {
		push (@cmd, '-s');
	}
	if ($HF::Globals::examine ne '') {
		push (@cmd, '-e', $HF::Globals::examine);
	}
	if ($HF::Globals::free_bytes > 0) {
		push (@cmd, '-a', $HF::Globals::free_bytes);
	}
	if ($HF::Globals::free_inodes > 0) {
		push (@cmd, '-A', $HF::Globals::free_inodes);
	}

	if ($HF::Globals::hosts->[$HF::Globals::hostrow]{'channel'} eq 'fork') {
		# good to go
	} elsif ($HF::Globals::hosts->[$HF::Globals::hostrow]{'channel'} eq 'null') {
		# good to go
	} elsif ($HF::Globals::hosts->[$HF::Globals::hostrow]{'channel'} eq 'ssh') {
		if ($HF::Globals::hosts->[$HF::Globals::hostrow]{'hostname'} ne '') {
			unshift (@cmd, $HF::Globals::hosts->[$HF::Globals::hostrow]{'hostname'}, '--');
			if (exists ($HF::Globals::hosts->[$HF::Globals::hostrow]{'sshopts'})) {
				unshift (@cmd, @{$HF::Globals::hosts->[$HF::Globals::hostrow]{'sshopts'}});
			}
			unshift (@cmd, 'ssh');
		} elsif ($HF::Globals::hosts->[$HF::Globals::hostrow]{'ip'} ne '') {
			unshift (@cmd, $HF::Globals::hosts->[$HF::Globals::hostrow]{'ip'} , '--');
			if (exists ($HF::Globals::hosts->[$HF::Globals::hostrow]{'sshopts'})) {
				unshift (@cmd, @{$HF::Globals::hosts->[$HF::Globals::hostrow]{'sshopts'}});
			}
			unshift (@cmd, 'ssh');
		} else {
			HF::Report->report (3, 1, "FATAL missing hostname and ip");
		}
	} else {
		HF::Report->report (3, 1, "FATAL bogus channel " . $HF::Globals::hosts->[$HF::Globals::hostrow]{'channel'});
	}


	if ($HF::Globals::hosts->[$HF::Globals::hostrow]{'channel'} eq 'null') {
		HF::Report->report (0, 6, "PROGRESS null channel");

		STDIN->autoflush;
		STDOUT->autoflush;

		$HF::Globals::hosts->[$HF::Globals::hostrow]{'write'} = \*STDOUT;
		$HF::Globals::hosts->[$HF::Globals::hostrow]{'read'} =  \*STDIN;
	} else {
		HF::Report->report (0, 6, "PROGRESS cmd is '" . join ("', '", @cmd) . "'");

		$popenresult = HF::Popen2way->new (@cmd);

		$HF::Globals::hosts->[$HF::Globals::hostrow]{'write'} = $popenresult->{'write'};
		$HF::Globals::hosts->[$HF::Globals::hostrow]{'read'} =  $popenresult->{'read'};
	}

	return (1);
}

sub toremote {

	my $arg = shift;

	if (!(print {$HF::Globals::hosts->[$HF::Globals::hostrow]{'write'}} $arg . "\n")) {
		HF::Report->report (2, 2, 'FAILURE write to remote', 'fail_connection');
		return (0);
	} else {
		return (1);
	}
}

sub fromremote {

	my $inline;

	$inline = $HF::Globals::hosts->[$HF::Globals::hostrow]{'read'}->getline;
	if (!(defined ($inline))) {
		HF::Report->report (2, 2, 'FAILURE read from remote, result is undef', 'fail_connection');
		return (undef);
	}
	chop ($inline);

	return ($inline);
}

1;
