#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

package HF::Misc;
use strict;

require "sys/syscall.ph";

# XXX replace from a standard lib, File::Basename or File::Spec?

sub joinpaths {

	my @inlist = @_;

	my $i;
	my $out;
	my $isroot;

	if (substr ($inlist[0], 0, 1) eq '/') {
		$isroot = 1;
	} else {
		$isroot = 0;
	}

	for ($i = 0;  $i <= $#inlist;  $i++) {

		$inlist[$i] =~ s,^/+,,;
		$inlist[$i] =~ s,/+$,,;
	}

	$out = join ('/', @inlist);

	if (($isroot == 1) && ($inlist[0] ne '')) {
		$out = '/'. $out;
	}

	return ($out);
}

# XXX replace from a standard lib, File::Basename or File::Spec?

sub dirpart {

	my $path = shift;

	$path =~ s,(^|/)[^/]+$,,;
	if ($path eq '') {
		$path = '.';
	}
	return ($path);
}

# XXX replace from a standard lib, File::Basename or File::Spec?

sub basepart {

	my $path = shift;

	if ($path eq '/') {
		return ($path);
	} else {
		$path =~ s,/$,,;
		$path =~ s,^.*/,,;
		return ($path);
	}
}

sub makeuniqfilename {

	my $name = shift;

	my $fullname;
	my $counter = 1;

	# Different logic than in omar/omar.pl:omar_makeuniqfilename,
	# This function never returns the name as-is, it always changes it

	do {
		$fullname = $name . "." . $counter++;
	} until ((!(-e $fullname)) && (!(-l $fullname)) && (!(defined ($HF::Globals::makeuniqfilename_used->{$fullname}))));
	$HF::Globals::makeuniqfilename_used->{$fullname} = 1;

	return ($fullname);
}

#  Handle rdev opaquely, like gtar; do not try to split into major/minor.

sub mknod {

	my $name = shift;
	my $mode = shift;
	my $rdev = shift;

	my $retval;

	$mode = $mode + 0;	# Convert to numeric
	$rdev = $rdev + 0;

	if (($retval = syscall (&SYS_mknod, $name, $mode, $rdev)) == 0) {
		return (1);
	} else {
		return ($retval);
	}
}

#  Like perl system, but never give commands to a shell

sub cleansystem {

	my @commands = @_;

	my $pid;
	my $result;

	$pid = open (CLEANSYSTEM, "-|");
	if (!(defined ($pid))) {
		HF::Report->report (4, 1, "FATAL fork failed, should not reach this point, error '$!'");
	}
	# parent
	if ($pid > 0) {
		wait;
	# "$?" to fix emacs syntax highlighting
		$result = (!("$?" >> 8));
		close (CLEANSYSTEM);
		return ($result);
	# child
	} else {
		exec { $commands[0] } @commands;
		HF::Report->report (4, 1, "FATAL exec failed, should not reach this point, error '$!'");
	}
}

# Change 12345678901 into 12,345,678,901

sub commify_number {

	my $number = shift;

	# In case of number like "500E6", but don't use for all
	# numbers because it will strip trailing .0
	if ($number =~ /E/) {
		$number += 0;
	}
	$number =~ s/(\d)(?=(\d{3})+(\D|$))/$1\,/g;
	return ($number);
}

# Change etc/foo/bar into ^(?!($|etc($|/foo($|/bar($|/)))))

sub regex_for_skip_all_but_this_path {

	my $wantbelow = shift;
	my @path = @_;

	my $regex;
	my @regexarr;
	my @patharr;
	my $i;
	my $j;

	if ($#path == -1) {
		HF::Report->report (3, 1, "FATAL path argument is empty");
	}
	for ($i = 0;  $i <= $#path;  $i++) {
		if ($path[$i] eq '') {
			HF::Report->report (3, 1, "FATAL path argument $i is zero length");
		}
		if ($path[$i] =~ /^\//) {
			HF::Report->report (3, 1, "FATAL leading slash should be removed from path '" . $path[$i] . "'");
		}
		if ($path[$i] =~ /\/$/) {
			HF::Report->report (3, 1, "FATAL trailing slash should be removed from path '" . $path[$i] . "'");
		}
		if ($path[$i] =~ /[\.\^\$\*\+\(\)\[\]\{\}\|\?\\]/) {
			HF::Report->report (2, 1, "WARNING path '" . $path[$i] . "'" . ' should not contain regex characters .^$*+()[]{}|?\\, quoting them', 'fail_layers');
			$path[$i] =~ s/([\.\^\$\*\+\(\)\[\]\{\}\|\?\\])/\\$1/g;
		}
	}

	for ($i = 0;  $i <= $#path;  $i++) {
		@patharr = split (/\//, $path[$i]);
		if ($wantbelow == 1) {
			$regex = '$|/';
		} else {
			$regex = '$';
		}
		for ($j = $#patharr;  $j > 0;  $j--) {
			$regex = '$|/' . $patharr[$j] . '(' . $regex . ')';
		}
		$regex = $patharr[0] . '(' . $regex . ')';
		push (@regexarr, $regex);
	}

	$regex = '^(?!((' . join (")|(", @regexarr) . ')))';

	return ($regex);
}

sub path_is_in {

	my $upperpath = shift;
	my $lowerpath = shift;

	my @upperarr;
	my @lowerarr;
	my $i;
	my $shorter;

	if ($upperpath eq '.') {
		HF::Report->report (4, 1, "FATAL path_is_in upperpath is dot upperpath '$upperpath'");
	}
	if ($upperpath =~ /\/\//) {
		HF::Report->report (4, 1, "FATAL path_is_in upperpath contains slashslash upperpath '$upperpath'");
	}
	if ($lowerpath =~ /\/\//) {
		HF::Report->report (4, 1, "FATAL path_is_in lowerpath contains slashslash lowerpath '$lowerpath'");
	}

	if (($upperpath eq '') || ($upperpath eq '/')) {
		return (1, $lowerpath);
	}
	if (($lowerpath eq '') || ($lowerpath eq '/')) {	# unless upperpath also /, covered previously
		return (0, undef);
	}

	@upperarr = split ('/', $upperpath);
	@lowerarr = split ('/', $lowerpath);

	for ($i = 0;  $i <= $#upperarr;  $i++) {
		if ($upperarr[$i] eq '..') {
			HF::Report->report (4, 1, "FATAL path_is_in upperpath contains dotdot upperpath '$upperpath'");
		}
	}
	for ($i = 0;  $i <= $#lowerarr;  $i++) {
		if ($lowerarr[$i] eq '..') {
			HF::Report->report (4, 1, "FATAL path_is_in lowerpath contains dotdot lowerpath '$lowerpath'");
		}
	}

	$shorter = $#upperarr;
	if ($#lowerarr < $shorter) {
		$shorter = $#lowerarr;
	}

	for ($i = 0;  $i <= $shorter;  $i++) {
		if ($upperarr[$i] ne $lowerarr[$i]) {
			return (0, undef);
		}
	}

	if ($#upperarr == $#lowerarr) {
		return (2, undef);
	} elsif ($#lowerarr > $shorter) {
		return (1, @lowerarr[($shorter + 1) .. $#lowerarr]);
	} else {
		return (0, undef);
	}
}

#  For oe and oel

sub dosetmath {

	my @setmath = @_;
	my $arg;
	my @intokens;
	my @netresult;
	my $i;
	my $j;
	my $neg;
	my @thiscontrib;
	my @contributors;
	my $hostlistfilename;
	my @tmpnetresult;

	while (defined ($arg = shift (@setmath))) {

		@intokens = split (/\s+/, $arg);

		foreach $i (@intokens) {

			if ($i eq '') {		# in case of a typo of leading whitespace
				next;
			}

			if ($i =~ /^\-/) {
				$neg = 1;
				substr ($i, 0, 1) = '';
			} else {
				$neg = 0;
			}

			if (-r $i) {
				if ($neg == 1) {
					push (@contributors, "neg file $i");
				} else {
					push (@contributors, "file $i");
				}
				@thiscontrib = &readlistfile ($i);
			} elsif (-r "$main::hfhostlists/$i") {
				$hostlistfilename = "$main::hfhostlists/$i";
				if ($neg == 1) {
					push (@contributors, "neg file $hostlistfilename");
				} else {
					push (@contributors, "file $hostlistfilename");
				}
				@thiscontrib = &readlistfile ($hostlistfilename);
			} else {
				if ($neg == 1) {
					push (@contributors, "neg literal $i");
				} else {
					push (@contributors, "literal $i");
				}
				@thiscontrib = ($i);
			}

			if ($neg == 1) {
				foreach $j (@thiscontrib) {
					@tmpnetresult = grep {!($_ eq $j)} @netresult;
					@netresult = @tmpnetresult;
				}
			} else {
				push (@netresult, @thiscontrib);
			}
		}
	}

	foreach $i (@contributors) {
		warn "# STATUS hostnames came from: $i\n";
	}

	return (@netresult);
}

#  For oe and oel

sub readlistfile {

	my $hostlistfilename = shift;
	my @listres;
	my $line;

	if (!(open (HOSTLISTFILE, '<', $hostlistfilename))) {
		die "FATAL open '$hostlistfilename' failed with error '$!'";
	}
	while (<HOSTLISTFILE>) {
		$line = $_;
		chop ($line);
		if ($line =~ /^#/) {
			next;
		}
		push (@listres, $line);
	}
	close (HOSTLISTFILE);

	return (@listres);
}

#  For sl and hf-rearrange

sub bashencode {

# From man bash:
#
#	\a     alert (bell)
#	\b     backspace
#	\e     an escape character
#	\f     form feed
#	\n     new line
#	\r     carriage return
#	\t     horizontal tab
#	\v     vertical tab
#	\\     backslash
#	\'     single quote
#	\"     double quote
#	\cx    a control-x character
#	\nnn   the eight-bit character whose value is the octal value

	my $input = shift;

	my $result;
	my $i;
	my $byte;

	$result = '$' . "'";

	for ($i = 0;  $i < length ($input);  $i++) {

		$byte = ord (substr ($input, $i, 1));

		if ((($byte >= 65) && ($byte <= 90)) ||
			(($byte >= 97) && ($byte <= 122)) ||
			(($byte >= 48) && ($byte <= 57)) ||
			($byte == 46) ||
			($byte == 47) ||
			($byte == 95)) {
			$result .= chr ($byte);
		} elsif ($byte == 7) {
			$result .= '\a';
		} elsif ($byte == 8) {
			$result .= '\b';
		} elsif ($byte == 9) {
			$result .= '\t';
		} elsif ($byte == 10) {
			$result .= '\n';
		} elsif ($byte == 11) {
			$result .= '\v';
		} elsif ($byte == 12) {
			$result .= '\f';
		} elsif ($byte == 13) {
			$result .= '\r';
		} elsif ($byte == 27) {
			$result .= '\e';
		} elsif ($byte == 34) {
			$result .= '\\"';
		} elsif ($byte == 39) {
			$result .= '\\' . "'";
		} elsif ($byte == 92) {
			$result .= '\\\\';
		} elsif (($byte >= 1) && ($byte <= 26)) {
			$result .= '\\c' . chr ($byte + 64);
		} else {
			$result .= '\\' . sprintf ("%03o", $byte);
		}
	}

	$result .= "'";

	return ($result);
}

sub getterminalsize {

	my $sizeres;
	my @sizearr;

	if ((-t STDERR) && (-e '/dev/stderr') && (-r '/dev/stderr')) {
		$sizeres = `stty -F /dev/stderr size`;
		chop ($sizeres);
		@sizearr = split (' ', $sizeres);
		$HF::Globals::stderrcols = $sizearr[1];
	}
	if ($HF::Globals::stderrcols == 0) {
		$HF::Globals::stderrcols = 9999;
	}
	$HF::Globals::gotterminalsize = 1;
}

1;
