#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

package HF::Popen2way;
use strict;

use IO::Pipe;

sub new {

	my $proto = shift;
	my @cmdline = @_;

	my $class = ref ($proto) || $proto;

	my $pipetoremote;
	my $pipetolocal;
	my $pid;

	$pipetoremote = IO::Pipe->new;
	$pipetolocal = IO::Pipe->new;

	if ($pid = fork) {

		# this is the parent

		$pipetoremote->writer;
		$pipetolocal->reader;

# XXX does this close unwanted pipe ends?

		$pipetoremote->autoflush;
		$pipetolocal->autoflush;

		my $self = {
			'write'		=> $pipetoremote,
			'read'		=> $pipetolocal,
		};

		bless ($self, $class);
		return ($self);

	} elsif (defined ($pid)) {

		# this is the child

		my $reassignedstdin;
		my $reassignedstdout;

		$pipetolocal->writer;
		$pipetoremote->reader;

# XXX does this close unwanted pipe ends?

		$pipetolocal->autoflush;
		$pipetoremote->autoflush;

		$reassignedstdin = IO::Handle->new_from_fd (fileno (STDIN), '<');
		$reassignedstdout = IO::Handle->new_from_fd (fileno (STDOUT), '>');

		$reassignedstdin->fdopen ($pipetoremote->fileno, '<');
		$reassignedstdout->fdopen ($pipetolocal->fileno, '>');

		exec { $cmdline[0] } @cmdline;
		HF::Report->report (4, 1, "FATAL can't exec in new child, error '$!'");
	} else {
		HF::Report->report (4, 1, "FATAL can't fork, error '$!'");
	}
}

1;
