#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

package HF::Remote;
use strict;

use IO::Handle;

sub main {

	my $pwdresult;
	my $from;
	my $configstuff;
	my $harmonizeptr;
	my $dumper;
	my $reports;
	my $starttime;
	my $returnval;

	STDIN->autoflush;
	STDOUT->autoflush;

# -----

	$HF::Globals::islocal = 0;
	if (!(defined ($HF::Globals::dowrite))) {
		$HF::Globals::dowrite = 0;
	}
	if (!(defined ($HF::Globals::dodelete))) {
		$HF::Globals::dodelete = 0;
	}
	if (!(defined ($HF::Globals::dochecksums))) {
		$HF::Globals::dochecksums = 0;
	}
	if (!(defined ($HF::Globals::checkreadiness))) {
		$HF::Globals::checkreadiness = 0;
	}
	if (($HF::Globals::dodelete) && (!($HF::Globals::dowrite))) {
		HF::Report->report (3, 1, "FATAL option -d without option -w is not sensible");
	}
	if (!(defined ($HF::Globals::opt_x))) {
		$HF::Globals::opt_x = 0;
	}
	if (!(defined ($HF::Globals::opt_X))) {
		$HF::Globals::opt_X = 0;
	}
	if (!(defined ($HF::Globals::swappushpull))) {
		$HF::Globals::swappushpull = 0;
	}
	if (!(defined ($HF::Globals::swaplayers))) {
		$HF::Globals::swaplayers = 0;
	}
	if (!(defined ($HF::Globals::examine))) {
		$HF::Globals::examine = '';
	}
	if (!(defined ($HF::Globals::free_bytes))) {
		$HF::Globals::free_bytes = 0;
	}
	if (!(defined ($HF::Globals::free_inodes))) {
		$HF::Globals::free_inodes = 0;
	}
	if (!(defined ($HF::Globals::donotfixhardlinks))) {
		$HF::Globals::donotfixhardlinks = 0;
	}

# -----

	$starttime = time;

	&remote_status_housekeeping;

	$HF::Globals::hostname = `hostname 2>&1`;
	chop ($HF::Globals::hostname);
	# "$?" to fix emacs syntax highlighting
	if (("$?" >> 8) != 0) {
		HF::Report->report (2, 2, "FAILURE hostname returned '$pwdresult', error '$!'", 'fail_connection');
	}

	HF::Report->report (0, 4, "PROGRESS remote starting up, pid " . $$ . ", hostname " . $HF::Globals::hostname);
	HF::Report->report (0, 6, "PROGRESS remote hftop " .
		$HF::Globals::hftop . ", hg version " .
		$HF::Globals::hgversion);

	$pwdresult = `/bin/pwd 2>&1`;
	chop ($pwdresult);
	# "$?" to fix emacs syntax highlighting
	if (("$?" >> 8) != 0) {
		HF::Report->report (3, 1, "FATAL /bin/pwd failed with result '$pwdresult' , error '$!'");
	}
	# These Globals values have been set by command line switches
	HF::Report->report (0, 6, "PROGRESS remote " .
		"dowrite " . $HF::Globals::dowrite . ", " .
		"dodelete " . $HF::Globals::dodelete . ", " .
		"dochecksums " . $HF::Globals::dochecksums . ", " .
		"donotfixhardlinks " . $HF::Globals::donotfixhardlinks . ", " .
		"opt_x " . $HF::Globals::opt_x . ", " .
		"opt_X " . $HF::Globals::opt_X . ", " .
		"swappushpull " . $HF::Globals::swappushpull . ", " .
		"swaplayers " . $HF::Globals::swaplayers . ", " .
		"examine '" . $HF::Globals::examine . "', " .
		"free_bytes " . $HF::Globals::free_bytes . ", " .
		"free_inodes " . $HF::Globals::free_inodes . ", " .
		"pwd " . HF::Encode->encode ($pwdresult));

# -----

	&tolocal ('hfdist wire protocol 1.24.1');

# -----

	$from = &fromlocal;
	if ($from eq 'shutdown') {
		HF::Report->report (0, 4, "PROGRESS remote shutting down");
		exit (0);
	}

# -----

	$configstuff = HF::Encode->decode ($from);

	eval $configstuff;

	if ($@ ne '') {
		HF::Report->report (4, 1, "FATAL eval failed of configstuff with error '$@'
$configstuff
");
	}

	$HF::Globals::remote_report->[$HF::Globals::hostrow] = { };
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'time_begin'} = $starttime;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'pid'} = $$;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'hftop'} = $HF::Globals::hftop;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'hgversion'} = $HF::Globals::hgversion;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'dowrite'} = $HF::Globals::dowrite;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'dodelete'} = $HF::Globals::dodelete;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'dochecksums'} = $HF::Globals::dochecksums;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'opt_x'} = $HF::Globals::opt_x;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'opt_X'} = $HF::Globals::opt_X;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'donotfixhardlinks'} = $HF::Globals::donotfixhardlinks;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'checkreadiness'} = $HF::Globals::checkreadiness;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'swappushpull'} = $HF::Globals::swappushpull;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'swaplayers'} = $HF::Globals::swaplayers;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'examine'} = $HF::Globals::examine;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'free_bytes'} = $HF::Globals::free_bytes;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'free_inodes'} = $HF::Globals::free_inodes;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'outputpl'} = $HF::Globals::outputpl;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'outputplbig'} = $HF::Globals::outputplbig;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'pwd'} = $pwdresult;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'euid'} = $>;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_audit'} = 0;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_connection'} = 0;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_layers'} = 0;
	$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_total'} = 0;

	$HF::Globals::remote_errors->[$HF::Globals::hostrow] = [ ];

	#  For use by Harmonize; Remote already knows Local is at STDIN and STDOUT
	$HF::Globals::hosts->[$HF::Globals::hostrow]{'write'} = \*STDOUT;
	$HF::Globals::hosts->[$HF::Globals::hostrow]{'read'} = \*STDIN;

# -----

	if ($HF::Globals::hosts->[$HF::Globals::hostrow]{'direction'} eq 'pull') {

		$harmonizeptr = HF::Harmonize->new (1);
		$harmonizeptr->source_main;

		$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_total'} =
			$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_connection'} +
			$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_audit'} +
			$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_layers'};

		$HF::Globals::remote_report->[$HF::Globals::hostrow]{'time_end'} = time;

		$dumper = Data::Dumper->new ([
$HF::Globals::remote_report->[$HF::Globals::hostrow],
$HF::Globals::remote_errors->[$HF::Globals::hostrow],
$HF::Globals::source_report->[$HF::Globals::hostrow],
$HF::Globals::source_skips->[$HF::Globals::hostrow]
],[
"HF::Globals::remote_report->[$HF::Globals::hostrow]",
"HF::Globals::remote_errors->[$HF::Globals::hostrow]",
"HF::Globals::source_report->[$HF::Globals::hostrow]",
"HF::Globals::source_skips->[$HF::Globals::hostrow]"
]);
	} else {
		$harmonizeptr = HF::Harmonize->new (0);
		$returnval = $harmonizeptr->target_main;

		$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_total'} =
			$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_connection'} +
			$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_audit'} +
			$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_layers'};

		$HF::Globals::target_report->[$HF::Globals::hostrow]{'target_diff_rows'} =
			$#{$HF::Globals::target_differences->[$HF::Globals::hostrow]} + 1;

#		hfdist -O log -x 1 of source into empty directory
#		= 474,229 rows of sl in target
#		= 474,213 rows in target_differences
#		= 5701m in VIRT column of top listing
#		= 83.18 target_differences rows/Megabyte

		if (($#{$HF::Globals::target_differences->[$HF::Globals::hostrow]} + 1) > 85176) {	# est. 1 Gig of memory
			HF::Report->report (1, 2, "WARNING too many target_differences to report, zeroing");
			$HF::Globals::target_differences->[$HF::Globals::hostrow] = [ ];
			$HF::Globals::target_report->[$HF::Globals::hostrow]{'target_diff_zeroed'} = 1;
		}

		$HF::Globals::remote_report->[$HF::Globals::hostrow]{'time_end'} = time;

		$dumper = Data::Dumper->new ([
$HF::Globals::remote_report->[$HF::Globals::hostrow],
$HF::Globals::remote_errors->[$HF::Globals::hostrow],
$HF::Globals::target_report->[$HF::Globals::hostrow],
$HF::Globals::target_differences->[$HF::Globals::hostrow],
$HF::Globals::target_skips->[$HF::Globals::hostrow]
],[
"HF::Globals::remote_report->[$HF::Globals::hostrow]",
"HF::Globals::remote_errors->[$HF::Globals::hostrow]",
"HF::Globals::target_report->[$HF::Globals::hostrow]",
"HF::Globals::target_differences->[$HF::Globals::hostrow]",
"HF::Globals::target_skips->[$HF::Globals::hostrow]"
]);
	}

# -----

	# can use a lot of memory for target_differences
	$reports = $dumper->Dump;
	chop ($reports);
	HF::Report->report (0, 4, "PROGRESS sending reports and logs");
	&tolocal (HF::Encode->encode ($reports));

# -----

	if ((defined ($HF::Globals::remote_status)) && (-e $HF::Globals::remote_status)) {
		if (!(unlink ($HF::Globals::remote_status))) {
			HF::Report->report (2, 2, "FAILURE 7 can't unlink '" . $HF::Globals::remote_status . "', error '$!'");
		}
	}

	HF::Report->report (0, 4, "PROGRESS remote shutting down");
	&tolocal ('goodbye');

	return ($returnval);
}

sub remote_status_housekeeping {

	my $globwalk;
	my $walkhash;
	my $thepid;
	my $thename;

	$globwalk = HF::GLOBstream->new ('/tmp/hfdist_remote_status\.*', undef, 0);
	$globwalk->next;	# discard dot

	while ($walkhash = $globwalk->next, defined ($walkhash->{'inode'})) {

		$thepid = $walkhash->{'name'};
		$thepid =~ s/^.+\.(.+)$/\1/;
		$thename = '/tmp/' . $walkhash->{'name'};

		if (!(-e "/proc/$thepid")) {
			if (!(unlink ($thename))) {
				HF::Report->report (2, 2, "FAILURE 8 can't unlink '" . $thename . "', error '$!'");
			}
		}
	}
}

sub remote_status_write {

	my $statusline;
	my @structtm;

	$HF::Globals::remote_status = "/tmp/hfdist_remote_status.$$";
	if (!(open (HFREMOTESTATUS, ">", $HF::Globals::remote_status))) {
		HF::Report->report (2, 2, "FAILURE can't open status file '" . $HF::Globals::remote_status . "', error '$!'");
	}
	if ($HF::Globals::dowrite == 1) {
		$statusline .= "w";
	} else {
		$statusline .= " ";
	}
	if ($HF::Globals::dodelete == 1) {
		$statusline .= "d";
	} else {
		$statusline .= " ";
	}
	if ($HF::Globals::dochecksums == 1) {
		$statusline .= "k";
	} else {
		$statusline .= " ";
	}
	if ($HF::Globals::donotfixhardlinks == 1) {
		$statusline .= "L";
	} else {
		$statusline .= " ";
	}
	if ($HF::Globals::opt_x != 0) {
		$statusline .= "x";
	} else {
		$statusline .= " ";
	}
	if ($HF::Globals::opt_X != 0) {
		$statusline .= "X";
	} else {
		$statusline .= " ";
	}
	if ($HF::Globals::swappushpull == 1) {
		$statusline .= "r";
	} else {
		$statusline .= " ";
	}
	if ($HF::Globals::swaplayers == 1) {
		$statusline .= "s";
	} else {
		$statusline .= " ";
	}
	if ($HF::Globals::examine ne '') {
		$statusline .= "e";
	} else {
		$statusline .= " ";
	}
	$statusline = sprintf ("%s %s\n", scalar (getpwuid ($HF::Globals::remote_report->[$HF::Globals::hostrow]{'euid'})), $statusline);
	$statusline .= substr ($HF::Globals::source_layers->[0]{'mount'}[1], -14) . "\n";
	$statusline .= substr ($HF::Globals::target_layers->[0]{'mount'}[1], -14) . "\n";
	@structtm = localtime ($HF::Globals::remote_report->[$HF::Globals::hostrow]{'time_begin'});
	$statusline .= sprintf ("hr %2d %02d:%02d:%02d\n", $HF::Globals::hostrow, $structtm[2], $structtm[1], $structtm[0]);
	print HFREMOTESTATUS $statusline;
	close (HFREMOTESTATUS);

# XXX less activity to give lcds on demo Raspberry Pi's a chance to be written cleanly
	if (($HF::Globals::hostname eq 'pirogue') || ($HF::Globals::hostname eq 'pinnace')) {
		sleep (2);
	}
}

sub tolocal {

	my $arg = shift;

	if (!(print STDOUT $arg . "\n")) {
		HF::Report->report (2, 2, 'FAILURE write to local', 'fail_connection');
		return (undef);
	} else {
		return (1);
	}
}

sub fromlocal {

	my $inline;

	$inline = STDIN->getline;
	if (!(defined ($inline))) {
		HF::Report->report (3, 1, 'FATAL read from local, result is undef');
	}
	chop ($inline);

	return ($inline);
}

1;
