#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

package HF::Report;
use strict;

sub setup {

	$HF::Report::forparsefh = \*STDOUT;
	$HF::Report::progressfh = \*STDERR;
	$HF::Report::noticefh = \*STDERR;
	$HF::Report::failurefh = \*STDERR;
	$HF::Report::fatalfh = \*STDERR;

	&HF::Misc::getterminalsize;
}

sub report {

	my $self = shift;
	my $severity = shift;
	my $verbosity = shift;
	my $message = shift;
	my $category = shift;

	my $locrem;

	if ($HF::Globals::gotterminalsize == 0) {
		&HF::Misc::getterminalsize;
	}

	if ($HF::Globals::islocal == 1) {
		$locrem = 'loc';
	} elsif ($HF::Globals::islocal == 0) {
		$locrem = 'rem';
	} else {
		$locrem = '???';
	}

	if ($severity == 0) {
		if ($HF::Globals::verbosity >= $verbosity) {
			print $HF::Report::progressfh substr (sprintf ("sev %d %s | %s\n", $severity, $locrem, $message), 0, $HF::Globals::stderrcols);
		}
	} elsif ($severity == 1) {
		if ($HF::Globals::verbosity >= $verbosity) {
			print $HF::Report::noticefh substr (sprintf ("sev %d %s | %s\n", $severity, $locrem, $message), 0, $HF::Globals::stderrcols);
		}
	} elsif ($severity == 2) {
		if ($HF::Globals::verbosity >= $verbosity) {
			print $HF::Report::failurefh sprintf ("sev %d %s | %s\n", $severity, $locrem, $message);
		}
		if ($category eq '') {
			$category = 'sev2';
		}
		if ($HF::Globals::islocal == 0) {
			$HF::Globals::remote_report->[$HF::Globals::hostrow]{$category}++;
			if (defined ($HF::Globals::outputpl)) {
				push (@{$HF::Globals::remote_errors->[$HF::Globals::hostrow]}, $message);
			}
		} else {
			$HF::Globals::local_report->[$HF::Globals::hostrow]{$category}++;
			if (defined ($HF::Globals::outputpl)) {
				push (@{$HF::Globals::local_errors->[$HF::Globals::hostrow]}, $message);
			}
		}
	} elsif ($severity == 3) {
		if ($HF::Globals::verbosity >= $verbosity) {
			print $HF::Report::fatalfh sprintf ("sev %d %s | %s\n", $severity, $locrem, $message);
		}

		if ((defined ($HF::Globals::remote_status)) && (-e $HF::Globals::remote_status)) {
			if (!(unlink ($HF::Globals::remote_status))) {
				print $HF::Report::failurefh sprintf ("sev %d %s | %s\n", $severity, $locrem,
					"FAILURE 5 can't unlink '" . $HF::Globals::remote_status . "', error '$!'");
			}
		}

		exit (3);
	} else {
		if ($HF::Globals::verbosity >= $verbosity) {
			print $HF::Report::fatalfh sprintf ("sev: %d | %s
message: %s
package: %s
filename: %s
line: %s
subroutine: %s
hasargs: %s
wantarray: %s
evaltext: %s
is_require: %s
hints: %s
bitmask: %s
hinthash: %s
",
			$severity, $locrem, $message, caller (1));
		}

		if ((defined ($HF::Globals::remote_status)) && (-e $HF::Globals::remote_status)) {
			 if (!(unlink ($HF::Globals::remote_status))) {
				print $HF::Report::failurefh sprintf ("sev %d %s | %s\n", $severity, $locrem,
					"FAILURE 6 can't unlink '" . $HF::Globals::remote_status . "', error '$!'");
			}
		}

		exit (4);
	}
}

BEGIN {

	&setup;
}

1;
