#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

package HF::Statreport;
use strict;

sub new {

	my $proto = shift;
	my $startdir = shift;

	my $class = ref ($proto) || $proto;

	my $self = {

		'linkarray'	=> { },
		'startdir'	=> $startdir,
		'nomore'	=> 0,
		'firsttime'	=> 1
	};

	bless ($self, $class);

	return ($self);
}

sub standalone {

	my $self = shift;

	my $checkpointtime;
	my $now;
	my $resstring;
	my $walkhash;
	my $rescount;
	my @reslinks;
	my @res;

	$HF::Globals::islocal = 1;
	$HF::Globals::dowrite = 0;	# believed not referenced, but just to be sure
	if (!(defined ($HF::Globals::dochecksums))) {
		$HF::Globals::dochecksums = 0;
	}
	if (!(defined ($HF::Globals::verbosity))) {
		$HF::Globals::verbosity = 0;
	} else {
	       	if ($HF::Globals::verbosity =~ /^\d+$/) {
			$HF::Globals::verbosity += 0;
		} else {
			HF::Report->report (3, 1, "FATAL verbose (-v ?) arg in wrong format");
		}
	}
	if (!(defined ($HF::Globals::norecurse))) {
		$HF::Globals::norecurse = 0;
	}
	if (!(defined ($HF::Globals::mtimeonly))) {
		$HF::Globals::mtimeonly = 0;
	}
	if (!(defined ($HF::Globals::nameonly))) {
		$HF::Globals::nameonly = 0;
	}
	if (!(defined ($HF::Globals::printls))) {
		$HF::Globals::printls = 0;
	}
	if (!(defined ($HF::Globals::printheadings))) {
		$HF::Globals::printheadings = 0;
	}
	if (!(defined ($HF::Globals::printleadpath))) {
		$HF::Globals::printleadpath = 0;
	}
	if (!(defined ($HF::Globals::printroot))) {
		$HF::Globals::printroot = 0;
	}
	if (!(defined ($HF::Globals::bashquoting))) {
		$HF::Globals::bashquoting = 0;
	}
	if ($HF::Globals::mtimeonly == 1) {
		$HF::Globals::norecurse = 1;
	}

# -----

	if ($HF::Globals::opt_x == 2) {
		$self->{'walk'} = HF::FTWstream->new ($self->{'startdir'}, undef, $HF::Globals::dochecksums);
		if (-d $self->{'startdir'}) {
			$self->{'rootstring'} = $self->{'startdir'};
		} else {
			$self->{'rootstring'} = &HF::Misc::dirpart ($self->{'startdir'});
		}
	} elsif ($HF::Globals::opt_x == 3) {
		# 'startdir' contains the sl file name
		$self->{'walk'} = HF::STLstream->new ($self->{'startdir'}, undef, $HF::Globals::dochecksums);
		$self->{'rootstring'} = $self->{'walk'}->getroot;
	} elsif ($HF::Globals::opt_x == 4) {
		# 'startdir' contains the glob
		$self->{'walk'} = HF::GLOBstream->new ($self->{'startdir'}, undef, $HF::Globals::dochecksums);
		$self->{'rootstring'} = $self->{'walk'}->getroot;
	} elsif ($HF::Globals::opt_x == 5) {
		$self->{'walk'} = HF::NULLstream->new;
		if (-d $self->{'startdir'}) {
			$self->{'rootstring'} = $self->{'startdir'};
		} else {
			$self->{'rootstring'} = &HF::Misc::dirpart ($self->{'startdir'});
		}
	} else {
		$self->{'walk'} = HF::FTGroveler->new ($self->{'startdir'}, undef, $HF::Globals::dochecksums);
		if (-d $self->{'startdir'}) {
			$self->{'rootstring'} = $self->{'startdir'};
		} else {
			$self->{'rootstring'} = &HF::Misc::dirpart ($self->{'startdir'});
		}
	}

# -----

	if ($HF::Globals::printheadings == 1) {
		if ($HF::Globals::nameonly == 1) {
			$resstring = '# name';
		} elsif ($HF::Globals::mtimeonly == 1) {
			$resstring = '# mtime';
		} elsif ($HF::Globals::printls == 1) {
			$resstring = '# t prm links owner group    size/rdev      mtime name target';
			if ($HF::Globals::dochecksums == 1) {
				$resstring .= "/checksum";
			}
		} else {
			$resstring = sprintf ("%s\t%5s\t%5s\t%5s\t%5s\t%10s\t%12s\t%s\t%6s\t%s",
				'# type', 'perms', 'links', 'owner', 'group', 'mtime', 'size', 'target', 'rdev', 'name');
			if ($HF::Globals::dochecksums == 1) {
				$resstring .= "\tchecksum";
			}
		}
		print STDOUT ($resstring . "\n");
	}

	$checkpointtime = time + 1;
	$rescount = 0;

	$walkhash = $self->{'walk'}->next;
	# necessary to call $self->next once with last undef $walkhash, to retrieve linkreport
	while (@res = $self->next ($walkhash)) {
		$rescount++;
		print STDOUT (join ("\n", @res) . "\n");	# join multi-line linkreport
		if (defined ($walkhash->{'name'})) {
			$walkhash = $self->{'walk'}->next;
		}

		$now = time;
		if (($now > $checkpointtime) && ($HF::Globals::printls == 0)) {
			print STDERR "PROGRESS '" . (split (/\t/, $res[$#res]))[9] . "'\n";
			$checkpointtime = $now + 1;
		}
	}

	if (($rescount == 0) ||	# meaning that not even one entry was walked
		(($HF::Globals::local_report->[$HF::Globals::hostrow]{'fail_layers'} +
			$HF::Globals::remote_report->[$HF::Globals::hostrow]{'fail_layers'}) != 0)) {

		return (0);
	} else {
		return (1);
	}
}

sub next {

	my $self = shift;
	my $walkhash = shift;

	my @reslinks;
	my @res;

	if ($self->{'nomore'} == 1) {
		return ( );
	}

	@res = ( );
	if (($self->{'firsttime'} == 1) && ($HF::Globals::printroot == 1)) {
		push (@res, "root\t" . HF::Encode->encode ($self->{'rootstring'}));
		$self->{'firsttime'} = 0;
	}

	if (defined ($walkhash->{'name'})) {
		$self->registerlinks ($walkhash);
		push (@res, $self->hashtostring ($walkhash));

		if ($HF::Globals::norecurse == 1) {
			$self->{'nomore'} = 1;
		}

		return (@res);
	} else {
		$self->{'nomore'} = 1;
		if (($HF::Globals::norecurse == 0) && ($HF::Globals::printls == 0)) {
			@reslinks= $self->linkreport;
			if ($#reslinks > -1) {
				return (@reslinks);
			}
		}
		return ( );
	}

	HF::Report->report (4, 1, "FATAL should never reach here");
}

sub hashtostring {

	my $self = shift;
	my $input = shift;

	my $result;
	my $pathtoprint;
	my $targettoprint;

	if ($HF::Globals::bashquoting == 0) {
		if ($HF::Globals::printleadpath == 1) {
			$pathtoprint = $input->{'encpath'};
			$targettoprint = $input->{'enctarget'};
		} else {
			$pathtoprint = $input->{'encname'};
			$targettoprint = $input->{'enctarget'};
		}
	} else {
		if ($HF::Globals::printleadpath == 1) {
			$pathtoprint = &HF::Misc::bashencode ($input->{'path'});
			$targettoprint = &HF::Misc::bashencode ($input->{'target'});
		} else {
			$pathtoprint = &HF::Misc::bashencode ($input->{'name'});
			$targettoprint = &HF::Misc::bashencode ($input->{'target'});
		}
	}

	if ($HF::Globals::nameonly == 1) {
		$result = $pathtoprint;
	} elsif ($HF::Globals::mtimeonly == 1) {
		$result = $input->{'mtime'};
	} elsif ($HF::Globals::printls == 1) {

		if ($input->{'type'} == 0100000) {	# Regular file

			$result = sprintf ("%s %04o %5s %5s %5s %12s %10s %s",
				'-',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				$input->{'size'},
				$input->{'mtime'},
				$pathtoprint);

			if ($HF::Globals::dochecksums == 1) {
				$result .= " " . $input->{'checksum'};
			}

		} elsif ($input->{'type'} == 0040000) {	# Directory

			$result = sprintf ("%s %04o %5s %5s %5s %12s %10s %s",
				'd',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				"",
				$input->{'mtime'},
				$pathtoprint);

		} elsif ($input->{'type'} == 0120000) {	# Symbolic link

			$result = sprintf ("%s %4s %5s %5s %5s %12s %10s %s -> %s",
				'l',
				"",
				"",
				"",
				"",
				"",
				"",
				$pathtoprint,
				$targettoprint);

		} elsif ($input->{'type'} == 0020000) {	# Character device

			$result = sprintf ("%s %04o %5s %5s %5s %12s %10s %s",
				'c',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				$input->{'rdev'},
				$input->{'mtime'},
				$pathtoprint);

		} elsif ($input->{'type'} == 0060000) {	# Block device

			$result = sprintf ("%s %04o %5s %5s %5s %12s %10s %s",
				'b',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				$input->{'rdev'},
				$input->{'mtime'},
				$pathtoprint);

		} elsif ($input->{'type'} == 0010000) {	# FIFO

			$result = sprintf ("%s %04o %5s %5s %5s %12s %10s %s",
				'p',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				"",
				$input->{'mtime'},
				$pathtoprint);

		} elsif ($input->{'type'} == 0140000) {	# Socket

			$result = sprintf ("%s %04o %5s %5s %5s %12s %10s %s",
				's',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				"",
				$input->{'mtime'},
				$pathtoprint);

		} else {
			HF::Report->report (4, 1, "FATAL unknown type " . $input->{'type'});
		}
	} else {
		if ($input->{'type'} == 0100000) {	# Regular file

			$result = sprintf ("%s\t0%04o\t%5s\t%5s\t%5s\t%10s\t%12s\t%s\t%5s\t%s",
				'file',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				$input->{'mtime'},
				$input->{'size'},
				"",
				"",
				$pathtoprint);

			if ($HF::Globals::dochecksums == 1) {
				$result .= "\t" . $input->{'checksum'};
			}

		} elsif ($input->{'type'} == 0040000) {	# Directory

			$result = sprintf ("%s\t0%04o\t%5s\t%5s\t%5s\t%10s\t%12s\t%s\t%5s\t%s",
				'dir',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				$input->{'mtime'},
				"",
				"",
				"",
				$pathtoprint);
			if ($HF::Globals::dochecksums == 1) {
				$result .= "\t";
			}

		} elsif ($input->{'type'} == 0120000) {	# Symbolic link

			$result = sprintf ("%s\t%s\t%5s\t%5s\t%5s\t%10s\t%12s\t%s\t%5s\t%s",
				'symlk',
				"",
				"",
				"",
				"",
				"",
				"",
				$targettoprint,
				"",
				$pathtoprint);
			if ($HF::Globals::dochecksums == 1) {
				$result .= "\t";
			}

		} elsif ($input->{'type'} == 0020000) {	# Character device

			$result = sprintf ("%s\t0%04o\t%5s\t%5s\t%5s\t%10s\t%12s\t%s\t%5s\t%s",
				'char',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				$input->{'mtime'},
				"",
				"",
				$input->{'rdev'},
				$pathtoprint);
			if ($HF::Globals::dochecksums == 1) {
				$result .= "\t";
			}

		} elsif ($input->{'type'} == 0060000) {	# Block device

			$result = sprintf ("%s\t0%04o\t%5s\t%5s\t%5s\t%10s\t%12s\t%s\t%5s\t%s",
				'block',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				$input->{'mtime'},
				"",
				"",
				$input->{'rdev'},
				$pathtoprint);
			if ($HF::Globals::dochecksums == 1) {
				$result .= "\t";
			}

		} elsif ($input->{'type'} == 0010000) {	# FIFO

			$result = sprintf ("%s\t0%04o\t%5s\t%5s\t%5s\t%10s\t%12s\t%s\t%5s\t%s",
				'fifo',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				$input->{'mtime'},
				"",
				"",
				"",
				$pathtoprint);
			if ($HF::Globals::dochecksums == 1) {
				$result .= "\t";
			}

		} elsif ($input->{'type'} == 0140000) {	# Socket

			$result = sprintf ("%s\t0%04o\t%5s\t%5s\t%5s\t%10s\t%12s\t%s\t%5s\t%s",
				'sock',
				$input->{'perms'},
				$input->{'links'},
				$input->{'owner'},
				$input->{'group'},
				$input->{'mtime'},
				"",
				"",
				"",
				$pathtoprint);
			if ($HF::Globals::dochecksums == 1) {
				$result .= "\t";
			}

		} else {
			HF::Report->report (4, 1, "FATAL unknown type " . $input->{'type'});
		}
	}

	return ($result);
}

sub registerlinks {

	my $self = shift;
	my $input = shift;

	my $dev;
	my $ino;

	if (($input->{'links'} > 1) &&
		(($input->{'type'} == 0100000) ||
		 ($input->{'type'} == 0020000) ||
		 ($input->{'type'} == 0060000) ||
		 ($input->{'type'} == 0010000) ||
		 ($input->{'type'} == 0140000))) {

		$dev = $input->{'dev'};
		$ino = $input->{'inode'};

		if (exists ($self->{'linkarray'}{"$dev,$ino"})) {

			if ($input->{'links'} != $self->{'linkarray'}{"$dev,$ino"}[0]) {
				HF::Report->report (2, 2, "FAILURE 2 inconsistent link count for '" . $input->{'encname'} .
					"', was " . $self->{'linkarray'}{"$dev,$ino"}[0] .
					", now " . $input->{'links'}, 'fail_audit');
			}

			push (@{$self->{'linkarray'}{"$dev,$ino"}},
				($HF::Globals::bashquoting ? &HF::Misc::bashencode ($input->{'name'}) : $input->{'encname'}));
		} else {
			push (@{$self->{'linkarray'}{"$dev,$ino"}}, $input->{'links'},
				($HF::Globals::bashquoting ? &HF::Misc::bashencode ($input->{'name'}) : $input->{'encname'}));
		}
	}

	return (1);
}

sub linkreport {

	my $self = shift;

	my $key;
	my @tmp;
	my $links;
	my @linkreport;
	my $result;

	foreach $key (keys (%{$self->{'linkarray'}})) {

		@tmp = @{$self->{'linkarray'}{"$key"}};

		$links = shift (@tmp);
		@tmp = sort (@tmp);

		if ($links != ($#tmp + 1)) {
			HF::Report->report (2, 2, 'FAILURE inconsistent link count, found ' . $#tmp + 1 .
				" of $links hardlinks for filenames @tmp", 'fail_audit');
		}
		push (@linkreport, join ("\t", 'hardlinked', @tmp));
	}

	@linkreport = sort (@linkreport);

	if ($#linkreport > -1) {
		return (@linkreport);
	} else {
		return ( );
	}
}

1;
