#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#  Expected use of signals: "^C" of hfdist-oe on command line
trap '
trap - INT
CLEANEXIT=1
kill -s INT $$' INT

export HFGENDB=${HFGENDB:-@@@ EXAMPLEDB @@@}
HFTOP='/@@@ HFTOP @@@'
GENTOPINST="$HFTOP/gen"
GENVERDIR="hfgen.out/$HFGENDB"

# Doesn't use getops because hostspec negations collide with dashed
# option parsing

WRITE=''
DELETE=''
CHECKSUM=''
CHECK=''
VERBOSE=''
OPTX=''
JOBS='4'
EXAMINE=''

while :
do
	case "${1:-}" in
		-w)	WRITE='-w'
			shift
			;;
		-d)	DELETE='-d'
			shift
			;;
		-wd)	WRITE='-w'
			DELETE='-d'
			shift
			;;
		-k)	CHECKSUM='-k'
			shift
			;;
		-c)	CHECK='-c'
			shift
			;;
		-v)	VERBOSE="${2:-}"
			if [ -z "$VERBOSE" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				VERBOSE="-v $VERBOSE"
				shift 2
			fi
			;;
		-x)	OPTX="${2:-}"
			if [ -z "$OPTX" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				OPTX="-x $OPTX"
				shift 2
			fi
			;;
		-j)	JOBS="${2:-}"
			if [ -z "$JOBS" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				shift 2
			fi
			;;
		-e)	EXAMINE="-e examine -r -s"
			shift
			;;
		*)	break
			;;
	esac
done

if [ $# -eq 0 ]
then
	print_version
	echo 1>&2
	print_usage
	CLEANEXIT=1
	exit 0
fi

if [ $# -lt 3 ]
then
	print_usage
	CLEANEXIT=1
	exit 1
fi

SOURCE="$1"
shift
TARGET="$1"
shift
HOSTSPEC="$@"

#-----

#echo "NOTICE HFGENDB=$HFGENDB" 1>&2

# Want hfdist-oe to run both in hg repo and against installed location.
# So it looks for dir hfgen.out or symlink to it in the current dir,
# and if it doesn't find it, uses the one in the install location

if [ ! -d hfgen.out ]
then
#	echo "NOTICE hfgen.out is not in cd, using data in $GENTOPINST/$GENVERDIR" 1>&2
	GENROOT="$GENTOPINST/$GENVERDIR"
else
	GENROOT="$(pwd)/$GENVERDIR"
fi

#-----

HFDIST_OE_TMP="/tmp/hfdist-oe.$$"
{	hflayers 0 "$SOURCE" "$TARGET" | \
		oe -q -L -s -j $JOBS "$HOSTSPEC" \
		hfdist $WRITE $DELETE $CHECKSUM $CHECK $VERBOSE $OPTX $EXAMINE -a 1e6 -A 1e4 -o hfdist-oe.hflog -l - -h $GENROOT'/hfdist/$OE_HOST/real.hfhosts' \
		9> $HFDIST_OE_TMP
} > /dev/null 2>&1 &

#  Whole loop in braces to discard error from read
{	while ! read -r HFDIST_OE_TMPINSIDE < $HFDIST_OE_TMP
	do
		sleep 1
	done
} 2> /dev/null

oewatch $HFDIST_OE_TMPINSIDE

cd $HFDIST_OE_TMPINSIDE/hosts

echo "stdout and exits are:"
for i in $(/bin/ls -1 | sort -t. -k 1,1 -n -k 2,2)
do
	(echo $i ; cat $i/oe_stdout) | fmt | awk '{ printf ("%-15s  %-25s  %-25s  exit ", $1, $2, $3) }'
	cat $i/oe_exit
done

echo "bash commands for detailed logs:"
for i in $(/bin/ls -1 | sort -t. -k 1,1 -n -k 2,2)
do
	echo "hflog $HFDIST_OE_TMPINSIDE/hosts/$i/hfdist-oe.hflog | less"
done
echo "hflog $HFDIST_OE_TMPINSIDE/hosts/*/hfdist-oe.hflog | less
less $HFDIST_OE_TMPINSIDE/hosts/*/oe_stderr"
if [ -n "$EXAMINE" ]
then
	echo "sl -tr $HFDIST_OE_TMPINSIDE/hosts/*/examine | less"
fi
echo "To cleanup rm -r $HFDIST_OE_TMPINSIDE"

rm -f $HFDIST_OE_TMP
