#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

use POSIX qw(:signal_h :errno_h :sys_wait_h);

sub winchhandler {
	$HF::Globals::gotterminalsize = 0;
}
$SIG{WINCH} = \&winchhandler;

$HF::Globals::hftop = '/@@@ HFTOP @@@';
$HF::Globals::hgversion = '@@@ HG VERSION @@@';

if ($#ARGV == -1) {
	&print_version;
	warn ("\n");
	&print_usage;
	exit (0);
}

if (!(getopts ('wdkLrcSsHv:h:l:o:O:t:x:X:R:P:a:A:e:'))) {
	&print_usage;
	exit (2);
}

if (!(defined ($main::opt_v))) {
	$HF::Globals::verbosity = 4;
} else {
       	if ($main::opt_v =~ /^\d+$/) {
		$HF::Globals::verbosity = $main::opt_v;
	} else {
		HF::Report->report (3, 1, "FATAL option -v arg must be numeric, quitting");
	}
}
if (defined ($main::opt_x)) {
	$HF::Globals::stderrcols = 9999;
}

$HF::Globals::dowrite = $main::opt_w;
$HF::Globals::dodelete = $main::opt_d;
$HF::Globals::dochecksums = $main::opt_k;
$HF::Globals::donotfixhardlinks = $main::opt_L;
$HF::Globals::checkreadiness = $main::opt_c;
$HF::Globals::outputpl = $main::opt_o;
$HF::Globals::outputplbig = $main::opt_O;
$HF::Globals::opt_x = $main::opt_x;
$HF::Globals::opt_X = $main::opt_X;
$HF::Globals::remotecmd = $main::opt_R;
$HF::Globals::swappushpull = $main::opt_r;
$HF::Globals::swaplayers = $main::opt_s;
$HF::Globals::examine = $main::opt_e;
$HF::Globals::free_bytes = $main::opt_a;
$HF::Globals::free_inodes = $main::opt_A;

# -----

if ($main::opt_S == 0) {

	# LOCAL side

# -----

	my $source_path;
	my $target_path;
	my @host_list;
	my $i;
	my $direction;
	my $treatment;
	my $config;
	my $sshopts;
	my $returnval;

	# Layers

	if (defined ($main::opt_l)) {
		&HF::Layers::eval_file_to_tmp ($main::opt_l);
	} else {
		$source_path = shift (@ARGV);
		$target_path = shift (@ARGV);

		if (length ($source_path) == 0) {
			warn "FATAL need cmd line arg for path of source-dir, quitting\n";
			&print_usage;
			exit (2);
		}
		if (length ($target_path) == 0) {
			warn "FATAL need cmd line arg for path of target-dir, quitting\n";
			&print_usage;
			exit (2);
		}

		# remove trailing slash from autocomplete, but permit single slash
		if (length ($source_path) >= 2) {
			$source_path =~ s|/$||;
		}
		if (length ($target_path) >= 2) {
			$target_path =~ s|/$||;
		}

		if ($main::opt_t) {
			$treatment = $main::opt_t;
		} else {
			$treatment = 0;
		}

		$config = {

'source_layers' => [
	{
		'mount' => [ '', $source_path, 'all' ],
		'treatments' => [ [ $treatment ] ]
	},
],

'target_layers' => [
	{
		'mount' => [ '', $target_path, 'all' ],
		'treatments' => [ [ 0 ] ]
	}
]

};
		&HF::Layers::pointer_to_tmp_layers ($config);
	}

	# Hosts

	if ((defined ($main::opt_h)) && (defined ($main::opt_H))) {
		warn "FATAL options -h and -H together are not sensible, quitting\n";
		&print_usage;
		exit (2);
	}

	if (defined ($main::opt_h)) {
		&HF::Hosts::eval_file_to_tmp ($main::opt_h);
	} elsif (defined ($main::opt_H)) {
		$HF::Tmp::hosts = { 'hosts' => [ { 'channel' => 'null' } ] };
	} else {
		@host_list = @ARGV;

		if ($#host_list == -1) {
			$HF::Tmp::hosts = { 'hosts' => [ { 'channel' => 'fork' } ] };
		} else {
			$HF::Tmp::hosts = { 'hosts' => [ ] };

			foreach $i (@host_list) {

				$direction = "push";

				# ip address
				if ($i =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) {
					push (@{$HF::Tmp::hosts->{'hosts'}}, { 'channel' => 'ssh', 'ip' => $i, 'direction' => $direction });
					if ($main::opt_P) {
						$HF::Tmp::hosts->{'hosts'}[$#{$HF::Tmp::hosts->{'hosts'}}]{'sshopts'} = eval ($main::opt_P);
					}
				# hostname
				} else {
					push (@{$HF::Tmp::hosts->{'hosts'}}, { 'channel' => 'ssh', 'hostname' => $i, 'direction' => $direction });
					if ($main::opt_P) {
						$HF::Tmp::hosts->{'hosts'}[$#{$HF::Tmp::hosts->{'hosts'}}]{'sshopts'} = eval ($main::opt_P);
					}
				}
			}
		}
	}

# -----

	&HF::Hosts::parse_tmp_to_globals;
	HF::Report->report (0, 18, "PROGRESS local hosts
" . &HF::Hosts::display);

	&HF::Layers::parse_tmp_to_globals;
	HF::Report->report (0, 18, "PROGRESS local layers
" . &HF::Layers::display);

	$returnval = &HF::Local::main;
	exit ($returnval);

} else {

	# REMOTE side

	my $returnval;

	$returnval = &HF::Remote::main;
	exit ($returnval);
}

HF::Report->report (4, 0, "FATAL 1 should never reach here");
