#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

use Data::Dumper;

$HF::Globals::verbosity = 3;

# -----

my $tailres;
my $result;
my $i;
my $dumper;
my @tail;
my $hasstdin;
my $filecontents;
my $evalcode;
my $cmd;
my $dodump = 0;
my $usedstdin = 0;

if ($#ARGV == -1) {
	&print_version;
	warn ("\n");
	&print_usage;
	exit (0);
}

$tailres = &split_command_tail (@ARGV);
if ($#{$tailres} == -1) {
	warn "FATAL no command given\n\n";
	&print_usage;
	exit (1);
}

if ((-t) || (eof STDIN)) {
	$hasstdin = 0;
} else {
	$hasstdin = 1;
}
$result = {
	'source_layers' => [ ],
	'target_layers' => [ ]
};

foreach $i (@{$tailres}) {
	@tail = @{$i};

	if ($#tail == -1) {
		warn "FATAL no command given\n\n";
		&print_usage;
		exit (1);
	}

	$cmd = shift (@tail);
	if ($cmd eq 'cat') {
		if ($hasstdin == 0) {
			warn "FATAL needs layers on stdin\n\n";
			&print_usage;
			exit (1);
		}
		&do_cat;
		$usedstdin = 1;
	} elsif ($cmd eq 'wantbelow') {
		if ($hasstdin == 0) {
			warn "FATAL needs layers on stdin\n\n";
			&print_usage;
			exit (1);
		}
		&do_wantonlyorbelow (1, @tail);
		$usedstdin = 1;
	} elsif ($cmd eq 'wantonly') {
		if ($hasstdin == 0) {
			warn "FATAL needs layers on stdin\n\n";
			&print_usage;
			exit (1);
		}
		&do_wantonlyorbelow (0, @tail);
		$usedstdin = 1;
	} elsif ($cmd eq 'wantskip') {
		if ($hasstdin == 0) {
			warn "FATAL needs layers on stdin\n\n";
			&print_usage;
			exit (1);
		}
		&do_wantskip (@tail);
		$usedstdin = 1;
	} else {
		if (($hasstdin == 1) && ($usedstdin == 0)) {
			{
				local ($/);
				$filecontents = <STDIN>;
			}
			$evalcode = '$HF::Tmp::layers =
' . $filecontents . ';

1;';
			eval $evalcode;
			if ($@ ne '') {
				warn "FATAL eval error of stdin, message is '$@'\n";
				exit (1);
			}
			$result = $HF::Tmp::layers;
		} else {

		}
		$usedstdin = 1;
		$dodump = 1;

		if ($cmd eq 'merge') {
			$result = &do_merge ($result, @tail);
		} elsif ($cmd eq 'eval') {
			if ($hasstdin == 0) {
				warn "FATAL needs layers on stdin\n\n";
				&print_usage;
				exit (1);
			}
		} elsif ($cmd eq '0') {
			$result = &do_0 ($result, @tail);
		} elsif ($cmd eq 'skip') {
			$result = &do_skip ($result, @tail);
		} elsif ($cmd eq 'skipsource') {
			$result = &do_skipsource ($result, @tail);
		} elsif ($cmd eq 'skiptarget') {
			$result = &do_skiptarget ($result, @tail);
		} elsif ($cmd eq 'copyone') {
			$result = &do_copyone ($result, @tail);
		} elsif ($cmd eq 'copyonemd') {
			$result = &do_copyonemd ($result, @tail);
		} elsif ($cmd eq 'both109') {
			$result = &do_both109 ($result, @tail);
		} elsif ($cmd eq 'ignoremtime') {
			$result = &do_ignoremtime ($result, @tail);
		} elsif ($cmd eq 'mount') {
			$result = &do_mount ($result, @tail);
		} elsif ($cmd eq 'overlay') {
			$result = &do_overlay ($result, @tail);
		} elsif ($cmd eq 'multidrop') {
			$result = &do_multidrop ($result, @tail);
		} elsif ($cmd eq 'glob') {
			$result = &do_glob ($result, @tail);
		} elsif ($cmd eq 'globnull') {
			$result = &do_globnull ($result, @tail);
		} elsif ($cmd eq 'sl') {
			$result = &do_sl ($result, @tail);
		} elsif ($cmd eq 'null') {
			$result = &do_null ($result, @tail);
		} elsif ($cmd eq 'perl') {
			$result = &do_perl ($result, @tail);
		} else {
			warn "FATAL unknown command '$cmd'\n\n";
			&print_usage;
			exit (1);
		}
	}
}

if ($dodump == 1) {
	$dumper = Data::Dumper->new ([ $result ]);
	$dumper->Terse (1);
	$dumper->Sortkeys (1);
	print ($dumper->Dump);
}

exit (0);

# -----

sub split_command_tail {

	my @tail = @_;
	my $tailindex = 0;
	my $tailarray = [ ];
	my $entry;

	while ($#tail > -1) {
		$entry = shift (@tail);
		if ($entry eq ';') {
			$tailindex++;
		} else {
			push (@{$tailarray->[$tailindex]}, $entry);
		}
	}
	return ($tailarray);
}

sub do_wantonlyorbelow {

	my $wantbelow = shift;
	my @sourcepathnames = @_;

	my $filetreeskip;
	my @quotedsourcepathnames;
	my $i;

	if ($#sourcepathnames == -1) {
		warn "FATAL no source pathname given\n\n";
		&print_usage;
		exit (1);
	}

	for ($i = 0;  $i <= $#sourcepathnames;  $i++) {
		$quotedsourcepathnames[$i] = $sourcepathnames[$i];
		if ($quotedsourcepathnames[$i] =~ /['\\]/) {
			$quotedsourcepathnames[$i] =~ s/(['\\])/\\$1/g;
		}
	}

	while (<STDIN>) {
		print $_;
	}

	$filetreeskip = &HF::Misc::regex_for_skip_all_but_this_path ($wantbelow, @sourcepathnames);

	if ($wantbelow == 1) {
		print q|;

# Generated by hflayers wantbelow '| . join ("' '", @sourcepathnames) . "'";
	} else {
		print q|;

# Generated by hflayers wantonly '| . join ("' '", @sourcepathnames) . "'";
	}

	print q|

my $i;
my $j;
my $k;
my @newskips;
my $newskipregex;
my $thisskipregex;
my $dowarn = 0;

@newskips = ('| . join ("', '", @quotedsourcepathnames) . q|');
$newskipregex = &HF::Misc::regex_for_skip_all_but_this_path (| . $wantbelow . q|, @newskips);

foreach $k ('source', 'target') {
	if ($dowarn) { warn "_${k} newskipregex $newskipregex\n"; }
	for ($i = 0;  $i <= $#{$HF::Tmp::layers->{"${k}_layers"}};  $i++) {
		if ($dowarn) { warn "_${k} i $i mount '" . $HF::Tmp::layers->{"${k}_layers"}[$i]{'mount'}[0] . "'\n"; }
		if ($HF::Tmp::layers->{"${k}_layers"}[$i]{'mount'}[0] eq '') {
			if ($dowarn) { warn "_${k} root match\n"; }
			unshift (@{$HF::Tmp::layers->{"${k}_layers"}[$i]{'skips'}}, $newskipregex);
		} elsif ($HF::Tmp::layers->{"${k}_layers"}[$i]{'mount'}[0] =~ /$newskipregex/) {
			if ($dowarn) { warn "_${k} DELETE_THIS_LAYER\n"; }
			$HF::Tmp::layers->{"${k}_layers"}[$i]{'mount'}[2] = 'DELETE_THIS_LAYER';
		} else {
			if ($dowarn) { warn "_${k} else, filter rule not for use as-is; will not delete layer, upper part good\n"; }
			for ($j = 0;  $j <= $#newskips;  $j++) {
				if ($dowarn) { warn "_${k} j " . $j . " newskips_j " . $newskips[$j] . "\n"; }
				$thisskipregex = &HF::Misc::regex_for_skip_all_but_this_path (| . $wantbelow . q|, $newskips[$j]);
				if ($dowarn) { warn "_${k} layers " . $HF::Tmp::layers->{"${k}_layers"}[$i]{'mount'}[0] . " thisskipregex " . $thisskipregex . "\n"; }
				if ($HF::Tmp::layers->{"${k}_layers"}[$i]{'mount'}[0] !~ /$thisskipregex/) {
					if ($dowarn) { warn "_${k} contained in regex\n"; }
					if (length ($newskips[$j]) > length ($HF::Tmp::layers->{"${k}_layers"}[$i]{'mount'}[0])) {
						if ($dowarn) { warn "_${k} unshift substr " .
							&HF::Misc::regex_for_skip_all_but_this_path (| . $wantbelow . q|, substr ($newskips[$j], length ($HF::Tmp::layers->{"${k}_layers"}[$i]{'mount'}[0]) + 1)) . "\n"; }
						unshift (@{$HF::Tmp::layers->{"${k}_layers"}[$i]{'skips'}},
							&HF::Misc::regex_for_skip_all_but_this_path (| . $wantbelow . q|, substr ($newskips[$j], length ($HF::Tmp::layers->{"${k}_layers"}[$i]{'mount'}[0]) + 1)));
					} else {
						if ($dowarn) { warn "_${k} do not unshift\n"; }
					}
				} else {
					if ($dowarn) { warn "_${k} not contained in regex\n"; }
				}
			}
		}
	}
}
|;
}

sub do_wantskip {

	my @skips = @_;

	my $i;
	my @quotedskips;

	if ($#skips == -1) {
		warn "FATAL no skips given\n\n";
		&print_usage;
		exit (1);
	}

	for ($i = 0;  $i <= $#skips;  $i++) {
		$quotedskips[$i] = $skips[$i];
		if ($quotedskips[$i] =~ /['\\]/) {
			$quotedskips[$i] =~ s/(['\\])/\\$1/g;
		}
	}

	while (<STDIN>) {
		print $_;
	}

	print q|;

# Generated by hflayers wantskip | . join (' ', @skips) . q|

my $i;
my $j;
my @result;

my @skips = ('| . join ("', '", @quotedskips) . q|');

for ($i = 0;  $i <= $#{$HF::Tmp::layers->{'source_layers'}};  $i++) {
	for ($j = 0;  $j <= $#skips;  $j++) {
		@result = &HF::Misc::path_is_in ($HF::Tmp::layers->{'source_layers'}[$i]{'mount'}[0], $skips[$j]);
		if ($result[0] == 1) {
			unshift (@{$HF::Tmp::layers->{'source_layers'}[$i]{'skips'}}, '^' . $result[1] . '$');
		} elsif ($result[0] == 2) {
			$HF::Tmp::layers->{'source_layers'}[$i]{'mount'}[2] = 'DELETE_THIS_LAYER';
		}
	}
}

for ($i = 0;  $i <= $#{$HF::Tmp::layers->{'target_layers'}};  $i++) {
	for ($j = 0;  $j <= $#skips;  $j++) {
		@result = &HF::Misc::path_is_in ($HF::Tmp::layers->{'target_layers'}[$i]{'mount'}[0], $skips[$j]);
		if ($result[0] == 1) {
			unshift (@{$HF::Tmp::layers->{'target_layers'}[$i]{'skips'}}, '^' . $result[1] . '$');
		} elsif ($result[0] == 2) {
			$HF::Tmp::layers->{'target_layers'}[$i]{'mount'}[2] = 'DELETE_THIS_LAYER';
		}
	}
}
|;
}

sub do_merge {

	my $result = shift;
	my @args = @_;
	my $i;

	if ($#args == -1) {
		warn "FATAL no merge filenames given\n\n";
		&print_usage;
		exit (1);
	}

	foreach $i (@args) {
		&HF::Layers::eval_file_to_tmp ($i);
		push (@{$result->{'source_layers'}}, @{$HF::Tmp::layers->{'source_layers'}});
		push (@{$result->{'target_layers'}}, @{$HF::Tmp::layers->{'target_layers'}});
	}

	return ($result);
}

sub do_cat {

	while (<STDIN>) {
		print $_;
	}
}

sub do_0 {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = shift;
	my @args = @_;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#args != -1)) {
		warn "FATAL no source path given, no target path given, or superfluous args\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers 0 ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ '', $sourcepathname, 'all' ],
		'treatments' => [ [0] ]
	});
	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers 0 ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ '', $targetpathname, 'all' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_skip {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = shift;
	my @skiparray = @_;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#skiparray == -1)) {
		warn "FATAL no source path given, no target path given, or no skips given\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers skip ' . $sourcepathname . ' ' . $targetpathname . ' ' . join (' ', @skiparray),
		'mount' => [ '', $sourcepathname, 'all' ],
		'skips' => [ map ('^' . $_ . '$', @skiparray) ],
		'treatments' => [ [0] ]
	});
	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers skip ' . $sourcepathname . ' ' . $targetpathname . ' ' . join (' ', @skiparray),
		'mount' => [ '', $targetpathname, 'all' ],
		'skips' => [ map ('^' . $_ . '$', @skiparray) ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_skipsource {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = shift;
	my @skiparray = @_;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#skiparray == -1)) {
		warn "FATAL no source path given, no target path given, or no skips given\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers skipsource ' . $sourcepathname . ' ' . $targetpathname . ' ' . join (' ', @skiparray),
		'mount' => [ '', $sourcepathname, 'all' ],
		'skips' => [ map ('^' . $_ . '$', @skiparray) ],
		'treatments' => [ [0] ]
	});
	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers skipsource ' . $sourcepathname . ' ' . $targetpathname . ' ' . join (' ', @skiparray),
		'mount' => [ '', $targetpathname, 'all' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_skiptarget {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = shift;
	my @skiparray = @_;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#skiparray == -1)) {
		warn "FATAL no source path given, no target path given, or no skips given\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers skiptarget ' . $sourcepathname . ' ' . $targetpathname . ' ' . join (' ', @skiparray),
		'mount' => [ '', $sourcepathname, 'all' ],
		'treatments' => [ [0] ]
	});
	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers skiptarget ' . $sourcepathname . ' ' . $targetpathname . ' ' . join (' ', @skiparray),
		'mount' => [ '', $targetpathname, 'all' ],
		'skips' => [ map ('^' . $_ . '$', @skiparray) ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_copyone {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = shift;
	my @args = @_;
	my $sourcepathdirname;
	my $sourcepathbasename;
	my $targetpathdirname;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#args != -1)) {
		warn "FATAL no source path given, no target path given, or superfluous args\n\n";
		&print_usage;
		exit (1);
	}

	$sourcepathdirname = &HF::Misc::dirpart ($sourcepathname);
	$sourcepathbasename = &HF::Misc::basepart ($sourcepathname);
	$targetpathdirname = &HF::Misc::dirpart ($targetpathname);

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers copyone ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ '', $sourcepathdirname, 'all' ],
		'skips' => [ '^(?!' . $sourcepathbasename . '$)' ],
		'treatments' => [ [0] ]
	});
	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers copyone ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ '', $targetpathdirname, 'all' ],
		'skips' => [ '^(?!' . $sourcepathbasename . '$)' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_copyonemd {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = shift;
	my @args = @_;
	my $sourcepathdirname;
	my $sourcepathbasename;
	my $targetpathdirname;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#args != -1)) {
		warn "FATAL no source path given, no target path given, or superfluous args\n\n";
		&print_usage;
		exit (1);
	}

	$sourcepathdirname = &HF::Misc::dirpart ($sourcepathname);
	$sourcepathbasename = &HF::Misc::basepart ($sourcepathname);
	$targetpathdirname = &HF::Misc::dirpart ($targetpathname);

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers copyonemd ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ $sourcepathdirname, $sourcepathdirname, 'all' ],
		'skips' => [ '^(?!' . $sourcepathbasename . '$)' ],
		'filters' => [ ['owner', 0], ['group', 0] ],
		'treatments' => [ [102], [0, ['^' . $sourcepathname . '$']] ]
	});
	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers copyonemd ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ $sourcepathdirname, $targetpathdirname, 'all' ],
		'skips' => [ '^(?!' . $sourcepathbasename . '$)' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_both109 {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = shift;
	my @args = @_;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#args != -1)) {
		warn "FATAL no source path given, no target path given, or superfluous args\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers both109 ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ '', $sourcepathname, 'all' ],
		'treatments' => [ [109] ]
	});
	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers both109 ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ '', $targetpathname, 'all' ],
		'treatments' => [ [109] ]
	});

	return ($result);
}

sub do_ignoremtime {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = shift;
	my @args = @_;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#args != -1)) {
		warn "FATAL no source path given, no target path given, or superfluous args\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers ignoremtime ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ '', $sourcepathname, 'all' ],
		'treatments' => [ [0] ]
	});
	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers ignoremtime ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ '', $sourcepathname, 'all' ],
		'treatments' => [ [113] ]
	});
	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers ignoremtime ' . $sourcepathname . ' ' . $targetpathname,
		'mount' => [ '', $targetpathname, 'all' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_mount {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = shift;
	my @args = @_;
	my $i;
	my $noroot;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#args == -1)) {
		warn "FATAL no source path given, no target path given, or no args given\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers mount ' . $sourcepathname . ' ' . $targetpathname . ' ' . join (' ', @args),
		'mount' => [ '', $sourcepathname, 'all' ],
		'treatments' => [ [0] ]
	});

	foreach $i (@args) {

		if ($i eq '/') {
			$noroot = '';
		} elsif ($i =~ /^\//) {
			$noroot = substr ($i, 1);
		} else {
			$noroot = $i;
		}

		push (@{$result->{'source_layers'}}, {
			'comment' => 'Generated by hflayers mount ' . $sourcepathname . ' ' . $targetpathname . ' ' . join (' ', @args),
			'mount' => [ $noroot, $i, 'all' ],
			'treatments' => [ [0] ]
		});
	}

	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers mount ' . $sourcepathname . ' ' . $targetpathname . ' ' . join (' ', @args),
		'mount' => [ '', $targetpathname, 'all' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_overlay {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = pop;
	my @args = @_;
	my $i;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#args == -1)) {
		warn "FATAL no source path given, no target path given, or no args given\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers overlay ' . $sourcepathname . ' ' . join (' ', @args) . ' ' . $targetpathname,
		'mount' => [ '', $sourcepathname, 'all' ],
		'treatments' => [ [0] ]
	});

	foreach $i (@args) {
		push (@{$result->{'source_layers'}}, {
			'comment' => 'Generated by hflayers overlay ' . $sourcepathname . ' ' . join (' ', @args) . ' ' . $targetpathname,
			'mount' => [ '', $i, 'all' ],
			'treatments' => [ [0] ]
		});
	}

	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers overlay ' . $sourcepathname . ' ' . join (' ', @args) . ' ' . $targetpathname,
		'mount' => [ '', $targetpathname, 'all' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_multidrop {

	my $result = shift;
	my $sourcepathname = shift;
	my $targetpathname = pop;
	my @args = @_;
	my $i;

	if (($sourcepathname eq '') || ($targetpathname eq '') || ($#args == -1)) {
		warn "FATAL no source path given, no target path given, or superfluous args\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers multidrop ' . $sourcepathname . ' ' . join (' ', @args) . ' ' . $targetpathname,
		'mount' => [ '', $sourcepathname, 'topdir' ],
		'treatments' => [ [102] ]
	});

	foreach $i (@args) {
		push (@{$result->{'source_layers'}}, {
			'comment' => 'Generated by hflayers multidrop ' . $sourcepathname . ' ' . join (' ', @args) . ' ' . $targetpathname,
			'mount' => [ $i, &HF::Misc::joinpaths ($sourcepathname, $i), 'all' ],
			'treatments' => [ [0] ]
		});
	}

	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers multidrop ' . $sourcepathname . ' ' . join (' ', @args) . ' ' . $targetpathname,
		'mount' => [ '', $targetpathname, 'topdir' ],
		'treatments' => [ [0] ]
	});

	foreach $i (@args) {
		push (@{$result->{'target_layers'}}, {
			'comment' => 'Generated by hflayers multidrop ' . $sourcepathname . ' ' . join (' ', @args) . ' ' . $targetpathname,
			'mount' => [ $i, &HF::Misc::joinpaths ($targetpathname, $i), 'all' ],
			'treatments' => [ [0] ]
		});
	}

	return ($result);
}

sub do_glob {

	my $result = shift;
	my $targetpathname = pop;
	my @args = @_;
	my $i;

	if (($targetpathname eq '') || ($#args == -1)) {
		warn "FATAL no target path given, or no args given\n\n";
		&print_usage;
		exit (1);
	}

	foreach $i (@args) {
		push (@{$result->{'source_layers'}}, {
			'comment' => 'Generated by hflayers glob ' . join (' ', @args) . ' ' . $targetpathname,
			'mount' => [ '', $i, 'glob' ],
			'treatments' => [ [0] ]
		});
	}

	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers glob ' . join (' ', @args) . ' ' . $targetpathname,
		'mount' => [ '', $targetpathname, 'all' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_globnull {

	my $result = shift;
	my @args = @_;
	my $i;

	if ($#args == -1) {
		warn "FATAL no args given\n\n";
		&print_usage;
		exit (1);
	}

	foreach $i (@args) {
		push (@{$result->{'source_layers'}}, {
			'comment' => 'Generated by hflayers globnull ' . join (' ', @args),
			'mount' => [ '', $i, 'glob' ],
			'treatments' => [ [0] ]
		});
	}

	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers globnull ' . join (' ', @args),
		'mount' => [ '', '', 'null' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_sl {

	my $result = shift;
	my $slfile = shift;
	my $targetpathname = shift;
	my @args = @_;

	if (($slfile eq '') || ($targetpathname eq '') || ($#args != -1)) {
		warn "FATAL no statlist file given, no target path given, or superfluous args\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers sl ' . $slfile . ' ' . $targetpathname,
		'mount' => [ '', $slfile, 'sl' ],
		'treatments' => [ [0] ]
	});
	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers sl ' . $slfile . ' ' . $targetpathname,
		'mount' => [ '', $targetpathname, 'all' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_null {

	my $result = shift;
	my $sourcepathname = shift;
	my @args = @_;

	if (($sourcepathname eq '') || ($#args != -1)) {
		warn "FATAL no source path given, or superfluous args\n\n";
		&print_usage;
		exit (1);
	}

	push (@{$result->{'source_layers'}}, {
		'comment' => 'Generated by hflayers null ' . $sourcepathname,
		'mount' => [ '', $sourcepathname, 'all' ],
		'treatments' => [ [0] ]
	});
	push (@{$result->{'target_layers'}}, {
		'comment' => 'Generated by hflayers null ' . $sourcepathname,
		'mount' => [ '', '', 'null' ],
		'treatments' => [ [0] ]
	});

	return ($result);
}

sub do_perl {

	my $result = shift;
	my $evalcode = shift;
	my @args = @_;

	if (($evalcode eq '') || ($#args != -1)) {
		warn "FATAL no perl code given, or superfluous args\n\n";
		&print_usage;
		exit (1);
	}

	eval $evalcode;
	if ($@ ne '') {
		warn "FATAL eval error of stdin, message is '$@'\n";
		exit (1);
	}

	return ($result);
}
