#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

$HF::Globals::verbosity = 3;

if (!(getopts ('dltrqh:fnNsvcC:xuUHLebT'))) {
	&print_usage;
	exit (1);
}

# -----

my $logfilename;
my $host;
my $banner;
my $haderrors;
my $hostlow;
my $hosthigh;

if ($#ARGV == -1) {
	&print_version;
	warn ("\n");
	&print_usage;
	exit (0);
}

foreach $logfilename (@ARGV) {

	if (!(-r $logfilename)) {
		die "FATAL log file '$logfilename' not readable\n";
	}

	if ($main::opt_n && $main::opt_N) {
		die "FATAL -n and -N cannot be used together\n";
	}

	if ($main::opt_u && $main::opt_U) {
		die "FATAL -u and -U cannot be used together\n";
	}

	if ($main::opt_r && $main::opt_u) {
		die "FATAL -r and -u cannot be used together\n";
	}

	if ($main::opt_r && $main::opt_U) {
		die "FATAL -r and -U cannot be used together\n";
	}

	if ($main::opt_r && $main::opt_x) {
		die "FATAL -r and -x cannot be used together\n";
	}

	if ($main::opt_T && $main::opt_x) {
		die "FATAL -T and -x cannot be used together\n";
	}

	if ($main::opt_u && $main::opt_x) {
		die "FATAL -u and -x cannot be used together\n";
	}

	if ($main::opt_U && $main::opt_x) {
		die "FATAL -U and -x cannot be used together\n";
	}

	if ($main::opt_U && $main::opt_T) {
		die "FATAL -U and -T cannot be used together\n";
	}

	if ($main::opt_U) {
		$main::opt_q = 1;
	}

	if ($main::opt_C ne '') {
		if (!(-r $main::opt_C)) {
			die "FATAL additional log file '$main::opt_C' not readable\n";
		} else {
			&eval_file_to_tmp ($main::opt_C);
			$HF::Tmp::hflog2 = $HF::Tmp::hflog;
		}
	}

	&eval_file_to_tmp ($logfilename);

	if ($#{$HF::Tmp::hflog->{'local_report'}} == -1) {

		die "FATAL no local_report log sections\n";
	}

	if ($main::opt_h ne '') {
		if ($main::opt_h < 1) {
			die "FATAL host numbers start at one\n";
		}
		if (($main::opt_h - 1) > $#{$HF::Tmp::hflog->{'local_report'}}) {
			die "FATAL not that many hosts\n";
		}
		$hostlow = $main::opt_h - 1;
		$hosthigh = $main::opt_h - 1;
	} else {
		$hostlow = 0;
		$hosthigh = $#{$HF::Tmp::hflog->{'local_report'}};
	}

	if (($main::opt_c || $main::opt_C) && (($#{$HF::Tmp::hflog->{'local_report'}} > 0) && (!($main::opt_h)))) {
		die "FATAL multiple hosts are incompatible with -c|-C\n";
	}

	if (!($main::opt_d || $main::opt_u || $main::opt_U || $main::opt_f || $main::opt_n ||
		$main::opt_N || $main::opt_s || $main::opt_v || $main::opt_c || $main::opt_C ||
		$main::opt_x || $main::opt_H || $main::opt_L || $main::opt_e)) {

		if (($HF::Tmp::hflog->{'local_report'}[0]{'opt_x'} != 0) ||
			($HF::Tmp::hflog->{'remote_report'}[0]{'opt_x'} != 0)) {

			$main::opt_x = 1;
		} else {
			$main::opt_d = 1;
		}
	}

	if (($main::opt_c || $main::opt_C) ||
		($main::opt_x && (($#{$HF::Tmp::hflog->{'local_report'}} == 0) || $main::opt_h))) {

		$main::opt_q = 1;
	}

	if (!($main::opt_q)) {
		print "# ---------- logfilename '$logfilename'\n";
	}

	$haderrors = 0;
	for ($host = $hostlow;  $host <= $hosthigh;  $host++) {

		if (!($main::opt_q)) {
			print &host_banner ($host) . "\n";
		}

		if ($main::opt_d || $main::opt_u || $main::opt_U) {
			&do_target_differences ($host, $main::opt_l, $main::opt_t, $main::opt_r, $main::opt_u, $main::opt_U) || ($haderrors = 1);
		} elsif ($main::opt_f) {
			&do_filecontents ($host) || ($haderrors = 1);
		} elsif ($main::opt_n || $main::opt_N) {
			&do_numbers ($host, $main::opt_N) || ($haderrors = 1);
		} elsif ($main::opt_s) {
			&do_skips ($host) || ($haderrors = 1);
		} elsif ($main::opt_v) {
			&do_versions ($host) || ($haderrors = 1);
		} elsif ($main::opt_c || $main::opt_C) {
			&do_rearrange ($host) || ($haderrors = 1);
		} elsif ($main::opt_x) {
			&do_opt_x ($host) || ($haderrors = 1);
		} elsif ($main::opt_H) {
			&do_hostslayers ($host, 0) || ($haderrors = 1);
		} elsif ($main::opt_L) {
			&do_hostslayers ($host, 1) || ($haderrors = 1);
		} elsif ($main::opt_e) {
			&do_errors ($host) || ($haderrors = 1);
		} else {
			die "FATAL should not reach here";
		}
	}
}

if ($haderrors) {
	exit (1);
} else {
	exit (0);
}

# -----

sub do_filecontents {

	my $host = shift;

	my @fileschanged;
	my $banner;
	my $key;
	my $size;
	my $encname;
	my @sortedlist;

	if ($#{$HF::Tmp::hflog->{'target_differences'}} == -1) {
		print "ERROR no target_differences log sections, skipping\n";
		return (0);
	}
	if ($HF::Tmp::hflog->{'target_report'}[$host]{'target_diff_zeroed'} == 1) {
		print "ERROR target difference log of " .
			&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'target_diff_rows'}) .
			" rows was deleted for extreme size, skipping\n";
		return (0);
	}

	@fileschanged = ();
	foreach $key (@{$HF::Tmp::hflog->{'target_differences'}[$host]}) {

		if (($key->[1] eq 'create') && ($key->[3] ne undef) && ($key->[3]{'type'} == 0100000)) {
			push (@fileschanged, [ $key->[1], '', $key->[3]{'size'}, '', $key->[0] ]);
		} elsif (($key->[1] eq 'delete') && ($key->[4] ne undef) && ($key->[4]{'type'} == 0100000)) {
			push (@fileschanged, [ $key->[1], '', '', $key->[4]{'size'}, $key->[0] ]);
		} elsif (($key->[1] eq 'replace') && ($key->[3] ne undef) && ($key->[3]{'type'} == 0100000)) {
			push (@fileschanged, [ $key->[1], $key->[2], $key->[3]{'size'}, $key->[4]{'size'}, $key->[0] ]);
		}
	}
	
	if ($#fileschanged > -1) {
		@sortedlist = sort {$b->[2] <=> $a->[2]} @fileschanged;

		if (!($main::opt_q)) {
			print "# what   why       size             have             name
# -----  --------  ---------------  ---------------  ----\n";
		}

		foreach $key (@sortedlist) {
			if (!($main::opt_b)) {
				$encname = HF::Encode->encode ($key->[4]);
			} else {
				$encname = &HF::Misc::bashencode ($key->[4]);
			}
			if (!($main::opt_q)) {
				printf ("%-7s  %-8s  %15s  %15s  %s\n", $key->[0], $key->[1], &HF::Misc::commify_number ($key->[2]), &HF::Misc::commify_number ($key->[3]), $encname);
			} else {
				printf ("%s\n", $encname);
			}
		}
	} else {
		if (!($main::opt_q)) {
			print "No file content changes.\n";
		}
	}

	return (1);
}

sub do_numbers {

	my $host = shift;
	my $shortreport = shift;

	my $timedelta;
	my @timesubstract;
	my $timedeltaprint;
	my $src_outline;
	my $tgt_outline;
	my $src_bytes_g;
	my $tgt_bytes_g;
	my $percent;
	my $src_count_g;
	my $tgt_count_g;
	my $src_diff_total_scaled;
	my $tgt_diff_total_scaled;

	if (!($shortreport)) {
		printf ("source_root '%s' target_root '%s'\n",
			$HF::Tmp::hflog->{'source_layers'}[$host]{'mount'}[1],
			$HF::Tmp::hflog->{'target_layers'}[$host]{'mount'}[1]);
		printf ("dowrite %d dodelete %d dochecksums %d donotfixhardlinks %d opt_x %d euid %d\n",
			$HF::Tmp::hflog->{'local_report'}[$host]{'dowrite'},
			$HF::Tmp::hflog->{'local_report'}[$host]{'dodelete'},
			$HF::Tmp::hflog->{'local_report'}[$host]{'dochecksums'},
			$HF::Tmp::hflog->{'local_report'}[$host]{'donotfixhardlinks'},
			$HF::Tmp::hflog->{'local_report'}[$host]{'opt_x'},
			$HF::Tmp::hflog->{'local_report'}[$host]{'euid'});
		printf ("fail_no_remote_answer %d fail_wrong_remote_version %d checkreadiness %d fail_connection %d\n\n",
			$HF::Tmp::hflog->{'local_report'}[$host]{'fail_no_remote_answer'},
			$HF::Tmp::hflog->{'local_report'}[$host]{'fail_wrong_remote_version'},
			$HF::Tmp::hflog->{'local_report'}[$host]{'checkreadiness'},
			$HF::Tmp::hflog->{'local_report'}[$host]{'fail_connection'});

		$timedelta = $HF::Tmp::hflog->{'local_report'}[$host]{'time_end'} - $HF::Tmp::hflog->{'local_report'}[$host]{'time_begin'};
		if ($timedelta < (3600 * 24)) {
			@timesubstract = gmtime ($timedelta);
			$timedeltaprint = sprintf ("%02d:%02d:%02d", $timesubstract[2], $timesubstract[1], $timesubstract[0]);
		} else {
			$timedeltaprint = sprintf ("days %3.2f", $timedelta / (3600.0 * 24));
		}

		printf ("time_begin %10d %-24s\ntime_end   %10d %-24s  total %s  hours %3.2f  mins %5.2f  secs %6d\n",
			$HF::Tmp::hflog->{'local_report'}[$host]{'time_begin'},
			scalar (localtime ($HF::Tmp::hflog->{'local_report'}[$host]{'time_begin'})),
			$HF::Tmp::hflog->{'local_report'}[$host]{'time_end'},
			scalar (localtime ($HF::Tmp::hflog->{'local_report'}[$host]{'time_end'})),
			$timedeltaprint,
			($timedelta) / 3600.0,
			($timedelta) / 60.0,
			$timedelta);
	}

	if (($HF::Tmp::hflog->{'source_report'}[$host]{'seen_all'} != 0) || ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_all'} != 0)) {

		if ($shortreport) {
			printf ("want_byte_sent %-17s  want_file %-9s\n",
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'want_sent_f_bytes'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'want_sent_f'}));
		} else {
			print "\n             file       dir   symlink      pipe   chardev    blkdev    socket       all    file_byte_seen\n";
			printf ("source %10s %9s %9s %9s %9s %9s %9s %9s %17s\n",
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'seen_f'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'seen_d'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'seen_l'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'seen_p'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'seen_c'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'seen_b'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'seen_s'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'seen_all'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'seen_f_bytes'}));
			printf ("target %10s %9s %9s %9s %9s %9s %9s %9s %17s\n",
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_f'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_d'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_l'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_p'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_c'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_b'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_s'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_all'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_f_bytes'}));

			$src_outline = &st_bargraph (98, 'source_report', $host);
			$tgt_outline = &st_bargraph (98, 'target_report', $host);
			print "source $src_outline\ntarget $tgt_outline\n\n";

			print "   want_byte_sent want_file      file_byte_sent file_sent   fixlink dir_mtime  diff_tot  fail_tot\n";
			printf ("%17s %9s   %17s %9s %9s %9s %9s %9s\n",
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'want_sent_f_bytes'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'want_sent_f'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'sent_f_bytes'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'source_report'}[$host]{'sent_f'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'fix_hardlinks'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'fix_dir_mtimes'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'diff_total'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'local_report'}[$host]{'fail_total'} +
					$HF::Tmp::hflog->{'remote_report'}[$host]{'fail_total'}));

			if ($HF::Tmp::hflog->{'source_report'}[$host]{'seen_f_bytes'} > 0) {
				$percent = int ($HF::Tmp::hflog->{'source_report'}[$host]{'sent_f_bytes'} / $HF::Tmp::hflog->{'source_report'}[$host]{'seen_f_bytes'});
				if ($percent > 1) {
					$percent = 1;
				}
				$src_bytes_g = 'b' x int (($percent * 98) + 0.5);
			} else {
				$src_bytes_g = '';
			}
			if ($HF::Tmp::hflog->{'target_report'}[$host]{'seen_f_bytes'} > 0) {
				$percent = int ($HF::Tmp::hflog->{'target_report'}[$host]{'want_sent_f_bytes'} / $HF::Tmp::hflog->{'target_report'}[$host]{'seen_f_bytes'});
				if ($percent > 1) {
					$percent = 1;
				}
				$tgt_bytes_g = 'b' x int (($percent * 98) + 0.5);
			} else {
				$tgt_bytes_g = '';
			}
			if ($HF::Tmp::hflog->{'target_report'}[$host]{'diff_total'} > $HF::Tmp::hflog->{'source_report'}[$host]{'seen_all'}) {
				$src_diff_total_scaled = $HF::Tmp::hflog->{'source_report'}[$host]{'seen_all'};
			} else {
				$src_diff_total_scaled = $HF::Tmp::hflog->{'target_report'}[$host]{'diff_total'};
			}
			if ($src_diff_total_scaled > 0) {
				$src_count_g = 'c' x (int (($src_diff_total_scaled / $HF::Tmp::hflog->{'source_report'}[$host]{'seen_all'}) * 98) + 0.5);
			} else {
				$src_count_g = '';
			}
			if ($HF::Tmp::hflog->{'target_report'}[$host]{'diff_total'} > $HF::Tmp::hflog->{'target_report'}[$host]{'seen_all'}) {
				$tgt_diff_total_scaled = $HF::Tmp::hflog->{'target_report'}[$host]{'seen_all'};
			} else {
				$tgt_diff_total_scaled = $HF::Tmp::hflog->{'target_report'}[$host]{'diff_total'};
			}
			if ($tgt_diff_total_scaled > 0) {
				$tgt_count_g = 'c' x (int (($tgt_diff_total_scaled / $HF::Tmp::hflog->{'target_report'}[$host]{'seen_all'}) * 98) + 0.5);
			} else {
				$tgt_count_g = '';
			}

			print "source " . &overlay_strings ($src_count_g, $src_bytes_g) . "\n";
			print "target " . &overlay_strings ($tgt_count_g, $tgt_bytes_g) . "\n\n";

			print "diffs create     modify     reload     delete | fails  connection      audit     layers\n";
			printf ("  %10s %10s %10s %10s | local  %10s %10s %10s\n",
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'diff_create'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'diff_modify'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'diff_reload'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'diff_delete'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'local_report'}[$host]{'fail_connection'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'local_report'}[$host]{'fail_audit'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'local_report'}[$host]{'fail_layers'}));

			$tgt_outline = &cmrd_bargraph (45, 'target_report', $host);
			printf ("%-45s | remote %10s %10s %10s\n",
				$tgt_outline,
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'remote_report'}[$host]{'fail_connection'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'remote_report'}[$host]{'fail_audit'}),
				&HF::Misc::commify_number ($HF::Tmp::hflog->{'remote_report'}[$host]{'fail_layers'}));
		}
	}

	return (1);
}

sub st_bargraph {

	my $bar_width = shift;
	my $whichend = shift;
	my $host = shift;

	my $outline = '';
	my @tosortarr;
	my @sortedarr;
	my @displayarr;
	my $unroundedbarwidth = 0;
	my $i;

	# Adjust the largest bar segment so the bar width is exactly right despite errors from quantitizating to letter width
	if ($HF::Tmp::hflog->{$whichend}[$host]{'seen_all'} > 0) {

		@tosortarr = (
			[ 0, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'seen_f'} / $HF::Tmp::hflog->{$whichend}[$host]{'seen_all'}) * $bar_width) + 0.5)) ],
			[ 1, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'seen_d'} / $HF::Tmp::hflog->{$whichend}[$host]{'seen_all'}) * $bar_width) + 0.5)) ],
			[ 2, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'seen_l'} / $HF::Tmp::hflog->{$whichend}[$host]{'seen_all'}) * $bar_width) + 0.5)) ],
			[ 3, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'seen_c'} / $HF::Tmp::hflog->{$whichend}[$host]{'seen_all'}) * $bar_width) + 0.5)) ],
			[ 4, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'seen_b'} / $HF::Tmp::hflog->{$whichend}[$host]{'seen_all'}) * $bar_width) + 0.5)) ],
			[ 5, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'seen_p'} / $HF::Tmp::hflog->{$whichend}[$host]{'seen_all'}) * $bar_width) + 0.5)) ],
			[ 6, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'seen_s'} / $HF::Tmp::hflog->{$whichend}[$host]{'seen_all'}) * $bar_width) + 0.5)) ]);

		# Index 0 bar segment is largest
		@sortedarr = (sort {$b->[1] <=> $a->[1]} @tosortarr);

		# Compute unadjusted bar width
		for ($i = 0;  $i <= $#sortedarr;  $i++) {
			$unroundedbarwidth += $sortedarr[$i]->[1];
		}

		# Adjust largest bar's width
		$sortedarr[0]->[1] = $sortedarr[0]->[1] - ($unroundedbarwidth - $bar_width);

		# Put bar segments back in display order
		@displayarr = (sort {$a->[0] <=> $b->[0]} @sortedarr);

		$outline  = '-' x @displayarr[0]->[1];
		$outline .= 'd' x @displayarr[1]->[1];
		$outline .= 'l' x @displayarr[2]->[1];
		$outline .= 'c' x @displayarr[3]->[1];
		$outline .= 'b' x @displayarr[4]->[1];
		$outline .= 'p' x @displayarr[5]->[1];
		$outline .= 's' x @displayarr[6]->[1];
	}
	return ($outline);
}

sub cmrd_bargraph {

	my $bar_width = shift;
	my $whichend = shift;
	my $host = shift;

	my $outline = '';
	my @tosortarr;
	my @sortedarr;
	my @displayarr;
	my $unroundedbarwidth = 0;
	my $i;

	# Adjust the largest bar segment so the bar width is exactly right despite errors from quantitizating to letter width
	if ($HF::Tmp::hflog->{$whichend}[$host]{'diff_total'} > 0) {

		@tosortarr = (
			[ 0, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'diff_create'} / $HF::Tmp::hflog->{$whichend}[$host]{'diff_total'}) * $bar_width) + 0.5)) ],
			[ 1, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'diff_modify'} / $HF::Tmp::hflog->{$whichend}[$host]{'diff_total'}) * $bar_width) + 0.5)) ],
			[ 2, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'diff_reload'} / $HF::Tmp::hflog->{$whichend}[$host]{'diff_total'}) * $bar_width) + 0.5)) ],
			[ 3, (int ((($HF::Tmp::hflog->{$whichend}[$host]{'diff_delete'} / $HF::Tmp::hflog->{$whichend}[$host]{'diff_total'}) * $bar_width) + 0.5)) ]);

		# Index 0 bar segment is largest
		@sortedarr = (sort {$b->[1] <=> $a->[1]} @tosortarr);

		# Compute unadjusted bar width
		for ($i = 0;  $i <= $#sortedarr;  $i++) {
			$unroundedbarwidth += $sortedarr[$i]->[1];
		}

		# Adjust largest bar's width
		$sortedarr[0]->[1] = $sortedarr[0]->[1] - ($unroundedbarwidth - $bar_width);

		# Put bar segments back in display order
		@displayarr = (sort {$a->[0] <=> $b->[0]} @sortedarr);

		$outline  = 'c' x @displayarr[0]->[1];
		$outline .= 'm' x @displayarr[1]->[1];
		$outline .= 'r' x @displayarr[2]->[1];
		$outline .= 'd' x @displayarr[3]->[1];
	}
	return ($outline);
}

sub overlay_strings {

	my $s1 = shift;
	my $s2 = shift;

	my $overlay;

	if (length ($s1) > length ($s2)) {
		$overlay = $s1;
		substr ($overlay, 0, length ($s2)) = $s2;
	} else {
		$overlay = $s2;
		substr ($overlay, 0, length ($s1)) = $s1;
	}

	return ($overlay);
}

sub do_skips {

	my $host = shift;

	my $name;
	my $encname;

	if ($#{$HF::Tmp::hflog->{'source_skips'}} == -1) {
		print "ERROR no source_skips log section, skipping\n";
	} else {
		if ($#{$HF::Tmp::hflog->{'source_skips'}[$host]} > -1) {
			if (!($main::opt_q)) {
				print "# source name\n";
				print "# -----------\n";
			}
			foreach $name (@{$HF::Tmp::hflog->{'source_skips'}[$host]}) {
				if (!($main::opt_b)) {
					$encname = HF::Encode->encode ($name);
				} else {
					$encname = &HF::Misc::bashencode ($name);
				}
				print $encname . "\n";
			}
			if (!($main::opt_q)) {
				print "\n";
			}
		} else {
			if (!($main::opt_q)) {
				print "No source skips.\n";
			}
		}
	}
	if ($#{$HF::Tmp::hflog->{'target_skips'}} == -1) {
		print "ERROR no target_skips log section, skipping\n";
	} else {
		if ($#{$HF::Tmp::hflog->{'target_skips'}[$host]} > -1) {
			if (!($main::opt_q)) {
				print "# target name\n";
				print "# -----------\n";
			}
			foreach $name (@{$HF::Tmp::hflog->{'target_skips'}[$host]}) {
				print HF::Encode->encode ($name) . "\n";
			}
		} else {
			if (!($main::opt_q)) {
				print "No target skips.\n";
			}
		}
	}

	return (1);
}

sub do_errors {

	my $host = shift;

	my $error;

	if ($#{$HF::Tmp::hflog->{'local_errors'}} == -1) {
		print "ERROR no local_errors log section\n";
	} else {
		if ($#{$HF::Tmp::hflog->{'local_errors'}[$host]} > -1) {
			if (!($main::opt_q)) {
				print "# local\n";
				print "# -----\n";
			}
			foreach $error (@{$HF::Tmp::hflog->{'local_errors'}[$host]}) {
				print "$error\n";
			}
			if (!($main::opt_q)) {
				print "\n";
			}
		} else {
			if (!($main::opt_q)) {
				print "No local errors.\n";
			}
		}
	}
	if ($#{$HF::Tmp::hflog->{'remote_errors'}} == -1) {
		print "ERROR no remote_errors log section\n";
	} else {
		if ($#{$HF::Tmp::hflog->{'remote_errors'}[$host]} > -1) {
			if (!($main::opt_q)) {
				print "# remote\n";
				print "# ------\n";
			}
			foreach $error (@{$HF::Tmp::hflog->{'remote_errors'}[$host]}) {
				print "$error\n";
			}
			if (!($main::opt_q)) {
				print "\n";
			}
		} else {
			if (!($main::opt_q)) {
				print "No remote errors.\n";
			}
		}
	}

	return (1);
}

sub do_versions {

	my $host = shift;

	printf ("local  hftop %s hgversion %s
remote hftop %s hgversion %s\n",
		$HF::Tmp::hflog->{'local_report'}[$host]{'hftop'},
		$HF::Tmp::hflog->{'local_report'}[$host]{'hgversion'},
		$HF::Tmp::hflog->{'remote_report'}[$host]{'hftop'},
		$HF::Tmp::hflog->{'remote_report'}[$host]{'hgversion'});

	return (1);
}

sub do_opt_x {

	my $host = shift;

	my $arrptr;
	my $line;
	my $sl;
	my $walkhash;
	my @res;

	if ($HF::Tmp::hflog->{'local_report'}[$host]{'opt_x'} eq undef) {
		print "ERROR opt_x unset on source, skipping\n";
		return (0);
	}

	if ($HF::Tmp::hflog->{'remote_report'}[$host]{'opt_x'} eq undef) {
		print "ERROR opt_x unset on target, skipping\n";
		return (0);
	}

	if ($HF::Tmp::hflog->{'local_report'}[$host]{'opt_x'} != $HF::Tmp::hflog->{'remote_report'}[$host]{'opt_x'}) {
		print "ERROR opt_x setting does not match between source and target, skipping\n";
		return (0);
	}

	if ((exists ($HF::Tmp::hflog->{'source_report'}[$host]{'opt_x_output'})) && (exists ($HF::Tmp::hflog->{'target_report'}[$host]{'opt_x_output'}))) {
		print "ERROR opt_x_output present for both source and target, skipping\n";
		return (0);
	}

	if ((!(exists ($HF::Tmp::hflog->{'source_report'}[$host]{'opt_x_output'}))) && (!(exists ($HF::Tmp::hflog->{'target_report'}[$host]{'opt_x_output'})))) {
		print "ERROR opt_x_output missing for both source and target, skipping\n";
		return (0);
	}

	if ($HF::Tmp::hflog->{'local_report'}[$host]{'opt_x'} == 22) {
		$arrptr = $HF::Tmp::hflog->{'source_report'}[$host]{'opt_x_output'};
		for ($line = 0;  $line <= $#{$arrptr};  $line++) {
			print $arrptr->[$line] . "\n";
		}
	} elsif ($HF::Tmp::hflog->{'local_report'}[$host]{'opt_x'} == 32) {
		$arrptr = $HF::Tmp::hflog->{'target_report'}[$host]{'opt_x_output'};
		for ($line = 0;  $line <= $#{$arrptr};  $line++) {
			print $arrptr->[$line] . "\n";
		}
	} elsif ($HF::Tmp::hflog->{'local_report'}[$host]{'opt_x'} == 21) {
		$sl = HF::Statreport->new ('.');  # dir is dummy value, there is no root line to display it
		$arrptr = $HF::Tmp::hflog->{'source_report'}[$host]{'opt_x_output'};
		for ($line = 0;  $line <= $#{$arrptr};  $line++) {
			@res = $sl->next ($arrptr->[$line][3]);
			print (join ("\n", @res) . "\n");
		}
		@res = $sl->next (undef);
		if (@res) {
			print (join ("\n", @res) . "\n");
		}
	} elsif ($HF::Tmp::hflog->{'local_report'}[$host]{'opt_x'} == 31) {
		$sl = HF::Statreport->new ('.');  # dir is dummy value, there is no root line to display it
		$arrptr = $HF::Tmp::hflog->{'target_report'}[$host]{'opt_x_output'};
		for ($line = 0;  $line <= $#{$arrptr};  $line++) {
		       @res = $sl->next ($arrptr->[$line][3]);
			print (join ("\n", @res) . "\n");
		}
		@res = $sl->next (undef);
		if (@res) {
			print (join ("\n", @res) . "\n");
		}
	} else {
		print "ERROR unknown opt_x setting '" . $HF::Tmp::hflog->{'local_report'}[$host]{'opt_x'} . "', skipping\n";
		return (0);
	}

	return (1);
}

sub do_hostslayers {

	my $host = shift;
	my $wantlayers = shift;

	my $dumper;
	my $all;

	if ($wantlayers == 1) {
		$all = {
			'layers_file'	=> $HF::Tmp::hflog->{'layers_file'},
			'source_layers'	=> $HF::Tmp::hflog->{'source_layers'},
			'target_layers'	=> $HF::Tmp::hflog->{'target_layers'}
		};
	} else {
		if ($main::opt_h ne '') {
			$all = $HF::Tmp::hflog->{'hosts'}[$main::opt_h];
		} else {
			$all = {
				'hosts_file'	=> $HF::Tmp::hflog->{'hosts_file'},
				'hosts'		=> $HF::Tmp::hflog->{'hosts'}
			};
		}
	}
	$dumper = Data::Dumper->new ([$all]);
	$dumper->Terse (1);
	print $dumper->Dump;

	return (1);
}

#  From the man page for rsync  version 3.0.9  protocol version 30
#
#  The "%i" escape has a cryptic output that is  11  letters  long.
#  The  general  format  is like the string YXcstpoguax, where Y is
#  replaced by the type of update being done, X is replaced by  the
#  file-type,  and  the other letters represent attributes that may
#  be output if they are being modified.
#
#  The update types that replace the Y are as follows:
#
#  o      A < means that a file is being transferred to the  remote
#         host (sent).
#
#  o      A  >  means that a file is being transferred to the local
#         host (received).
#
#  o      A c means that a local change/creation is  occurring  for
#         the  item  (such  as  the  creation of a directory or the
#         changing of a symlink, etc.).
#
#  o      A h means that the item is a hard link  to  another  item
#         (requires --hard-links).
#
#  o      A  .  means that the item is not being updated (though it
#         might have attributes that are being modified).
#
#  o      A * means that the rest of the itemized-output area  con‐
#         tains a message (e.g. "deleting").
#
#  The  file-types  that replace the X are: f for a file, a d for a
#  directory, an L for a symlink, a D for a device, and a S  for  a
#  special file (e.g. named sockets and fifos).
#
#  The  other  letters  in  the string above are the actual letters
#  that will be output if the associated attribute for the item  is
#  being  updated or a "." for no change.  Three exceptions to this
#  are: (1) a newly created item replaces each letter with  a  "+",
#  (2)  an identical item replaces the dots with spaces, and (3) an
#  unknown attribute replaces each letter with a "?" (this can hap‐
#  pen when talking to an older rsync).
#
#  The attribute that is associated with each letter is as follows:
#
#  o      A  c  means  either  that  a regular file has a different
#         checksum (requires --checksum) or that a symlink, device,
#         or  special  file  has a changed value.  Note that if you
#         are sending files to an rsync prior to 3.0.1, this change
#         flag  will be present only for checksum-differing regular
#         files.
#
#  o      A s means the size of a regular  file  is  different  and
#         will be updated by the file transfer.
#
#  o      A t means the modification time is different and is being
#         updated to the sender’s  value  (requires  --times).   An
#         alternate  value  of  T  means that the modification time
#         will be set to the transfer time, which  happens  when  a
#         file/symlink/device is updated without --times and when a
#         symlink is changed and the receiver can’t set  its  time.
#         (Note:  when  using  an rsync 3.0.0 client, you might see
#         the s flag combined with t instead of the proper  T  flag
#         for this time-setting failure.)
#
#  o      A  p  means  the  permissions are different and are being
#         updated to the sender’s value (requires --perms).
#
#  o      An o means the owner is different and is being updated to
#         the sender’s value (requires --owner and super-user priv‐
#         ileges).
#
#  o      A g means the group is different and is being updated  to
#         the sender’s value (requires --group and the authority to
#         set the group).
#
#  o      The u slot is reserved for future use.
#
#  o      The a means that the ACL information changed.
#
#  o      The x  means  that  the  extended  attribute  information
#         changed.
#
#  One  other  output  is  possible:  when deleting files, the "%i"
#  will output the string "*deleting" for each item that  is  being
#  removed  (assuming that you are talking to a recent enough rsync
#  that it logs deletions instead of outputting them as  a  verbose
#  message).

sub do_target_differences {

	my $host = shift;
	my $more = shift;
	my $indents = shift;
	my $rsync = shift;
	my $filewritesonly = shift;
	my $diskusage = shift;

	my $key;
	my $diff_count;
	my $typeword;
	my $rsynccreate;
	my $rsyncdir;
	my $lastname;
	my $lastrec;
	my $thisrec;
	my $onlymtime;
	my $layersptr;
	my $walkhash;
	my $arrptr;
	my $line;

	if ($#{$HF::Tmp::hflog->{'target_differences'}} == -1) {
		print "ERROR no target_differences log sections, substituting numbers\n";
		&do_numbers ($host);
		return (0);
	} elsif ($HF::Tmp::hflog->{'target_report'}[$host]{'target_diff_zeroed'} == 1) {
		print "ERROR target difference log of " .
			&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'target_diff_rows'}) .
			" rows was deleted for extreme size, skipping\n";
		return (0);
	} elsif ($#{$HF::Tmp::hflog->{'target_differences'}[$host]} == -1 ) {
		if (!($main::opt_q)) {
			print "No target differences.\n";
		}
		return (1);
	} else {
		if ($diskusage) {
			$HF::Globals::opt_x_output = [ ];
			$layersptr = HF::Layers->new;
		}

		$diff_count = 0;
		$lastname = '';
		$lastrec = '';
		$thisrec = '';
		$onlymtime = 0;
		foreach $key (@{$HF::Tmp::hflog->{'target_differences'}[$host]}) {
	
			if ($key->[1] eq 'ignore') {
				next;
			}

			if ($diff_count == 0) {
				if ($rsync == 1) {
					if (!($main::opt_q)) {
						print "# rsync     name
# --------- -------------------------------------------------------------------\n";
					}
				} else {
					if ((!($main::opt_q)) && ($indents == 1)) {
						print "# what   why       want           have           t  source	both	# target
# -----  --------  -------------  -------------  -  ---------------------------------\n";
					} elsif (!($main::opt_q)) {
						print "# what   why       want           have           t  name
# -----  --------  -------------  -------------  -  ---------------------------------\n";
					}
					if ((!($main::opt_q)) && ($more == 1)) {
						print "#  which t  mode lnk  user group size/rdev       mtime      mtime                    target/checksum
# ------ - ----- --- ----- ----- --------------  ---------- ------------------------ ---------------\n"
					}
				}
			}
			++$diff_count;

			if ($key->[0] ne $lastname) {
				if (! (($main::opt_T == 1) && ($onlymtime == 1))) {
					print $lastrec;
				}
				$lastrec = '';
				$thisrec = '';
			}
			$onlymtime = 0;

			if ($key->[1] eq 'create') {
				if ($rsync == 1) {
					if ($key->[3]{'type'} == 0100000) {	# file
						if ($HF::Tmp::hflog->{'local_report'}[$host]{'swappushpull'} == 1) {
							$rsynccreate = "<";
						} else {
							$rsynccreate = ">";
						}
					} else {
						$rsynccreate = "c";
					}
					if ($key->[3]{'type'} == 0040000) {	# dir
						$rsyncdir = "/";
					} else {
						$rsyncdir = "";
					}
					if ($key->[3]{'type'} == 0120000) {	# symlink
						$rsyncdir = " -> " . $key->[3]{'enctarget'};
					}
					$thisrec .= sprintf ("%s%s+++++++++ %s%s\n", $rsynccreate, &type_to_text ($key->[3]{'type'}, 0, 1), HF::Encode->encode ($key->[0]), $rsyncdir);
				} else {
					if (($filewritesonly || $diskusage) && ($key->[3]{'type'} != 0100000)) {	# file
						next;
					} else {
						if ($diskusage) {
							$walkhash->{'encname'} = HF::Encode->encode ($key->[0]);
							$walkhash->{'type'} = $key->[3]{'type'};
							$walkhash->{'size'} = $key->[3]{'size'};
							$layersptr->diskusage_record ($walkhash);
							next;
						} else {
							$thisrec .= sprintf ("%-7s  %-8s  %13s                 %s  %s\n",
								$key->[1], &type_to_text ($key->[3]{'type'}, 1, 0), &HF::Misc::commify_number ($key->[3]{'size'}),
								&type_to_text ($key->[3]{'type'}, 0, 0), HF::Encode->encode ($key->[0]));
							if ($more == 1) {
								$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
							}
						}
					}
				}

			} elsif ($key->[1] eq 'delete') {
				if ($filewritesonly || $diskusage) {
					next;
				} elsif ($rsync == 1) {
					if ($key->[4]{'type'} == 0040000) {	# dir
						$rsyncdir = "/";
					} else {
						$rsyncdir = "";
					}
					if ($key->[3]{'type'} == 0120000) {	# symlink
						$rsyncdir = " -> " . $key->[3]{'enctarget'};
					}
					$thisrec .= sprintf ("*deleting   %s%s\n", HF::Encode->encode ($key->[0]), $rsyncdir);
				} else {
					$thisrec .= sprintf ("%-7s  %-8s                 %13s  %s  %s\n",
						$key->[1], &type_to_text ($key->[4]{'type'}, 1, 0), &HF::Misc::commify_number ($key->[4]{'size'}),
						&type_to_text ($key->[4]{'type'}, 0, 0), ("\t" x ($indents * 3)) . HF::Encode->encode ($key->[0]));
					if ($more == 1) {
						$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
					}
				}

			} elsif ($key->[1] eq 'unlink') {
				if ($filewritesonly || $diskusage) {
					next;
				} elsif ($rsync == 1) {
					# rsync doesn't do this, there is no corresponding message code
				} else {
					$thisrec .= sprintf ("%-7s  %-8s  %13s  %13s  %s  %s\n",
						$key->[1], &type_to_text ($key->[4]{'type'}, 1, 0), $key->[4]{'mtime'}, &HF::Misc::commify_number ($key->[4]{'size'}),
						&type_to_text ($key->[4]{'type'}, 0, 0), ("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
					if ($more == 1) {
						$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
					}
				}

			} elsif ($key->[1] eq 'relink') {
				if ($filewritesonly || $diskusage) {
					next;
				} elsif ($rsync == 1) {
					$thisrec .= sprintf ("h%s          %s => %s\n", &type_to_text ($key->[3]{'type'}, 0, 1), HF::Encode->encode ($key->[0]), $key->[3]{'encname'});
				} else {
					$thisrec .= sprintf ("%-7s  %-8s  %-13s  %-13s  %s  %s\n",
						$key->[1], &type_to_text ($key->[4]{'type'}, 1, 0), $key->[3]{'encname'}, $key->[4]{'encname'},
						&type_to_text ($key->[3]{'type'}, 0, 0), ("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
					if ($more == 1) {
						$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
						$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
					}
				}

			} elsif ($key->[1] eq 'replace') {
				if ($key->[2] eq 'type') {
					if ($rsync == 1) {
						if ($key->[3]{'type'} == 0100000) {	# file
							if ($HF::Tmp::hflog->{'local_report'}[$host]{'swappushpull'} == 1) {
								$rsynccreate = "<";
							} else {
								$rsynccreate = ">";
							}
						} else {
							$rsynccreate = "c";
						}
						if ($key->[3]{'type'} == 0040000) {	# dir
							$rsyncdir = "/";
						} else {
							$rsyncdir = "";
						}
						if ($key->[3]{'type'} == 0120000) {	# symlink
							$rsyncdir = " -> " . HF::Encode->encode ($key->[3]{'target'});
						}
						$thisrec .= sprintf ("%s%s+++++++++ %s%s\n", $rsynccreate, &type_to_text ($key->[3]{'type'}, 0, 1),
							HF::Encode->encode ($key->[0]), $rsyncdir);
					} else {
						if (($filewritesonly || $diskusage) && ($key->[3]{'type'} != 0100000)) {	# file
							next;
						} else {
							if ($diskusage) {
								$walkhash->{'encname'} = HF::Encode->encode ($key->[0]);
								$walkhash->{'type'} = $key->[3]{'type'};
								$walkhash->{'size'} = $key->[3]{'size'};
								$layersptr->diskusage_record ($walkhash);
								next;
							} else {
								$thisrec .= sprintf ("%-7s  %-8s        %7s        %7s  %s  %s\n",
									$key->[1], $key->[2], &type_to_text ($key->[3]{'type'}, 1, 0), &type_to_text ($key->[4]{'type'}, 1, 0),
									&type_to_text ($key->[4]{'type'}, 0, 0), ("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
								if ($more == 1) {
									$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
									$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
								}
							}
						}
					}

				} elsif ($key->[2] eq 'target') {
					if ($filewritesonly || $diskusage) {
						next;
					} elsif ($rsync == 1) {
						$thisrec .= sprintf ("cLc........ %s -> %s\n", HF::Encode->encode ($key->[0]), $key->[3]{'enctarget'});
					} else {
						$thisrec .= sprintf ("%-7s  %-8s  %-13s  %-13s  %s  %s\n",
							$key->[1], $key->[2], $key->[3]{'enctarget'}, $key->[4]{'enctarget'},
							&type_to_text ($key->[3]{'type'}, 0, 0), ("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
						if ($more == 1) {
							$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
							$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
						}
					}

				} elsif ($key->[2] eq 'checksum') {
					if ($rsync == 1) {
						if ($HF::Tmp::hflog->{'local_report'}[$host]{'swappushpull'} == 1) {
							$rsynccreate = "<";
						} else {
							$rsynccreate = ">";
						}
						if ($thisrec eq '') {
							$thisrec .= sprintf ("%s%sc........ %s\n", $rsynccreate, &type_to_text ($key->[3]{'type'}, 0, 1), HF::Encode->encode ($key->[0]));
						} else {
							substr ($thisrec, 2, 1) = 'c';
							if ($key->[3]{'type'} == 0100000) {	# file
								substr ($thisrec, 0, 1) = $rsynccreate;
							}
						}
					} else {
						if ($diskusage) {
							$walkhash->{'encname'} = HF::Encode->encode ($key->[0]);
							$walkhash->{'type'} = $key->[3]{'type'};
							$walkhash->{'size'} = $key->[3]{'size'};
							$layersptr->diskusage_record ($walkhash);
							next;
						} else {
							$thisrec .= sprintf ("%-7s  %-8s  %-13s  %-13s  %s  %s\n",
								$key->[1], $key->[2], substr ($key->[3]{'checksum'}, 0, 12) . '+', substr ($key->[4]{'checksum'}, 0, 12) . '+',
								&type_to_text ($key->[3]{'type'}, 0, 0), ("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
							if ($more == 1) {
								$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
								$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
							}
						}
					}

				} elsif ($key->[2] eq 'mtime') {
					if ($rsync == 1) {
						if ($HF::Tmp::hflog->{'local_report'}[$host]{'swappushpull'} == 1) {
							$rsynccreate = "<";
						} else {
							$rsynccreate = ">";
						}
						if ($thisrec eq '') {
							$thisrec .= sprintf ("%s%s..t...... %s\n", $rsynccreate, &type_to_text ($key->[3]{'type'}, 0, 1), HF::Encode->encode ($key->[0]));
						} else {
							substr ($thisrec, 4, 1) = 't';
							if ($key->[3]{'type'} == 0100000) {	# file
								substr ($thisrec, 0, 1) = $rsynccreate;
							}
						}
					} else {
						if (($filewritesonly || $diskusage) && ($key->[3]{'type'} != 0100000)) {	# file
							next;
						} else {
							if ($diskusage) {
								$walkhash->{'encname'} = HF::Encode->encode ($key->[0]);
								$walkhash->{'type'} = $key->[3]{'type'};
								$walkhash->{'size'} = $key->[3]{'size'};
								$layersptr->diskusage_record ($walkhash);
								next;
							} else {
								$thisrec .= sprintf ("%-7s  %-8s  %13d  %13d  %s  %s\n",
									$key->[1], $key->[2], $key->[3]{'mtime'}, $key->[4]{'mtime'},
									&type_to_text ($key->[3]{'type'}, 0, 0), ("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
								if ($more == 1) {
									$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
									$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
								}
							}
						}
					}
				} elsif ($key->[2] eq 'rdev') {
					if ($filewritesonly || $diskusage) {
						next;
					} elsif ($rsync == 1) {
						if ($HF::Tmp::hflog->{'local_report'}[$host]{'swappushpull'} == 1) {
							$rsynccreate = "<";
						} else {
							$rsynccreate = ">";
						}
						$thisrec .= sprintf ("c%sc........ %s\n", &type_to_text ($key->[3]{'type'}, 0, 1), HF::Encode->encode ($key->[0]));
					} else {
						$thisrec .= sprintf ("%-7s  %-8s  %-13s  %-13s  %s  %s\n",
							$key->[1], $key->[2], $key->[3]{'rdev'}, $key->[4]{'rdev'},
							&type_to_text ($key->[3]{'type'}, 0, 0), ("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
						if ($more == 1) {
							$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
							$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
						}
					}
				} elsif ($key->[2] eq 'size') {
					if ($rsync == 1) {
						if ($HF::Tmp::hflog->{'local_report'}[$host]{'swappushpull'} == 1) {
							$rsynccreate = "<";
						} else {
							$rsynccreate = ">";
						}
						if ($thisrec eq '') {
							$thisrec .= sprintf ("%s%s.s....... %s\n", $rsynccreate, &type_to_text ($key->[3]{'type'}, 0, 1), HF::Encode->encode ($key->[0]));
						} else {
							substr ($thisrec, 3, 1) = 's';
							if ($key->[3]{'type'} == 0100000) {	# file
								substr ($thisrec, 0, 1) = $rsynccreate;
							}
						}
					} else {
						if (($filewritesonly || $diskusage) && ($key->[3]{'type'} != 0100000)) {	# file
							next;
						} else {
							if ($diskusage) {
								$walkhash->{'encname'} = HF::Encode->encode ($key->[0]);
								$walkhash->{'type'} = $key->[3]{'type'};
								$walkhash->{'size'} = $key->[3]{'size'};
								$layersptr->diskusage_record ($walkhash);
								next;
							} else {
								$thisrec .= sprintf ("%-7s  %-8s  %13s  %13s  %s  %s\n",
									$key->[1], $key->[2], &HF::Misc::commify_number ($key->[3]{'size'}),
									&HF::Misc::commify_number ($key->[4]{'size'}),
									&type_to_text ($key->[3]{'type'}, 0, 0), ("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
								if ($more == 1) {
									$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
									$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
								}
							}
						}
					}

				} else {
					HF::Report->report (4, 1, "FATAL unknown subkey '" . $key->[1] . "', '" . $key->[2] . "'")
				}

			} elsif ($key->[1] eq 'fix') {
				if ($filewritesonly || $diskusage) {
					next;
				} elsif ($key->[2] eq 'owner') {
					if ($rsync == 1) {
						if ($key->[3]{'type'} == 0040000) {	# dir
							$rsyncdir = "/";
						} else {
							$rsyncdir = "";
						}
						if ($thisrec eq '') {
							$thisrec .= sprintf (".%s....o.... %s%s\n", &type_to_text ($key->[4]{'type'}, 0, 1), HF::Encode->encode ($key->[0]), $rsyncdir);
						} else {
							substr ($thisrec, 6, 1) = 'o';
						}
					} else {
						$thisrec .= sprintf ("%-7s  %-8s          %5d          %5d  %s  %s\n",
							$key->[1], $key->[2], $key->[3]{'owner'}, $key->[4]{'owner'}, &type_to_text ($key->[3]{'type'}, 0, 0),
							("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
						if ($more == 1) {
							$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
							$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
						}
					}

				} elsif ($key->[2] eq 'group') {
					if ($rsync == 1) {
						if ($key->[3]{'type'} == 0040000) {	# dir
							$rsyncdir = "/";
						} else {
							$rsyncdir = "";
						}
						if ($thisrec eq '') {
							$thisrec .= sprintf (".%s.....g... %s%s\n", &type_to_text ($key->[3]{'type'}, 0, 1), HF::Encode->encode ($key->[0]), $rsyncdir);
						} else {
							substr ($thisrec, 7, 1) = 'g';
						}
					} else {
						$thisrec .= sprintf ("%-7s  %-8s          %5d          %5d  %s  %s\n",
							$key->[1], $key->[2], $key->[3]{'group'}, $key->[4]{'group'}, &type_to_text ($key->[3]{'type'}, 0, 0),
							("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
						if ($more == 1) {
							$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
							$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
						}
					}

				} elsif ($key->[2] eq 'mtime') {
					if ($rsync == 1) {
						if ($key->[3]{'type'} == 0040000) {	# dir
							$rsyncdir = "/";
						} else {
							$rsyncdir = "";
						}
						if ($HF::Tmp::hflog->{'local_report'}[$host]{'swappushpull'} == 1) {
							$rsynccreate = "<";
						} else {
							$rsynccreate = ">";
						}
						if ($thisrec eq '') {
							$onlymtime = 1;
							$thisrec .= sprintf (".%s..t...... %s%s\n", &type_to_text ($key->[3]{'type'}, 0, 1), HF::Encode->encode ($key->[0]), $rsyncdir);
						} else {
							$onlymtime = 0;
							substr ($thisrec, 4, 1) = 't';
							if ($key->[3]{'type'} == 0100000) {	# file
								substr ($thisrec, 0, 1) = $rsynccreate;
							}
						}
					} else {
						$onlymtime = 1;
						if ($main::opt_T != 1) {
							$thisrec .= sprintf ("%-7s  %-8s  %13d  %13d  %s  %s\n",
								$key->[1], $key->[2], $key->[3]{'mtime'}, $key->[4]{'mtime'}, &type_to_text ($key->[3]{'type'}, 0, 0),
								("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
							if ($more == 1) {
								$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
								$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
							}
						}
					}
				} elsif ($key->[2] eq 'perms') {
					if ($rsync == 1) {
						if ($key->[3]{'type'} == 0040000) {	# dir
							$rsyncdir = "/";
						} else {
							$rsyncdir = "";
						}
						if ($thisrec eq '') {
							$thisrec .= sprintf (".%s...p..... %s%s\n", &type_to_text ($key->[4]{'type'}, 0, 1), HF::Encode->encode ($key->[0]), $rsyncdir);
						} else {
							substr ($thisrec, 5, 1) = 'p';
						}
					} else {
						$thisrec .= sprintf ("%-7s  %-8s           %04o           %04o  %s  %s\n",
							$key->[1], $key->[2], $key->[3]{'perms'}, $key->[4]{'perms'}, &type_to_text ($key->[3]{'type'}, 0, 0),
							("\t" x ($indents * 2)) . HF::Encode->encode ($key->[0]));
						if ($more == 1) {
							$thisrec .= '    want ' . &hash_to_ls ($key->[3]) . "\n";
							$thisrec .= '    have ' . &hash_to_ls ($key->[4]) . "\n";
						}
					}

				} else {
					HF::Report->report (4, 1, "FATAL unknown subkey '" . $key->[1] . "', '" . $key->[2] . "'")
				}

			} else {
				HF::Report->report (4, 1, "FATAL unknown key '" . $key->[1] . "'")
			}

			$lastname = $key->[0];
			$lastrec = $thisrec;
		}

		if ($thisrec ne '') {
			if (! (($main::opt_T == 1) && ($onlymtime == 1))) {
				print $thisrec;
			}
		}

		if (($diff_count > 0) && (!($main::opt_q || $diskusage))) {
			print "# ----- diff_count $diff_count\n";
		}

		if ($diskusage) {
			$layersptr->diskusage_report;
			$arrptr = $HF::Globals::opt_x_output;
			for ($line = 0;  $line <= $#{$arrptr};  $line++) {
				print $arrptr->[$line] . "\n";
			}
		}
	}

	return (1);
}

sub do_rearrange {

	my $host = shift;

	my $key;
	my %wanted_to_cp;
	my %avail_to_cp;
	my %avail_to_cp_print;
	my %do_cp;
	my $arrptr;
	my $line;
	my $asidefilename;

	if ($#{$HF::Tmp::hflog->{'target_differences'}} == -1) {
		print "ERROR no target_differences log sections, skipping\n";
		return (0);
	} elsif ($HF::Tmp::hflog->{'target_report'}[$host]{'target_diff_zeroed'} == 1) {
		print "ERROR target difference log of " .
			&HF::Misc::commify_number ($HF::Tmp::hflog->{'target_report'}[$host]{'target_diff_rows'}) .
			" rows was deleted for extreme size, skipping\n";
		return (0);
	} elsif ($#{$HF::Tmp::hflog->{'target_differences'}[$host]} == -1 ) {
		print "ERROR no target differences, skipping\n";
		return (0);
	} elsif (($HF::Tmp::hflog->{'local_report'}[$host]{'dochecksums'} != 1) ||
		($HF::Tmp::hflog->{'remote_report'}[$host]{'dochecksums'} != 1)) {
		print "ERROR checksums not enabled, skipping\n";
		return (0);
	} elsif ($main::opt_C ne '') {
		if (($HF::Tmp::hflog2->{'local_report'}[$host]{'dochecksums'} != 1) ||
			($HF::Tmp::hflog2->{'remote_report'}[$host]{'dochecksums'} != 1)) {

			print "ERROR checksums not enabled in alternate log, skipping\n";
			return (0);
		}
		if (($HF::Tmp::hflog2->{'local_report'}[$host]{'opt_x'} != 31) ||
			($HF::Tmp::hflog2->{'remote_report'}[$host]{'opt_x'} != 31)) {

			print "ERROR opt_x not 31 in alternate log, skipping\n";
			return (0);
		}
		if (!(exists ($HF::Tmp::hflog2->{'target_report'}[$host]{'opt_x_output'}))) {
			print "ERROR opt_x output missing for target in alternate log, skipping\n";
			return (0);
		}
	}

	print q{#!/bin/bash -euET

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

set -x

};
	print "cd " . &HF::Misc::bashencode ($HF::Tmp::hflog->{'target_layers'}[$host]{'mount'}[1]) . "\n";

	print "\n# mkdirs\n";
	foreach $key (@{$HF::Tmp::hflog->{'target_differences'}[$host]}) {
	
		if (($key->[1] eq 'create') && ($key->[3]{'type'} == 0040000)) {
			print "mkdir -p " . &HF::Misc::bashencode ($key->[3]{'name'}) . "\n";
		}

		# may be undef despite -k if perms don't allow reading
		if (($key->[1] eq 'create') && ($key->[3]{'type'} == 0100000) && (defined ($key->[3]{'checksum'}))) {
			$wanted_to_cp{$key->[3]{'checksum'}} = $key->[3]{'name'};
		}
		if (($key->[1] eq 'replace') && ($key->[3]{'type'} == 0100000) && (defined ($key->[3]{'checksum'}))) {
			$wanted_to_cp{$key->[3]{'checksum'}} = $key->[3]{'name'};
		}
		if (($key->[1] eq 'replace') && ($key->[4]{'type'} == 0100000) && (defined ($key->[4]{'checksum'}))) {
			$avail_to_cp{$key->[4]{'checksum'}} = $key->[4]{'name'};
		}
		if (($key->[1] eq 'delete') && ($key->[4]{'type'} == 0100000) && (defined ($key->[4]{'checksum'}))) {
			$avail_to_cp{$key->[4]{'checksum'}} = $key->[4]{'name'};
		}
	}

	if ($main::opt_C ne '') {
		$arrptr = $HF::Tmp::hflog2->{'target_report'}[$host]{'opt_x_output'};
		for ($line = 0;  $line <= $#{$arrptr};  $line++) {
			if (($arrptr->[$line][3]{'type'} == 0100000) && (defined ($arrptr->[$line][3]{'checksum'}))) {
				# end up with file containing last duplicate checksum
				$avail_to_cp{$arrptr->[$line][3]{'checksum'}} = $arrptr->[$line][3]{'name'};
			}
		}
	}

	print "\n# save existing files under hopefully non-colliding names\n";
	foreach $key (sort (keys (%wanted_to_cp))) {
		if (exists ($avail_to_cp{$key})) {
			# XXX may still collide on filenames because we don't have target filesystem available to test
			$asidefilename = $avail_to_cp{$key} . '_UnIqUe';
			$HF::Globals::uniqfilename_used->{$asidefilename} = 1;
			$avail_to_cp_print{&HF::Misc::bashencode ($avail_to_cp{$key})} = &HF::Misc::bashencode ($asidefilename);
			$do_cp{$wanted_to_cp{$key}} = $asidefilename;
		}
	}
	foreach $key (sort (keys (%avail_to_cp_print))) {
		print 'cp -a ' . $key . " " . $avail_to_cp_print{$key} . "\n";
	}

	print "\n# copy saved files into new places\n";
	foreach $key (sort { $do_cp{$a} cmp $do_cp{$b} } (keys (%do_cp))) {
		print 'cp -a ' . &HF::Misc::bashencode ($do_cp{$key}) . " " . &HF::Misc::bashencode ($key) . "\n";
	}

	print "\n# delete saved files\n";
	foreach $key (sort (keys (%{$HF::Globals::uniqfilename_used}))) {
		print 'rm ' . &HF::Misc::bashencode ($key) . "\n";
	}

	print q{
#-----

CLEANEXIT=1
exit 0
};
	return (1);
}

sub host_banner {

	my $host = shift;

	my $banner;

	$banner = "# ----- host " . ($host + 1) . " direction '" . $HF::Tmp::hflog->{'hosts'}[$host]{'direction'} . "' channel '" . $HF::Tmp::hflog->{'hosts'}[$host]{'channel'} . "'";
	if (defined ($HF::Tmp::hflog->{'hosts'}[$host]{'hostname'})) {
		$banner .= " hostname '" . $HF::Tmp::hflog->{'hosts'}[$host]{'hostname'} . "'";
	}
	if (defined ($HF::Tmp::hflog->{'hosts'}[$host]{'ip'})) {
		$banner .= " ip '" . $HF::Tmp::hflog->{'hosts'}[$host]{'ip'} . "'";
	}
	if (defined ($HF::Tmp::hflog->{'hosts'}[$host]{'sshopts'})) {
		$banner .= " sshopts '" . join ("', '", @{$HF::Tmp::hflog->{'hosts'}[$host]{'sshopts'}}) . "'";
	}

	return ($banner);
}

sub type_to_text {

	my $type = shift;
	my $long = shift;
	my $rsync = shift;

	my $line;

	if ($type == 0100000) {		# file
		if ($long == 1) {
			$line = 'file';
		} else {
			if ($rsync == 1) {
				$line = 'f';
			} else {
				$line = '-';
			}
		}
	} elsif ($type == 0040000) {	# dir
		if ($long == 1) {
			$line = 'dir';
		} else {
			$line = 'd';
		}
	} elsif ($type == 0120000) {	# symlink
		if ($long == 1) {
			$line = 'symlink';
		} else {
			if ($rsync == 1) {
				$line = 'L';
			} else {
				$line = 'l';
			}
		}
	} elsif ($type == 0020000) {	# char dev
		if ($long == 1) {
			$line = 'char';
		} else {
			if ($rsync == 1) {
				$line = 'D';
			} else {
				$line = 'c';
			}
		}
	} elsif ($type == 0060000) {	# block dev
		if ($long == 1) {
			$line = 'block';
		} else {
			if ($rsync == 1) {
				$line = 'D';
			} else {
				$line = 'b';
			}
		}
	} elsif ($type == 0010000) {	# fifo
		if ($long == 1) {
			$line = 'pipe';
		} else {
			if ($rsync == 1) {
				$line = 'S';
			} else {
				$line = 'p';
			}
		}
	} elsif ($type == 0140000) {	# socket
		if ($long == 1) {
			$line = 'socket';
		} else {
			if ($rsync == 1) {
				$line = 'S';
			} else {
				$line = 's';
			}
		}
	} else {			# can happen from ro dist
		if ($long == 1) {
			$line = '??????';
		} else {
			$line = '?';
		}
	}

	return ($line);
}

# XXX relation to Harmonize::hash_to_ls ?
sub hash_to_ls {

	my $ftwhash = shift;

	my $line;
	my $date;

	if (!(defined ($ftwhash->{'inode'}))) {
		HF::Report->report (2, 1, "FAILURE hash_to_ls inode undefined'");
		return ('hash_to_ls NO FILE');
	}

	$line = &type_to_text ($ftwhash->{'type'}, 0, 0);

	if ($ftwhash->{'type'} != 0120000) {
		$line .= sprintf (' 0%04o %3d %5d %5d', $ftwhash->{'perms'}, $ftwhash->{'links'}, $ftwhash->{'owner'}, $ftwhash->{'group'});
	} else {
		$line .= '                     ';
	}

	if ($ftwhash->{'type'} == 0100000) {
		$line .= sprintf (' %14s', &HF::Misc::commify_number ($ftwhash->{'size'}));
	} elsif (($ftwhash->{'type'} == 0020000) || ($ftwhash->{'type'} == 0060000)) {
		$line .= sprintf (' %14d', $ftwhash->{'rdev'});
	} else {
		$line .= '               ';
	}

	if ($ftwhash->{'type'} != 0120000) {
		$date = localtime ($ftwhash->{'mtime'});
		$line .= sprintf ("  %10d %-24s", $ftwhash->{'mtime'}, $date);
	} else {
		$line .= '                                     ';
	}

	if (($ftwhash->{'type'} == 0100000) && ($ftwhash->{'checksum'} ne '')) {
		$line .= ' ' . $ftwhash->{'checksum'};
	} elsif ($ftwhash->{'type'} == 0120000) {
		$line .= ' ' . $ftwhash->{'enctarget'};
	} 

	return ($line);
}

# XXX relation to Layers::eval_file_to_tmp ?
sub eval_file_to_tmp {

	my $filename = shift;

	my $filecontents;
	my $evalcode;

	if (-z $filename) {
		die "FATAL log file '$filename' is zero size\n";
	}

	if (!(open (HFLOGFILE, '<', $filename))) {
		die "FATAL open '$filename' failed with error '$!'\n";
	}
	{
		local ($/);
		$filecontents = <HFLOGFILE>;
	}
	close (HFLOGFILE);
	$evalcode = '$HF::Tmp::hflog = 
' . $filecontents . ';

1;';

	eval $evalcode;

	if ($@ eq '') {
		return (1);
	} else {
		die "FATAL eval error message for log file '$filename' is '$@'\n";
	}
}
