#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

if (!(getopts ('kdnmlLtprbx:'))) {
	&print_usage;
	exit (2);
}

$HF::Globals::verbosity = 4;
$HF::Globals::dochecksums = $main::opt_k;
$HF::Globals::norecurse = $main::opt_d;
$HF::Globals::nameonly = $main::opt_n;
$HF::Globals::mtimeonly = $main::opt_m;
$HF::Globals::printls = $main::opt_l;
$HF::Globals::actls = $main::opt_L;
$HF::Globals::printheadings = $main::opt_t;
$HF::Globals::printleadpath = $main::opt_p;
$HF::Globals::printroot = $main::opt_r;
$HF::Globals::bashquoting = $main::opt_b;
$HF::Globals::opt_x = $main::opt_x;

my $startdir;
my $sl;
my $someerror;
my $i;
my $status;

$HF::Globals::stderrcols = 9999;
if ($ARGV[0] eq '') {
	$ARGV[0] = '.';
}

if (($HF::Globals::nameonly == 1) && ($HF::Globals::mtimeonly == 1)) {
	HF::Report->report (3, 1, "FATAL options -n and -m can't be used together");
}
if (($HF::Globals::printls == 1) &&
	(($HF::Globals::nameonly == 1) || ($HF::Globals::mtimeonly == 1) || ($HF::Globals::printroot == 1) ||
	($HF::Globals::norecurse == 1) || ($HF::Globals::printleadpath == 1))) {

	HF::Report->report (3, 1, "FATAL option -l can't be used with options -n, -m, -r, -d, or -p");
}
if (($HF::Globals::actls == 1) &&
	(($HF::Globals::nameonly == 1) || ($HF::Globals::mtimeonly == 1) || ($HF::Globals::printroot == 1) ||
	($HF::Globals::norecurse == 1) || ($HF::Globals::printleadpath == 1) ||
	($HF::Globals::printheadings == 1) || ($HF::Globals::printls == 1) || ($HF::Globals::opt_x == 1))) {

	HF::Report->report (3, 1, "FATAL option -L can't be used with options -n, -m, -r, -d, -p, -t, -l, or -x");
}

$someerror = 0;

if ($HF::Globals::actls == 1) {
	$HF::Globals::printls = 1;
	$HF::Globals::norecurse = 1;
	$HF::Globals::printleadpath = 1;
	$ENV{'LANG'} = 'C';
}

$HF::Globals::hostrow = 0;
foreach $startdir (@ARGV) {

	$HF::Globals::local_errors->[$HF::Globals::hostrow] = { };	# used by error reporting in FTWstream
	$HF::Globals::remote_errors->[$HF::Globals::hostrow] = { };

	if (length ($startdir) > 1) {
		$startdir =~ s|/$||;
	}

	if ($HF::Globals::actls == 1) {
		if ($HF::Globals::hostrow > 0) {
			print STDOUT ("\n");
		}
		if (-d $startdir) {
			if ($#ARGV > 0) {
				print STDOUT ("$startdir\n");
			}
			if (!($status = opendir (STARTINGDIRH, '.'))) {
				HF::Report->report (3, 1, "FATAL can't opendir '.'");
			}
			if (!($status = chdir ($startdir))) {
				HF::Report->report (3, 1, "FATAL can't chdir into '" . HF::Encode->encode ($startdir) . "'");
			}
			foreach $i (sort (glob ('{.*,*}'))) {
				$sl = HF::Statreport->new ($i);
				if (!($sl->standalone)) {
					$someerror++;
				}
				$HF::Globals::hostrow++;
			}
			if (!($status = chdir (STARTINGDIRH))) {
				HF::Report->report (3, 1, "FATAL can't chdir back to normal cwd after chdir into '" . HF::Encode->encode ($startdir) . "'");
			}
			if (!($status = closedir (STARTINGDIRH))) {
				HF::Report->report (3, 1, "FATAL can't closedir '.' after chdir into '" . HF::Encode->encode ($startdir) . "'");
			}
		} else {
			$sl = HF::Statreport->new ($startdir);
			if (!($sl->standalone)) {
				$someerror++;
			}
			$HF::Globals::hostrow++;
		}
	} else {
		$sl = HF::Statreport->new ($startdir);
		if (!($sl->standalone)) {
			$someerror++;
		}
		$HF::Globals::hostrow++;
	}
}

if ($someerror) {
#	warn "ERROR some problem; does '$startdir' exist in dir '" . substr (`pwd`, 0, -1) . "'?\n";
	warn "ERROR some problem occurred\n";
	exit (1);
} else {
	exit (0);
}
