#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#  Intention is to generate layer file fragments for scripts which
#  boil down multiple host captures into a model.  There will be
#  multiple layers files produced, for leaving data-in-flight alone
#  for audit, factory resetting it back to a known state, collecting
#  it as a software update, or deleting it as per-host during model
#  assembly.

$HF::Globals::verbosity = 3;

use Data::Dumper;

# -----

my $composefile;
my $auditfile = 'audit.hflayers';
my $loadfile = 'load.hflayers';
my $resetfile = 'reset.hflayers';
my $collectfile = 'collect.hflayers';
my $setupfile = 'setup.sh';
my $auditstruct;
my $loadstruct;
my $resetstruct;
my $collectstruct;
my $setupsh;
my $dumper;
my $touchmtime;
my $i;
my $dir;
my $file;
my $setup_mkdir_touch;
my $auditstruct_source_layers_body;
my $auditstruct_source_treatments;
my $loadstruct_source_layers_body;
my $resetstruct_source_layers_body;
my $collectstruct_source_layers_body;
my $collectstruct_source_layers_body_114;
my $collectstruct_target_layers_body;

if ($#ARGV == -1) {
	&print_version;
	warn ("\n");
	&print_usage;
	exit (0);
}

$composefile = shift (@ARGV);

&eval_file_to_tmp ($composefile);

# -----

$setupsh = q@#!/bin/bash -euET

@ . "# Generated setup by hfcompose $composefile" . q@

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" >&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

if ! TEMP=`getopt -o w -n "$0" -- "$@ . '@' . q@"`
then
	CLEANEXIT=1
	exit 1
fi

eval set -- "$TEMP"

WRITE=''

while :
do
	case "$1" in
		-w)	WRITE=1
			shift
			;;
		--)	shift
			break
			;;
		*)	echo "$0: ERROR parsing getopt" >&2
			exit 1
			;;
	esac
done

#-----

if [ -z "${1:-}" ]
then
	echo "Usage: setup.sh [-w] host1 [host2] ..." 1>&2
	CLEANEXIT=1
	exit 1
fi

#-----

@;

$auditstruct_source_treatments = [ [0] ];

if ($#{$HF::Tmp::hfcompose->{'audit_source_treatments'}} > -1) {
	push (@{$auditstruct_source_treatments}, @{$HF::Tmp::hfcompose->{'audit_source_treatments'}});
}

$auditstruct_source_layers_body = [ {
		'mount' => [ '', $HF::Tmp::hfcompose->{'merge'} . '/0', 'all' ],
		'treatments' => $auditstruct_source_treatments
	} ];

$loadstruct_source_layers_body = [ {
		'mount' => [ '', $HF::Tmp::hfcompose->{'merge'} . '/0', 'all' ],
		'treatments' => [
			[0]
		]
	} ];

$resetstruct_source_layers_body = [ {
		'mount' => [ '', $HF::Tmp::hfcompose->{'merge'} . '/0', 'all' ],
		'treatments' => [
			[0]
		]
	} ];

$collectstruct_source_layers_body_114 = [ [0] ];

if (($#{$HF::Tmp::hfcompose->{'fortreat104'}} > -1) || ($#{$HF::Tmp::hfcompose->{'fortreat105'}} > -1)) {

	# if one of these lists is empty it adds no empty/blank/undef entries
	push (@{$collectstruct_source_layers_body_114},
		[ 114, [ &uniq (map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat104'}}),
				map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat105'}}))]]);
}

$collectstruct_source_layers_body = [ {
		'mount' => [ '', $HF::Tmp::hfcompose->{'collect_from'}, 'all' ],
		'treatments' => $collectstruct_source_layers_body_114,
		'skips' => $HF::Tmp::hfcompose->{'collect_source_skips'}
	} ];

$collectstruct_target_layers_body = [ {
		'mount' => [ '', $HF::Tmp::hfcompose->{'store_here'} . '/oh/0', 'all' ],
		'treatments' => [ [0] ]
	} ];

$setup_mkdir_touch = '';
$touchmtime = 1E5 + 0;

if ($#{$HF::Tmp::hfcompose->{'fortreat103'}} > -1) {

	foreach $i (@{$HF::Tmp::hfcompose->{'fortreat103'}}) {

		$dir = $i->[0];
		$file = $i->[1];

		$setup_mkdir_touch .= '	mkdir -p ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/0/$dir") . ' ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/103/$dir") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/0/$dir/$file") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/103/$dir/$file") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/0/$dir") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/103/$dir") . "\n";
	}

	push (@{$auditstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/103', 'all' ],
		'treatments' => [
			[0],
			[ 103, [ map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat103'}}) ] ]
		]
	});

	push (@{$loadstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/103', 'all' ],
		'treatments' => [
			[0],
			[ 103, [ map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat103'}}) ] ]
		]
	});

	push (@{$resetstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/103', 'all' ],
		'treatments' => [
			[0],
			[ 103, [ map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat103'}}) ] ]
		]
	});

	push (@{$collectstruct_target_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'store_here'} . '/oh/103', 'all' ],
		'treatments' => [
			[0],
			[ 103, [ map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat103'}}) ] ]
		]
	});
}

if ($#{$HF::Tmp::hfcompose->{'fortreat113'}} > -1) {

	foreach $i (@{$HF::Tmp::hfcompose->{'fortreat113'}}) {

		$dir = $i->[0];
		$file = $i->[1];

		$setup_mkdir_touch .= '	mkdir -p ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/0/$dir") . ' ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/113/$dir") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/0/$dir/$file") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/113/$dir/$file") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/0/$dir") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/113/$dir") . "\n";
	}

	push (@{$auditstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/113', 'all' ],
		'treatments' => [
			[0],
			[ 113, [ map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat113'}}) ] ]
		]
	});

	push (@{$loadstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/113', 'all' ],
		'treatments' => [
			[0],
			# 103 not 113
			[ 103, [ map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat113'}}) ] ]
		]
	});

	push (@{$resetstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/113', 'all' ],
		'treatments' => [
			[0],
			# 103 not 113
			[ 103, [ map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat113'}}) ] ]
		]
	});

	push (@{$collectstruct_target_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'store_here'} . '/oh/113', 'all' ],
		'treatments' => [
			[0],
			# 103 not 113
			[ 103, [ map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat113'}}) ] ]
		]
	});
}

if ($#{$HF::Tmp::hfcompose->{'fortreat104'}} > -1) {

	foreach $i (@{$HF::Tmp::hfcompose->{'fortreat104'}}) {

		$dir = $i->[0];
		$file = $i->[1];

		$setup_mkdir_touch .= '	mkdir -p ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/104/$dir") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/104/$dir/$file") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/104/$dir") . "\n";
	}

	push (@{$auditstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/104', 'all' ],
		'treatments' => [
			[0],
			[ 104, [ &uniq (map ('^' . $_->[0] . '$', @{$HF::Tmp::hfcompose->{'fortreat104'}})),
				map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat104'}}) ] ]
		]
	});

	push (@{$loadstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/104', 'all' ],
		'treatments' => [
			[0]
		]
	});

	push (@{$resetstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/104', 'all' ],
		'treatments' => [
			[0]
		]
	});

	push (@{$collectstruct_target_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'store_here'} . '/oh/104', 'all' ],
		'treatments' => [
			[0]
		]
	});
}

if ($#{$HF::Tmp::hfcompose->{'fortreat105'}} > -1) {

	foreach $i (@{$HF::Tmp::hfcompose->{'fortreat105'}}) {

		$dir = $i->[0];
		$file = $i->[1];

		$setup_mkdir_touch .= '	mkdir -p ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/105/$dir") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/105/$dir/$file") . "\n";
		$setup_mkdir_touch .= '	touch -d @$((($LOOP*10000000)+' . $touchmtime++ . ')) ' .
			'$i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'} . "/oh/105/$dir") . "\n";
	}

	push (@{$auditstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/105', 'all' ],
		'treatments' => [
			[0],
			[ 104, [ &uniq (map ('^' . $_->[0] . '$', @{$HF::Tmp::hfcompose->{'fortreat105'}})) ] ],
			[ 105, [ map ('^' . $_->[0] . '/' . $_->[1] . '$', @{$HF::Tmp::hfcompose->{'fortreat105'}}) ] ]
		]
	});

	push (@{$loadstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/105', 'all' ],
		'treatments' => [
			[0]
		]
	});

	push (@{$resetstruct_source_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'diffs'} . '/105', 'all' ],
		'treatments' => [
			[0]
		]
	});

	push (@{$collectstruct_target_layers_body}, {
		'mount' => [ '', $HF::Tmp::hfcompose->{'store_here'} . '/oh/105', 'all' ],
		'treatments' => [
			[0]
		]
	});
}

$auditstruct = {
'source_layers' => $auditstruct_source_layers_body,
'target_layers' => [
	{
		'mount' => [ '', $HF::Tmp::hfcompose->{'audit_me'}, 'all' ],
		'treatments' => [ [0] ],
		'skips' => $HF::Tmp::hfcompose->{'collect_source_skips'}
	}
]
};

$loadstruct = {
'source_layers' => $loadstruct_source_layers_body,
'target_layers' => [
	{
		'mount' => [ '', $HF::Tmp::hfcompose->{'load_me'}, 'all' ],
		'treatments' => [ [0] ]
	}
]
};

$resetstruct = {
'source_layers' => $resetstruct_source_layers_body,
'target_layers' => [
	{
		'mount' => [ '', $HF::Tmp::hfcompose->{'reset_me'}, 'all' ],
		'treatments' => [ [0] ]
	}
]
};

$collectstruct = {
'source_layers' => $collectstruct_source_layers_body,
'target_layers' => $collectstruct_target_layers_body
};

#------------------------------------------------------------------------------------------------------------------------

$setupsh .= 'LOOP=1
for i
do
	rm -rf $i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'}) . '
	mkdir -p $i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'}) . '
	(cd $i/' . &HF::Misc::bashencode ($HF::Tmp::hfcompose->{'store_here'}) . ' ; oh init)

' . $setup_mkdir_touch . '
	LOOP=$(($LOOP+1))
done

#-----

CLEANEXIT=1
exit 0
';

# -----

&dump_file ($auditfile, $auditstruct, "# Generated audit by hfcompose $composefile");
&dump_file ($loadfile, $loadstruct, "# Generated load by hfcompose $composefile");
&dump_file ($resetfile, $resetstruct, "# Generated reset by hfcompose $composefile");
&dump_file ($collectfile, $collectstruct, "# Generated collect by hfcompose $composefile");

if (!(open (SETUPFILE, '>', $setupfile))) {
	die "FATAL open '$setupfile' failed with error '$!'\n";
}
print "Generating $setupfile\n";
print SETUPFILE $setupsh;
close (SETUPFILE);
if (!(chmod (0755, $setupfile))) {
	die "FATAL chmod '$setupfile' failed with error '$!'\n";
}

exit (0);

# -----

sub dump_file  {

	my $filename = shift;
	my $struct = shift;
	my $comment = shift;

	if (!(open (DUMPFILE, '>', $filename))) {
		die "FATAL open '$filename' failed with error '$!'\n";
	}
	print "Generating $filename\n";
	print DUMPFILE "$comment\n";
	$dumper = Data::Dumper->new ([$struct]);
	$dumper->Sortkeys (1);
	$dumper->Terse (1);
	print DUMPFILE $dumper->Dump;
	close (DUMPFILE);

	return (1);
}

sub eval_file_to_tmp {

	my $filename = shift;

	my $filecontents;
	my $evalcode;

	if (-z $filename) {
		die "FATAL compose file '$filename' is zero size\n";
	}

	if (!(open (HFCOMPOSEFILE, '<', $filename))) {
		die "FATAL open '$filename' failed with error '$!'\n";
	}
	{
		local ($/);
		$filecontents = <HFCOMPOSEFILE>;
	}
	close (HFCOMPOSEFILE);
	$evalcode = '$HF::Tmp::hfcompose = 
' . $filecontents . ';

1;';

	eval $evalcode;

	if ($@ eq '') {
		return (1);
	} else {
		die "FATAL eval error message for compose file '$filename' is '$@'\n";
	}
}

sub uniq  {

	my @list = @_;

	my @unique;
	my %seen;

	return (grep { !$seen{$_}++ } @list);
}
