
CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
case "${CLEANEXIT:-0}" in
	1)
	:
	;;
	2)
		ENDTIME=`date "+%s"`
		# $(( not same as backquote here
		echo "STATUS total elapsed time $((ENDTIME - STARTTIME)) seconds aka `echo scale=1 \; \( $ENDTIME - $STARTTIME \) / 60 | bc -l` minutes"
	;;
	*)
		echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
		exit $SAVEDSTATUS
	;;
esac' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

function print_version {

	echo "oh version @@@ HFVER @@@
Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew
All rights reserved.
Written by Brian Bartholomew." 1>&2
}

function print_usage {

	cat << 'EOF' 1>&2
Usage: oh init
       oh wrap
       oh unwrap
       oh status
       oh [-b maxblockbytes] commit [message ...]
       oh snapshot [message ...]
       oh fix [-y]

       init	creates an empty Mercurial repository and omar support to
       		allow Mercurial of a Linux root partition filetree
       wrap	massage the filetree so that it is ready for Mercurial commit
       status	what state is the repository in?
       [-b maxblockbytes] commit [message ...]
		do Mercurial commit, remaining words on tail put into message
       unwrap	massage the filetree so that it is ready to have files written
       fix [-y]
		make working set files consistent with Mercurial revision
		-y don't ask for confirmation before deleting files
       snapshot [message ...]
		do oh wrap, commit, unwrap
		remaining words on command tail put into message
       --help   display usage message
       --version
                display version of program

Operation keywords can be truncated to minimum unique characters.
EOF
}

if [ "${1:-}" = '--version' ]
then
	print_version
	CLEANEXIT=1
	exit 0
fi

if [ \( $# -eq 0 \) -o \( "${1:-}" = '--help' \) ]
then
	print_version
	echo 1>&2
	print_usage
	CLEANEXIT=1
	exit 0
fi

if [ "${1:-}" = '-b' ]
then
	shift || :
	OPT_B="${1:-}"
	if [ -z "$OPT_B" ]
	then
		print_usage
		CLEANEXIT=1
		exit 1
	fi
	shift || :
else
	OPT_B=''
fi

STARTTIME=$(date '+%s')

if ! type -p hg > /dev/null
then
	echo 'FATAL Mercurial not installed, quitting' 1>&2
	CLEANEXIT=1
	exit 1
fi

#  For /usr/bin/time
export TIME='%U user %S system %E elapsed'

OHVER='2.0'
OHWRAPDIR="oh-$OHVER"
OHUNWRAPDIR='oh'
OMARVER=$(/@@@ HFTOP @@@/bin/omar --versionfile)
OMARFILE="omar-$OMARVER"

if ! pwd -P > /dev/null 2>&1
then
	echo "FATAL pwd error, did you oh delete the '$OHUNWRAPDIR' dir you were in?, quitting" 1>&2
	CLEANEXIT=1
	exit 1
fi

CMD="${1:-}"
shift || :

case "$CMD" in
	i*)	:
		if [ -n "${1:-}" ]
		then
			print_usage
			CLEANEXIT=1
			exit 1
		fi

		echo "STATUS doing init"

		if [ -d .hg ]
		then
			echo "FATAL should not be dir '.hg', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ -d $OHWRAPDIR ]
		then
			echo "FATAL should not be dir '$OHWRAPDIR', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ -d $OHUNWRAPDIR ]
		then
			echo "FATAL should not be dir '$OHUNWRAPDIR', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ -f $OMARFILE ]
		then
			echo "FATAL should not be file '$OMARFILE', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ -f $OMARFILE.part ]
		then
			echo "FATAL should not be file '$OMARFILE.part', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		hg init
		mkdir $OHWRAPDIR
		(cd $OHWRAPDIR ; /@@@ HFTOP @@@/bin/omar -cf ../$OMARFILE .)
		hg add
		hg commit -m "Empty initial checkin"
		mv $OHWRAPDIR $OHUNWRAPDIR
		echo "STATUS init done"
		echo "STATUS next step is to write files to the '$OHUNWRAPDIR' dir"
		CLEANEXIT=2
		exit 0
		;;
	w*)	:
		if [ -n "${1:-}" ]
		then
			print_usage
			CLEANEXIT=1
			exit 1
		fi

		echo "STATUS doing wrap"
		cd $(hg root)

		if [ -f $OMARFILE.part ]
		then
			echo "FATAL should not be file '$OMARFILE.part', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ ! -f $OMARFILE ]
		then
			echo "FATAL should be file '$OMARFILE', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ -d $OHWRAPDIR ]
		then
			echo "FATAL should not be dir '$OHWRAPDIR', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ ! -d $OHUNWRAPDIR ]
		then
			echo "FATAL should be dir '$OHUNWRAPDIR', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		mv $OHUNWRAPDIR $OHWRAPDIR
		echo "STATUS doing omar create"
		(cd $OHWRAPDIR ; /usr/bin/time /@@@ HFTOP @@@/bin/omar -cf ../$OMARFILE .)
		echo "STATUS wrap done"
		echo "STATUS next step is to oh commit"
		CLEANEXIT=2
		exit 0
		;;
	u*)	:
		if [ -n "${1:-}" ]
		then
			print_usage
			CLEANEXIT=1
			exit 1
		fi

		echo "STATUS doing unwrap"
		cd $(hg root)

		if [ -f $OMARFILE.part ]
		then
			echo "FATAL should not be file '$OMARFILE.part', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ ! -f $OMARFILE ]
		then
			echo "FATAL should be file '$OMARFILE', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ ! -d $OHWRAPDIR ]
		then
			echo "FATAL should be dir '$OHWRAPDIR', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ -d $OHUNWRAPDIR ]
		then
			echo "FATAL should not be dir '$OHUNWRAPDIR', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		echo "STATUS doing hg update"
		hg -v --time update -c -r $(hg id -n | sed 's/\+//') | cat -v
		echo "STATUS doing omar extract"
		(cd $OHWRAPDIR ; /usr/bin/time /@@@ HFTOP @@@/bin/omar -xf ../$OMARFILE)
		mv $OHWRAPDIR $OHUNWRAPDIR

		echo "STATUS unwrap done"
		echo "STATUS next step is to write files to the '$OHUNWRAPDIR' dir"
		CLEANEXIT=2
		exit 0
		;;
	st*)	:
		if [ -n "${1:-}" ]
		then
			print_usage
			CLEANEXIT=1
			exit 1
		fi

		echo "STATUS doing status"
		cd $(hg root)

		if [ -f $OMARFILE.part ]
		then
			echo "FATAL should not be file '$OMARFILE.part', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ ! -f $OMARFILE ]
		then
			echo "FATAL should be file '$OMARFILE', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ \( -d $OHWRAPDIR \) -a \( ! -d $OHUNWRAPDIR \) ]
		then
			echo "STATUS looks wrapped"
			echo "STATUS doing hg status"
			hg --time status | cat -v | tee /tmp/oh.$$.status
			COUNT=$(wc -l < /tmp/oh.$$.status)
			rm /tmp/oh.$$.status
			echo "STATUS $COUNT hg status lines"
			echo "STATUS $(wc -l < $OMARFILE) omar lines"
			echo "STATUS status done"
			if [ $COUNT -eq 0 ]
			then
				echo "STATUS next step is to oh unwrap"
			else
				echo "STATUS if hg status lines are sensible, next step is to oh commit"
			fi

			CLEANEXIT=2
			exit 0
		fi

		if [ \( ! -d $OHWRAPDIR \) -a \( -d $OHUNWRAPDIR \) ]
		then
			echo "STATUS looks unwrapped"
			echo "STATUS don't do hg status, results may be bogus"
			echo "STATUS doing omar listing"
			(cd $OHUNWRAPDIR ; /usr/bin/time /@@@ HFTOP @@@/bin/omar -tf /tmp/oh.$$.omar .)
			if diff $OMARFILE /tmp/oh.$$.omar
			then
				rm /tmp/oh.$$.omar
				echo "STATUS status done"
				echo "STATUS next step is to write files to the '$OHUNWRAPDIR' dir"
				CLEANEXIT=2
				exit 0
			else
				echo "STATUS that was diff $OMARFILE /tmp/oh.$$.omar"
				echo "STATUS status done"
				echo "STATUS next step is to oh wrap"
				CLEANEXIT=2
				exit 1
			fi
		fi

		echo "FATAL repository condition not understood, exiting" 1>&2
		CLEANEXIT=1
		exit 1
		;;
	c*)	:
		echo "STATUS doing commit"
		cd $(hg root)

		if [ -f $OMARFILE.part ]
		then
			echo "FATAL should not be file '$OMARFILE.part', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ ! -f $OMARFILE ]
		then
			echo "FATAL should be file '$OMARFILE', exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		if [ \( -d $OHWRAPDIR \) -a \( ! -d $OHUNWRAPDIR \) ]
		then
			if [ -n "$OPT_B" ]
			then
				/@@@ HFTOP @@@/bin/oh-blocked-commit -b "$OPT_B" "$OMARFILE" "$*"
			else
				# if no message supplied then editor appears to ask for one
				/@@@ HFTOP @@@/bin/oh-blocked-commit "$OMARFILE" "$*"
			fi
			echo "STATUS commit done"
			echo "STATUS next step is to oh unwrap"
			CLEANEXIT=2
			exit 0
		else
			echo "FATAL repository not wrapped, exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi

		echo "FATAL repository condition not understood, exiting" 1>&2
		CLEANEXIT=1
		exit 1
		;;
	sn*)	:
		echo "STATUS doing snapshot"
		/@@@ HFTOP @@@/bin/oh wrap
		# if no message supplied then editor appears to ask for one
		/@@@ HFTOP @@@/bin/oh commit "$*"
		/@@@ HFTOP @@@/bin/oh unwrap
		echo "STATUS snapshot done"
		CLEANEXIT=2
		exit 0
		;;
	f*)	:
		echo "STATUS doing fix"
		YES="${1:-}"
		shift || :

		if [ \( \( -n "$YES" \) -a \( "$YES" != '-y' \) \) -o \( -n "${1:-}" \) ]
		then
			print_usage
			CLEANEXIT=1
			exit 1
		fi

		cd $(hg root)
		if [ \( ! -f .hg/store/data/${OMARFILE}.i \) -o \( ! -d .hg/store/data/$OHWRAPDIR \) ]
		then
			echo "FATAL doesn't look like an oh repository, exiting" 1>&2
			CLEANEXIT=1
			exit 1
		fi
		if [ "$YES" = '-y' ]
		then
			# delete everything but .hg and $OHUNWRAPDIR; save $OHUNWRAPDIR in case another shell is cd into it
			find . -maxdepth 2 -path ./.hg -prune -o \( \( ! -path . -a ! -path ./$OHUNWRAPDIR \) -print0 \) | \
xargs --null --verbose --no-run-if-empty rm -rf
		else
			echo "STATUS if a question appears, answer with y <RET> to approve deletion"
			# delete everything but .hg and $OHUNWRAPDIR; save $OHUNWRAPDIR in case another shell is cd into it
			# and confirm before deletion
			find . -maxdepth 2 -path ./.hg -prune -o \( \( ! -path . -a ! -path ./$OHUNWRAPDIR \) -print0 \) | \
xargs --null --verbose --no-run-if-empty --interactive rm -rf
		fi
		# save oh in case another shell is cd into it
		if [ -d $OHUNWRAPDIR ]
		then
			mv $OHUNWRAPDIR $OHWRAPDIR
		fi

		# can't just touch omar, mkdir $OHWRAPDIR, and do unwrap,
		# hg status will complain about locally mod files
		hg -v --time update --clean -r $(hg id -n | sed 's/\+//') | cat -v
		/@@@ HFTOP @@@/bin/oh status
		echo "STATUS fix done, hg local revision number is $(hg id -n)"
		echo "STATUS next step is to oh unwrap"

		CLEANEXIT=1
		exit 0
		;;
	*)	:
		print_usage
		CLEANEXIT=1
		exit 1
		;;
esac

#-----

CLEANEXIT=1
exit 0
