#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

use POSIX qw(:signal_h :errno_h :sys_wait_h);

my $hfhostlists;
my $arg;
my $opt_p_script;
my @hostsetmath;
my @commandtail;
my $filename;
my $tmpbasepid;
my $tmpbase;
my $tmpstdin;
my $tmphosts;
my $tmphostsstdout;
my $tmphostsstderr;
my $tmphostspid;
my $tmphostsexit;
my @host;
my $widest;
my $i;
my $line;
my $inplay;
my $pid;
my $ifreapfork;
my $waitpidres;
my $exitvalue;
my $exitresult;
my $oecwd;
my $pargroup;
my $out;
my $wrap;
my $onecommandpid;
my $cmd;
my $hfhostsfile;

# -----------------------------------------------------------------------------

if ($#ARGV == -1) {
	&print_version;
	warn ("\n");
	&print_usage;
	exit (0);
}

#  Sanity checks

if ($ENV{HFGENDB} eq "") {
	$ENV{HFGENDB} = '@@@ EXAMPLEDB @@@';
}

$oecwd = `pwd`;
chop ($oecwd);
if (-d 'hfgen.out') {
	$main::hfhostlists = "$oecwd/hfgen.out/$ENV{HFGENDB}/hostlists";
	$main::hfhosts = "$oecwd/hfgen.out/$ENV{HFGENDB}/hfdist";
} else {
	$main::hfhostlists = "/@@@ HFTOP @@@/gen/hfgen.out/$ENV{HFGENDB}/hostlists";
	$main::hfhosts = "/@@@ HFTOP @@@/gen/hfgen.out/$ENV{HFGENDB}/hfdist";
}

if (!(-d $main::hfhostlists)) {
	die "FATAL host list dir does not exist: '$main::hfhostlists'\n";
}

# -----------------------------------------------------------------------------

#  Parse options
#
#  Doesn't use getops because oe option letters may collide with (and
#  thus remove) command tail option letters.

while (defined ($arg = shift (@ARGV))) {

	if ($arg eq "-q") {
		$main::opt_q = 1;
		next;
	}
	if ($arg eq "-l") {
		$main::opt_l = 1;

		if ($main::opt_h == 1) {
			warn "FATAL -l incompatible with -h\n";
			&print_usage;
			exit (1);
		} elsif ($main::opt_L == 1) {
			warn "FATAL -l incompatible with -L\n";
			&print_usage;
			exit (1);
		} else {
			next;
		}
	}
	if ($arg eq "-L") {
		$main::opt_L = 1;

		if ($main::opt_h == 1) {
			warn "FATAL -L incompatible with -h\n";
			&print_usage;
			exit (1);
		} elsif ($main::opt_l == 1) {
			warn "FATAL -L incompatible with -l\n";
			&print_usage;
			exit (1);
		} else {
			next;
		}
	}
	if ($arg eq "-h") {
		$main::opt_h = 1;

		if ($main::opt_l == 1) {
			warn "FATAL -h incompatible with -l\n";
			&print_usage;
			exit (1);
		} elsif ($main::opt_L == 1) {
			warn "FATAL -h incompatible with -L\n";
			&print_usage;
			exit (1);
		} else {
			next;
		}
	}
	if ($arg eq "-j") {
		$main::opt_j = shift (@ARGV);

		if (!(($main::opt_j =~ /^\d+$/) && ($main::opt_j gt 0))) {
			warn "FATAL '$main::opt_j' not an integer greater than zero\n";
			&print_usage;
			exit (1);
		} else {
			next;
		}
	}
	if ($arg eq "-s") {
		$main::opt_s = 1;
		next;
	}
	if ($arg eq "-p") {
		$main::opt_p = 1;
		$opt_p_script = shift (@ARGV);

		if (!(-x $opt_p_script)) {
			warn "FATAL '$opt_p_script' not executable\n";
			&print_usage;
			exit (1);
		} else {
			next;
		}
	}
	@hostsetmath = &HF::Misc::dosetmath ($arg);
	last;
}

# Command tail is in remainder of @ARGV
@commandtail = @ARGV;

if ($#hostsetmath == -1) {
	warn "FATAL no hostspec given\n";
	&print_usage;
	exit (1);
}

if (!(defined ($commandtail[0]))) {
	warn "FATAL no command tail given\n";
	&print_usage;
	exit (1);
}

# -----------------------------------------------------------------------------

$tmpbasepid = $$;
$tmpbase = "/tmp/oe.$tmpbasepid";
mkdir ($tmpbase, 0700) || die "FATAL mkdir '$tmpbase' failed with error '$!'\n";
$ENV{OE_TMPBASE} = $tmpbase;
$0 = "oe $tmpbasepid master @hostsetmath @commandtail";

#  Two expected uses of signals: 1) "^C" of oe on command line, 2) "kill PID" for PID in CHECKPOINT list
sub inthandler {
	$filename = "$tmpbase/done";
	open (DONEFILE, '>', $filename) || warn "ERROR open '$filename' failed with error '$!'\n";
	close (DONEFILE);
	warn "\nFATAL quitting, to clean up do rm -r $tmpbase\n";
	$SIG{INT} = 'DEFAULT';
	kill ('INT', $$);
}
$SIG{INT} = \&inthandler;

if (open (FDNINE, '>&=9')) {
	print FDNINE "$tmpbase\n";
	close (FDNINE);
}

# -----------------------------------------------------------------------------

$filename = "$tmpbase/commandtail";
open (TMPCOMMANDTAIL, '>', $filename) || die "FATAL open '$filename' failed with error '$!'\n";
print TMPCOMMANDTAIL (join ("\n", @commandtail) . "\n");
close (TMPCOMMANDTAIL);

#  Save stdin data where it can be shared with all contacted machines

$tmpstdin = "$tmpbase/stdin";
open (SAVESTDIN, '>', $tmpstdin) || die "FATAL open '$tmpstdin' failed with error '$!'\n";
if (! -t STDIN) {
	while (<STDIN>) {
		print SAVESTDIN "$_";
	}
}
close (SAVESTDIN);

# -----------------------------------------------------------------------------

#  Process list of hosts

$tmphosts = "$tmpbase/hosts";
mkdir ($tmphosts) || die "FATAL mkdir '$tmphosts' failed with error '$!'\n";
$widest = 0;
$filename = "$tmpbase/hostlist";
open (TMPHOSTLIST, '>', $filename) || die "FATAL open '$filename' failed with error '$!'\n";
for ($i = 0;  $i <= $#hostsetmath;  $i++) {
	$host[$i] = { 'state' => 'never', 'raw' => $hostsetmath[$i], 'clean' => &uniquehostname ($hostsetmath[$i]), 'pid' => undef };
	print TMPHOSTLIST "$i\t" . $host[$i]->{'raw'} . "\t" . $host[$i]->{'clean'} . "\n";
	if (length ($host[$i]->{'clean'}) > $widest) {
		$widest = length ($host[$i]->{'clean'});
	}
	$filename = "$tmphosts/$host[$i]->{'clean'}";
	mkdir ($filename) || die "FATAL mkdir '$filename' failed with error '$!'\n";
}
close (TMPHOSTLIST);

# -----------------------------------------------------------------------------

#  Parallel group size

$pargroup = $#hostsetmath + 1;
if ((defined ($main::opt_j)) && ($pargroup > $main::opt_j)) {
	$pargroup = $main::opt_j;
}

$filename = "$tmpbase/pargroup";
open (PARGROUPFILE, '>', $filename) || die "FATAL open '$filename' failed with error '$!'\n";
print PARGROUPFILE "$pargroup\n";
close (PARGROUPFILE);

$filename = "$tmpbase/ready";
open (READYFILE, '>', $filename) || die "FATAL open '$filename' failed with error '$!'\n";
close (READYFILE);

# -----------------------------------------------------------------------------

#  Main loop, do wait/fork/print

$inplay = 0;
while ((grep {$_->{'state'} eq 'done'} @host) <= $#host) {

	$ifreapfork = 0;
	# Reap before fork so that status print, done after fork, shows new pids running not old pids missing
	while ($pid = waitpid (-1, &WNOHANG), (($pid != 0) && ($pid != -1))) {
		$waitpidres=$?;
REAPLOOP:	for ($i = 0;  $i <= $#host;  $i++) {
			if ($host[$i]->{'pid'} == $pid) {
				$host[$i]->{'exit'} = &WEXITSTATUS ($waitpidres);
				$host[$i]->{'state'} = 'done';
				$inplay--;
				$ifreapfork = 1;
				if (&WIFEXITED ($waitpidres)) {
					warn sprintf ("STATUS host %-${widest}s exited with value %d\n",
						$host[$i]->{'clean'}, $host[$i]->{'exit'});
				} elsif (&WIFSIGNALED ($waitpidres)) {
					warn sprintf ("STATUS host %-${widest}s terminated by signal %d\n",
						$host[$i]->{'clean'}, &WTERMSIG ($waitpidres));
				} else {
					die "FATAL host " . $host[$i]->{'clean'} . " mentioned by waitpid for unknown reason";
				}
				last REAPLOOP;
			}
		}
	}

	# Fork
	for ($i = 0;  $i <= $#host;  $i++) {
		if ($inplay == $pargroup) {
			last;
		}

		if ($host[$i]->{'state'} eq 'never') {
			$inplay++;
			$tmphostspid = "$tmphosts/$host[$i]->{'clean'}/oe_pid";
			warn sprintf ("STATUS host %-${widest}s starting, %3d of %3d originally named %s\n", $host[$i]->{'clean'}, ($i + 1), ($#host + 1), $host[$i]->{'raw'});

			$pid = fork ();
			if ($pid) {
				# Parent
				open (TMPHOSTSPIDFILE, '>', $tmphostspid) || die "FATAL open '$tmphostspid' failed with error '$!'\n";
				print TMPHOSTSPIDFILE "$pid\n";
				close (TMPHOSTSPIDFILE);
				$host[$i]->{'pid'} = $pid;
				$host[$i]->{'state'} = 'running';
				$ifreapfork = 1;
			} elsif ($pid == 0) {
				# Child
				$tmphostsexit = "$tmphosts/$host[$i]->{'clean'}/oe_exit";
				$0 = "oe $tmpbasepid wrap $host[$i]->{'clean'}";

				if (($main::opt_l != 1) && ($main::opt_L != 1) && ($main::opt_h != 1) && ($host[$i]->{'raw'} ne '_fork')) {
					if (system ("ping -c 2 -w 3 $host[$i]->{'raw'} > /dev/null") / 256) {
						warn "ERROR ping $host[$i]->{'raw'} not happy, dropping host\n";
						open (TMPHOSTSEXITFILE, '>', $tmphostsexit) || die "FATAL open '$tmphostsexit' failed with error '$!'\n";
						print TMPHOSTSEXITFILE "no ping\n";
						close (TMPHOSTSEXITFILE);
						unlink ($tmphostspid);	# no result check, may not exist
						exit (1);
					}
				}

				$tmphostsstdout = "$tmphosts/$host[$i]->{'clean'}/oe_stdout";
				$tmphostsstderr = "$tmphosts/$host[$i]->{'clean'}/oe_stderr";
				$ENV{OE_CWD} = $oecwd;
				$ENV{OE_HOST} = $host[$i]->{'raw'};
				$ENV{OE_HOSTDIR} = "$tmphosts/$host[$i]->{'clean'}";
				chdir ("$tmphosts/$host[$i]->{'clean'}") || die "FATAL chdir '$tmphosts/$host[$i]->{'clean'}' failed with error '$!'\n";

				$onecommandpid = fork ();
				if ($onecommandpid) {
					# Parent
					setpgrp ($onecommandpid, 0);

					#  Two expected uses of signals: 1) "^C" of oe on command line, 2) "kill PID" for PID in CHECKPOINT list
					sub termhandler2 {
						kill (-15, $onecommandpid);
					}
					$SIG{TERM} = \&termhandler2;

					sub inthandler2 {
						kill (-2, $onecommandpid);
					}
					$SIG{INT} = \&inthandler2;

					waitpid ($onecommandpid, 0);
					$exitresult=$?;

				} elsif ($onecommandpid == 0) {
					# Child
					$SIG{INT} = 'DEFAULT';
					$SIG{TERM} = 'DEFAULT';

					if ($host[$i]->{'raw'} eq '_fork') {
						exec ("@commandtail < $tmpstdin > $tmphostsstdout 2> $tmphostsstderr");
					} elsif ($main::opt_l == 1) {
						exec ("@commandtail $host[$i]->{'raw'} < $tmpstdin > $tmphostsstdout 2> $tmphostsstderr");
					} elsif ($main::opt_L == 1) {
						exec ("@commandtail < $tmpstdin > $tmphostsstdout 2> $tmphostsstderr");
					} elsif ($main::opt_h == 1) {
						$hfhostsfile = "$main::hfhosts/$host[$i]->{'raw'}/real.hfhosts";
						if ( ! -r $hfhostsfile) {
							die "FATAL can't read hfhosts file '$hfhostsfile'\n";
						}
						&HF::Hosts::eval_file_to_tmp ($hfhostsfile);
						if ($HF::Tmp::hosts->{'hosts'}[0]{'channel'} eq 'fork') {
							die "FATAL unusable channel 'fork'\n";
						} elsif ($HF::Tmp::hosts->{'hosts'}[0]{'channel'} eq 'null') {
							die "FATAL unusable channel 'null'\n";
						} elsif ($HF::Tmp::hosts->{'hosts'}[0]{'channel'} eq 'ssh') {
							$cmd = "ssh -o 'BatchMode yes' -o 'ConnectTimeout 5' -o 'ServerAliveInterval 15' -o 'SetupTimeOut 10' -x";
							if (exists ($HF::Tmp::hosts->{'hosts'}[0]{'sshopts'})) {
								$cmd .= " @{$HF::Tmp::hosts->{'hosts'}[0]{'sshopts'}}";
							}
							if ($HF::Tmp::hosts->{'hosts'}[0]{'hostname'} ne '') {
								$cmd .= " $HF::Tmp::hosts->{'hosts'}[0]{'hostname'}";
							} elsif ($HF::Tmp::hosts->{'hosts'}[0]{'ip'} ne '') {
								$cmd .= " $HF::Tmp::hosts->{'hosts'}[0]{'ip'}";
							} else {
								die "FATAL missing hostname and ip\n";
							}
							$cmd .= " @commandtail < $tmpstdin > $tmphostsstdout 2> $tmphostsstderr";
						} else {
							die "FATAL bogus channel '$HF::Tmp::hosts->{'hosts'}[0]{'channel'}'\n";
						}
						exec ($cmd);
					} else {
						exec ("ssh -o 'BatchMode yes' -o 'ConnectTimeout 5' -o 'ServerAliveInterval 15' -o 'SetupTimeOut 10' -x $host[$i]->{'raw'} @commandtail < $tmpstdin > $tmphostsstdout 2> $tmphostsstderr");
					}

				} else {
					die "FATAL fork error with error '$!'\n";
				}

				if ($exitresult == -1) {
					open (TMPHOSTSEXITFILE, '>', $tmphostsexit) || die "FATAL open '$tmphostsexit' failed with error '$!'\n";
					print TMPHOSTSEXITFILE "PerlSystemNoExecute=$!\n";
					close (TMPHOSTSEXITFILE);
					unlink ($tmphostspid);	# no result check, may not exist
					exit (1);
				} elsif (&WIFEXITED ($exitresult)) {
					$exitvalue = &WEXITSTATUS ($exitresult);
					open (TMPHOSTSEXITFILE, '>', $tmphostsexit) || die "FATAL open '$tmphostsexit' failed with error '$!'\n";
					print TMPHOSTSEXITFILE "$exitvalue\n";
					close (TMPHOSTSEXITFILE);
					unlink ($tmphostspid);	# no result check, may not exist
					exit ($exitvalue);
				} elsif (&WIFSIGNALED ($exitresult)) {
					open (TMPHOSTSEXITFILE, '>', $tmphostsexit) || die "FATAL open '$tmphostsexit' failed with error '$!'\n";
					print TMPHOSTSEXITFILE "KilledBySignal=" . &WTERMSIG ($exitresult) . "\n";
					close (TMPHOSTSEXITFILE);
					unlink ($tmphostspid);	# no result check, may not exist
					exit (1);
				} else {
					open (TMPHOSTSEXITFILE, '>', $tmphostsexit) || die "FATAL open '$tmphostsexit' failed with error '$!'\n";
					print TMPHOSTSEXITFILE "PerlSystemReturn=$exitvalue\n";
					close (TMPHOSTSEXITFILE);
					unlink ($tmphostspid);	# no result check, may not exist
					exit (1);
				}
			} else {
				die "FATAL fork error with error '$!'\n";
			}
		}
	}

	# Print status
	if (($ifreapfork == 1) && ($inplay > 0)) {
		$out = "CHECKPOINT waiting on $inplay children with results in $tmpbase\n";
		$wrap = 0;
		for ($i = 0;  $i <= $#host;  $i++) {
			if ($host[$i]->{'state'} eq 'running') {
				$wrap++;
				$out .= sprintf ("%12s %5d", $host[$i]->{'clean'}, $host[$i]->{'pid'});
				if ((($wrap % 4) == 0) && ($wrap != $inplay)) {
					$out .= "\n";
				} else {
					$out .= "  ";
				}
			}
		}
		warn "$out\n";
	}

	if ($ifreapfork == 0) {
		sleep (1);
	}
}

# -----------------------------------------------------------------------------

# Merge the outputs from each host into a report

if (!($main::opt_q)) {
	for ($i = 0;  $i <= $#host;  $i++) {
		$tmphostsstdout = "$tmphosts/$host[$i]->{'clean'}/oe_stdout";
		if (-r $tmphostsstdout) {
			open (HOSTSSTDOUT, '<', $tmphostsstdout) || die "FATAL open '$tmphostsstdout' failed with error '$!'\n";
			while (<HOSTSSTDOUT>) {
				$line = $_;
				chop ($line);
				$line = substr ($line, 0, ($HF::Globals::stderrcols - $widest - 1));
				printf ("%-${widest}s %s\n", $host[$i]->{'clean'}, $line);
			}
			close (HOSTSSTDOUT);
		}
		$tmphostsstderr = "$tmphosts/$host[$i]->{'clean'}/oe_stderr";
		if (-r $tmphostsstderr) {
			open (HOSTSSTDERR, '<', $tmphostsstderr) || die "FATAL open '$tmphostsstderr' failed with error '$!'\n";
			while (<HOSTSSTDERR>) {
				$line = $_;
				chop ($line);
				$line = substr ($line, 0, ($HF::Globals::stderrcols - $widest - 1));
				printf ("%-${widest}s %s\n", $host[$i]->{'clean'}, $line);
			}
			close (HOSTSSTDERR);
		}
	}
}

# -----------------------------------------------------------------------------

# Run the post-run-command
# This doesn't cd into per-host dir because then starting dir is hard to get back to

if ($main::opt_p) {
	for ($i = 0;  $i <= $#host;  $i++) {
		(!(system ($opt_p_script, $host[$i]->{'raw'}, $host[$i]->{'clean'}, "$tmphosts/$host[$i]->{'clean'}") / 256)) || warn "ERROR non-zero exit from post-run-command\n";
	}
}

# -----------------------------------------------------------------------------

$filename = "$tmpbase/done";
open (DONEFILE, '>', $filename) || die "FATAL open '$filename' failed with error '$!'\n";
close (DONEFILE);

if (!($main::opt_s)) {
	(!(system ('rm', '-r', $tmpbase) / 256)) || warn "ERROR 'rm -r $tmpbase' failed with error '$!'\n";
} else {
	warn `cat $tmpbase/hosts/*/oe_exit | fmt -66 | sed 's/^/STATUS exits /g'` .
"STATUS inputs more $tmpbase/{commandtail,hostlist,stdin}
STATUS outputs more $tmpbase/hosts/*/oe_{exit,stdout,stderr}
STATUS outputs retained, to delete do rm -r $tmpbase\n";
}

exit (0);

# -----

sub uniquehostname {

	my $string = shift;

	my $counter;
	my $result;

	$counter = 2;
	$result = $string;

	while (exists ($main::hostinuse{$result})) {
		$result = "$string.$counter";
		$counter++;
	}

	$main::hostinuse{$result} = 1;
	return ($result);
}
