#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

trap '
trap - INT
CLEANEXIT=1
kill -s INT $$' INT

set -o functrace
set -o errtrace

#-----

if ! TEMP=`getopt -o ewW -n "ERROR" -- "$@"`
then
	echo >&2
	print_usage
	CLEANEXIT=1
	exit 1
fi

eval set -- "$TEMP"

EXITS=0
WAITFOR=0
WAITFORONE=0

while :
do
	case "$1" in
		-e)	EXITS=1
			shift
			;;
		-w)	if [ -z "$WAITFORONE" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				WAITFOR=1
				shift
			fi
			;;
		-W)	if [ -z "$WAITFOR" ]
			then
				print_usage
				CLEANEXIT=1
				exit 1
			else
				WAITFORONE=1
				shift
			fi
			;;
		--)	shift
			break
			;;
		*)	echo "$0: ERROR parsing getopt" 1>&2
			exit 1
			;;
	esac
done

if [ \( $WAITFOR -eq 1 -o $WAITFORONE -eq 1 \) -a \( $EXITS -eq 1 -o -n "${1:-}" \) ]
then
	echo "FATAL -w and -W incompatible with other options" 1>&2
	CLEANEXIT=1
	exit 1
fi

#-----

function wrapped_exits {

	{	for i in $DIRS
		do
			if [ -s $i/oe_exit ]
			then
				cat $i/oe_exit
			fi
		done
	} | fmt | sed 's/^/EXITS /g'
}

function single_oewatch {

	BASE=''

	if [ -n "${1:-}" ]
	then
		BASE="$1"
	else
		LASTOE=$( (/bin/ls -1trd /tmp/oe.* 2> /dev/null || :) | tail -1)
		if [ -n "$LASTOE" ]
		then
			BASE="$LASTOE"
		fi
	fi

	if [ -z "$BASE" ]
	then
		echo "FATAL no /tmp/oe.* found" 1>&2
		CLEANEXIT=1
		exit 1
	fi

	if [ ! -d "$BASE" ]
	then
		echo "FATAL '$BASE' is not a dir" 1>&2
		CLEANEXIT=1
		exit 1
	fi

	if [ ! -r "$BASE" -o ! -x "$BASE" ]
	then
		echo "FATAL no permissions to read and search '$BASE'" 1>&2
		CLEANEXIT=1
		exit 1
	fi

	while [ ! -e "$BASE/ready" ]
	do
		echo "STATUS oe session '$BASE' not ready for oewatch" 1>&2
		sleep 1
	done

	# cd command tries to determine present directory before switching, but it may be deleted
	cd "$BASE/hosts" 2> /dev/null

	DIRS=$(/bin/ls -1 | sort -t. -k 1,1 -n -k 2,2)

	#-----

	if [ $EXITS -eq 1 -o -f $BASE/done ]
	then
		echo "$(date)    ${BASE}"
		sed 's/^/COMMANDTAIL /g' < ../commandtail
		if [ -f ../stdin ]
		then
			if [ `sl -d ../stdin | awk -F $'\t' '{print $7}'` -lt 300 ]
			then
				strings ../stdin | sed 's/^/STDIN /g'
			else
				echo "STATUS stdin too large to display"
			fi
		else
			echo "STATUS no stdin"
		fi
		if /bin/ls -1 */oe_exit > /dev/null 2>&1
		then
			if [ $EXITS -eq 1 ]
			then
				for i in $DIRS
				do
					printf '%-20s  %s\n' $i "`cat $i/oe_exit 2> /dev/null`"
				done
			fi
			wrapped_exits
		else
			echo "STATUS no exit files"
		fi
		CLEANEXIT=1
		exit 0
	fi

	#-----

	set -- $(stty size)
	SCRROWS=$1
	SCRCOLS=$2
	COLS=$SCRCOLS
	SHOWROWS=$(( SCRROWS - 2 ))
	HEADROWS=$(( SCRROWS - 1 ))

	if ! PARGROUPCOUNT=$(< ../pargroup) 2>&1
	then
		# oe finished quickly and directory deleted out from under us
		return 0
	fi

	SPACER='
'
	NOTSHOWN=''
	ROWS=$(( (SHOWROWS - PARGROUPCOUNT) / PARGROUPCOUNT ))
	if [ $ROWS -eq 0 ]
	then
		SPACER=''
		ROWS=$(( SHOWROWS / PARGROUPCOUNT ))
		if [ $ROWS -eq 0 ]
		then
			ROWS=1
			NOTSHOWN="    $(( PARGROUPCOUNT - SHOWROWS )) not shown"
		fi
	fi

	#-----

	while :
	do
		if ! pwd -P > /dev/null 2>&1
		then
			# oe finished and directory deleted out from under us
			return 0
		fi

		echo "$(date)    ${BASE}${NOTSHOWN}" > /tmp/oewatch.$$
		for i in $DIRS
		do
			(	if [ $i/oe_stderr -nt $i/oe_stdout ]
				then
					tail -$ROWS $i/oe_stdout 2> /dev/null || :
					tail -$ROWS $i/oe_stderr 2> /dev/null || :
				else
					tail -$ROWS $i/oe_stderr 2> /dev/null || :
					tail -$ROWS $i/oe_stdout 2> /dev/null || :
				fi
			) | tail -$ROWS | sed "s|^|$i  |" | cut -c1-$COLS >> /tmp/oewatch.$$
			echo -n "$SPACER" >> /tmp/oewatch.$$
		done

		clear
		head -$HEADROWS /tmp/oewatch.$$

		if [ -f $BASE/done ]
		then
			# oe finished
			wrapped_exits
			return 0
		fi

		sleep 1
	done
}

#-----

if [ $WAITFOR -eq 1 -o $WAITFORONE -eq 1 ]
then
	echo "STATUS waiting for live oe session to appear"

	# Exited by ^C
	while :
	do
		LASTOE=$( (/bin/ls -1trd /tmp/oe.* 2> /dev/null || :) | tail -1)

		if [ -n "$LASTOE" -a -d "$LASTOE" ]
		then
			BASE=$LASTOE
			if [ ! -f $BASE/done ]
			then
				single_oewatch $BASE

				if [ $WAITFORONE -eq 1 ]
				then
					CLEANEXIT=1
					exit 0
				else
					echo "STATUS waiting for live oe session to appear"
				fi
			fi
		fi

		sleep 1
	done
else
	RESULT=0
	single_oewatch "${1:-}"
	CLEANEXIT=1
	exit $?
fi

CLEANEXIT=1
exit 0
