#!/bin/bash -euET

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#
#  Usage:
#
#	HFGENDB=foo ./get-usernames
#

CLEANEXIT=0
trap '
SAVEDSTATUS=$?
set +x
if [ "${CLEANEXIT:-0}" -ne 1 ]
then
	echo "${0##*/}: FATAL Unexpected return value of ${SAVEDSTATUS:-?}, line number ${ERR_LINENO:-?}, quitting" 1>&2
	exit $SAVEDSTATUS
fi' EXIT

trap 'ERR_LINENO=$LINENO' ERR

set -o pipefail

#-----

: ${HFGENDB:?FATAL env var HFGENDB not defined, should be name of gen database, quitting}

mysql -uhfgen -p$(< ~/.hfgen-password) --skip-column-names $HFGENDB << EOF | \
awk -F'	' '
{
	name = ""
	if ($2 != "") {
		name = sprintf ("%s %s", name, $2)
	}
	if ($3 != "") {
		name = sprintf ("%s %s", name, $3)
	}
	if ($4 != "") {
		name = sprintf ("%s %s", name, $4)
	}
	if ($5 != "") {
		name = sprintf ("%s %s", name, $5)
	}
	if ($6 != "") {
		name = sprintf ("%s %s", name, $6)
	}
	if ($7 != "") {
		name = sprintf ("%s %s", name, $7)
	}
	gsub (/^ /, "", name)
	gsub (/ $/, "", name)
	printf ("%s\t%s\n", $1, name)
}'
select usernames.username, persons.honorific, persons.given, persons.middle1, persons.middle2, persons.family, persons.generation
from persons, usernames
where usernames.usertype = 'person'
	and usernames.userdata = persons.emplid
order by usernames.username;
EOF

#-----

CLEANEXIT=1
exit 0
