
#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#  Usage:
#
#	mkt_names [-n] [1]
#
#	[test dist]
#
#	mkt_names [-n] 2
#
#	[test dist]
#
#	mkt_names [-n] 3
#
#  -n  Avoids creating files and directories with unreadable modes

use Socket;
use Carp;

srand (999);

sub dirpart {

	my $path = shift;

	$path =~ s,(^|/)[^/]+$,,;
	if ($path eq '') {
		$path = '.';
	}
	return ($path);
}

sub basepart {

	my $path = shift;

	if ($path eq '/') {
		return ($path);
	} else {
		$path =~ s,/$,,;
		$path =~ s,^.*/,,;
		return ($path);
	}
}

sub makesocket {

	my $socketname = shift;

	my $shortsocketname;
	my $shortsocketdir;

	# silent truncation of pathnames longer than around 108 in bind(); chdir to dir first to shorten
	$shortsocketname = &basepart ($socketname);
	$shortsocketdir = &dirpart ($socketname);
	if (($socketname eq '/') || ($shortsocketname eq '/')) {
		die "socket name should not be slash";
	}
	opendir (MAKESOCKETDIRH, '.') || die "$!";
	chdir ($shortsocketdir) || die "$!";
	unlink ($shortsocketname);
	socket (theserver, &main::PF_UNIX, &main::SOCK_STREAM, 0) || die "$!";
	bind (theserver, &main::sockaddr_un ($shortsocketname)) || die "$!";
	chdir (MAKESOCKETDIRH) || die "$!";
	closedir (MAKESOCKETDIRH);
}

sub chownifroot {

	my $owner = shift;
	my $group = shift;
	my $filename = shift;

	if ($> == 0) {
		return (chown ($owner, $group, $filename));
	} else {
		return (1);
	}
}

sub touch {

	$filename = shift;
	$contents = shift;

	$filename =~ s#^(\s)#./$1#;
	open (TOUCH, '>', "$filename\0") || die "$!, filename $filename";
	if (defined ($contents)) {
		print TOUCH $contents || die "$!";
	}
	close (TOUCH) || die "$!";
	&chownifroot (0, 0, $filename) || die "$!";
	chmod (0644, $filename) || die "$!";
	utime (1000000000, 1000000000, $filename) || die "$!";
}

$opt_n = 0;
if ($ARGV[0] eq '-n') {
	shift (@ARGV);
	$opt_n = 1;
}

if (($ARGV[0] eq '') || ($ARGV[0] eq '1')) {

	# make test directory under cd

	chmod (0755, 't_names/unreadabledir');
	(!(system ('rm', '-rf', 't_names') / 256)) || die "$!";
	mkdir ('t_names', 0755) || die "$!";
	&chownifroot (0, 0, 't_names') || die "$!";
	chdir ('t_names') || die "$!";

	# one of each kind

	&touch ('file');
	&chownifroot (1, 2, 'file') || die "$!";
	utime (1000000000, 1000000000, 'file') || die "$!";

	mkdir ('dir', 0755) || die "$!";
	&chownifroot (3, 4, 'dir') || die "$!";
	chmod (0755, 'dir') || die "$!";
	utime (1000000000, 1000000000, 'dir') || die "$!";
	
	mkdir ('.dir', 0755) || die "$!";
	&chownifroot (3, 4, '.dir') || die "$!";
	chmod (0755, '.dir') || die "$!";
	utime (1000000000, 1000000000, '.dir') || die "$!";
	
	symlink ('file', 'symlink') || die "$!";

	(!(system ('mkfifo', 'pipe') / 256)) || die "$!";
	&chownifroot (5, 6, 'pipe') || die "$!";
	chmod (0755, 'pipe') || die "$!";
	utime (1000000000, 1000000000, 'pipe') || die "$!";

	&makesocket ('socket') || die "$!";
	&chownifroot (15, 16, 'socket') || die "$!";
	chmod (0640, 'socket') || die "$!";
	utime (1000000000, 1000000000, 'socket') || die "$!";

	# silent truncation of pathnames longer than around 108 in bind(); chdir to dir first to shorten
	foreach $i (1, 10, 20, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 50, 60, 70, 80, 90, 100, 120, 130, 150, 200) {
		$longdir = 'socketdir_' . 'a'x$i;
		mkdir ($longdir, 0755) || die "$!";
		&chownifroot (3, 4, $longdir) || die "$!";
		chmod (0755, $longdir) || die "$!";

		$longpath = $longdir . '/socket___';
		&makesocket ($longpath) || die "$!";
		&chownifroot (15, 16, $longpath) || die "$!";
		chmod (0640, $longpath) || die "$!";
		utime (1000000000, 1000000000, $longpath) || die "$!";

		utime (1000000000, 1000000000, $longdir) || die "$!";
	}

	(!(system ('mkdir', '-p', 'dev/.udev/links/cdrom') / 256)) || die "$!";
	symlink ("/devices/pci0000:00/0000:00:1f.1/host0/target0:0:0/0:0:0:0/block/sr0", "dev/.udev/links/cdrom/b11:0") || die "$!";
	utime (1000000000, 1000000000, 'dev/.udev/links/cdrom') || die "$!";
	utime (1000000000, 1000000000, 'dev/.udev/links') || die "$!";
	utime (1000000000, 1000000000, 'dev/.udev') || die "$!";
	utime (1000000000, 1000000000, 'dev') || die "$!";

	(!(system ('mkdir', '-p', 'e9/d/c/b/a') / 256)) || die "$!";
	utime (1000000000, 1000000000, 'e9/d/c/b/a') || die "$!";
	utime (1000000000, 1000000000, 'e9/d/c/b') || die "$!";
	utime (1000000000, 1000000000, 'e9/d/c') || die "$!";
	utime (1000000000, 1000000000, 'e9/d') || die "$!";
	utime (1000000000, 1000000000, 'e9') || die "$!";

	if ($> == 0) {
		(!(system ('mknod', 'block', 'b', '1', '1') / 256)) || die "$!";
		&chownifroot (7, 8, 'block') || die "$!";
		chmod (0755, 'block') || die "$!";
		utime (1000000000, 1000000000, 'block') || die "$!";

		(!(system ('mknod', 'char', 'c', '1', '1') / 256)) || die "$!";
		&chownifroot (9, 10, 'char') || die "$!";
		chmod (0755, 'char') || die "$!";
		utime (1000000000, 1000000000, 'char') || die "$!";

		(!(system ('mkfifo', 'pipesugt') / 256)) || die "$!";
		&chownifroot (5, 6, 'pipesugt') || die "$!";
		chmod (07113, 'pipesugt') || die "$!";
		utime (1000000000, 1000000000, 'pipesugt') || die "$!";
	}

	# permissions problems

	if ($opt_n == 0) {
		&touch ('unreadablefile', "put some contents in the file\n");
		chmod (0000, 'unreadablefile') || die "$!";
		utime (1000000000, 1000000000, 'unreadablefile') || die "$!";

		mkdir ('unreadabledir', 0755) || die "$!";
		&touch ('unreadabledir/inside');
		&chownifroot (0, 0, 'unreadabledir') || die "$!";
		chmod (0000, 'unreadabledir') || die "$!";
		utime (1000000000, 1000000000, 'unreadabledir') || die "$!";
	}

	# shell metachars

	&touch (' ');
	&touch ('space between');
	&touch ("newline\nbetween");
	&touch ("tab\tbetween");

	&touch ("lowbinary\001\002\003");
	&touch ("delete\177");
	&touch ("highbinary\375\376\377");

	# privledged mode bits

	&touch ('setuid');
	chmod (04755, 'setuid') || die "$!";

	&touch ('setgid');
	chmod (02755, 'setgid') || die "$!";

	&touch ('sticky');
	chmod (01755, 'sticky') || die "$!";

	&touch ('everythingset');
	chmod (07777, 'everythingset') || die "$!";

	# lower half except NUL, forward slash, 0-9, A-Z, a-z

# XXX too long when quoted by visname for omar renaming
###	&touch ("\001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023\024\025\026\027\030\031\032\033\034\035\036\037\040\041\042\043\044\045\046\047\050\051\052\053\054\055\056\072\073\074\075\076\077\100\133\134\135\136\137\140\173\174\175\176\177");

	&touch ("\001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023\024\025");
	&touch ("\026\027\030\031\032\033\034\035\036\037\040\041\042\043\044\045\046\047\050\051\052");
	&touch ("\053\054\055\056\072\073\074\075\076\077\100\133\134\135\136\137\140\173\174\175\176\177");

	# upper half complete

# XXX too long when quoted by visname for omar renaming
###	&touch ("\200\201\202\203\204\205\206\207\210\211\212\213\214\215\216\217\220\221\222\223\224\225\226\227\230\231\232\233\234\235\236\237\240\241\242\243\244\245\246\247\250\251\252\253\254\255\256\257\260\261\262\263\264\265\266\267\270\271\272\273\274\275\276\277\300\301\302\303\304\305\306\307\310\311\312\313\314\315\316\317\320\321\322\323\324\325\326\327\330\331\332\333\334\335\336\337\340\341\342\343\344\345\346\347\350\351\352\353\354\355\356\357\360\361\362\363\364\365\366\367\370\371\372\373\374\375\376\377");

	&touch ("\200\201\202\203\204\205\206\207\210\211\212\213\214\215\216\217\220\221\222\223\224");
	&touch ("\225\226\227\230\231\232\233\234\235\236\237\240\241\242\243\244\245\246\247\250\251");
	&touch ("\252\253\254\255\256\257\260\261\262\263\264\265\266\267\270\271\272\273\274\275\276");
	&touch ("\277\300\301\302\303\304\305\306\307\310\311\312\313\314\315\316\317\320\321\322\323");
	&touch ("\324\325\326\327\330\331\332\333\334\335\336\337\340\341\342\343\344\345\346\347\350");
	&touch ("\351\352\353\354\355\356\357\360\361\362\363\364\365\366\367\370\371\372\373\374\375\376\377");

	&touch ('.dotfile');

	&touch ('-i');

	&touch ('__FILE__');
	&touch ('__LINE__');
	&touch ('__PACKAGE__');
	&touch ('__END__');
	&touch ('__DATA__');
	&touch ("\004");
	&touch ("\032");
	&touch ('v65');
	&touch ('v65.66');
	&touch ('65.66.67');
	&touch (v9786);
	&touch ("\x{263a}\x{263a}");
	&touch ('\x{263a}');

	# long filenames

# XXX filename now too long??
	warn "XXX why are these long filenames no longer working??";
###	&touch ('........10........20........30........40........50........60........70........80........90.......100.......110.......120.......130.......140.......150.......160.......170.......180.......190.......200.......210.......220.......230.......240.......250.2345');
###	&touch ('........10........20........30........40........50........60........70........80........90.......100.......110.......120.......130.......140.......150.......160.......170.......180.......190.......200.......210.......220.......230.......240.......250.234');
###	&touch ('........10........20........30........40........50........60........70........80........90.......100.......110.......120.......130.......140.......150.......160.......170.......180.......190.......200.......210.......220.......230.......240.......250.23X');
###	&touch ('........10........20........30........40........50........60........70........80........90.......100.......110.......120.......130.......140.......150.......160.......170.......180.......190.......200.......210.......220.......230.......240.......250.23');
###	&touch ('........10........20........30........40........50........60........70........80........90.......100.......110.......120.......130.......140...');
	&touch ('........10........20........30........40........50........60........70........80........90.......100.......110.......120.......130.......');

	# symlink with shell metachars

# XXX too long when quoted by visname for omar renaming
###	symlink ('file', "symlink \001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023\024\025\026\027\030\031\032\033\034\035\036\037\040\041\042\043\044\045\046\047\050\051\052\053\054\055\056\072\073\074\075\076\077\100\133\134\135\136\137\140\173\174\175\176\177") || die "$!";

	symlink ('file', "symlink \001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023") || die "$!";
	symlink ('file', "symlink \024\025\026\027\030\031\032\033\034\035\036\037\040\041\042\043\044\045\046") || die "$!";
	symlink ('file', "symlink \047\050\051\052\053\054\055\056\072\073\074\075\076\077\100\133\134\135\136") || die "$!";
	symlink ('file', "symlink \137\140\173\174\175\176\177") || die "$!";

	# symlink target with shell metachars

# XXX too long when quoted by visname for omar renaming
###	symlink ("\001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023\024\025\026\027\030\031\032\033\034\035\036\037\040\041\042\043\044\045\046\047\050\051\052\053\054\055\056\072\073\074\075\076\077\100\133\134\135\136\137\140\173\174\175\176\177", 'uglytarget') || die "$!";

	symlink ("\001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023\024\025", 'uglytarget1') || die "$!";
	symlink ("\026\027\030\031\032\033\034\035\036\037\040\041\042\043\044\045\046\047\050\051\052", 'uglytarget2') || die "$!";
	symlink ("\053\054\055\056\072\073\074\075\076\077\100\133\134\135\136\137\140\173\174\175\176\177", 'uglytarget3') || die "$!";

	# symlink to nowhere

	symlink ('roadtonowhere', 'nosuchfilename') || die "$!";

	# quoting perl open

	&touch ('> writeto');
	&touch ('< readfrom');
	&touch (' spaceinfront');
	&touch ('  twospaces');
	&touch ('	tab');
	&touch (' 	spacetab');
	&touch ('spaceatend ');
	&touch ('| od');
	&touch ('ls |');
	&touch ('./-');
	&touch ('./--');
	&touch ('./---');
	&touch ('./>');
	&touch ('./<');

	# file with holes

	&touch ('holefile');
	open (HOLEFILE, '>', 'holefile') || die "$!";
	seek (HOLEFILE, 100000, 0) || die "$!";
	print HOLEFILE "it's lonely out here\n";
	close (HOLEFILE) || die "$!";
	utime (1000000000, 1000000000, 'holefile') || die "$!";

	# things to change in pass two

	symlink ('wrongtarget', 'symlinkchangingtarget') || die "$!";

	&makesocket ('deletethissocket') || die "$!";
	&chownifroot (105, 106, 'deletethissocket') || die "$!";
	chmod (0640, 'deletethissocket') || die "$!";
	utime (1000000000, 1000000000, 'deletethissocket') || die "$!";

	&touch ('deletethisfile');

# XXX too long when quoted by visname for omar renaming
###	&touch ("deletemetachars \001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023\024\025\026\027\030\031\032\033\034\035\036\037\040\041\042\043\044\045\046\047\050\051\052\053\054\055\056\072\073\074\075\076\077\100\133\134\135\136\137\140\173\174\175\176\177");

	&touch ("deletemetachars \001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021");
	&touch ("deletemetachars \022\023\024\025\026\027\030\031\032\033\034\035\036\037\040\041\042");
	&touch ("deletemetachars \043\044\045\046\047\050\051\052\053\054\055\056\072\073\074\075\076");
	&touch ("deletemetachars \077\100\133\134\135\136\137\140\173\174\175\176\177");

	&touch ('filebecomesadir');

	mkdir ('dirbecomesafile', 0755) || die "$!";
	&chownifroot (0, 0, 'dirbecomesafile') || die "$!";
	chmod (0755, 'dirbecomesafile') || die "$!";
	utime (1000000000, 1000000000, 'dirbecomesafile') || die "$!";

	&touch ('thisfileshrinks');
	open (SHRINKFILE, '>', 'thisfileshrinks') || die "$!";
	print SHRINKFILE "abcde\n";
	close (SHRINKFILE) || die "$!";
	utime (1000000000, 1000000000, 'thisfileshrinks') || die "$!";

	&touch ('thisfilegrows');
	open (GROWFILE, '>', 'thisfilegrows') || die "$!";
	print GROWFILE "........10........20........30........40........50........60........70........80........90.......100.......110.......120.......130.......140.......150.......160.......170.......180.......190.......200.......210.......220.......230.......240.......250.2345\n";
	close (GROWFILE) || die "$!";
	utime (1000000000, 1000000000, 'thisfilegrows') || die "$!";

	symlink ('0', 'zerotarget') || die "$!";

	&touch ('sameallbutcontents');
	open (GROWFILE, '>', 'sameallbutcontents') || die "$!";
	print GROWFILE "abc\n";
	close (GROWFILE) || die "$!";
	utime (1000000000, 1000000000, 'sameallbutcontents') || die "$!";

	mkdir ('dir2', 0755) || die "$!";
	&chownifroot (443, 444, 'dir2') || die "$!";
	chmod (0755, 'dir2') || die "$!";
	utime (1000000000, 1000000000, 'dir2') || die "$!";
	
	&touch ('dir2/file', 'these are some file contents');
	&chownifroot (441, 442, 'dir2/file') || die "$!";
	utime (1000000000, 1000000000, 'dir2/file') || die "$!";

	utime (1000000000, 1000000000, 'dir2') || die "$!";

	foreach $i ('blah_11', 'blah_11-blah', 'blah_12', 'blah_12-blah', 'blah_13', 'blah_13-blah', ) {
		mkdir ($i, 0755) || die "$!";
		&chownifroot (1443, 1444, $i) || die "$!";
		chmod (0755, $i) || die "$!";
		&touch ("$i/file_inside", "some file contents");
		utime (1000000000, 1000000000, "$i/file_inside") || die "$!";
		utime (1000000000, 1000000000, $i) || die "$!";
	}

	# hard links

	&touch ('hardlinkedto');
	link ('hardlinkedto', 'hardlink1') || die "$!";
	link ('hardlinkedto', 'hardlink2') || die "$!";
	link ('hardlinkedto', 'hardlink3') || die "$!";
	utime (1000000000, 1000000000, 'hardlinkedto') || die "$!";

	# hard links that cross directories

	mkdir ('linkdir1', 0755) || die "$!";
	&chownifroot (0, 0, 'linkdir1') || die "$!";
	chmod (0755, 'linkdir1') || die "$!";

	mkdir ('linkdir2', 0755) || die "$!";
	&chownifroot (0, 0, 'linkdir2') || die "$!";
	chmod (0755, 'linkdir2') || die "$!";

	&touch ('linkdir1/a');
	&touch ('linkdir1/b');

	link ('linkdir1/a', 'linkdir2/c') || die "$!";
	link ('linkdir1/b', 'linkdir2/d') || die "$!";

	link ('linkdir1/a', 'linkdir2/e') || die "$!";
	link ('linkdir1/a', 'linkdir2/f') || die "$!";

	utime (1000000000, 1000000000, 'linkdir1/a') || die "$!";
	utime (1000000000, 1000000000, 'linkdir1/b') || die "$!";
	utime (1000000000, 1000000000, 'linkdir2/c') || die "$!";
	utime (1000000000, 1000000000, 'linkdir2/d') || die "$!";
	utime (1000000000, 1000000000, 'linkdir2/e') || die "$!";
	utime (1000000000, 1000000000, 'linkdir2/f') || die "$!";
	utime (1000000000, 1000000000, 'linkdir1') || die "$!";
	utime (1000000000, 1000000000, 'linkdir2') || die "$!";

	# hard links that mutate

	&touch ('hardlinkmutate1');
	link ('hardlinkmutate1', 'hardlinkmutate2') || die "$!";
	link ('hardlinkmutate1', 'hardlinkmutate4') || die "$!";
	&touch ('hardlinkmutate3');
	&touch ('hardlinkmutate5');
	utime (1000000000, 1000000000, 'hardlinkmutate1') || die "$!";
	utime (1000000000, 1000000000, 'hardlinkmutate3') || die "$!";
	utime (1000000000, 1000000000, 'hardlinkmutate5') || die "$!";

	# hard link file mutates into a dir

	&touch ('hl-a');
	link ('hl-a', 'hl-b') || die "$!";
	link ('hl-a', 'hl-c') || die "$!";
	utime (1000000000, 1000000000, 'hl-a') || die "$!";

	# hard link inside a deleted dir

	mkdir ('prod', 0755) || die "$!";
	mkdir ('prod/.hg', 0755) || die "$!";
	&touch ('prod/.hg/requires');
	utime (1000000000, 1000000000, 'prod/.hg/requires') || die "$!";
	utime (1000000000, 1000000000, 'prod/.hg') || die "$!";
	utime (1000000000, 1000000000, 'prod') || die "$!";

	# hfdist protocol keywords

	&touch ('walk');
	&touch ('suspendcompare');
	&touch ('nomoresourcetreewalk');
	&touch ('abort');
	&touch ('writethisfile');
	&touch ('shutdown');
	&touch ('goodbye');
	&touch ('opt_x');
	&touch ("\t");

	&touch ('filewritten');
	utime (1000000000, 1000000000, 'filewritten') || die "$!";
	&touch ('filewritten.1');
	utime (1000000000, 1000000000, 'filewritten.1') || die "$!";
	&touch ('filewritten.2');
	utime (1000000000, 1000000000, 'filewritten.2') || die "$!";

	# simple test cases which have been useful

	&touch ('foo');
	chmod (0610, 'foo') || die "$!";
	open (FOO, '>', 'foo') || die "$!";
	print FOO "foo 123\n";
	close (FOO) || die "$!";
	utime (1000000000, 1000000000, 'foo') || die "$!";

	&touch ('baz');
	chmod (0610, 'baz') || die "$!";
	open (BAZ, '>', 'baz') || die "$!";
	print BAZ "baz\n";
	close (BAZ) || die "$!";
	utime (1000000000, 1000000000, 'baz') || die "$!";

	link ('baz', 'quux') || die "$!";

	&touch ('foo@');
	chmod (0610, 'foo@') || die "$!";
	open (FOOAT, '>', 'foo@') || die "$!";
	print FOOAT "foo 123\n";
	close (FOOAT) || die "$!";
	utime (1000000000, 1000000000, 'foo@') || die "$!";

	&touch ('baz@');
	chmod (0610, 'baz@') || die "$!";
	open (BAZAT, '>', 'baz@') || die "$!";
	print BAZAT "baz\n";
	close (BAZAT) || die "$!";
	utime (1000000000, 1000000000, 'baz@') || die "$!";

	link ('baz@', 'quux@') || die "$!";
# hard link to above test dir, so is not found in scans
###	link ('foo', '../bar') || die "$!";
	symlink ('def', 'abc@') || die "$!";
	symlink ('jkl', 'ghi') || die "$!";

	mkdir ('a@', 0755) || die "$!";
	mkdir ('a@/b@', 0755) || die "$!";
	mkdir ('a@/b@/c@', 0755) || die "$!";
	&touch ('a@/one@');
	&touch ('a@/b@/two@');
	&touch ('a@/b@/c@/three@');
	utime (1000000000, 1000000000, 'a@/b@/c@') || die "$!";
	utime (1000000000, 1000000000, 'a@/b@') || die "$!";
	utime (1000000000, 1000000000, 'a@') || die "$!";

	mkdir ("tab\tdir", 0755) || die "$!";
	mkdir ("tab\tdir/.hg", 0755) || die "$!";
	utime (1000000000, 1000000000, "tab\tdir/.hg") || die "$!";
	utime (1000000000, 1000000000, "tab\tdir") || die "$!";

	# informally generated collection of names, not tested for reachability or coverage
	foreach $i ('basename', 'basename.1', 'basename.2', '_OMAR_basename', '_OMAR_basename.1', '_OMAR_basename.2', '_OMAR__OMAR_basename', '_OMAR__OMAR_basename.1', 'basename.hftmp', 'basename.hftmp.1', 'basename.hftmp.2', '_OMAR_basename.hftmp', '_OMAR_basename.hftmp.1', '_OMAR__OMAR_basename.hftmp.1.hftmp', '_OMAR_b@', '_OMAR_c@', '_OMAR_.hg', '_OMAR__OMAR_basename.1.1', '_OMAR__OMAR_basename.2', '_OMAR__OMAR_basename.2.1', '_OMAR_one@', '_OMAR_three@', '_OMAR_two@',
'basename10.@......20........30........40........50',
'basename10.@......20........30........40........50........60',
'basename10.@......20........30........40........50........60........70') {
		&touch ($i);
		utime (1000000000, 1000000000, $i) || die "$!";
	}

	mkdir ('.hg', 0755) || die "$!";
	&touch ('.hg/hgrc');
	utime (1000000000, 1000000000, '.hg') || die "$!";

	# non-files which get deleted, then dirs are deleted by hg

	mkdir ('sub1', 0755) || die "$!";
	mkdir ('sub2', 0755) || die "$!";
	mkdir ('sub3', 0755) || die "$!";

	symlink ('nowhere', 'sub1/symlink') || die "$!";

	if ($> == 0) {
		(!(system ('mknod', 'sub2/block', 'b', '1', '1') / 256)) || die "$!";
		&chownifroot (7, 8, 'sub2/block') || die "$!";
		chmod (0644, 'sub2/block') || die "$!";
		utime (1000000000, 1000000000, 'sub2/block') || die "$!";

		link ('sub2/block', 'sub3/block') || die "$!";
	}

	utime (1000000000, 1000000000, 'sub1') || die "$!";
	utime (1000000000, 1000000000, 'sub2') || die "$!";
	utime (1000000000, 1000000000, 'sub3') || die "$!";

	# subdirs of sgid dir

	mkdir ('sgiddir', 0755) || die "$!";
	mkdir ('sgiddir/foo', 0755) || die "$!";
	&chownifroot (0, 1, 'sgiddir') || die "$!";
	&chownifroot (0, 0, 'sgiddir/foo') || die "$!";
	chmod (02775, 'sgiddir') || die "$!";
	utime (1000000000, 1000000000, 'sgiddir/foo') || die "$!";
	utime (1000000000, 1000000000, 'sgiddir') || die "$!";

	# complicated? bug

	mkdir ('layers2-bbb', 0755) || die "$!";
	mkdir ('layers2-bbb/foo', 0755) || die "$!";
	utime (1000000000, 1000000000, 'layers2-bbb') || die "$!";
	utime (1000000000, 1000000000, 'layers2-bbb/foo') || die "$!";

	# just random

	mkdir ('random', 0755) || die "$!";
	for ($i = 0;  $i < 100;  $i++) {
		$name = '';
		for ($j = 0;  $j < 80;  $j++) {
			$cn = int (rand (256));
			if (($cn == 0) || ($cn == 47)) {	# NUL, forward slash
				$cn = 1;
			}
			$name .= chr ($cn);
		}
		&touch ("random/$name", "a\n");
	}
	utime (1000000000, 1000000000, 'random') || die "$!";

	# top level directory mtime

	utime (1000000000, 1000000000, '.') || die "$!";

} elsif ($ARGV[0] eq '2') {

	chdir ('t_names') || die "$!";

	unlink ('deletethisfile') || die "$!";

	unlink ('deletethissocket') || die "$!";

	unlink ('-i') || die "$!";

# XXX too long when quoted by visname for omar renaming
###	unlink ("deletemetachars \001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023\024\025\026\027\030\031\032\033\034\035\036\037\040\041\042\043\044\045\046\047\050\051\052\053\054\055\056\072\073\074\075\076\077\100\133\134\135\136\137\140\173\174\175\176\177") || die "$!";

	unlink ("deletemetachars \001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021") || die "$!";
	unlink ("deletemetachars \022\023\024\025\026\027\030\031\032\033\034\035\036\037\040\041\042") || die "$!";
	unlink ("deletemetachars \043\044\045\046\047\050\051\052\053\054\055\056\072\073\074\075\076") || die "$!";
	unlink ("deletemetachars \077\100\133\134\135\136\137\140\173\174\175\176\177") || die "$!";

	unlink ('filebecomesadir') || die "$!";
	mkdir ('filebecomesadir', 0755) || die "$!";
	&chownifroot (0, 0, 'filebecomesadir') || die "$!";
	chmod (0755, 'filebecomesadir') || die "$!";
	utime (1000001000, 1000001000, 'filebecomesadir') || die "$!";

	rmdir ('dirbecomesafile') || die "$!";
	&touch ('dirbecomesafile');

	&touch ('thisfileshrinks');
	open (SHRINKFILE, '>', 'thisfileshrinks') || die "$!";
	print SHRINKFILE "a\n";
	close (SHRINKFILE) || die "$!";
	utime (1000001000, 1000001000, 'thisfileshrinks') || die "$!";

	&touch ('thisfilegrows');
	open (GROWFILE, '>>', 'thisfilegrows') || die "$!";
	print GROWFILE "........10........20........30........40........50........60........70........80........90.......100.......110.......120.......130.......140.......150.......160.......170.......180.......190.......200.......210.......220.......230.......240.......250.......260.......270.......280.......290.......300\n";
	close (GROWFILE) || die "$!";
	utime (1000001000, 1000001000, 'thisfilegrows') || die "$!";

	open (GROWFILE, '>', 'sameallbutcontents') || die "$!";
	print GROWFILE "def\n";
	close (GROWFILE) || die "$!";
	utime (1000000000, 1000000000, 'sameallbutcontents') || die "$!";

	unlink ('symlinkchangingtarget') || die "$!";
	symlink ('righttarget', 'symlinkchangingtarget') || die "$!";

	&touch ('dir2/file', 'these are different file contents right here');
	&chownifroot (441, 442, 'dir2/file') || die "$!";
	utime (1000001000, 1000001000, 'dir2/file') || die "$!";

	# scrambled hard links

	unlink ('linkdir2/e') || die "$!";

	rename ('linkdir2/f', 'linkdir1/f') || die "$!";

	unlink ('linkdir2/c') || die "$!";
	unlink ('linkdir2/d') || die "$!";

	link ('linkdir1/a', 'linkdir2/d') || die "$!";
	link ('linkdir1/b', 'linkdir2/c') || die "$!";

	utime (1000001000, 1000001000, 'linkdir1/a') || die "$!";
	utime (1000001000, 1000001000, 'linkdir1/b') || die "$!";
	utime (1000001000, 1000001000, 'linkdir1/f') || die "$!";
	utime (1000001000, 1000001000, 'linkdir2/c') || die "$!";
	utime (1000001000, 1000001000, 'linkdir2/d') || die "$!";
	utime (1000001000, 1000001000, 'linkdir1') || die "$!";
	utime (1000001000, 1000001000, 'linkdir2') || die "$!";

	# hard links that mutate

	(!(system ('cp', '-a', 'hardlinkmutate2', 'hardlinkmutate5') / 256)) || die "$!";
	unlink ('hardlinkmutate2') || die "$!";
	link ('hardlinkmutate5', 'hardlinkmutate2') || die "$!";
	link ('hardlinkmutate5', 'hardlinkmutate6') || die "$!";
	unlink ('hardlinkmutate3') || die "$!";
	link ('hardlinkmutate1', 'hardlinkmutate3') || die "$!";
	utime (1000000000, 1000000000, 'hardlinkmutate1') || die "$!";
	utime (1000000000, 1000000000, 'hardlinkmutate2') || die "$!";

	unlink ('hl-c') || die "$!";
	mkdir ('hl-c', 0755) || die "$!";
	utime (1000000000, 1000000000, 'hl-c') || die "$!";

	# hard link inside a later-named deleted dir

	mkdir ('zzzprod', 0755) || die "$!";
	mkdir ('zzzprod/.hg', 0755) || die "$!";
	link ('prod/.hg/requires', 'zzzprod/.hg/requires') || die "$!";
	utime (1000000000, 1000000000, 'zzzprod/.hg') || die "$!";
	utime (1000000000, 1000000000, 'zzzprod') || die "$!";

	# change just one stat

	&chownifroot (11, 2, 'file') || die "$!";
	utime (1000001000, 1000001000, 'file') || die "$!";

	&chownifroot (3, 12, 'dir') || die "$!";
	utime (1000001000, 1000001000, 'dir') || die "$!";

	if ($opt_n == 0) {
		utime (1000001000, 1000001000, 'unreadablefile') || die "$!";
	}

	open (GREW, '>', 'space between') || die "$!";
	print GREW "got bigger\n";
	close (GREW) || die "$!";
	utime (1000000000, 1000000000, 'space between') || die "$!";

	if ($> == 0) {
		&chownifroot (7, 8, 'pipesugt') || die "$!";
		# chown strips suid/sgid/sticky
		chmod (07113, 'pipesugt') || die "$!";
		utime (1000000000, 1000000000, 'pipesugt') || die "$!";
	}

	# subdirs of sgid dir

	mkdir ('sgiddir/bar', 0755) || die "$!";
	chmod (0755, 'sgiddir/bar') || die "$!";
	&chownifroot (0, 0, 'sgiddir/bar') || die "$!";
	&touch ('sgiddir/baz');
	chmod (0644, 'sgiddir/baz') || die "$!";
	&chownifroot (0, 0, 'sgiddir/baz') || die "$!";
	utime (1000001000, 1000001000, 'sgiddir/bar') || die "$!";
	utime (1000001000, 1000001000, 'sgiddir/baz') || die "$!";
	utime (1000001000, 1000001000, 'sgiddir') || die "$!";

	# complicated? bug

	rmdir ('layers2-bbb/foo') || die "$!";
	rmdir ('layers2-bbb') || die "$!";
	mkdir ('layers2-aaa', 0755) || die "$!";
	mkdir ('layers2-aaa/foo', 0755) || die "$!";
	utime (1000000000, 1000000000, 'layers2-aaa') || die "$!";
	utime (1000000000, 1000000000, 'layers2-aaa/foo') || die "$!";
	symlink ('layers2-aaa', 'layers2-bbb') || die "$!";

	# top level directory mtime

	utime (1000001000, 1000001000, '.') || die "$!";

} elsif ($ARGV[0] eq '3') {

	chmod (0755, 't_names/unreadabledir');
	(!(system ('rm', '-rf', 't_names') / 256)) || die "$!";

} else {

	die "Usage: $0 [-n] [1|2|3]\n";
}
