
#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

use strict;

use Socket;

$main::unique_file_utime_pair_counter = 0;

sub unique_file_utime_pair {

	my $uniqtime;

	++$main::unique_file_utime_pair_counter;

	if ($main::unique_file_utime_pair_counter > 9999) {
		die "unique_file_utime_pair: over 9999 files created, ran out of numbers";
	}

# 133668720
#  1234
#      1234

	$uniqtime = sprintf ("1%04d%04d", $ARGV[0], $main::unique_file_utime_pair_counter);

	return ($uniqtime, $uniqtime);
}

sub makefile {

	my $filename = shift;

	my $mode;

	$filename =~ s#^(\s)#./$1#;
	open (TOUCH, '>', "$filename\0") || die "$!";
	print TOUCH rand (1000) . "\n";
	close (TOUCH) || die "$!";
	&chownifroot (int (rand (1000)), int (rand (1000)), $filename) || die "$!";
	$mode = int (rand (4095));
	if ($> != 0) {
		$mode |= 0400;
	}
	chmod ($mode, $filename) || die "$!";
	utime (&unique_file_utime_pair, $filename) || die "$!";
}

sub chownifroot {

	my $owner = shift;
	my $group = shift;
	my $filename = shift;

	if ($> == 0) {
		return (chown ($owner, $group, $filename));
	} else {
		return (1);
	}
}

sub makedir {

	my $filename = shift;

	my $mode;

	mkdir ($filename, 0755) || die "$!";
	&chownifroot (int (rand (1000)), int (rand (1000)), $filename) || die "$!";
	$mode = int (rand (4095));
	if ($> != 0) {
		$mode |= 0700;
	}
	chmod ($mode, $filename) || die "$!";
	utime (1000000000 + int (rand (1000)), 1000000000 + int (rand (1000)), $filename) || die "$!";
}	

sub makesymlink {

	my $filename = shift;

	symlink (int (rand (1000)), $filename) || die "$!";
}	

sub makechar {

	my $filename = shift;

	if ($> eq 0) {
		(!(system ('mknod', $filename, 'c', int (rand (100)), int (rand (100)) / 256))) || die "$!";
		&chownifroot (int (rand (1000)), int (rand (1000)), $filename) || die "$!";
		chmod (int (rand (4095)), $filename) || die "$!";
		utime (1000000000 + int (rand (1000)), 1000000000 + int (rand (1000)), $filename) || die "$!";
	}
}	

sub makeblock {

	my $filename = shift;

	if ($> eq 0) {
		(!(system ('mknod', $filename, 'b', int (rand (100)), int (rand (100)) / 256))) || die "$!";
		&chownifroot (int (rand (1000)), int (rand (1000)), $filename) || die "$!";
		chmod (int (rand (4095)), $filename) || die "$!";
		utime (1000000000 + int (rand (1000)), 1000000000 + int (rand (1000)), $filename) || die "$!";
	}
}	

sub makefifo {

	my $filename = shift;

	(!(system ('mkfifo', $filename) / 256)) || die "$!";
	&chownifroot (int (rand (1000)), int (rand (1000)), $filename) || die "$!";
	chmod (int (rand (4095)), $filename) || die "$!";
	utime (1000000000 + int (rand (1000)), 1000000000 + int (rand (1000)), $filename) || die "$!";
}	

sub makesocket {

	my $filename = shift;

	unlink ($filename);
	socket (theserver, &main::PF_UNIX, &main::SOCK_STREAM, 0) || die "$!";
	bind (theserver, &main::sockaddr_un ($filename)) || die "$!";
	&chownifroot (int (rand (1000)), int (rand (1000)), $filename) || die "$!";
	chmod (int (rand (4095)), $filename) || die "$!";
	utime (1000000000 + int (rand (1000)), 1000000000 + int (rand (1000)), $filename) || die "$!";
}

if (!(defined ($ARGV[0]))) {
	die "Usage: $0 0|1|2|...|9999\n";
}

(!(system ('rm', '-rf', 't_random_tree') / 256)) || die "$!";

if (($ARGV[0] >= 1) && ($ARGV[0] <= 9999)) {

	# make test directory under cd

	mkdir ('t_random_tree', 0755) || die "$!";
	&chownifroot (0, 0, 't_random_tree') || die "$!";
	chdir ('t_random_tree') || die "$!";

	# weakly random, repeatable on a given host and perl version

	srand ($ARGV[0]);

	my $i;

	for ($i = 0;  $i < 500;  $i++) {
		&fill_with_random_contents;
	}

	mkdir ('sub', 0755) || die "$!";
	&chownifroot (0, 0, 'sub') || die "$!";
	chdir ('sub') || die "$!";

	for ($i = 0;  $i < 500;  $i++) {
		&fill_with_random_contents;
	}

	chdir ('..') || die "$!";

	utime (1000000000, 1000000000, 'sub') || die "$!";
	utime (1000000000, 1000000000, '.') || die "$!";
}

sub fill_with_random_contents {

	my $filename;
	my $whichtype;
	my $i;
	my $randval;

	# fill with random contents

	$filename = int (rand (1000));
	if ((!(-e $filename)) && (!(-l $filename))) {

		$whichtype = int (rand (10));

		if ($whichtype <= 4) {
			# Directory
			&makedir ($filename);

			$randval = int (rand (100));
			if ($randval > 90) {
				chdir ($filename) || die "$!";
				for ($i = 0;  $i < 10;  $i++) {
					&fill_with_random_contents;
				}
				chdir ('..') || die "$!";
			}
		} elsif ($whichtype == 5) {
			# Regular file
			&makefile ($filename);
		} elsif ($whichtype == 6) {
			# Symbolic link
			&makesymlink ($filename);
		} elsif ($whichtype == 7) {
			# Character device
			&makechar ($filename);
		} elsif ($whichtype == 8) {
			# Block device
			&makeblock ($filename);
		} elsif ($whichtype == 9) {
			# FIFO
			&makefifo ($filename);
		} elsif ($whichtype == 10) {
			# Socket
			&makesocket ($filename);
		} else {
			# do nothing
		}
	}
}
