
#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#  Usage:
#
#	mkt_treatment_tree 1
#
#	[test dist]
#
#	mkt_treatment_tree 2
#
#	[test dist]

use Socket;
use Carp;

sub makesocket {

	my $socketname = shift;

	unlink ($socketname);
	socket (theserver, &main::PF_UNIX, &main::SOCK_STREAM, 0) || die "$!";
	bind (theserver, &main::sockaddr_un ($socketname)) || die "$!";
}

sub chownifroot {

	my $owner = shift;
	my $group = shift;
	my $filename = shift;

	if ($> == 0) {
		chown ($owner, $group, $filename) || die "$!";
	}
}

sub utimeifroot {

	my $mtime = shift;
	my $name = shift;

	if ($> == 0) {
		utime ($mtime, $mtime, $name) || die "$!";
	}
}

sub chmodifroot {

	my $mode = shift;
	my $name = shift;

	if ($> == 0) {
		chmod ($mode, $name) || die "$!";
	}
}

sub mknodifroot {

	my $filename = shift;
	my $cb = shift;
	my $rdev = shift;

	if ($> == 0) {
		(!(system ('mknod', $filename, $cb, $rdev, '1') / 256)) || die "$!";
	}
}

sub touch {

	my $filename = shift;
	my $contents = shift;

	$filename =~ s#^(\s)#./$1#;
	open (TOUCH, '>', "$filename\0") || die "$!, filename $filename";
	if (defined ($contents)) {
		print TOUCH $contents;
	}
	close (TOUCH) || die "$!";
}

sub makeit {

	my $type = shift;
	my $owner = shift;
	my $group = shift;
	my $mode = shift;
	my $mtime = shift;
	my $content = shift;
	my $symlinktarget = shift;
	my $rdev = shift;
	my $name = shift;

###	warn "sub makeit type $type name $name\n";

	if ($type eq "f") {
		&touch ($name, $content);
		&chownifroot ($owner, $group, $name);
		chmod ($mode, $name) || die "$!";
		utime ($mtime, $mtime, $name) || die "$!";
	} elsif ($type eq "d") {
		mkdir ($name, $mode) || die "$!";
		&chownifroot ($owner, $group, $name);
		chmod ($mode, $name) || die "$!";
		utime ($mtime, $mtime, $name) || die "$!";
	} elsif ($type eq "s") {
		symlink ($symlinktarget, $name) || die "$!";
	} elsif ($type eq "p") {
		(!(system ('mkfifo', $name) / 256)) || die "$!";
		&chownifroot ($owner, $group, $name);
		chmod ($mode, $name) || die "$!";
		utime ($mtime, $mtime, $name) || die "$!";
	} elsif ($type eq "c") {
		&mknodifroot ($name, 'c', $rdev);
		&chownifroot ($owner, $group, $name);
		&chmodifroot ($mode, $name);
		&utimeifroot ($mtime, $name);
	} elsif ($type eq "b") {
		&mknodifroot ($name, 'b', $rdev);
		&chownifroot ($owner, $group, $name);
		&chmodifroot ($mode, $name);
		&utimeifroot ($mtime, $name);
	} elsif ($type eq "k") {
		&makesocket ($name);
		&chownifroot ($owner, $group, $name);
		chmod ($mode, $name) || die "$!";
		utime ($mtime, $mtime, $name) || die "$!";
	} else {
		die "unknown type $type";
	}
}

#--

if (($ARGV[0] eq '1') || ($ARGV[0] eq '')) {

#--

(!(system ('rm', '-rf', 't_treatment') / 256)) || die "$!";
mkdir ('t_treatment', 0755) || die "$!";
chdir ('t_treatment') || die "$!";

mkdir ("source") || die "$!";
mkdir ("source_right") || die "$!";
mkdir ("target") || die "$!";
mkdir ("target_right") || die "$!";

#--

$counter = 100;

#==============================================================================
#==============================================================================
#==============================================================================

$i = $treatment = 101;

mkdir ("source/$i") || die "$!";
mkdir ("target/$i") || die "$!";
mkdir ("target_right/$i") || die "$!";

#--

foreach $type ("f", "d", "s", "p", "c", "b", "k") {

	$name = "$type-same";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-new";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-changed-owner";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner + 100, $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner + 100, $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-changed-group";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-changed-owner-group";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner + 100, $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner + 100, $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-wrong-mode";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00711;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $othermode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-wrong-mtime";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-wrong-mode-time";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00711;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $othermode, $mtime + 10000, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content, $symlinktarget, $rdev, "target_right/$i/$name");
}

foreach $type ("f") {

	$name = "$type-wrong-mtime-size-contents";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime,         $content,      $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime + 10000, $othercontent, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime,         $content,      $symlinktarget, $rdev, "target_right/$i/$name");
}

foreach $type ("s") {

	$name = "$type-wrong-target";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "source/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $othersymlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "target_right/$i/$name");
}

foreach $type ("c", "b") {

	$name = "$type-wrong-rdev";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "source/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev + 100, "target/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "target_right/$i/$name");
}

#--

utime (1000000000, 1000000000, "source/$i") || die "$!";
utime (1000000000, 1000000000, "target/$i") || die "$!";
utime (1000000000, 1000000000, "target_right/$i") || die "$!";

#==============================================================================
#==============================================================================
#==============================================================================

$i = $treatment = 102;

mkdir ("source/$i") || die "$!";
mkdir ("target/$i") || die "$!";
mkdir ("target_right/$i") || die "$!";

#--

foreach $type ("f", "d", "s", "p", "c", "b", "k") {

	$name = "$type-same";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$content = "e";
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,       $content,      $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,       $content,      $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,       $content,      $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-old";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$othermode = 00600;
	$mtime = $counter++;
	$content = "a";
	$othercontent = "aa";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner + 100, $group + 100, $othermode, $mtime + 10000, $othercontent, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner + 100, $group + 100, $othermode, $mtime + 10000, $othercontent, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-new";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$content = "b";
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,       $mtime,      $content,     $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,       $mtime,      $content,     $symlinktarget, $rdev, "target_right/$i/$name");
}

foreach $type ("s") {

	$name = "$type-wrong-target";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "source/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $othersymlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "target_right/$i/$name");
}

foreach $type ("c", "b") {

	$name = "$type-wrong-rdev";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "source/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev + 100, "target/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "target_right/$i/$name");
}

#--

utime (1000000000, 1000000000, "source/$i") || die "$!";
utime (1000000000, 1000000000, "target/$i") || die "$!";
utime (1000000000, 1000000000, "target_right/$i") || die "$!";

#==============================================================================
#==============================================================================
#==============================================================================

$i = $treatment = 104;

mkdir ("source/$i") || die "$!";
mkdir ("target/$i") || die "$!";
mkdir ("target_right/$i") || die "$!";

#--

foreach $type ("f", "d", "s", "p", "c", "b", "k") {

	$name = "$type-same";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$content = "h";
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,       $content,      $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,       $content,      $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,       $content,      $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-old";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$othermode = 00600;
	$mtime = $counter++;
	$content = "g";
	$othercontent = "gg";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;
	$otherrdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $othercontent, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $othercontent, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-old-changed-mtime-only";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$othermode = 00600;
	$mtime = $counter++;
	$content = "g";
	$othercontent = "gg";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;
	$otherrdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,      "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content,      $symlinktarget,      $rdev,      "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content,      $symlinktarget,      $rdev,      "target_right/$i/$name");

	$name = "$type-new";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$content = "b";
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,       $mtime,      $content,     $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,       $mtime,      $content,     $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-wrong-owner";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner + 100, $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-wrong-group";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-wrong-owner-group";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner + 100, $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-wrong-mode";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00711;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $othermode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");
}

foreach $type ("s") {

	$name = "$type-wrong-target";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "source/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $othersymlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "target_right/$i/$name");
}

foreach $type ("c", "b") {

	$name = "$type-wrong-rdev";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "source/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev + 100, "target/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "target_right/$i/$name");
}

#--

utime (1000000000, 1000000000, "source/$i") || die "$!";
utime (1000000000, 1000000000, "target/$i") || die "$!";
utime (1000000000, 1000000000, "target_right/$i") || die "$!";

#==============================================================================
#==============================================================================
#==============================================================================

$treatment = 105;

mkdir ("source/$treatment") || die "$!";
mkdir ("target/$treatment") || die "$!";
mkdir ("target_right/$treatment") || die "$!";

#--

foreach $type ("f", "d", "s", "p", "c", "b", "k") {

	$name = "$type-same";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$content = "h";
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,       $content,      $symlinktarget, $rdev, "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,       $content,      $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,       $content,      $symlinktarget, $rdev, "target_right/$treatment/$name");

	$name = "$type-old";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$othermode = 00600;
	$mtime = $counter++;
	$content = "g";
	$othercontent = "gg";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;
	$otherrdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget, $rdev, "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $othercontent, $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $othercontent, $symlinktarget, $rdev, "target_right/$treatment/$name");

	$name = "$type-old-changed-mtime-only";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$othermode = 00600;
	$mtime = $counter++;
	$content = "g";
	$othercontent = "gg";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;
	$otherrdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,      "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content,      $symlinktarget,      $rdev,      "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content,      $symlinktarget,      $rdev,      "target_right/$treatment/$name");

	$name = "$type-new";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$content = "b";
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,       $mtime,      $content,     $symlinktarget, $rdev, "source/$treatment/$name");

	$name = "$type-wrong-owner";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$treatment/$name");
	&makeit ($type, $owner + 100, $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/$name");

	$name = "$type-wrong-group";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$treatment/$name");
	&makeit ($type, $owner,       $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/$name");

	$name = "$type-wrong-owner-group";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$treatment/$name");
	&makeit ($type, $owner + 100, $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/$name");

	$name = "$type-wrong-mode";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00711;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $othermode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/$name");
}

foreach $type ("s") {

	$name = "$type-wrong-target";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "source/$treatment/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $othersymlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "target_right/$treatment/$name");
}

foreach $type ("c", "b") {

	$name = "$type-wrong-rdev";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev + 100, "target/$treatment/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "target_right/$treatment/$name");
}

#--

utime (1000000000, 1000000000, "source/$treatment") || die "$!";
utime (1000000000, 1000000000, "target/$treatment") || die "$!";
utime (1000000000, 1000000000, "target_right/$treatment") || die "$!";

#==============================================================================
#==============================================================================
#==============================================================================

$treatment = 106;

mkdir ("source/$treatment") || die "$!";
mkdir ("source/$treatment/dir") || die "$!";
mkdir ("target/$treatment") || die "$!";
mkdir ("target/$treatment/dir") || die "$!";
mkdir ("target_right/$treatment") || die "$!";
mkdir ("target_right/$treatment/dir") || die "$!";

mkdir ("$treatment") || die "$!";
mkdir ("$treatment/dir") || die "$!";

#--

foreach $type ("f", "d", "s", "p", "c", "b", "k") {

	$name = "$type-same";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/dir/$name");

	$name = "$type-new";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/dir/$name");

	$name = "$type-changed-owner";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner + 100, $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/dir/$name");

	$name = "$type-changed-group";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner,       $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/dir/$name");

	$name = "$type-changed-owner-group";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner + 100, $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/dir/$name");

	$name = "$type-wrong-mode";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00711;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $othermode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/dir/$name");

	$name = "$type-wrong-mtime";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content, $symlinktarget, $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content, $symlinktarget, $rdev, "target/$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content, $symlinktarget, $rdev, "target_right/$treatment/dir/$name");

	$name = "$type-wrong-mode-time";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00711;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content, $symlinktarget, $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $othermode, $mtime + 10000, $content, $symlinktarget, $rdev, "target/$treatment/dir/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content, $symlinktarget, $rdev, "target_right/$treatment/dir/$name");
}

foreach $type ("f") {

	$name = "$type-wrong-mtime-size-contents";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime,         $content,      $symlinktarget, $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner, $group, $mode, $mtime + 10000, $othercontent, $symlinktarget, $rdev, "target/$treatment/dir/$name");
	&makeit ($type, $owner, $group, $mode, $mtime,         $content,      $symlinktarget, $rdev, "target_right/$treatment/dir/$name");
}

foreach $type ("d") {

	$name = "$type-wrong-ogmt-correct-to-up";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00751;
	$othermod2 = 00750;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget, $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner + 100, $group + 100, $othermode, $mtime + 10000, $content,      $symlinktarget, $rdev, "source/$treatment/dir/$name");
	&makeit ($type, $owner + 200, $group + 200, $othermod2, $mtime + 20000, $content,      $symlinktarget, $rdev, "target/$treatment/dir/$name");
	&makeit ($type, $owner + 100, $group + 100, $othermode, $mtime + 10000, $content,      $symlinktarget, $rdev, "target_right/$treatment/dir/$name");
}

foreach $type ("s") {

	$name = "$type-wrong-target";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "$treatment/dir/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $othersymlinktarget, $rdev, "target/$treatment/dir/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "target_right/$treatment/dir/$name");
}

foreach $type ("c", "b") {

	$name = "$type-wrong-rdev";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "$treatment/dir/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev + 100, "target/$treatment/dir/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "target_right/$treatment/dir/$name");
}

#--

utime (1000000000, 1000000000, "source/$treatment/dir") || die "$!";
utime (1000000000, 1000000000, "source/$treatment") || die "$!";
utime (1000000000, 1000000000, "target/$treatment/dir") || die "$!";
utime (1000000000, 1000000000, "target/$treatment") || die "$!";
utime (1000000000, 1000000000, "target_right/$treatment/dir") || die "$!";
utime (1000000000, 1000000000, "target_right/$treatment") || die "$!";

utime (1010101010, 1010101010, "$treatment/dir") || die "$!";
utime (1010101010, 1010101010, "$treatment") || die "$!";

#==============================================================================
#==============================================================================
#==============================================================================

$i = $treatment = 107;

mkdir ("source/$i") || die "$!";
mkdir ("target/$i") || die "$!";
mkdir ("target_right/$i") || die "$!";

#--

foreach $type ("f", "d", "s", "p", "c", "b", "k") {

	$name = "$type-same";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-new";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-changed-owner";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner + 100, $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner + 100, $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-changed-group";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-changed-owner-group";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner + 100, $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner + 100, $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-wrong-mode";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00711;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $othermode, $mtime, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-wrong-mtime";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content, $symlinktarget, $rdev, "target_right/$i/$name");

	$name = "$type-wrong-mode-time";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00711;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content, $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner,       $group,       $othermode, $mtime + 10000, $content, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content, $symlinktarget, $rdev, "target_right/$i/$name");
}

foreach $type ("f") {

	$name = "$type-wrong-mtime-size-contents";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime,         $content,      $symlinktarget, $rdev, "source/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime + 10000, $othercontent, $symlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime,         $content,      $symlinktarget, $rdev, "target_right/$i/$name");
}

foreach $type ("s") {

	$name = "$type-wrong-target";
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "source/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $othersymlinktarget, $rdev, "target/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget,      $rdev, "target_right/$i/$name");
}

foreach $type ("c", "b") {

	$name = "$type-wrong-rdev";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00644;
	$mtime = $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "source/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev + 100, "target/$i/$name");
	&makeit ($type, $owner, $group, $mode, $mtime, $content, $symlinktarget, $rdev,       "target_right/$i/$name");
}

#--

utime (1000000000, 1000000000, "source/$i") || die "$!";
utime (1000000000, 1000000000, "target/$i") || die "$!";
utime (1000000000, 1000000000, "target_right/$i") || die "$!";

#==============================================================================
#==============================================================================
#==============================================================================

$treatment = 103;

mkdir ("source/$treatment") || die "$!";
mkdir ("source/$treatment/dir") || die "$!";
mkdir ("target/$treatment") || die "$!";
mkdir ("target/$treatment/dir") || die "$!";
mkdir ("target_right/$treatment") || die "$!";
mkdir ("target_right/$treatment/dir") || die "$!";

mkdir ("$treatment") || die "$!";
mkdir ("$treatment/dir") || die "$!";

#--

foreach $type ("f", "d", "s", "p", "c", "b", "k") {

	$name = "$type-all-pieces";
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "def",               4,           "$treatment/dir/$name");
	&makeit ($type, 5,            6,            0752,       70000,          "ghi",         "jkl",               8,           "source/$treatment/dir/$name");
	&makeit ($type, 9,            10,           0753,       11000,          "mno",         "pqr",               12,          "target/$treatment/dir/$name");
    if ($type eq "d") {
	&makeit ($type, 5,            6,            0752,       70000,          "",            "",                  0,           "target_right/$treatment/dir/$name");
    } else {
	&makeit ($type, 5,            6,            0752,       3,              "abc",         "def",               4,           "target_right/$treatment/dir/$name");
    }

	$name = "$type-no-source";
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "def",               4,           "$treatment/dir/$name");
	&makeit ($type, 9,            10,           0753,       11000,          "mno",         "pqr",               12,          "target/$treatment/dir/$name");
    if ($type eq "d") {
	&makeit ($type, 0,            0,            0755,       3,              "",            "",                  0,           "target_right/$treatment/dir/$name");
    } else {
	&makeit ($type, 0,            0,            0644,       3,              "abc",         "def",               4,           "target_right/$treatment/dir/$name");
    }

	$name = "$type-no-target";
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "def",               4,           "$treatment/dir/$name");
	&makeit ($type, 5,            6,            0752,       70000,          "ghi",         "jkl",               8,           "source/$treatment/dir/$name");
    if ($type eq "d") {
	&makeit ($type, 5,            6,            0752,       70000,          "",            "",                  0,           "target_right/$treatment/dir/$name");
    } else {
	&makeit ($type, 5,            6,            0752,       3,              "abc",         "def",               4,           "target_right/$treatment/dir/$name");
    }

	$name = "$type-no-source-no-target";
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "def",               4,           "$treatment/dir/$name");
    if ($type eq "d") {
	&makeit ($type, 0,            0,            0755,       3,              "",            "",                  0,           "target_right/$treatment/dir/$name");
    } else {
	&makeit ($type, 0,            0,            0644,       3,              "abc",         "def",               4,           "target_right/$treatment/dir/$name");
    }
}

foreach $type ("f") {

	$name = "$type-mtime-wrong";
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "source/$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       40000,          "abc",         "",                  0,           "target/$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "target_right/$treatment/dir/$name");

	$name = "$type-size-contents-wrong";
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "source/$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "ABCDEF",      "",                  0,           "target/$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "target_right/$treatment/dir/$name");

	$name = "$type-mtime-size-contents-wrong";
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "source/$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       40000,          "ABCDEF",      "",                  0,           "target/$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "target_right/$treatment/dir/$name");

	$name = "$type-contents-wrong";
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "source/$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "DEF",         "",                  0,           "target/$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "target_right/$treatment/dir/$name");

	$name = "$type-ogm-wrong";
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "source/$treatment/dir/$name");
	&makeit ($type, 11,           12,           0752,       3,              "abc",         "",                  0,           "target/$treatment/dir/$name");
	&makeit ($type, 1,            2,            0751,       3,              "abc",         "",                  0,           "target_right/$treatment/dir/$name");

	$name = "$type-no-source-mtime-wrong";
	&makeit ($type, 1,            2,            0755,       3,              "abc",         "",                  0,           "$treatment/dir/$name");
	&makeit ($type, 0,            0,            0644,       40000,          "abc",         "",                  0,           "target/$treatment/dir/$name");
	&makeit ($type, 0,            0,            0644,       3,              "abc",         "",                  0,           "target_right/$treatment/dir/$name");

	$name = "$type-no-source-size-contents-wrong";
	&makeit ($type, 1,            2,            0755,       3,              "abc",         "",                  0,           "$treatment/dir/$name");
	&makeit ($type, 0,            0,            0644,       3,              "ABCDEF",      "",                  0,           "target/$treatment/dir/$name");
	&makeit ($type, 0,            0,            0644,       3,              "abc",         "",                  0,           "target_right/$treatment/dir/$name");

	$name = "$type-no-source-mtime-size-contents-wrong";
	&makeit ($type, 1,            2,            0755,       3,              "abc",         "",                  0,           "$treatment/dir/$name");
	&makeit ($type, 0,            0,            0644,       40000,          "ABCDEF",      "",                  0,           "target/$treatment/dir/$name");
	&makeit ($type, 0,            0,            0644,       3,              "abc",         "",                  0,           "target_right/$treatment/dir/$name");

	$name = "$type-no-source-contents-wrong";
	&makeit ($type, 1,            2,            0755,       3,              "abc",         "",                  0,           "$treatment/dir/$name");
	&makeit ($type, 0,            0,            0644,       3,              "DEF",         "",                  0,           "target/$treatment/dir/$name");
	&makeit ($type, 0,            0,            0644,       3,              "abc",         "",                  0,           "target_right/$treatment/dir/$name");

	$name = "$type-no-source-ogm-wrong";
	&makeit ($type, 1,            2,            0755,       3,              "abc",         "",                  0,           "$treatment/dir/$name");
	&makeit ($type, 11,           12,           0660,       3,              "abc",         "",                  0,           "target/$treatment/dir/$name");
	&makeit ($type, 0,            0,            0644,       3,              "abc",         "",                  0,           "target_right/$treatment/dir/$name");
}

#--

utime (1000000000, 1000000000, "source/$treatment/dir") || die "$!";
utime (1000000000, 1000000000, "source/$treatment") || die "$!";
utime (1000000000, 1000000000, "target/$treatment/dir") || die "$!";
utime (1000000000, 1000000000, "target/$treatment") || die "$!";
utime (1000000000, 1000000000, "target_right/$treatment/dir") || die "$!";
utime (1000000000, 1000000000, "target_right/$treatment") || die "$!";

utime (1010101010, 1010101010, "$treatment/dir") || die "$!";
utime (1010101010, 1010101010, "$treatment") || die "$!";

#==============================================================================
#==============================================================================
#==============================================================================

$treatment = 207;

mkdir ("source/$treatment") || die "$!";
mkdir ("target/$treatment") || die "$!";
mkdir ("target_right/$treatment") || die "$!";

#--

foreach $type ("f", "d", "s", "p", "c", "b", "k") {

	$name = "$type-new";
	$owner = 4000;
	$group = 4001;
	$mode = 00751;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/$name");

	$name = "$type-wrong-owner";
	$owner = 4000;
	$group = 4001;
	$mode = 00751;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner + 100, $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/$name");

	$name = "$type-wrong-group";
	$owner = 4000;
	$group = 4001;
	$mode = 00751;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/$name");

	$name = "$type-wrong-owner-group";
	$owner = 4000;
	$group = 4001;
	$mode = 00751;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner + 100, $group + 100, $mode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode, $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/$name");

	$name = "$type-wrong-mode";
	$owner = 4000;
	$group = 4001;
	$mode = 00751;
	$othermode = 00711;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $othermode, $mtime, $content, $symlinktarget, $rdev, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime, $content, $symlinktarget, $rdev, "target_right/$treatment/$name");
}

#--

utime (1000000000, 1000000000, "source/$treatment") || die "$!";
utime (1000000000, 1000000000, "target/$treatment") || die "$!";
utime (1000000000, 1000000000, "target_right/$treatment") || die "$!";

#==============================================================================
#==============================================================================
#==============================================================================

$treatment = 113;

mkdir ("source/$treatment") || die "$!";
mkdir ("target/$treatment") || die "$!";
mkdir ("target_right/$treatment") || die "$!";

mkdir ("$treatment") || die "$!";

#--

foreach $type ("f", "d", "s", "p", "c", "b", "k") {

	$name = "$type-changed-mtime";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00711;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content,      $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");

	$name = "$type-new";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");

	$name = "$type-changed-owner-mtime";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner + 100, $group,       $mode,      $mtime + 10000, $content,      $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner + 200, $group,       $mode,      $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner + 100, $group,       $mode,      $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");

	$name = "$type-changed-group-mtime";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group + 100, $mode,      $mtime + 10000, $content,      $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner,       $group + 200, $mode,      $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner,       $group + 100, $mode,      $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");

	$name = "$type-changed-owner-group-mtime";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner + 100, $group + 100, $mode,      $mtime + 10000, $content,      $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner + 200, $group + 200, $mode,      $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner + 100, $group + 100, $mode,      $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");

	$name = "$type-wrong-mode-changed-mtime";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$othermode = 00711;
	$othermod2 = 00700;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group,       $othermode, $mtime + 10000, $content,      $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $othermod2, $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $othermode, $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");
}

foreach $type ("f") {

	$name = "$type-same-mtime-right-size-right-contents";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");

	$name = "$type-changed-mtime-right-size-right-contents";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $content,      $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 20000, $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");

	$name = "$type-changed-mtime-wrong-size-wrong-contents";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "mno";
	$othercontent = "pqrstu";
	$otherconten2 = "vwxyza";

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $otherconten2, $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 20000, $othercontent, $symlinktarget,      $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");

	$name = "$type-changed-mtime-right-size-wrong-contents";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "abc";
	$othercontent = "def";
	$otherconten2 = "ghi";

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 10000, $otherconten2, $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime + 20000, $othercontent, $symlinktarget,      $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");

	$name = "$type-same-mtime-right-size-wrong-contents";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "ghi";
	$othercontent = "jkl";
	$otherconten2 = "def";

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $otherconten2, $symlinktarget,      $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $othercontent, $symlinktarget,      $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");
}

foreach $type ("s") {

	$name = "$type-wrong-target";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$othersymlinktarge2 = "$counter$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $othersymlinktarget, $rdev,       "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $othersymlinktarge2, $rdev,       "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");
}

foreach $type ("c", "b") {

	$name = "$type-wrong-rdev";
	$owner = $counter++;
	$group = $counter++;
	$mode = 00755;
	$mtime = $counter++;
	$content = "$counter"; $counter++;
	$othercontent = "$counter$counter"; $counter++;
	$symlinktarget = "$counter" ; $counter++;
	$othersymlinktarget = "$counter$counter" ; $counter++;
	$othersymlinktarge2 = "$counter$counter" ; $counter++;
	$rdev = $counter++;

	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev + 100, "source/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev + 200, "target/$treatment/$name");
	&makeit ($type, $owner,       $group,       $mode,      $mtime,         $content,      $symlinktarget,      $rdev,       "target_right/$treatment/$name");
}

#--

utime (1000000000, 1000000000, "source/$treatment") || die "$!";
utime (1000000000, 1000000000, "target/$treatment") || die "$!";
utime (1000000000, 1000000000, "target_right/$treatment") || die "$!";

utime (1010101010, 1010101010, "$treatment") || die "$!";

#==============================================================================
#==============================================================================
#==============================================================================

utime (1000000000, 1000000000, "source") || die "$!";
utime (1000000000, 1000000000, "target") || die "$!";
utime (1000000000, 1000000000, "target_right") || die "$!";
utime (1000000000, 1000000000, ".") || die "$!";

(!(system ("(cd source ; tar -cf - . 2> /dev/null) | (cd source_right ; tar -xpf -)") / 256)) || die "$!";

utime (1000000000, 1000000000, "source_right") || die "$!";

exit (0);

} elsif ($ARGV[0] eq '2') {

chdir ('t_treatment') || die "$!";

#foreach $treatment (101, 102, 104, 105, 207, 113) {
#
#	foreach $type ("f", "d", "s", "p", "c", "b", "k") {
#
#		$name = "$type-new-new-new";
#		unlink ("target/$treatment/$name");
#		&makeit ($type, 119,          1110,         0773,       800000,        "xxyyzz",      "zzEEpp",           1112,         "target/$treatment/$name");
#		unlink ("source_right/$treatment/$name");
#		&makeit ($type, 115,          116,          0773,       800000,        "xxyyzz",      "zzEEpp",           1112,         "source_right/$treatment/$name");
#	}
#}

#foreach $treatment (106, 103) {
foreach $treatment (103) {

	foreach $type ("f", "d", "s", "p", "c", "b", "k") {

		$name = "$type-new-new-new";
		unlink ("target/$treatment/dir/$name");
		&makeit ($type, 119,          1110,         0773,       800000,        "xxyyzz",      "zzEEpp",           1112,         "target/$treatment/dir/$name");
		unlink ("source_right/$treatment/dir/$name");
		&makeit ($type, 115,          116,          0773,       800000,        "xxyyzz",      "zzEEpp",           1112,         "source_right/$treatment/dir/$name");
	}
}

exit (0);

} else {
	die "Usage: $0 [[1]|2]\n";
}
