#!/usr/bin/perl

# Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

use strict;

my $trtables;
my $tmt;
my $latexfilename;
my $perlfilename;

$trtables = {

'0' => {
	'normal' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'r',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
	},
	'inverse' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'r',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		}
	}
},

'101' => {
	'normal' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'i',  'i',  'u',  'r',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'i',  'i',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'i',  'i',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'i',  'i',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'i',  'i',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'i',  'i',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 't',  't',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 't',  't',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 't',  't',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 't',  't',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 't',  't',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 't',  't',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
	},
	'inverse' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'i',  'i',  'u',  'r',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'i',  'i',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'i',  'i',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'i',  'i',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'i',  'i',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'i',  'i',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 't',  't',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 't',  't',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 't',  't',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 't',  't',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 't',  't',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 't',  't',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		}
	}
},

'102' => {
	'normal' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'i',  'i',  'i',  'i',  'i',  'i',  'i',  ' ',  ' ' ],
				'd' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 't',  't',  't',  't',  't',  't',  't',  ' ',  ' ' ],
				'd' => [ 't',  't',  't',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 't',  't',  't',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 't',  't',  't',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 't',  't',  't',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 't',  't',  't',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
	},
	'inverse' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'i',  'i',  'i',  'i',  'i',  'i',  'i',  ' ',  ' ' ],
				'd' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 't',  't',  't',  't',  't',  't',  't',  ' ',  ' ' ],
				'd' => [ 't',  't',  't',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 't',  't',  't',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 't',  't',  't',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 't',  't',  't',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 't',  't',  't',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		}
	}
},

'103' => {
	'normal' => {
		'split_stats_content' => 'source',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'u',  'u',  'u',  'u',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'u',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
	},
	'inverse' => {
		'split_stats_content' => 'target',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'stats' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'i',  'i',  'i',  'i',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'i',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'i' ],
				'b' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'i' ],
				'k' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
# XXX is this t for f t correct??
				'f' => [ 's',  's',  's',  't',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
		'content' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'i',  'i',  'i',  'r',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
# XXX is this t for f t correct??
				'f' => [ 's',  's',  's',  't',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
# XXX is this t for f t correct??
				'f' => [ 's',  's',  's',  't',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		}
	}
},

'113' => {
	'normal' => {
		'split_stats_content' => 'source',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'u',  'u',  'u',  'u',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'i',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  't',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
	},
	'inverse' => {
		'split_stats_content' => 'target',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'stats' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'i',  'i',  'i',  'i',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'i',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'i' ],
				'b' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'i' ],
				'k' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  't',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
		'content' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'i',  'i',  'i',  'i',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'i',  'i',  'i',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  't',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		}
	}
},

'104' => {
	'normal' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'i',  'i',  'i',  'i',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  't',  't',  't',  't',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  '0',  ' ',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
	},
	'inverse' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'i',  'i',  'i',  'i',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  't',  't',  't',  't',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  '0',  ' ',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		}
	}
},

'105' => {
	'normal' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'i',  'i',  'i',  'i',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  't',  't',  't',  't',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
	},
	'inverse' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'i',  'i',  'i',  'i',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  't',  't',  't',  't',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		}
	}
},

'106' => {
	'normal' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'r',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
	},
	'inverse' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'r',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		}
	}
},

'107' => {
	'normal' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'i',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  't',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
	},
	'inverse' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'u',  'u',  'u',  'i',  'r',  'r',  'r',  ' ',  ' ' ],
				'd' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
				'p' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'b' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  'r' ],
				'k' => [ 'u',  'u',  'u',  'i',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  't',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  't',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		}
	}
},

'109' => {
	'normal' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'i',  'i',  'i',  'u',  'i',  'i',  'i',  ' ',  ' ' ],
				'd' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'i',  ' ' ],
				'p' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  'i' ],
				'b' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  'i' ],
				'k' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		},
	},
	'inverse' => {
		'split_stats_content' => '',
		# Up / Mine
		'layers' => {
			       #  o     g     m     t     s     c     k     l     r
			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
		},
		'both' => {
			# Ignore / Update / Recreate
			'compare' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 'i',  'i',  'i',  'u',  'i',  'i',  'i',  ' ',  ' ' ],
				'd' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'i',  ' ' ],
				'p' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  'i' ],
				'b' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  'i' ],
				'k' => [ 'i',  'i',  'i',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'update' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			},
			# Source / Target
			'recreate' => {
				       #  o     g     m     t     s     c     k     l     r
				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
			}
		}
	}
},

#'108' => {
#	'normal' => {
#		'split_stats_content' => '',
#		# Up / Mine
#		'layers' => {
#			       #  o     g     m     t     s     c     k     l     r
#			'f' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			'd' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
#			'p' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			'c' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			'b' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			'k' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#		},
#		'both' => {
#			# Ignore / Update / Recreate
#			'compare' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'd' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'i',  ' ' ],
#				'p' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'b' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'k' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			},
#			# Source / Target
#			'update' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'd' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  't',  ' ' ],
#				'p' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'b' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'k' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			},
#			# Source / Target
#			'recreate' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'd' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
#				'p' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'b' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'k' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			}
#		},
#	},
#	'inverse' => {
#		'split_stats_content' => '',
#		# Up / Mine
#		'layers' => {
#			       #  o     g     m     t     s     c     k     l     r
#			'f' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			'd' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
#			'p' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			'c' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			'b' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			'k' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#		},
#		'both' => {
#			# Ignore / Update / Recreate
#			'compare' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'd' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'i',  ' ' ],
#				'p' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'b' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'k' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			},
#			# Source / Target
#			'update' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'd' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  't',  ' ' ],
#				'p' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'b' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'k' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			},
#			# Source / Target
#			'recreate' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'd' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
#				'p' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'b' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'k' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			}
#		}
#	}
#},
#
#'207' => {
#	'normal' => {
#		'split_stats_content' => '',
#		# Up / Mine
#		'layers' => {
#			       #  o     g     m     t     s     c     k     l     r
#			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
#			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
#			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
#			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
#			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#		},
#		'both' => {
#			# Ignore / Update / Recreate
#			'compare' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ 'u',  'u',  'u',  'r',  'r',  'r',  'r',  ' ',  ' ' ],
#				'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
#				'p' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
#				'b' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
#				'k' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			},
#			# Source / Target
#			'update' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
#				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
#				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
#				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
#				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			},
#			# Source / Target
#			'recreate' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
#				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
#				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
#				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
#				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			}
#		},
#	},
#	'inverse' => {
#		'split_stats_content' => '',
#		# Up / Mine
#		'layers' => {
#			       #  o     g     m     t     s     c     k     l     r
#			'f' => [ 'm',  'm',  'm',  'm',  'm',  'm',  'm',  ' ',  ' ' ],
#			'd' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'm',  ' ' ],
#			'p' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#			'c' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
#			'b' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  'm' ],
#			'k' => [ 'm',  'm',  'm',  'm',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#		},
#		'both' => {
#			# Ignore / Update / Recreate
#			'compare' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ 'u',  'u',  'u',  'r',  'r',  'r',  'r',  ' ',  ' ' ],
#				'd' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  'r',  ' ' ],
#				'p' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
#				'b' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  'r' ],
#				'k' => [ 'u',  'u',  'u',  'u',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			},
#			# Source / Target
#			'update' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
#				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
#				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
#				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
#				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			},
#			# Source / Target
#			'recreate' => {
#				       #  o     g     m     t     s     c     k     l     r
#				'f' => [ 's',  's',  's',  's',  's',  's',  's',  ' ',  ' ' ],
#				'd' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				's' => [ ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  's',  ' ' ],
#				'p' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ],
#				'c' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
#				'b' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  's' ],
#				'k' => [ 's',  's',  's',  's',  ' ',  ' ',  ' ',  ' ',  ' ' ]
#			}
#		}
#	}
#}

};

#-------------------------------------------------------------------------------

#&print_trtables ($trtables);
#exit (99);

&write_layers;

foreach $tmt (sort (keys ($trtables))) {
	&latexify_treatment ($trtables, $tmt);
}

$perlfilename = "mkt_trtab2.pl";
open (PERLFILE, '>', $perlfilename) || die "open '$perlfilename' failed with error '$!'";
print PERLFILE &perl_top;
print PERLFILE &perl_middle;
print PERLFILE &perl_bottom;
close (PERLFILE);
chmod (0755, $perlfilename) || die "$!";

exit (0);

#-----

sub write_layers {

	my $layers;
	my $i;

	foreach $i ({ 'sts' => 'source', 'stt' => 'target', 'ni' => 'normal' }, { 'sts' => 'target', 'stt' => 'source', 'ni' => 'inverse' }) {

		$layers = q|
#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

{

'source_layers' => [

#	{
#		'mount' => [ '', 't_trtab2/source_upup', 'all' ],
#		'treatments' => [
#			[0],
#		]
#	},
	{
		'mount' => [ '', 't_trtab2/| . $i->{'sts'} . q|-both-| . $i->{'ni'} . q|', 'all' ],
		'treatments' => [
			[0]
		]
	},
	{
		'mount' => [ '101', 't_trtab2/| . $i->{'sts'} . q|-both-| . $i->{'ni'} . q|/101', 'all' ],
		'treatments' => [
			[101]
		]
	},
	{
		'mount' => [ '102', 't_trtab2/| . $i->{'sts'} . q|-both-| . $i->{'ni'} . q|/102', 'all' ],
		'treatments' => [
			[102]
		]
	},
	{
		'mount' => [ '104', 't_trtab2/| . $i->{'sts'} . q|-both-| . $i->{'ni'} . q|/104', 'all' ],
		'treatments' => [
			[104]
		]
	},
	{
		'mount' => [ '105', 't_trtab2/| . $i->{'sts'} . q|-both-| . $i->{'ni'} . q|/105', 'all' ],
		'treatments' => [
			[105]
		]
	},
	{
		'mount' => [ '106', 't_trtab2/| . $i->{'sts'} . q|-both-| . $i->{'ni'} . q|/106', 'all' ],
		'treatments' => [
			[106]
		]
	},
	{
		'mount' => [ '107', 't_trtab2/| . $i->{'sts'} . q|-both-| . $i->{'ni'} . q|/107', 'all' ],
		'treatments' => [
			[107]
		]
	},
	{
		'mount' => [ '109', 't_trtab2/| . $i->{'sts'} . q|-both-| . $i->{'ni'} . q|/109', 'all' ],
		'treatments' => [
			[109]
		]
	},
#	{
#		'mount' => [ '108', 't_trtab2/| . $i->{'sts'} . q|-both-| . $i->{'ni'} . q|/108', 'all' ],
#		'treatments' => [
#			[108]
#		]
#	},
	{
		'mount' => [ '103', 't_trtab2/| . $i->{'sts'} . q|-stats-| . $i->{'ni'} . q|/103', 'all' ],
		'treatments' => [
			[0]
		]
	},
	{
		'mount' => [ '103', 't_trtab2/| . $i->{'sts'} . q|-content-| . $i->{'ni'} . q|/103', 'all' ],
		'treatments' => [
			[103]
		]
	},
	{
		'mount' => [ '113', 't_trtab2/| . $i->{'sts'} . q|-stats-| . $i->{'ni'} . q|/113', 'all' ],
		'treatments' => [
			[0]
		]
	},
	{
		'mount' => [ '113', 't_trtab2/| . $i->{'sts'} . q|-content-| . $i->{'ni'} . q|/113', 'all' ],
		'treatments' => [
			[113]
		]
	}
],

'target_layers' => [
	{
		'mount' => [ '', 't_trtab2/| . $i->{'stt'} . q|-both-| . $i->{'ni'} . q|', 'all' ],
		'treatments' => [
			[0],
#			[207,	[
#				'^207/'
#				],
#				[ 33, 33, 0733 ]	# owner, group, mode
#			]
		]
	}
]

}
|;

		open (LAYERSFILE, '>', "test-trtab2-$i->{'ni'}.hflayers") || die "open 'test-trtab2-$i->{'ni'}.hflayers' failed with error '$!'";
		print LAYERSFILE $layers;
		close (LAYERSFILE);
	}
}

#-----

sub print_trtables {

	my $trtables = shift;

	my $tmt;
	my $ni;
	my $sc;
	my $row;
	my $col;
	my @splitlist;

	foreach $tmt (sort (keys ($trtables))) {
		warn "treatment $tmt\n";
		foreach $ni ('normal', 'inverse') {
			foreach $row ('f', 'd', 's', 'p', 'c', 'b', 'k') {
				warn "tmt $tmt ni $ni LAYERS\trow $row  " .
sprintf ("o=%s  g=%s  m=%s  t=%s  s=%s  c=%s  k=%s  l=%s  r=%s", @{$trtables->{$tmt}{$ni}{'layers'}{$row}}) . "\n";
			}
#			if (($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'source') || ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target')) {
			if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
				@splitlist = ('stats', 'content');
			} else {
				@splitlist = ('both');
			}

			foreach $sc (@splitlist) {
				foreach $row ('f', 'd', 's', 'p', 'c', 'b', 'k') {
					warn "tmt $tmt ni $ni sc $sc COMPARE\trow $row  " .
sprintf ("o=%s  g=%s  m=%s  t=%s  s=%s  c=%s  k=%s  l=%s  r=%s", @{$trtables->{$tmt}{$ni}{$sc}{'compare'}{$row}}) . "\n";
				}
				foreach $row ('f', 'd', 's', 'p', 'c', 'b', 'k') {
					warn "tmt $tmt ni $ni sc $sc UPDATE\trow $row  " .
sprintf ("o=%s  g=%s  m=%s  t=%s  s=%s  c=%s  k=%s  l=%s  r=%s", @{$trtables->{$tmt}{$ni}{$sc}{'update'}{$row}}) . "\n";
				}
				foreach $row ('f', 'd', 's', 'p', 'c', 'b', 'k') {
					warn "tmt $tmt ni $ni sc $sc RECREATE\trow $row  " .
sprintf ("o=%s  g=%s  m=%s  t=%s  s=%s  c=%s  k=%s  l=%s  r=%s", @{$trtables->{$tmt}{$ni}{$sc}{'recreate'}{$row}}) . "\n";
				}
			}
		}
	}
}

sub latexify_treatment {

	my $trtables = shift;
	my $tmt = shift;

	my $ni;
	my $sc;
	my $latex;
	my $latexfilename;

	foreach $ni ('normal', 'inverse') {

		$latex = &latexify_both ($trtables->{$tmt}{$ni}{'layers'}, $tmt, $ni, 'both', 'layers');

#		if (($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'source') || ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target')) {
		if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
			$latex .= &latexify_stats_content ($trtables->{$tmt}{$ni}, $tmt, $ni, 'compare');
			$latex .= &latexify_stats_content ($trtables->{$tmt}{$ni}, $tmt, $ni, 'update');
			$latex .= &latexify_stats_content ($trtables->{$tmt}{$ni}, $tmt, $ni, 'recreate');
		} else {
			$latex .= &latexify_both ($trtables->{$tmt}{$ni}{'both'}{'compare'}, $tmt, $ni, 'both', 'compare');
			$latex .= &latexify_both ($trtables->{$tmt}{$ni}{'both'}{'update'}, $tmt, $ni, 'both', 'update');
			$latex .= &latexify_both ($trtables->{$tmt}{$ni}{'both'}{'recreate'}, $tmt, $ni, 'both', 'recreate');
		}

		$latexfilename = "$tmt-$ni.tex";
		open (LATEXFILE, '>', $latexfilename) || die "open '$latexfilename' failed with error '$!'";
		print LATEXFILE $latex;
		close (LATEXFILE);
	}

	return (1);
}

sub latexify_both {

	my $proptable = shift;
	my $tmt = shift;
	my $ni = shift;
	my $sc = shift;
	my $func = shift;

	my $row;
	my $latex;
	my $caption;
	my $label;

	$latex = q|\begin{table}[!htp]
\centering
\begin{tabular}{p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}}
\toprule
  && \multicolumn{9}{c}{| . 'Properties' . q|} \\\\
T && o & g & m & t & s & c & k & l & r \\\\
\midrule
|;
	foreach $row ('f', 'd', 's', 'p', 'c', 'b', 'k') {
		$latex .= sprintf ("%s && %s & %s & %s & %s & %s & %s & %s & %s & %s \\\\\n",
			$row, @{$proptable->{$row}});
	}

	if ($func eq 'compare') {
		$label = "Treatment $tmt direction " . ucfirst ($ni) . " function " . ucfirst ($func);
		$caption = ucfirst ($func) . " [Ignore / Update / Recreate]";
	} elsif (($func eq 'update') || ($func eq 'recreate')) {
		$label = "Treatment $tmt direction " . ucfirst ($ni) . " function " . ucfirst ($func);
		$caption = ucfirst ($func) . " [Ignore / Source / Target]";
	} elsif ($func eq 'layers') {
		$label = "Treatment $tmt direction " . ucfirst ($ni) . " function " . ucfirst ($func);
		$caption = ucfirst ($func) . " [Up / Mine]";
	} else {
		die "unknown func $func";
	}
	$label =~ s/\s/_/g;

	$latex .= q|\bottomrule
\end{tabular}
\caption{| . $caption . q|}
\label{table:| . $label . q|}
\end{table}
|;
	return ($latex);
}

sub latexify_stats_content {

	my $proptree = shift;
	my $tmt = shift;
	my $ni = shift;
	my $func = shift;

	my $row;
	my $latex;
	my $caption;
	my $label;

	$latex = q|\begin{table}[!htp]
\centering
\begin{tabular}{p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}p{0.5em}}
\toprule
  && \multicolumn{9}{c}{| . 'Stats' . q|} && \multicolumn{9}{c}{| . 'Contents' . q|} \\\\
T && o & g & m & t & s & c & k & l & r && o & g & m & t & s & c & k & l & r \\\\
\midrule
|;
	foreach $row ('f', 'd', 's', 'p', 'c', 'b', 'k') {
		$latex .= sprintf ("%s && %s & %s & %s & %s & %s & %s & %s & %s & %s && ",
			$row, @{$proptree->{'stats'}{$func}{$row}});
		$latex .= sprintf ("%s & %s & %s & %s & %s & %s & %s & %s & %s \\\\\n",
			@{$proptree->{'content'}{$func}{$row}});
	}

	if ($func eq 'compare') {
		$label = "Treatment $tmt direction " . ucfirst ($ni) . " function " . ucfirst ($func);
		$caption = ucfirst ($func) . " [Ignore / Update / Recreate]";
	} elsif (($func eq 'update') || ($func eq 'recreate')) {
		$label = "Treatment $tmt direction " . ucfirst ($ni) . " function " . ucfirst ($func);
		$caption = ucfirst ($func) . " [Ignore / Source / Target]";
	} elsif ($func eq 'layers') {
		$label = "Treatment $tmt direction " . ucfirst ($ni) . " function " . ucfirst ($func);
		$caption = ucfirst ($func) . " [Up / Mine]";
	} else {
		die "unknown func $func";
	}
	$label =~ s/\s/_/g;

	$latex .= q|\bottomrule
\end{tabular}
\caption{| . $caption . q|}
\label{table:| . $label . q|}
\end{table}
|;
	return ($latex);
}

sub perl_top {

	return (q{#!/usr/bin/perl

#  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2002, 2003, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Brian Bartholomew

#  Usage:
#
#	treatment_test2.pl
#
#	[forward test dist]
#	[reverse test dist]

use Socket;
use Carp;

#--

(!(system ('rm', '-rf', 't_trtab2') / 256)) || die "$!";
mkdir ('t_trtab2', 0755) || die "$!";
chdir ('t_trtab2') || die "$!";

mkdir ("source-both-normal") || die "$!";
mkdir ("source-stats-normal") || die "$!";
mkdir ("source-content-normal") || die "$!";
mkdir ("target-both-normal") || die "$!";
mkdir ("target-both-normal-right") || die "$!";

mkdir ("source-both-inverse") || die "$!";
mkdir ("target-both-inverse") || die "$!";
mkdir ("target-both-inverse-right") || die "$!";
mkdir ("target-stats-inverse") || die "$!";
mkdir ("target-stats-inverse-right") || die "$!";
mkdir ("target-content-inverse") || die "$!";
mkdir ("target-content-inverse-right") || die "$!";

$n = 0;

#--

sub mkdir_top {

	my $treatment = shift;

#	if (!(-e "source/$treatment")) {
#		mkdir ("source/$treatment") || die "$!";
#		mkdir ("target/$treatment") || die "$!";
#		mkdir ("target_right/$treatment") || die "$!";
#
#		mkdir ("source_sr/$treatment") || die "$!";
#		mkdir ("target_sr/$treatment") || die "$!";
#		mkdir ("source_sr_right/$treatment") || die "$!";
#
#		mkdir ("source_up/$treatment") || die "$!";
#	}

	mkdir ("source-both-inverse/$treatment") || die "$!";
	mkdir ("source-both-normal/$treatment") || die "$!";
	mkdir ("source-content-normal/$treatment") || die "$!";
	mkdir ("source-stats-normal/$treatment") || die "$!";
	mkdir ("target-both-inverse/$treatment") || die "$!";
	mkdir ("target-both-inverse-right/$treatment") || die "$!";
	mkdir ("target-both-normal/$treatment") || die "$!";
	mkdir ("target-both-normal-right/$treatment") || die "$!";
	mkdir ("target-content-inverse/$treatment") || die "$!";
	mkdir ("target-content-inverse-right/$treatment") || die "$!";
	mkdir ("target-stats-inverse/$treatment") || die "$!";
	mkdir ("target-stats-inverse-right/$treatment") || die "$!";

}

sub utime_bottom {

	my $treatment = shift;

	utime (1000000000, 1000000000, "source-both-inverse/$treatment") || die "$!";
	utime (1000000000, 1000000000, "source-both-normal/$treatment") || die "$!";
	utime (1000000000, 1000000000, "source-content-normal/$treatment") || die "$!";
	utime (1000000000, 1000000000, "source-stats-normal/$treatment") || die "$!";
	utime (1000000000, 1000000000, "target-both-inverse/$treatment") || die "$!";
	utime (1000000000, 1000000000, "target-both-inverse-right/$treatment") || die "$!";
	utime (1000000000, 1000000000, "target-both-normal/$treatment") || die "$!";
	utime (1000000000, 1000000000, "target-both-normal-right/$treatment") || die "$!";
	utime (1000000000, 1000000000, "target-content-inverse/$treatment") || die "$!";
	utime (1000000000, 1000000000, "target-content-inverse-right/$treatment") || die "$!";
	utime (1000000000, 1000000000, "target-stats-inverse/$treatment") || die "$!";
	utime (1000000000, 1000000000, "target-stats-inverse-right/$treatment") || die "$!";
}

sub mkdir_top_alt {

	my $treatment = shift;

#	if (!(-e "source_alt/$treatment")) {
#		mkdir ("source_alt/$treatment") || die "$!";
#
#		mkdir ("source_alt_sr/$treatment") || die "$!";
#		mkdir ("source_alt_sr_right/$treatment") || die "$!";
#	}
}

sub utime_bottom_alt {

	my $treatment = shift;

#	utime (1000000000, 1000000000, "source_alt/$treatment") || die "$!";
#
#	utime (1000000000, 1000000000, "source_alt_sr/$treatment") || die "$!";
#	utime (1000000000, 1000000000, "source_alt_sr_right/$treatment") || die "$!";
}

#--

#$A   = "00";
#$bb  = "11";
#$CCC = "22";

$A   = "11";
$bb  = "22";
$CCC = "33";

});

}

sub perl_middle {

# these stats aren't checked: set them to $A in source and 77 in target
#
# these stats cause updates: do one test with all of them set them to
# $bb in target and check the results
#
# these stats cause recreates: test each stat changed one at a time from
# $A to bb and check each result set against the same set of declared
# results
#
# new file, check result against declared recreate result set
#
# exercise various ATTENTION conditions and check results

	my $perl;
	my $tmt;
	my $ni;
	my $sc;
	my $row;
	my $col;
	my @rowval_ss;
	my @rowval_sc;
	my @rowval_sb;
	my @rowval_ts;
	my @rowval_tc;
	my @rowval_tb;
	my @rowval_rs;
	my @rowval_rc;
	my @rowval_rb;
	my @rowval_r;
	my @rowval_same;
	my $ur;
	my $gen;
	my $rowval_ss_ref;
	my $rowval_sc_ref;
	my $rowval_sb_ref;
	my $rowval_rs_ref;
	my $rowval_rc_ref;
	my $rowval_rb_ref;

	$perl = '';

	@rowval_same = ('$A  ', '$A  ', '$A  ', '$A  ', '$A  ', '$A  ', '$A  ', '$A  ', '$A  ');

	foreach $tmt (sort (keys ($trtables))) {
		$perl .= "\&mkdir_top ($tmt);\n";
		foreach $ni ('normal', 'inverse') {
			foreach $row ('f', 'd', 's', 'p', 'c', 'b', 'k') {
				$perl .= "# tmt $tmt ni $ni row $row\n";

	@rowval_ss = @rowval_sc = @rowval_sb = @rowval_ts = @rowval_tc = @rowval_tb = @rowval_rs = @rowval_rc = @rowval_rb = @rowval_r = @rowval_same;
if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'source') {
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'source-stats-$ni/$tmt/$row-sam'")          . ", '" . $row . "', " . join (' ,', @rowval_ss) . ");\n";
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'source-content-$ni/$tmt/$row-sam'")        . ", '" . $row . "', " . join (' ,', @rowval_sc) . ");\n";
} else {
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'source-both-$ni/$tmt/$row-sam'")           . ", '" . $row . "', " . join (' ,', @rowval_sb) . ");\n";
}
if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-stats-$ni/$tmt/$row-sam'")          . ", '" . $row . "', " . join (' ,', @rowval_ts) . ");\n";
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-content-$ni/$tmt/$row-sam'")        . ", '" . $row . "', " . join (' ,', @rowval_tc) . ");\n";
} else {
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-both-$ni/$tmt/$row-sam'")           . ", '" . $row . "', " . join (' ,', @rowval_tb) . ");\n";
}
if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-stats-$ni-right/$tmt/$row-sam'")    . ", '" . $row . "', " . join (' ,', @rowval_sb) . "); # LOOK 3\n";
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-content-$ni-right/$tmt/$row-sam'")  . ", '" . $row . "', " . join (' ,', @rowval_sb) . "); # LOOK 4\n";
} else {
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-both-$ni-right/$tmt/$row-sam'")     . ", '" . $row . "', " . join (' ,', @rowval_sb) . "); # LOOK 1\n";
}

	@rowval_ss = @rowval_sc = @rowval_sb = @rowval_ts = @rowval_tc = @rowval_tb = @rowval_rs = @rowval_rc = @rowval_rb = @rowval_r = @rowval_same;
if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'source') {
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'source-stats-$ni/$tmt/$row-new'")          . ", '" . $row . "', " . join (' ,', @rowval_ss) . ");\n";
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'source-content-$ni/$tmt/$row-new'")        . ", '" . $row . "', " . join (' ,', @rowval_sc) . ");\n";
} else {
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'source-both-$ni/$tmt/$row-new'")           . ", '" . $row . "', " . join (' ,', @rowval_sb) . ");\n";
}
if (!((($tmt == 105) && ($ni eq 'normal')) || (($tmt == 109) && ($ni eq 'normal')))) {
    if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
if (!((($tmt == 103) && ($ni eq 'inverse')) || (($tmt == 113) && ($ni eq 'inverse')))) {
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-stats-$ni-right/$tmt/$row-new'")    . ", '" . $row . "', " . join (' ,', @rowval_sb) . "); # LOOK 5\n";
}
	$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-content-$ni-right/$tmt/$row-new'")  . ", '" . $row . "', " . join (' ,', @rowval_sb) . "); # LOOK 6\n";
    } else {
	if (($row eq 'f') && ($tmt == 104) && ($ni eq 'normal')) {
	    # XXX hack, should respond to zero in table
	    $perl .= q|&makeit2 (| . sprintf ("%40s", "'target-both-$ni-right/$tmt/$row-new'")     . q|, 'f', $A   ,$A   ,$A   ,$A   ,0    ,''   ,$A   ,$A   ,$A  ); # LOOK 222 XXX
|;
	} else {
	    $perl .= q|&makeit2 (| . sprintf ("%40s", "'target-both-$ni-right/$tmt/$row-new'")     . ", '" . $row . "', " . join (' ,', @rowval_sb) . "); # LOOK 2\n";
	}
    }
}

	for ($col = 0;  $col <= 8;  $col++) {
if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
		if (!(
			(($trtables->{$tmt}{$ni}{'stats'}{'compare'}{$row}[$col] eq ' ') && ($trtables->{$tmt}{$ni}{'content'}{'compare'}{$row}[$col] eq ' ')) ||
			(($trtables->{$tmt}{$ni}{'stats'}{'compare'}{$row}[$col] eq 'i') && ($trtables->{$tmt}{$ni}{'content'}{'compare'}{$row}[$col] eq 'i')) ||
			(($trtables->{$tmt}{$ni}{'stats'}{'compare'}{$row}[$col] eq 'i') && ($trtables->{$tmt}{$ni}{'content'}{'compare'}{$row}[$col] eq 'u')) ||
			(($trtables->{$tmt}{$ni}{'stats'}{'compare'}{$row}[$col] eq 'i') && ($trtables->{$tmt}{$ni}{'content'}{'compare'}{$row}[$col] eq 'r')) ||
			(($trtables->{$tmt}{$ni}{'stats'}{'compare'}{$row}[$col] eq 'u') && ($trtables->{$tmt}{$ni}{'content'}{'compare'}{$row}[$col] eq 'i')) ||
			(($trtables->{$tmt}{$ni}{'stats'}{'compare'}{$row}[$col] eq 'r') && ($trtables->{$tmt}{$ni}{'content'}{'compare'}{$row}[$col] eq 'i'))		)) {

			die "invalid combination of field letters in stats and content compare";
		}
}
#		foreach $ur ('u', 'r', 'i') {
		foreach $ur ('u', 'r') {
			$gen = 0;
			@rowval_ss = @rowval_sc = @rowval_sb = @rowval_ts = @rowval_tc = @rowval_tb = @rowval_rs = @rowval_rc = @rowval_rb = @rowval_r = @rowval_same;
#			if (($ur eq 'u') || ($ur eq 'r')) {
if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'source') {
				if ($trtables->{$tmt}{$ni}{'both'}{'compare'}{$row}[$col] eq $ur) {
					$rowval_ss[$col] = '$bb ';
					$rowval_sc[$col] = '$CCC';
### $rowval_r[$col] = ???
					$gen = 1;
				}
} elsif ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
				if ($trtables->{$tmt}{$ni}{'stats'}{'compare'}{$row}[$col] eq $ur) {
					$rowval_ts[$col] = '$bb ';
### $rowval_r[$col] = ???
					$gen = 1;
				}
				if ($trtables->{$tmt}{$ni}{'content'}{'compare'}{$row}[$col] eq $ur) {
					$rowval_tc[$col] = '$CCC';
### $rowval_r[$col] = ???
					$gen = 1;
				}
} else {
				if ($trtables->{$tmt}{$ni}{'both'}{'compare'}{$row}[$col] eq $ur) {
					$rowval_sb[$col] = $rowval_r[$col] = '$bb ';
					$gen = 1;
				}
			}

			if ($gen == 1) {
if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'source') {
				$perl .= q|&makeit2 (| . sprintf ("%40s", "'source-stats-$ni/$tmt/$row-$ur-$col'")         . ", '" . $row . "', " . join (' ,', @rowval_ss) . ");\n";
				$perl .= q|&makeit2 (| . sprintf ("%40s", "'source-content-$ni/$tmt/$row-$ur-$col'")       . ", '" . $row . "', " . join (' ,', @rowval_sc) . ");\n";
} else {
				$perl .= q|&makeit2 (| . sprintf ("%40s", "'source-both-$ni/$tmt/$row-$ur-$col'")          . ", '" . $row . "', " . join (' ,', @rowval_sb) . ");\n";
}
if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
				$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-stats-$ni/$tmt/$row-$ur-$col'")         . ", '" . $row . "', " . join (' ,', @rowval_ts) . ");\n";
				$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-content-$ni/$tmt/$row-$ur-$col'")       . ", '" . $row . "', " . join (' ,', @rowval_tc) . ");\n";
} else {
				$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-both-$ni/$tmt/$row-$ur-$col'")          . ", '" . $row . "', " . join (' ,', @rowval_tb) . ");\n";
}
if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'source') {
				$rowval_rb_ref = &ur_results (5, $tmt, $ni, $row, $ur, \@rowval_ss, \@rowval_sc, \@rowval_tb, undef      );
				$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-both-$ni-right/$tmt/$row-$ur-$col'")     . ", '" . $row . "', " . join (' ,', @{$rowval_rb_ref}) . "); # LOOK 13\n";
} elsif ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
				($rowval_rs_ref, $rowval_rc_ref) = &ur_results (6, $tmt, $ni, $row, $ur, \@rowval_sb, undef,       \@rowval_ts, \@rowval_tc);
				$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-stats-$ni-right/$tmt/$row-$ur-$col'")    . ", '" . $row . "', " . join (' ,', @{$rowval_rs_ref}) . "); # LOOK 10\n";
				$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-content-$ni-right/$tmt/$row-$ur-$col'")  . ", '" . $row . "', " . join (' ,', @{$rowval_rc_ref}) . "); # LOOK 11\n";
} else {
				$rowval_rb_ref = &ur_results (7, $tmt, $ni, $row, $ur, \@rowval_sb, undef      , \@rowval_tb, undef      );
				$perl .= q|&makeit2 (| . sprintf ("%40s", "'target-both-$ni-right/$tmt/$row-$ur-$col'")     . ", '" . $row . "', " . join (' ,', @{$rowval_rb_ref}) . "); # LOOK 12\n";
}



			}
		}
	}

			}
		}
		$perl .= "\&utime_bottom ($tmt);\n";
	}

	return ($perl);
}

sub perl_bottom {

	return (q{
#--

utime (1000000000, 1000000000, "source-both-normal") || die "$!";
utime (1000000000, 1000000000, "source-stats-normal") || die "$!";
utime (1000000000, 1000000000, "source-content-normal") || die "$!";
utime (1000000000, 1000000000, "target-both-normal") || die "$!";
utime (1000000000, 1000000000, "target-both-normal-right") || die "$!";

utime (1000000000, 1000000000, "source-both-inverse") || die "$!";
utime (1000000000, 1000000000, "target-both-inverse") || die "$!";
utime (1000000000, 1000000000, "target-both-inverse-right") || die "$!";
utime (1000000000, 1000000000, "target-stats-inverse") || die "$!";
utime (1000000000, 1000000000, "target-stats-inverse-right") || die "$!";
utime (1000000000, 1000000000, "target-content-inverse") || die "$!";
utime (1000000000, 1000000000, "target-content-inverse-right") || die "$!";

utime (1000000000, 1000000000, ".") || die "$!";

#--

exit (0);

#-----

sub makesocket {

	my $socketname = shift;

	unlink ($socketname);
	socket (theserver, &main::PF_UNIX, &main::SOCK_STREAM, 0) || die "$!";
	bind (theserver, &main::sockaddr_un ($socketname)) || die "$!";
}

sub chownifroot {

	my $owner = shift;
	my $group = shift;
	my $filename = shift;

	if ($> == 0) {
		chown ($owner, $group, $filename) || die "$!";
	}
}

sub utimeifroot {

	my $mtime = shift;
	my $name = shift;

	if ($> == 0) {
		utime ($mtime, $mtime, $name) || die "$!";
	}
}

sub chmodifroot {

	my $mode = shift;
	my $name = shift;

	if ($> == 0) {
		chmod ($mode, $name) || die "$!";
	}
}

sub mknodifroot {

	my $filename = shift;
	my $cb = shift;
	my $rdev = shift;

	if ($> == 0) {
		(!(system ('mknod', $filename, $cb, $rdev, '1') / 256)) || die "$!";
	}
}

sub touch {

	my $filename = shift;
	my $contents = shift;

	$filename =~ s#^(\s)#./$1#;
	open (TOUCH, '>', "$filename\0") || die "open '$filename' failed with error '$!'";
	if (defined ($contents)) {
		print TOUCH $contents;
	}
	close (TOUCH) || die "$!";
}

sub fakemakeit {

	warn join (', ', @_) . "\n";
}

sub makeit2 {

	my $name = shift;
	my $type = shift;
	my $owner = shift;
	my $group = shift;
	my $mode = shift;
	my $mtime = shift;
	my $size = shift;
	my $content = shift;
	my $checksum = shift;
	my $symlinktarget = shift;
	my $rdev = shift;

#	warn "sub makeit name '$name', type '$type', owner '$owner', group '$group', mode '$mode', mtime '$mtime', content '$content', symlinktarget '$symlinktarget', rdev '$rdev'\n";

	if ($type eq "f") {
		&touch ($name, $content);
		&chownifroot ($owner, $group, $name);
		chmod ($mode, $name) || die "$!";
		utime ($mtime, $mtime, $name) || die "$!";
	} elsif ($type eq "d") {
		mkdir ($name, $mode) || die "name $name, error $!";
		&chownifroot ($owner, $group, $name);
		chmod ($mode, $name) || die "$!";
		utime ($mtime, $mtime, $name) || die "$!";
	} elsif ($type eq "s") {
		symlink ($symlinktarget, $name) || die "$!, symlinktarget '$symlinktarget', name '$name'";
	} elsif ($type eq "p") {
		(!(system ('mkfifo', $name) / 256)) || die "$!";
		&chownifroot ($owner, $group, $name);
		chmod ($mode, $name) || die "$!";
		utime ($mtime, $mtime, $name) || die "$!";
	} elsif ($type eq "c") {
		&mknodifroot ($name, 'c', $rdev);
		&chownifroot ($owner, $group, $name);
		&chmodifroot ($mode, $name);
		&utimeifroot ($mtime, $name);
	} elsif ($type eq "b") {
		&mknodifroot ($name, 'b', $rdev);
		&chownifroot ($owner, $group, $name);
		&chmodifroot ($mode, $name);
		&utimeifroot ($mtime, $name);
	} elsif ($type eq "k") {
		&makesocket ($name);
		&chownifroot ($owner, $group, $name);
		chmod ($mode, $name) || die "$!";
		utime ($mtime, $mtime, $name) || die "$!";
	} else {
		die "unknown type $type";
	}
}
});

}

#  ($rowval_rs_ref, $rowval_rc_ref) = &ur_results ($tmt, $ni, $row, $ur, \@rowval_sb, undef,       \@rowval_ts, \@rowval_tc);
#                    $rowval_rb_ref = &ur_results ($tmt, $ni, $row, $ur, \@rowval_ss, \@rowval_sc, \@rowval_tb, undef      );
#                    $rowval_rb_ref = &ur_results ($tmt, $ni, $row, $ur, \@rowval_sb, undef      , \@rowval_tb, undef      );

sub ur_results {

	my $call = shift;
	my $tmt = shift;
	my $ni = shift;
	my $row = shift;
	my $ur = shift;
	my $rowval_ss_ref = shift;
	my $rowval_sc_ref = shift;
	my $rowval_ts_ref = shift;
	my $rowval_tc_ref = shift;

	my $rowval_sb_ref = $rowval_ss_ref;
	my $rowval_tb_ref = $rowval_ts_ref;

	my $col;
	my $urlong;
	my $rowval_rs_ref = [ ];
	my $rowval_rc_ref = [ ];
	my $rowval_rb_ref = [ ];

#	warn "call $call tmt $tmt, ni $ni, ur $ur, row $row\n";

	if ($ur eq 'u') {
		$urlong = 'update';
	} elsif ($ur eq 'r') {
		$urlong = 'recreate';
	} else {
		die "unknown ur '$ur'";
	}

	for ($col = 0;  $col <= 8;  $col++) {

#  $rowval_rb_ref = &ur_results ($tmt, $ni, $row, $ur, \@rowval_sb, undef      , \@rowval_tb, undef      );
		# one source and one result
		if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq '') {
			if ($trtables->{$tmt}{$ni}{'both'}{$urlong}{$row}[$col] eq 's') {
				$rowval_rb_ref->[$col] = $rowval_sb_ref->[$col];
			} elsif ($trtables->{$tmt}{$ni}{'both'}{$urlong}{$row}[$col] eq 't') {
				$rowval_rb_ref->[$col] = $rowval_tb_ref->[$col];
			} else {
				$rowval_rb_ref->[$col] = '031 ';
			}
#  $rowval_rb_ref = &ur_results ($tmt, $ni, $row, $ur, \@rowval_ss, \@rowval_sc, \@rowval_tb, undef      );
		# two sources and one result
		} elsif ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'source') {
			if ($trtables->{$tmt}{$ni}{'both'}{$urlong}{$row}[$col] eq 's') {
				if ($trtables->{$tmt}{$ni}{'layers'}{$row}[$col] eq 'u') {
					$rowval_rb_ref->[$col] = $rowval_ss_ref->[$col];
				} elsif ($trtables->{$tmt}{$ni}{'layers'}{$row}[$col] eq 'm') {
					$rowval_rb_ref->[$col] = $rowval_sc_ref->[$col];
				} else {
					die "not u or m";
				}
			} elsif ($trtables->{$tmt}{$ni}{'both'}{$urlong}{$row}[$col] eq 't') {
				$rowval_rb_ref->[$col] = $rowval_tb_ref->[$col];
			} else {
				$rowval_rb_ref->[$col] = '031 ';
			}
#  ($rowval_rs_ref, $rowval_rc_ref) = &ur_results ($tmt, $ni, $row, $ur, \@rowval_sb, undef,       \@rowval_ts, \@rowval_tc);
		# one source and two results
		} elsif ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
			if ($trtables->{$tmt}{$ni}{'stats'}{$urlong}{$row}[$col] eq 's') {
				$rowval_rs_ref->[$col] = $rowval_sb_ref->[$col];
			} elsif ($trtables->{$tmt}{$ni}{'stats'}{$urlong}{$row}[$col] eq 't') {
				$rowval_rs_ref->[$col] = $rowval_ts_ref->[$col];
			} else {
				$rowval_rs_ref->[$col] = '031 ';
			}
			if ($trtables->{$tmt}{$ni}{'content'}{$urlong}{$row}[$col] eq 's') {
				$rowval_rc_ref->[$col] = $rowval_sb_ref->[$col];
			} elsif ($trtables->{$tmt}{$ni}{'content'}{$urlong}{$row}[$col] eq 't') {
				$rowval_rc_ref->[$col] = $rowval_tc_ref->[$col];
			} else {
				$rowval_rc_ref->[$col] = '051 ';
			}
		} else {
			die "unknown split_stats_content value";
		}
	}

	if ($trtables->{$tmt}{$ni}{'split_stats_content'} eq '') {
		return ($rowval_rb_ref);
	} elsif ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'source') {
		return ($rowval_rb_ref);
	} elsif ($trtables->{$tmt}{$ni}{'split_stats_content'} eq 'target') {
		return ($rowval_rs_ref, $rowval_rc_ref);
	} else {
		die "unknown split_stats_content value";
	}
}
